@@ -77,7 +77,7 @@ class Daemon {
      *            client applications.
      * @param published
      *            The published AddressBook. This address book is published on
-     *            the user's eepsite so that others may subscribe to it.
+     *            the user's I2P Site so that others may subscribe to it.
      *            May be null.
      *            If non-null, overwrite with the new addressbook.
      * @param subscriptions
@@ -111,7 +111,7 @@ class Daemon {
      *            The NamingService to update, generally the root NamingService from the context.
      * @param published
      *            The published AddressBook. This address book is published on
-     *            the user's eepsite so that others may subscribe to it.
+     *            the user's I2P Site so that others may subscribe to it.
      *            May be null.
      *            If non-null, overwrite with the new addressbook.
      * @param subscriptions
@@ -697,7 +697,7 @@ class Daemon {
             StringBuilder buf = new StringBuilder(16);
             final int sz = dests.size();
             for (int i = 0; i < sz; i++) {
-                buf.append(dests.get(i).toBase64().substring(0, 6));
+                buf.append(dests.get(i).toBase64(), 0, 6);
                 if (i != sz - 1)
                     buf.append(", ");
             }
