@@ -58,7 +58,7 @@ public class MetaInfo
   private final String name_utf8;
   private final List<List<String>> files;
   private final List<List<String>> files_utf8;
-  private final BitField paddingFileBitfield;
+  private final List<String> attributes;
   private final List<Long> lengths;
   private final int piece_length;
   private final byte[] piece_hashes;
@@ -82,8 +82,9 @@ public class MetaInfo
    *  @param created_by may be null
    *  @param url_list may be null
    *  @param comment may be null
+   *  @since public since 0.9.53, was package private
    */
-  MetaInfo(String announce, String name, String name_utf8, List<List<String>> files, List<Long> lengths,
+  public MetaInfo(String announce, String name, String name_utf8, List<List<String>> files, List<Long> lengths,
            int piece_length, byte[] piece_hashes, long length, boolean privateTorrent,
            List<List<String>> announce_list, String created_by, List<String> url_list, String comment)
   {
@@ -104,7 +105,7 @@ public class MetaInfo
     this.url_list = url_list;
 
     // TODO BEP 52 hybrid torrent with piece layers, meta version and file tree
-    this.paddingFileBitfield = null;
+    this.attributes = null;
 
     // TODO if we add a parameter for other keys
     //if (other != null) {
@@ -185,11 +186,20 @@ public class MetaInfo
     if (val == null) {
         this.url_list = null;
     } else {
-        List<BEValue> bl1 = val.getList();
-        this.url_list = new ArrayList<String>(bl1.size());
-        for (BEValue bev : bl1) {
-            this.url_list.add(bev.getString());
+        List<String> urllist;
+        try {
+            List<BEValue> bl1 = val.getList();
+            urllist = new ArrayList<String>(bl1.size());
+            for (BEValue bev : bl1) {
+                urllist.add(bev.getString());
+            }
+        } catch (InvalidBEncodingException ibee) {
+            // BEP 19 says it's a list but the example there
+            // is for a single byte string, and we've seen this
+            // in the wild.
+            urllist = Collections.singletonList(val.getString());
         }
+        this.url_list = urllist;
     }
 
     // misc. optional  top-level stuff
@@ -281,7 +291,7 @@ public class MetaInfo
         files = null;
         files_utf8 = null;
         lengths = null;
-        paddingFileBitfield = null;
+        attributes = null;
       }
     else
       {
@@ -299,7 +309,7 @@ public class MetaInfo
         List<List<String>> m_files = new ArrayList<List<String>>(size);
         List<List<String>> m_files_utf8 = null;
         List<Long> m_lengths = new ArrayList<Long>(size);
-        BitField paddingBitfield = null;
+        List<String> m_attributes = null;
         long l = 0;
         for (int i = 0; i < list.size(); i++)
           {
@@ -361,18 +371,23 @@ public class MetaInfo
             val = desc.get("attr");
             if (val != null) {
                 String s = val.getString();
-                if (s.contains("p")) {
-                    if (paddingBitfield == null)
-                        paddingBitfield = new BitField(size);
-                    paddingBitfield.set(i);
+                if (m_attributes == null) {
+                    m_attributes = new ArrayList<String>(size);
+                    for (int j = 0; j < i; j++) {
+                        m_attributes.add("");
+                    }
+                    m_attributes.add(s);
                 }
+            } else {
+                if (m_attributes != null)
+                    m_attributes.add("");
             }
           }
         files = Collections.unmodifiableList(m_files);
         files_utf8 = m_files_utf8 != null ? Collections.unmodifiableList(m_files_utf8) : null;
         lengths = Collections.unmodifiableList(m_lengths);
         length = l;
-        paddingFileBitfield = paddingBitfield;
+        attributes = m_attributes;
       }
 
     info_hash = calculateInfoHash();
@@ -437,9 +452,12 @@ public class MetaInfo
   }
 
   /**
-   * Returns the piece hashes. Only used by storage so package local.
+   *  Returns the piece hashes.
+   *
+   *  @return not a copy, do not modify
+   *  @since public since 0.9.53, was package private
    */
-  byte[] getPieceHashes()
+  public byte[] getPieceHashes()
   {
     return piece_hashes;
   }
@@ -477,9 +495,9 @@ public class MetaInfo
    * @since 0.9.48
    */
   public boolean isPaddingFile(int filenum) {
-      if (paddingFileBitfield == null)
+      if (attributes == null)
           return false;
-      return paddingFileBitfield.get(filenum);
+      return attributes.get(filenum).indexOf('p') >= 0;
   }
 
   /**
@@ -711,8 +729,9 @@ public class MetaInfo
     if (infoMap != null)
         return Collections.unmodifiableMap(infoMap);
     // we should only get here if serving a magnet on a torrent we created
-    if (_log.shouldLog(Log.WARN))
-        _log.warn("Creating new infomap", new Exception());
+    // or on edit torrent save
+    if (_log.shouldDebug())
+        _log.debug("Creating new infomap", new Exception());
     // otherwise we must create it
     Map<String, BEValue> info = new HashMap<String, BEValue>();
     info.put("name", new BEValue(DataHelper.getUTF8(name)));
@@ -749,6 +768,12 @@ public class MetaInfo
                 file.put("path.utf-8", new BEValue(beufiles));
             }
             file.put("length", new BEValue(lengths.get(i)));
+            String attr = null;
+            if (attributes != null) {
+                attr = attributes.get(i);
+                if (attr.length() > 0)
+                    file.put("attr", new BEValue(DataHelper.getASCII(attr)));
+            }
             l.add(new BEValue(file));
           }
         info.put("files", new BEValue(l));
