@@ -42,14 +42,14 @@ import org.klomp.snark.bencode.InvalidBEncodingException;
 
 public class Peer implements Comparable<Peer>
 {
-  private final Log _log = I2PAppContext.getGlobalContext().logManager().getLog(Peer.class);
+  protected final Log _log = I2PAppContext.getGlobalContext().logManager().getLog(getClass());
   // Identifying property, the peer id of the other side.
   private final PeerID peerID;
 
   private final byte[] my_id;
   private final byte[] infohash;
   /** will start out null in magnet mode */
-  private MetaInfo metainfo;
+  protected MetaInfo metainfo;
   private Map<String, BEValue> handshakeMap;
 
   // The data in/output streams set during the handshake and used by
@@ -61,8 +61,11 @@ public class Peer implements Comparable<Peer>
   private final AtomicLong downloaded = new AtomicLong();
   private final AtomicLong uploaded = new AtomicLong();
 
-  // Keeps state for in/out connections.  Non-null when the handshake
-  // was successful, the connection setup and runs
+  /**                                                                    `
+   *  Keeps state for in/out connections.  Non-null when the handshake
+   *  was successful, the connection setup and runs.
+   *  Do not access directly. All actions should be through Peer methods.
+   */
   volatile PeerState state;
 
   /** shared across all peers on this torrent */
@@ -77,8 +80,8 @@ public class Peer implements Comparable<Peer>
 
   final static long CHECK_PERIOD = PeerCoordinator.CHECK_PERIOD; // 40 seconds
   final static int RATE_DEPTH = PeerCoordinator.RATE_DEPTH; // make following arrays RATE_DEPTH long
-  private long uploaded_old[] = {-1,-1,-1};
-  private long downloaded_old[] = {-1,-1,-1};
+  private final long uploaded_old[] = {-1,-1,-1};
+  private final long downloaded_old[] = {-1,-1,-1};
 
   private static final byte[] HANDSHAKE = DataHelper.getASCII("BitTorrent protocol");
   // See BEP 4 for definitions
@@ -341,7 +344,6 @@ public class Peer implements Comparable<Peer>
   private byte[] handshake(InputStream in, OutputStream out)
     throws IOException
   {
-    din = new DataInputStream(in);
     dout = new DataOutputStream(out);
     
     // Handshake write - header
@@ -366,6 +368,7 @@ public class Peer implements Comparable<Peer>
         _log.debug("Wrote my shared hash and ID to " + toString());
     
     // Handshake read - header
+    din = new DataInputStream(in);
     byte b = din.readByte();
     if (b != HANDSHAKE.length)
       throw new IOException("Handshake failure, expected 19, got "
@@ -789,4 +792,12 @@ public class Peer implements Comparable<Peer>
   void setTotalCommentsSent(int count) {
     _totalCommentsSent = count;
   }
+
+  /**
+   * @return false
+   * @since 0.9.49
+   */
+  public boolean isWebPeer() {
+      return false;
+  }
 }
