@@ -87,7 +87,7 @@ public class Storage implements Closeable
   /** bigger than this will be rejected */
   public static final int MAX_PIECE_SIZE = 32*1024*1024;
   /** The maximum number of pieces in a torrent. */
-  public static final int MAX_PIECES = 32*1024;
+  public static final int MAX_PIECES = 64*1024;
   public static final long MAX_TOTAL_SIZE = MAX_PIECE_SIZE * (long) MAX_PIECES;
   public static final int PRIORITY_SKIP = -9;
   public static final int PRIORITY_NORMAL = 0;
@@ -288,9 +288,13 @@ public class Storage implements Closeable
    *  @throws IOException if too many total files
    */
   private void addFiles(List<File> l, File f) throws IOException {
+    int max = _util.getMaxFilesPerTorrent();
     if (!f.isDirectory()) {
-        if (l.size() >= SnarkManager.MAX_FILES_PER_TORRENT)
-            throw new IOException("Too many files, limit is " + SnarkManager.MAX_FILES_PER_TORRENT + ", zip them?");
+        if (l.size() >= max)
+            throw new IOException(_util.getString("Too many files in \"{0}\" ({1})!", metainfo.getName(), l.size()) +
+                                  " - limit is " + max + ", zip them or set " +
+                                  SnarkManager.PROP_MAX_FILES_PER_TORRENT + '=' + l.size() + " in " +
+                                  SnarkManager.CONFIG_FILE + " and restart");
         l.add(f);
     } else {
         File[] files = f.listFiles();
@@ -1388,6 +1392,7 @@ public class Storage implements Closeable
                       //rafs[i].write(bs, off + written, len);
                       pp.write(raf, written, len);
                   } catch (IOException ioe) {
+                      try { tf.closeRAF(); } catch (IOException ioe2) {}
                       // get the file name in the logs
                       IOException ioe2 = new IOException("Error writing " + tf.RAFfile.getAbsolutePath());
                       ioe2.initCause(ioe);
@@ -1494,6 +1499,7 @@ public class Storage implements Closeable
                 raf.seek(start);
                 raf.readFully(bs, read, len);
             } catch (IOException ioe) {
+                try { tf.closeRAF(); } catch (IOException ioe2) {}
                 // get the file name in the logs
                 IOException ioe2 = new IOException("Error reading " + tf.RAFfile.getAbsolutePath());
                 ioe2.initCause(ioe);
