@@ -38,6 +38,7 @@ import net.i2p.data.Destination;
 import net.i2p.data.Hash;
 import net.i2p.i2ptunnel.localServer.LocalHTTPServer;
 import net.i2p.util.ConvertToHash;
+import net.i2p.util.DNSOverHTTPS;
 import net.i2p.util.EventDispatcher;
 import net.i2p.util.Log;
 import net.i2p.util.PortMapper;
@@ -64,11 +65,11 @@ import net.i2p.util.PortMapper;
  * resolve correctly, cookies won't work, etc.
  *
  * Note that http://$b64key/... and http://$b64key.i2p/... are NOT supported, as
- * a b64 key may contain '=' and '~', both of which are illegal host name characters.
+ * a b64 key may contain '=' and '~', both of which are illegal hostname characters.
  * Rewrite as http://i2p/$b64key/...
  *
  * If the $site resolves with the I2P naming service, then it is directed towards
- * that eepsite, otherwise it is directed towards this client's outproxy (typically
+ * that I2P Site, otherwise it is directed towards this client's outproxy (typically
  * "squid.i2p").  Only HTTP and HTTPS are supported (no ftp, mailto, etc).  Both GET
  * and POST have been tested, though other $methods should work.
  *
@@ -76,7 +77,7 @@ import net.i2p.util.PortMapper;
 public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runnable {
 
     /**
-     *  Map of host name to base64 destination for destinations collected
+     *  Map of hostname to base64 destination for destinations collected
      *  via address helper links
      */
     private final ConcurrentHashMap<String, String> addressHelpers = new ConcurrentHashMap<String, String>(8);
@@ -92,7 +93,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                                          "\r\n";
     // ESR version of Firefox, same as Tor Browser
     private static final String UA_CLEARNET = "User-Agent: " +
-                                              "Mozilla/5.0 (Windows NT 10.0; rv:78.0) Gecko/20100101 Firefox/78.0" +
+                                              DNSOverHTTPS.UA_CLEARNET +
                                               "\r\n";
     // overrides
     private static final String PROP_UA_I2P = "httpclient.userAgent.i2p";
@@ -106,7 +107,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: REQUEST DENIED</H1>" +
             "You attempted to connect to a non-I2P website or location.<BR>";
@@ -129,7 +129,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: No outproxy found</H1>" +
             "Your request was for a site outside of I2P, but you have no " +
@@ -140,7 +139,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: Destination key conflict</H1>" +
             "The addresshelper link you followed specifies a different destination key " +
@@ -157,7 +155,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: Helper key not resolvable.</H1>" +
             "The helper key you put for i2paddresshelper= is not resolvable. " +
@@ -169,11 +166,10 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>New Host Name with Address Helper</H1>" +
-            "The address helper link you followed is for a new host name that is not in your address book. " +
-            "You may either save the destination for this host name to your address book, or remember it only until your router restarts. " +
+            "The address helper link you followed is for a new hostname that is not in your address book. " +
+            "You may either save the destination for this hostname to your address book, or remember it only until your router restarts. " +
             "If you save it to your address book, you will not see this message again. " +
             "If you do not wish to visit this host, click the \"back\" button on your browser.";
 
@@ -182,7 +178,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: NON-HTTP PROTOCOL</H1>" +
             "The request uses a bad protocol. " +
@@ -193,7 +188,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: INVALID REQUEST URI</H1>" +
             "The request URI is invalid, and probably contains illegal characters. " +
@@ -204,7 +198,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: REQUEST DENIED</H1>" +
             "Your browser is misconfigured. Do not use the proxy to access the router console or other localhost destinations.<BR>";
@@ -214,7 +207,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: SSL to I2P address rejected</H1>" +
             "SSL to .i2p addresses denied by configuration." +
@@ -269,6 +261,8 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
      * Warning, this does not make a copy of I2PTunnel's client options,
      * it modifies them directly.
      * unused?
+     *
+     * This will throw IAE on tunnel build failure
      */
     @Override
     protected I2PSocketOptions getDefaultOptions() {
@@ -292,6 +286,8 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
      * Warning, this does not make a copy of I2PTunnel's client options,
      * it modifies them directly.
      * Do not use overrides for per-socket options.
+     *
+     * This will throw IAE on tunnel build failure
      */
     @Override
     protected I2PSocketOptions getDefaultOptions(Properties overrides) {
@@ -417,6 +413,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             String referer = null;
             URI origRequestURI = null;
             boolean preserveConnectionHeader = false;
+            boolean allowGzip = false;
             while((line = reader.readLine(method)) != null) {
                 line = line.trim();
                 if(_log.shouldLog(Log.DEBUG)) {
@@ -573,7 +570,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
 
                     int port = requestURI.getPort();
 
-                    // Go through the various types of host names, set
+                    // Go through the various types of hostnames, set
                     // the host and destination variables accordingly,
                     // and transform the first line.
                     // For all i2p network hosts, ensure that the host is a
@@ -622,7 +619,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                             break;
                         }
                     } else if(hostLowerCase.endsWith(".i2p")) {
-                        // Destination gets the host name
+                        // Destination gets the hostname
                         destination = host;
                         // Host becomes the destination's "{b32}.b32.i2p" string, or "i2p" on lookup failure
                         host = getHostName(destination);
@@ -678,7 +675,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                                         // allow i2paddresshelper=<b32>.b32.i2p syntax.
                                         /*
                                         also i2paddresshelper=name.i2p for aliases
-                                        i.e. on your eepsite put
+                                        i.e. on your I2P Site put
                                         <a href="?i2paddresshelper=name.i2p">This is the name I want to be called.</a>
                                          */
                                         Destination _dest = _context.namingService().lookup(ahelperKey);
@@ -911,7 +908,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                             // ignore
                         }
                         return;
-                    }   // end host name processing
+                    }   // end hostname processing
 
                     boolean isValid = usingInternalOutproxy || usingWWWProxy ||
                                       usingInternalServer || isSupportedAddress(host, protocol);
@@ -985,8 +982,9 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                         // strip the accept-blah headers, as they vary dramatically from
                         // browser to browser
                         // But allow Accept-Encoding: gzip, deflate
-                        if(!lowercaseLine.startsWith("accept-encoding: ") &&
-                           !Boolean.parseBoolean(getTunnel().getClientOptions().getProperty(PROP_ACCEPT))) {
+                        if (lowercaseLine.startsWith("accept-encoding: ")) {
+                            allowGzip = lowercaseLine.contains("gzip");
+                        } else if (!Boolean.parseBoolean(getTunnel().getClientOptions().getProperty(PROP_ACCEPT))) {
                             line = null;
                             continue;
                         }
@@ -1006,7 +1004,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                                         line = null;
                                         continue; // completely strip the line if everything doesn't match
                                     }
-                                    // Strip to a relative URI, to hide the original host name
+                                    // Strip to a relative URI, to hide the original hostname
                                     StringBuilder buf = new StringBuilder();
                                     buf.append("Referer: ");
                                     String refererPath = refererURI.getRawPath();
@@ -1163,7 +1161,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                             Boolean.parseBoolean(getTunnel().getClientOptions().getProperty(PROP_DISABLE_HELPER))) {
                         out.write(ERR_HELPER_DISABLED.getBytes("UTF-8"));
                     } else {
-                        LocalHTTPServer.serveLocalFile(_context, sockMgr, out, method, internalPath, internalRawQuery, _proxyNonce);
+                        LocalHTTPServer.serveLocalFile(_context, sockMgr, out, method, internalPath, internalRawQuery, _proxyNonce, allowGzip);
                     }
                 } catch (IOException ioe) {
                     // ignore
@@ -1351,7 +1349,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             }
 
             // Redirect to non-addresshelper URL to not clog the browser address bar
-            // and not pass the parameter to the eepsite.
+            // and not pass the parameter to the I2P Site.
             // This also prevents the not-found error page from looking bad
             // Syndie can't handle a redirect of a POST
             if (ahelperPresent && !"POST".equals(method) && !"PUT".equals(method)) {
@@ -1363,7 +1361,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                     out.write(("HTTP/1.1 301 Address Helper Accepted\r\n" +
                         "Location: " + uri + "\r\n" +
                         "Connection: close\r\n"+
-                        "Proxy-Connection: close\r\n"+
                         "\r\n").getBytes("UTF-8"));
                 } catch (IOException ioe) {
                     // ignore
@@ -1376,7 +1373,21 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             // 1 == disconnect.  see ConnectionOptions in the new streaming lib, which i
             // dont want to hard link to here
             //opts.setProperty("i2p.streaming.inactivityTimeoutAction", ""+1);
-            I2PSocketOptions sktOpts = getDefaultOptions(opts);
+            I2PSocketOptions sktOpts;
+            try {
+                sktOpts = getDefaultOptions(opts);
+            } catch (RuntimeException re) {
+                // tunnel build failure
+                StringBuilder buf = new StringBuilder(128);
+                buf.append("HTTP/1.1 503 Service Unavailable");
+                if (re.getMessage() != null)
+                    buf.append(" - ").append(re.getMessage());
+                buf.append("\r\n\r\n");
+                try {
+                    out.write(buf.toString().getBytes("UTF-8"));
+                } catch (IOException ioe) {}
+                throw re;
+            }
             if (remotePort > 0)
                 sktOpts.setPort(remotePort);
             i2ps = createI2PSocket(clientDest, sktOpts);
@@ -1426,16 +1437,20 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
         }
     }
 
-    /** @since 0.8.7 */
+    /**
+     * @param destination the hostname
+     * @since 0.8.7
+     */
     private void writeHelperSaveForm(OutputStream outs, String destination, String ahelperKey,
                                      String targetRequest, String referer) throws IOException {
         if(outs == null)
             return;
+        String idn = decodeIDNHost(destination);
         Writer out = new BufferedWriter(new OutputStreamWriter(outs, "UTF-8"));
         String header = getErrorPage("ahelper-new", ERR_AHELPER_NEW);
         out.write(header);
         out.write("<table id=\"proxyNewHost\">\n<tr><td align=\"right\">" + _t("Host") +
-                "</td><td>" + destination + "</td></tr>\n");
+                "</td><td>" + idn + "</td></tr>\n");
         try {
             String b32 = Base32.encode(SHA256Generator.getInstance().calculateHash(Base64.decode(ahelperKey)).getData());
             out.write("<tr><td align=\"right\">" + _t("Base32") + "</td>" +
@@ -1447,38 +1462,39 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                   "</td></tr>\n</table>\n" + "<hr>\n" +
 
                 // FIXME if there is a query remaining it is lost
-                "<form method=\"GET\" action=\"" + targetRequest + "\">\n" +
-                "<h4>" + _t("Continue to {0} without saving", destination) + "</h4>\n<p>" +
+                "<form method=\"GET\" action=\"" + targetRequest + "\" class=\"hostaddform\">\n" +
+                "<div class=\"formaction hostaddaction\"><button type=\"submit\" class=\"go hostadd\">" +
+                "<span class=\"unicodeicon\">&#10143;</span><h4>" + _t("Continue to {0} without saving", idn) + "</h4>\n<p>" +
                 _t("You can browse to the site without saving it to the address book. The address will be remembered until you restart your I2P router.") +
-                "</p>\n<div class=\"formaction\"><button type=\"submit\" class=\"go\">" + _t("Continue without saving") + "</button></div>" + "\n</form>\n" +
+                "</p>\n</button></div>" + "\n</form>\n" +
 
-                "<form method=\"GET\" action=\"http://" + LOCAL_SERVER + "/add\">\n" +
+                "<form method=\"GET\" action=\"http://" + LOCAL_SERVER + "/add\" class=\"hostaddform\">\n" +
                 "<input type=\"hidden\" name=\"host\" value=\"" + destination + "\">\n" +
                 "<input type=\"hidden\" name=\"dest\" value=\"" + ahelperKey + "\">\n" +
-                "<input type=\"hidden\" name=\"nonce\" value=\"" + _proxyNonce + "\">\n" +
+                "<input type=\"hidden\" name=\"nonce\" value=\"" + _proxyNonce + "\">\n");
 
-                "<h4>" + _t("Save {0} to router address book and continue to website", destination) + "</h4>\n<p>" +
-                _t("This address will be saved to your Router address book where your subscription-based addresses are stored."));
-        if(_context.namingService().getName().equals("BlockfileNamingService")) {
-            out.write(" " + _t("If you want to keep track of sites you have added manually, add to your Local or Private address book instead."));
-        }
         // FIXME wasn't escaped
         String label = _t("Save & continue").replace("&", "&amp;");
-        out.write("</p>\n<div class=\"formaction\"><button type=\"submit\" class=\"accept\" name=\"router\" value=\"router\">" +
-                  label + "</button></div>\n");
+        out.write("<div class=\"formaction hostaddaction\"><button type=\"submit\" class=\"accept hostadd\" name=\"router\" value=\"router\">" +
+                "<span class=\"unicodeicon\">&#10143;</span><h4>" + _t("Save {0} to router address book and continue to website", idn) + "</h4>\n<p>" +
+                _t("This address will be saved to your Router address book where your subscription-based addresses are stored."));
+                if(_context.namingService().getName().equals("BlockfileNamingService")) {
+                    out.write(" " + _t("If you want to keep track of sites you have added manually, add to your Local or Private address book instead."));
+                }
+                out.write("</p>\n</button></div>\n");
 
         if(_context.namingService().getName().equals("BlockfileNamingService")) {
             // only blockfile supports multiple books
 
-            out.write("<h4>" + _t("Save {0} to local address book and continue to website", destination) + "</h4>\n<p>" +
+            out.write("<div class=\"formaction hostaddaction\"><button type=\"submit\" class=\"accept hostadd\" name=\"local\" value=\"local\">" +
+            "<span class=\"unicodeicon\">&#10143;</span><h4>" + _t("Save {0} to local address book and continue to website", idn) + "</h4>\n<p>" +
             _t("This address will be saved to your Local address book. Select this option for addresses you wish to keep separate from the main router address book, but don't mind publishing.") +
-            "</p>\n<div class=\"formaction\"><button type=\"submit\" class=\"accept\" name=\"local\" value=\"local\">" +
-            label + "</button></div>\n");
+            "</p>\n</button></div>\n");
 
-            out.write("<h4>" + _t("Save {0} to private address book and continue to website", destination) + "</h4>\n<p>" +
+            out.write("<div class=\"formaction hostaddaction\"><button type=\"submit\" class=\"accept hostadd\" name=\"private\" value=\"private\">" +
+            "<span class=\"unicodeicon\">&#10143;</span><h4>" + _t("Save {0} to private address book and continue to website", idn) + "</h4>\n<p>" +
             _t("This address will be saved to your Private address book, ensuring it is never published.") +
-            "</p>\n<div class=\"formaction\"><button type=\"submit\" class=\"accept\" name=\"private\" value=\"private\">" +
-            label + "</button></div>\n");
+             "</p>\n</button></div>\n");
 
         }
         // Firefox (and others?) don't send referer to meta refresh target, which is
@@ -1528,7 +1544,7 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
                       "</p>\n" +
 
                       "<p><b>PSK:</b> " + _t("Enter PSK encryption key") +
-                      ": ]</p>\n" +
+                      ":</p>\n" +
                       "<input type=\"text\" size=\"55\" name=\"privkey\" value=\"\">\n" +
                       "<p><b>DH:</b> " + _t("Generate new DH encryption key") +
                       ": <div class=\"formaction_xx\">" +
@@ -1614,10 +1630,11 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
 
     public static final String DEFAULT_JUMP_SERVERS =
             //"http://i2host.i2p/cgi-bin/i2hostjump?," +
-            "http://stats.i2p/cgi-bin/jump.cgi?a=";
+            "http://stats.i2p/cgi-bin/jump.cgi?a=," +
             //"http://no.i2p/jump/," +
-            //"http://i2pjump.i2p/jump/";
+            "http://i2pjump.i2p/jump/," +
             //"http://i2jump.i2p/";
+            "http://notbob.i2p/cgi-bin/jump.cgi?q=";
 
     /** @param host ignored */
     private static boolean isSupportedAddress(String host, String protocol) {
@@ -1654,7 +1671,6 @@ public class I2PTunnelHTTPClient extends I2PTunnelHTTPClientBase implements Runn
             "HTTP/1.1 403 Disabled\r\n" +
             "Content-Type: text/plain\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "Address helpers disabled";
 
