@@ -7,6 +7,7 @@ import java.io.BufferedWriter;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
@@ -14,6 +15,7 @@ import java.io.Reader;
 import java.io.StringReader;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
+import java.net.IDN;
 import java.net.Socket;
 import java.net.SocketTimeoutException;
 import java.net.URI;
@@ -40,6 +42,7 @@ import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.i2cp.MessageStatusMessage;
+import net.i2p.i2ptunnel.localServer.LocalHTTPServer;
 import net.i2p.util.EepGet;
 import net.i2p.util.EventDispatcher;
 import net.i2p.util.InternalSocket;
@@ -88,12 +91,9 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
             "Content-Type: text/html; charset=UTF-8\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
-            "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.5\r\n" + // try to get a UTF-8-encoded response back for the password
-            "Proxy-Authenticate: ";
+            "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.5\r\n"; // try to get a UTF-8-encoded response back for the password
     // put the auth type and realm in between
     private static final String ERR_AUTH2 =
-            "\r\n" +
             "\r\n" +
             "<html><body><H1>I2P ERROR: PROXY AUTHENTICATION REQUIRED</H1>" +
             "This proxy is configured to require authentication.";
@@ -105,7 +105,6 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
          "Content-Type: text/html; charset=iso-8859-1\r\n"+
          "Cache-Control: no-cache\r\n"+
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n"+
          "<html><body><H1>I2P ERROR: No outproxy found</H1>"+
          "Your request was for a site outside of I2P, but you have no "+
@@ -116,7 +115,6 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
             "Content-Type: text/html; charset=iso-8859-1\r\n" +
             "Cache-Control: no-cache\r\n" +
             "Connection: close\r\n"+
-            "Proxy-Connection: close\r\n"+
             "\r\n" +
             "<html><body><H1>I2P ERROR: DESTINATION NOT FOUND</H1>" +
             "That I2P Destination was not found. Perhaps you pasted in the " +
@@ -142,6 +140,19 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
     // very simple, remember last-failed only
     private String _lastFailedSSLProxy;
 
+    /** available as of Java 6 and Android API 9 */
+    private static final boolean _haveIDN;
+    static {
+        boolean h;
+        try {
+            Class.forName("java.net.IDN", false, ClassLoader.getSystemClassLoader());
+            h = true;
+        } catch (ClassNotFoundException cnfe) {
+            h = false;
+        }
+        _haveIDN = h;
+    }
+
     protected String getPrefix(long requestId) {
         return "HTTPClient[" + _clientId + '/' + requestId + "]: ";
     }
@@ -200,6 +211,8 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
         String s = getTunnel().getClientOptions().getProperty(PROP_SSL_OUTPROXIES);
         if (s == null)
             return null;
+        // default config typo fix
+        s = s.replace("exit.storymcloud.i2p", "exit.stormycloud.i2p");
         String[] p = DataHelper.split(s, "[,; \r\n\t]");
         int size = p.length;
         if (size == 0)
@@ -330,6 +343,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
     /** new style MD5 auth */
     public static final String PROP_PROXY_DIGEST_PREFIX = "proxy.auth.";
     public static final String PROP_PROXY_DIGEST_SUFFIX = ".md5";
+    public static final String PROP_PROXY_DIGEST_SHA256_SUFFIX = ".sha256";
     public static final String BASIC_AUTH = "basic";
     public static final String DIGEST_AUTH = "digest";
 
@@ -510,6 +524,19 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
                 _log.info("Bad digest request: " + DataHelper.toString(args));
             return AuthResult.AUTH_BAD_REQ;
         }
+        // RFC 7616
+        String algorithm = args.get("algorithm");
+        boolean isSHA256 = false;
+        if (algorithm != null) {
+            algorithm = algorithm.toLowerCase(Locale.US);
+            if (algorithm.equals("sha-256")) {
+                isSHA256 = true;
+            } else if (!algorithm.equals("md5")) {
+                if (_log.shouldLog(Log.INFO))
+                    _log.info("Bad digest request: " + DataHelper.toString(args));
+                return AuthResult.AUTH_BAD_REQ;
+            }
+        }
         // nonce check
         AuthResult check = verifyNonce(nonce, nc);
         if (check != AuthResult.AUTH_GOOD) {
@@ -519,17 +546,17 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
         }
         // get H(A1) == stored password
         String ha1 = getTunnel().getClientOptions().getProperty(PROP_PROXY_DIGEST_PREFIX + user +
-                                                                PROP_PROXY_DIGEST_SUFFIX);
+                                                                (isSHA256 ? PROP_PROXY_DIGEST_SHA256_SUFFIX : PROP_PROXY_DIGEST_SUFFIX));
         if (ha1 == null) {
             _log.logAlways(Log.WARN, "HTTP proxy authentication failed, user: " + user);
             return AuthResult.AUTH_BAD;
         }
         // get H(A2)
         String a2 = method + ':' + uri;
-        String ha2 = PasswordManager.md5Hex(a2);
+        String ha2 = isSHA256 ? PasswordManager.sha256Hex(a2) : PasswordManager.md5Hex(a2);
         // response check
         String kd = ha1 + ':' + nonce + ':' + nc + ':' + cnonce + ':' + qop + ':' + ha2;
-        String hkd = PasswordManager.md5Hex(kd);
+        String hkd = isSHA256 ? PasswordManager.sha256Hex(kd) : PasswordManager.md5Hex(kd);
         if (!response.equals(hkd)) {
             _log.logAlways(Log.WARN, "HTTP proxy authentication failed, user: " + user);
             if (_log.shouldLog(Log.INFO))
@@ -615,17 +642,43 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
      */
     protected String getAuthError(boolean isStale) {
         boolean isDigest = isDigestAuthRequired();
-        return
-            ERR_AUTH1 +
-            (isDigest ? "Digest" : "Basic") +
-            " realm=\"" + getRealm() + '"' +
-            (isDigest ? ", nonce=\"" + getNonce() + "\"," +
-                        " algorithm=MD5," +
-                        " charset=UTF-8," +     // RFC 7616/7617
-                        " qop=\"auth\"" +
-                        (isStale ? ", stale=true" : "")
-                      : "") +
-            ERR_AUTH2;
+        StringBuilder buf = new StringBuilder(512);
+        buf.append(ERR_AUTH1)
+           .append("Proxy-Authenticate: ")
+           .append(isDigest ? "Digest" : "Basic")
+           .append(" realm=\"" + getRealm() + '"');
+        if (isDigest) {
+            String nonce = getNonce();
+            // RFC 7616 most-preferred first, client accepts first that he supports
+            // This is also compatible with eepget < 0.9.56 that will use the last one
+            // Do we have a SHA256 hash for any user?
+            for (String k : getTunnel().getClientOptions().stringPropertyNames()) {
+                if (k.startsWith(PROP_PROXY_DIGEST_PREFIX) &&
+                    k.endsWith(PROP_PROXY_DIGEST_SHA256_SUFFIX)) {
+                    // SHA-256, RFC 7616
+                    buf.append(", nonce=\"" + nonce + "\"," +
+                               " algorithm=SHA-256," +
+                               " charset=UTF-8," +
+                               " qop=\"auth\"");
+                    if (isStale)
+                        buf.append(", stale=true");
+                    buf.append("\r\n" +
+                               "Proxy-Authenticate: Digest" +
+                               " realm=\"" + getRealm() + '"');
+                    break;
+                }
+            }
+
+            buf.append(", nonce=\"" + nonce + "\"," +
+                       " algorithm=MD5," +
+                       " charset=UTF-8," +     // RFC 7616/7617
+                       " qop=\"auth\"");
+            if (isStale)
+                buf.append(", stale=true");
+        }
+        buf.append("\r\n")
+           .append(ERR_AUTH2);
+        return buf.toString();
     }
 
     /**
@@ -670,8 +723,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
      *  @since 0.9.4 moved from I2PTunnelHTTPClient
      */
     protected static String getErrorPage(I2PAppContext ctx, String base, String backup) {
-        File errorDir = new File(ctx.getBaseDir(), "docs");
-        File file = new File(errorDir, base + "-header.ht");
+        String file = "proxy/" + base + "-header.ht";
         try {
             return readFile(ctx, file);
         } catch(IOException ioe) {
@@ -683,20 +735,24 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
     private static final String BUNDLE_NAME = "net.i2p.i2ptunnel.proxy.messages";
 
     /**
+     *  As of 0.9.49, loads the error pages from the jar, not the file system.
      *  @since 0.9.4 moved from I2PTunnelHTTPClient
      */
-    private static String readFile(I2PAppContext ctx, File file) throws IOException {
+    private static String readFile(I2PAppContext ctx, String file) throws IOException {
         Reader reader = null;
         char[] buf = new char[512];
         StringBuilder out = new StringBuilder(2048);
+        InputStream in = LocalHTTPServer.getResource(file);
+        if (in == null)
+            throw new IOException();
         try {
             boolean hasSusiDNS = ctx.portMapper().isRegistered(PortMapper.SVC_SUSIDNS);
             boolean hasI2PTunnel = ctx.portMapper().isRegistered(PortMapper.SVC_I2PTUNNEL);
             if (hasSusiDNS && hasI2PTunnel) {
-                reader = new TranslateReader(ctx, BUNDLE_NAME, new FileInputStream(file));
+                reader = new TranslateReader(ctx, BUNDLE_NAME, in);
             } else {
                 // strip out the addressbook links
-                reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
+                reader = new InputStreamReader(in, "UTF-8");
                 int len;
                 while((len = reader.read(buf)) > 0) {
                     out.append(buf, 0, len);
@@ -734,6 +790,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
             String rv = out.toString();
             return rv;
         } finally {
+            try { in.close(); } catch (IOException ioe) {}
             try {
                 if(reader != null)
                     reader.close();
@@ -756,7 +813,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
         private final boolean _isSSL;
 
         /**
-         *  @param target the URI for an HTTP request, or the host name for CONNECT
+         *  @param target the URI for an HTTP request, or the hostname for CONNECT
          */
         public OnTimeout(Socket s, OutputStream out, String target, boolean usingProxy, String wwwProxy, long id) {
             _socket = s;
@@ -770,7 +827,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
         }
 
         /**
-         *  @param target the URI for an HTTP request, or the host name for CONNECT
+         *  @param target the URI for an HTTP request, or the hostname for CONNECT
          *  @param targetHost if non-null, call noteProxyResult() with this as host
          *  @param isSSL to pass to noteProxyResult(). FALSE for ConnectClient.
          *  @since 0.9.39
@@ -923,7 +980,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
             out.write(uri);
             out.write("\">");
             // Long URLs are handled in CSS
-            out.write(uri);
+            out.write(decodeIDNURI(uri));
             out.write("</a>");
             if (usingWWWProxy) {
                 out.write("<br><br><b>");
@@ -965,7 +1022,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
 
                     if (first) {
                         first = false;
-                        out.write("<br><br>\n<div id=\"jumplinks\">\n<h4>");
+                        out.write("<br>\n<div id=\"jumplinks\">\n<h4>");
                         out.write(_t("Click a link below for an address helper from a jump service"));
                         out.write("</h4>\n");
                     } else {
@@ -975,7 +1032,7 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
                     out.write(jurl);
                     out.write(uri);
                     out.write("\">");
-                    // Translators: parameter is a host name
+                    // Translators: parameter is a hostname
                     out.write(_t("{0} jump service", jumphost));
                     out.write("</a>\n");
                 }
@@ -990,6 +1047,45 @@ public abstract class I2PTunnelHTTPClientBase extends I2PTunnelClientBase implem
         writeFooter(out);
     }
 
+    /**
+     *  Decode the host part of a URI for display.
+     *  Returns original string on any error.
+     *
+     *  @since 0.9.50
+     */
+    private static String decodeIDNURI(String uri) {
+        if (!_haveIDN)
+             return uri;
+        if (!uri.contains("xn--"))
+             return uri;
+        try {
+            URI u = new URI(uri);
+            String h = u.getHost();
+            String hu = IDN.toUnicode(h);
+            if (hu == null || h.equals(hu))
+                return uri;
+            int idx = uri.indexOf(h);
+            if (idx < 0)
+                return uri;
+            return uri.substring(0, idx) + hu + uri.substring(idx + h.length(), uri.length());
+         } catch(URISyntaxException use) {}
+         return uri;
+    }
+
+    /**
+     *  Decode a hostname for display.
+     *  Returns original string on any error.
+     *
+     *  @since 0.9.50
+     */
+    public static String decodeIDNHost(String host) {
+        if (!_haveIDN)
+             return host;
+        if (!host.contains("xn--"))
+             return host;
+        return IDN.toUnicode(host);
+    }
+
     /**
      *  Flushes.
      *
