@@ -28,6 +28,7 @@ import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.atomic.AtomicLong;
 
 import net.i2p.I2PException;
 import net.i2p.I2PAppContext;
@@ -233,7 +234,7 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
             try {
                 _filter = FilterFactory.createFilter(context, filterDefinition);
             } catch (IOException | InvalidDefinitionException bad) {
-                throw new IllegalArgumentException("Can't create socket manager "+bad.getMessage(), bad);
+                throw new IllegalArgumentException("Bad filter definition file: " + bad.getMessage(), bad);
             }
         }
 
@@ -289,14 +290,14 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
                 if (remaining <= 10*60*1000) {
                     String msg;
                     if (remaining > 0)
-                        msg = "Offline signature for tunnel alternate destination expires " + DataHelper.formatTime(exp);
+                        msg = "Offline signature for tunnel " + name + " alternate destination expires in " + DataHelper.formatTime(exp);
                     else
-                        msg = "Offline signature for tunnel alternate destination expired " + DataHelper.formatTime(exp);
+                        msg = "Offline signature for tunnel " + name + " alternate destination expired " + DataHelper.formatTime(exp);
                     _log.log(Log.CRIT, msg);
                     throw new IllegalArgumentException(msg);
                 }
                 if (remaining < 60*24*60*60*1000L) {
-                    String msg = "Offline signature for tunnel alternate destination expires in " + DataHelper.formatDuration(remaining);
+                    String msg = "Offline signature for tunnel " + name + " alternate destination expires in " + DataHelper.formatDuration(remaining);
                     _log.logAlways(Log.WARN, msg);
                     l.log("WARNING: " + msg);
                 }
@@ -327,18 +328,25 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
         if (session.isOffline()) {
             long exp = session.getOfflineExpiration();
             long remaining = exp - getTunnel().getContext().clock().now();
+            Properties props = getTunnel().getClientOptions();
+            String name = props.getProperty("inbound.nickname");
+            if (name == null) {
+                name = props.getProperty("outbound.nickname");
+                if (name == null)
+                    name = "";
+            }
             // if expires before the LS expires...
             if (remaining <= 10*60*1000) {
                 String msg;
                 if (remaining > 0)
-                    msg = "Offline signature for tunnel expires " + DataHelper.formatTime(exp);
+                    msg = "Offline signature for tunnel " + name + " expires in " + DataHelper.formatTime(exp);
                 else
-                    msg = "Offline signature for tunnel expired " + DataHelper.formatTime(exp);
+                    msg = "Offline signature for tunnel " + name + " expired " + DataHelper.formatTime(exp);
                 _log.log(Log.CRIT, msg);
                 throw new IllegalArgumentException(msg);
             }
             if (remaining < 60*24*60*60*1000L) {
-                String msg = "Offline signature for tunnel expires in " + DataHelper.formatDuration(remaining);
+                String msg = "Offline signature for tunnel " + name + " expires in " + DataHelper.formatDuration(remaining);
                 _log.logAlways(Log.WARN, msg);
                 l.log("WARNING: " + msg);
             }
@@ -373,7 +381,9 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
                 else
                     msg = "Unable to connect to the router at " + getTunnel().host + ':' + portNum +
                              " and build tunnels for the server at " + remoteHost.getHostAddress() + ':' + remotePort;
-                if (++retries < MAX_RETRIES) {
+                String exmsg = ise.getMessage();
+                boolean fail = exmsg != null && exmsg.contains("session limit exceeded");
+                if (!fail && ++retries < MAX_RETRIES) {
                     msg += ", retrying in " + (RETRY_DELAY / 1000) + " seconds";
                     this.l.log(msg);
                     _log.error(msg);
@@ -667,6 +677,7 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
                 } catch (InterruptedException ie) {}
                 // This should be the same as before, but we have to call getServerSocket()
                 // so sockMgr will call ConnectionManager.setAllowIncomingConnections(true) again
+                _log.logAlways(Log.WARN, "Reconnecting to router after restart");
                 i2pss = sockMgr.getServerSocket();
             } catch (I2PException ipe) {
                 String s = "Error accepting - KILLING THE TUNNEL SERVER";
@@ -723,6 +734,7 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
     /** just to set the name and set Daemon */
     private static class CustomThreadFactory implements ThreadFactory {
         private final String _name;
+        private final AtomicLong _executorThreadCount = new AtomicLong();
 
         public CustomThreadFactory(String name) {
             _name = name;
@@ -730,7 +742,7 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
 
         public Thread newThread(Runnable r) {
             Thread rv = Executors.defaultThreadFactory().newThread(r);
-            rv.setName(_name);
+            rv.setName(_name + ' ' + _executorThreadCount.incrementAndGet());
             rv.setDaemon(true);
             return rv;
         }
@@ -822,7 +834,9 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
                 port = isa.getPort();
             }
         }
-        return getSocket(from, host, port);
+        // don't do SSL-over-SSL
+        boolean force = incomingPort == 443 || incomingPort == 22;
+        return getSocket(from, host, port, force);
     }
 
     /**
@@ -833,8 +847,19 @@ public class I2PTunnelServer extends I2PTunnelTask implements Runnable {
      *  @since 0.9.9
      */
     protected Socket getSocket(Hash from, InetAddress remoteHost, int remotePort) throws IOException {
+        return getSocket(from, remoteHost, remotePort, false);
+    }
+
+    /**
+     *  Get a regular or SSL socket depending on config.
+     *  The SSL config applies to all hosts/ports, unless forced off.
+     *
+     *  @param forceNonSSL override config
+     *  @since 0.9.50
+     */
+    private Socket getSocket(Hash from, InetAddress remoteHost, int remotePort, boolean forceNonSSL) throws IOException {
         String opt = getTunnel().getClientOptions().getProperty(PROP_USE_SSL);
-        if (Boolean.parseBoolean(opt)) {
+        if (!forceNonSSL && Boolean.parseBoolean(opt)) {
             synchronized(sslLock) {
                 if (_sslFactory == null) {
                     try {
