@@ -5,6 +5,7 @@ package net.i2p.i2ptunnel.localServer;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.HashMap;
 import java.util.Map;
@@ -27,6 +28,7 @@ import net.i2p.data.Destination;
 import net.i2p.data.PrivateKey;
 import net.i2p.data.PublicKey;
 import net.i2p.data.SigningPublicKey;
+import net.i2p.i2ptunnel.GunzipOutputStream;
 import net.i2p.i2ptunnel.I2PTunnelHTTPClientBase;
 import net.i2p.util.FileUtil;
 import net.i2p.util.PortMapper;
@@ -50,7 +52,6 @@ public abstract class LocalHTTPServer {
          "HTTP/1.1 404 Not Found\r\n"+
          "Content-Type: text/plain\r\n"+
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n"+
          "HTTP Proxy local file not found";
 
@@ -58,7 +59,6 @@ public abstract class LocalHTTPServer {
          "HTTP/1.1 409 Bad\r\n"+
          "Content-Type: text/plain\r\n"+
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n"+
          "Add to addressbook failed - bad parameters";
 
@@ -66,7 +66,6 @@ public abstract class LocalHTTPServer {
          "HTTP/1.1 400 Bad\r\n"+
          "Content-Type: text/plain\r\n"+
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n"+
          "B32 update failed - bad parameters";
 
@@ -75,7 +74,6 @@ public abstract class LocalHTTPServer {
          "Content-Type: text/plain\r\n" +
          "Cache-Control: max-age=86400\r\n" +
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n"+
          "I2P HTTP proxy OK";
 
@@ -85,7 +83,6 @@ public abstract class LocalHTTPServer {
          "Referrer-Policy: no-referrer\r\n"+
          "Cache-Control: no-cache\r\n" +
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
          "\r\n";
 
     /**
@@ -110,10 +107,11 @@ public abstract class LocalHTTPServer {
      *  @param sockMgr only for /b32, otherwise ignored
      *  @param targetRequest decoded path only, non-null
      *  @param query raw (encoded), may be null
+     *  @param allowGzip may we send a gzipped response?
      */
     public static void serveLocalFile(I2PAppContext context, I2PSocketManager sockMgr,
                                       OutputStream out, String method, String targetRequest,
-                                      String query, String proxyNonce) throws IOException {
+                                      String query, String proxyNonce, boolean allowGzip) throws IOException {
         //System.err.println("targetRequest: \"" + targetRequest + "\"");
         // a home page message for the curious...
         if (targetRequest.equals("/")) {
@@ -124,32 +122,41 @@ public abstract class LocalHTTPServer {
         if ((method.equals("GET") || method.equals("HEAD")) &&
             targetRequest.startsWith("/themes/") &&
             !targetRequest.contains("..")) {
-            String filename = null;
-            try {
-                filename = targetRequest.substring(8); // "/themes/".length
-            } catch (IndexOutOfBoundsException ioobe) {
-                 return;
-            }
+            String filename = targetRequest.substring(1);
             // theme hack
-            if (filename.startsWith("console/default/"))
+            if (filename.startsWith("themes/console/default/"))
                 filename = filename.replaceFirst("default", context.getProperty("routerconsole.theme", "light"));
-            File themesDir = new File(context.getBaseDir(), "docs/themes");
-            File file = new File(themesDir, filename);
-            if (file.exists() && !file.isDirectory()) {
-                String type;
-                if (filename.endsWith(".css"))
-                    type = "text/css";
-                else if (filename.endsWith(".ico"))
-                    type = "image/x-icon";
-                else if (filename.endsWith(".png"))
-                    type = "image/png";
-                else if (filename.endsWith(".jpg"))
-                    type = "image/jpeg";
-                else type = "text/html";
-                out.write("HTTP/1.1 200 OK\r\nContent-Type: ".getBytes("UTF-8"));
-                out.write(type.getBytes("UTF-8"));
-                out.write("\r\nCache-Control: max-age=86400\r\nConnection: close\r\nProxy-Connection: close\r\n\r\n".getBytes("UTF-8"));
-                FileUtil.readFile(filename, themesDir.getAbsolutePath(), out);
+            if (filename.endsWith(".css"))
+                filename = filename + ".gz";
+            InputStream in = getResource(filename);
+            if (in != null) {
+                try {
+                    String type;
+                    if (filename.endsWith(".css.gz"))
+                        type = "text/css; charset=UTF-8";
+                    else if (filename.endsWith(".ico"))
+                        type = "image/x-icon";
+                    else if (filename.endsWith(".png"))
+                        type = "image/png";
+                    else if (filename.endsWith(".jpg"))
+                        type = "image/jpeg";
+                    else type = "text/html; charset=UTF-8";
+                    out.write("HTTP/1.1 200 OK\r\nContent-Type: ".getBytes("UTF-8"));
+                    out.write(type.getBytes("UTF-8"));
+                    if (allowGzip && filename.endsWith(".gz"))
+                        out.write("\r\nContent-Encoding: gzip".getBytes("UTF-8"));
+                    out.write("\r\nCache-Control: max-age=86400\r\nConnection: close\r\n\r\n".getBytes("UTF-8"));
+                    if (!allowGzip && filename.endsWith(".gz")) {
+                        // gunzip on the fly. should be very rare, all browsers should support gzip
+                        OutputStream out2 = new GunzipOutputStream(out);
+                        DataHelper.copy(in, out2);
+                        out2.flush();
+                    } else {
+                        DataHelper.copy(in, out);
+                    }
+                } finally {
+                    try { in.close(); } catch (IOException ioe) {}
+                }
                 return;
             }
         }
@@ -288,7 +295,7 @@ public abstract class LocalHTTPServer {
                                .append(_t("Your new encryption key"))
                                .append("</title>\n" +
                                        "<link rel=\"shortcut icon\" href=\"http://proxy.i2p/themes/console/images/favicon.ico\" >\n" +
-                                       "<link href=\"http://proxy.i2p/themes/console/default/console.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
+                                       "<link href=\"http://proxy.i2p/themes/console/default/proxy.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
                                        "</head><body>\n" +
                                        "<div class=logo>\n" +
                                        "<a href=\"")
@@ -299,7 +306,7 @@ public abstract class LocalHTTPServer {
                                .append(conURL).append("help.jsp\">").append(_t("Help")).append("</a>");
                             if (pm.isRegistered(PortMapper.SVC_SUSIDNS)) {
                                 buf.append(" <a href=\"").append(conURL).append("susidns/index\">")
-                                   .append(_t("Addressbook")).append("</a>\n");
+                                   .append(_t("Address book")).append("</a>\n");
                             }
                             buf.append("</div>" +
                                        "<div class=warning id=warning>\n" +
@@ -353,29 +360,29 @@ public abstract class LocalHTTPServer {
 
         PortMapper pm = I2PAppContext.getGlobalContext().portMapper();
         String conURL = pm.getConsoleURL();
+        String idn = I2PTunnelHTTPClientBase.decodeIDNHost(host);
         out.write(("HTTP/1.1 200 OK\r\n"+
                   "Content-Type: text/html; charset=UTF-8\r\n"+
                   "Referrer-Policy: no-referrer\r\n"+
                   "Connection: close\r\n"+
-                  "Proxy-Connection: close\r\n"+
                   "\r\n"+
                   "<html><head>"+
-                  "<title>" + _t("Redirecting to {0}", host) + "</title>\n" +
+                  "<title>" + _t("Redirecting to {0}", idn) + "</title>\n" +
                   "<link rel=\"shortcut icon\" href=\"http://proxy.i2p/themes/console/images/favicon.ico\" >\n" +
-                  "<link href=\"http://proxy.i2p/themes/console/default/console.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
+                  "<link href=\"http://proxy.i2p/themes/console/default/proxy.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
                   "<meta http-equiv=\"Refresh\" content=\"1; url=" + url + "\">\n" +
                   "</head><body>\n" +
                   "<div class=logo>\n" +
                   "<a href=\"" + conURL + "\" title=\"" + _t("Router Console") + "\"><img src=\"http://proxy.i2p/themes/console/images/i2plogo.png\" alt=\"I2P Router Console\" border=\"0\"></a><hr>\n" +
                   "<a href=\"" + conURL + "config\">" + _t("Configuration") + "</a> <a href=\"" + conURL + "help.jsp\">" + _t("Help") + "</a>").getBytes("UTF-8"));
         if (pm.isRegistered(PortMapper.SVC_SUSIDNS))
-            out.write((" <a href=\"" + conURL + "susidns/index\">" + _t("Addressbook") + "</a>\n").getBytes("UTF-8"));
+            out.write((" <a href=\"" + conURL + "susidns/index\">" + _t("Address Book") + "</a>\n").getBytes("UTF-8"));
         out.write(("</div>" +
                   "<div class=warning id=warning>\n" +
                   "<h3>" +
                   (success ?
-                           _t("Saved {0} to the {1} addressbook, redirecting now.", host, tbook) :
-                           _t("Failed to save {0} to the {1} addressbook, redirecting now.", host, tbook)) +
+                           _t("Saved {0} to the {1} address book, redirecting now.", idn, tbook) :
+                           _t("Failed to save {0} to the {1} address book, redirecting now.", idn, tbook)) +
                   "</h3>\n<p><a href=\"" + url + "\">" +
                   _t("Click here if you are not redirected automatically.") +
                   "</a></p></div>").getBytes("UTF-8"));
@@ -387,27 +394,27 @@ public abstract class LocalHTTPServer {
     private static void writeB32RedirectPage(OutputStream out, String host, String url) throws IOException {
         PortMapper pm = I2PAppContext.getGlobalContext().portMapper();
         String conURL = pm.getConsoleURL();
+        String idn = I2PTunnelHTTPClientBase.decodeIDNHost(host);
         out.write(("HTTP/1.1 200 OK\r\n"+
                   "Content-Type: text/html; charset=UTF-8\r\n"+
                   "Referrer-Policy: no-referrer\r\n"+
                   "Connection: close\r\n"+
-                  "Proxy-Connection: close\r\n"+
                   "\r\n"+
                   "<html><head>"+
-                  "<title>" + _t("Redirecting to {0}", host) + "</title>\n" +
+                  "<title>" + _t("Redirecting to {0}", idn) + "</title>\n" +
                   "<link rel=\"shortcut icon\" href=\"http://proxy.i2p/themes/console/images/favicon.ico\" >\n" +
-                  "<link href=\"http://proxy.i2p/themes/console/default/console.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
+                  "<link href=\"http://proxy.i2p/themes/console/default/proxy.css\" rel=\"stylesheet\" type=\"text/css\" >\n" +
                   "<meta http-equiv=\"Refresh\" content=\"1; url=" + url + "\">\n" +
                   "</head><body>\n" +
                   "<div class=logo>\n" +
                   "<a href=\"" + conURL + "\" title=\"" + _t("Router Console") + "\"><img src=\"http://proxy.i2p/themes/console/images/i2plogo.png\" alt=\"I2P Router Console\" border=\"0\"></a><hr>\n" +
                   "<a href=\"" + conURL + "config\">" + _t("Configuration") + "</a> <a href=\"" + conURL + "help.jsp\">" + _t("Help") + "</a>").getBytes("UTF-8"));
         if (pm.isRegistered(PortMapper.SVC_SUSIDNS))
-            out.write((" <a href=\"" + conURL + "susidns/index\">" + _t("Addressbook") + "</a>\n").getBytes("UTF-8"));
+            out.write((" <a href=\"" + conURL + "susidns/index\">" + _t("Address Book") + "</a>\n").getBytes("UTF-8"));
         out.write(("</div>" +
                   "<div class=warning id=warning>\n" +
                   "<h3>" +
-                  _t("Saved the authentication for {0}, redirecting now.", host) +
+                  _t("Saved the authentication for {0}, redirecting now.", idn) +
                   "</h3>\n<p><a href=\"" + url + "\">" +
                   _t("Click here if you are not redirected automatically.") +
                   "</a></p></div>").getBytes("UTF-8"));
@@ -483,6 +490,15 @@ public abstract class LocalHTTPServer {
         return buf.toString();
     }
 
+    /**
+     *  @param resource relative path
+     *  @return stream or null if not found
+     *  @since 0.9.49
+     */
+    public static InputStream getResource(String resource) {
+            return LocalHTTPServer.class.getResourceAsStream("/net/i2p/i2ptunnel/resources/" + resource);
+    }
+
     /** these strings go in the jar, not the war */
     private static final String BUNDLE_NAME = "net.i2p.i2ptunnel.proxy.messages";
 
