@@ -10,7 +10,10 @@ package net.i2p.i2ptunnel.web;
 
 import java.io.File;
 import java.io.IOException;
+import java.text.Collator;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
@@ -351,13 +354,48 @@ public class IndexBean {
             if (_context.getProperty(PROP_DISABLE_OLD, DEFAULT_DISABLE_OLD))
                 theme = "light";
         }
-        return "/themes/i2ptunnel/" + theme + "/";
+        return "themes/" + theme + "/";
     }
 
     public int getTunnelCount() {
         if (_group == null) return 0;
         return _group.getControllers().size();
     }
+
+    /**
+     *  Return tunnel numbers of clients or servers only, sorted by tunnel name
+     *  @param isClient true for clients, false for servers
+     *  @return non-null, may be empty
+     *  @since 0.9.57
+     */
+    public List<Integer> getControllerNumbers(boolean isClient) {
+        if (_group == null)
+            return Collections.emptyList();
+        List<TunnelController> all = _group.getControllers();
+        List<Integer> rv = new ArrayList<Integer>(all.size());
+        for (int i = 0; i < all.size(); i++) {
+            TunnelController tc = all.get(i);
+            if (tc.isClient() == isClient)
+                rv.add(Integer.valueOf(i));
+        }
+        if (rv.size() > 1)
+            Collections.sort(rv, new TCComparator());
+        return rv;
+    }
+
+    /**
+     *  Sort tunnel numbers by the name of the tunnel
+     *  @since 0.9.57
+     */
+    private class TCComparator implements Comparator<Integer> {
+         private final Collator _comp = Collator.getInstance();
+         public int compare(Integer l, Integer r) {
+             int rv = _comp.compare(getTunnelName(l), getTunnelName(r));
+             if (rv != 0)
+                 return rv;
+             return l.compareTo(r);
+        }
+    }
     
     /**
      *  Is it a client or server in the UI and I2P side?
@@ -672,7 +710,7 @@ public class IndexBean {
     String getType() { return _config.getType(); }
     
     /** Short name of the tunnel */
-    public void setName(String name) { 
+    public void setNofilter_name(String name) {
         _config.setName(name);
     }
     /** one line description */
@@ -1083,7 +1121,10 @@ public class IndexBean {
 
     /** all proxy auth @since 0.8.2 */
     public void setProxyAuth(String s) {
-        _config.setProxyAuth(I2PTunnelHTTPClientBase.DIGEST_AUTH);
+        String type = getType();
+        boolean isSOCKS = TunnelController.TYPE_SOCKS.equals(type) ||
+                          TunnelController.TYPE_SOCKS_IRC.equals(type);
+        _config.setProxyAuth(isSOCKS ? "true" : I2PTunnelHTTPClientBase.DIGEST_AUTH);
     }
     
     public void setProxyUsername(String s) {
@@ -1116,6 +1157,15 @@ public class IndexBean {
         _config.setUseOutproxyPlugin(true);
     }
 
+    /**
+     *  @param s "connect" or "socks"
+     *  @since 0.9.57
+     */
+    public void setOutproxyType(String s) {
+        _config.setOutproxyType(s);
+    }
+
+
     public void setLimitMinute(String s) {
         if (s != null) {
             try {
