@@ -1,28 +1,28 @@
 package org.rrd4j.core;
 
 import java.io.IOException;
-import java.lang.reflect.UndeclaredThrowableException;
 import java.net.URI;
-import java.util.HashSet;
 import java.util.Optional;
-import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Semaphore;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
+import java.util.stream.Stream;
 
 /**
  * <p>This class should be used to synchronize access to RRD files
  * in a multithreaded environment. This class should be also used to prevent opening of
  * too many RRD files at the same time (thus avoiding operating system limits).
  * </p>
- * <p>It should not be called directly. Use {@link RrdDb.Builder#usePool()} instead.</p>
+ * <p>It can also be used a factory for RrdDb, using a default backend factory.</p>
+ * <p>In case of interruptions, it throws IllegalStateException.
  */
 public class RrdDbPool {
     private static class RrdDbPoolSingletonHolder {
@@ -38,8 +38,8 @@ public class RrdDbPool {
     }
 
     /**
-     * Initial capacity of the pool i.e. maximum number of simultaneously open RRD files. The pool will
-     * never open too many RRD files at the same time.
+     * Initial capacity of the pool i.e. maximum number of simultaneously open RRD. The pool will
+     * never open too many RRD at the same time.
      */
     public static final int INITIAL_CAPACITY = 200;
 
@@ -80,7 +80,7 @@ public class RrdDbPool {
             if (placeholder) {
                 return String.format("RrdEntry [placeholder, uri=%s]", uri);
             } else {
-                return String.format("RrdEntry [count=%d, rrdDb=%s, uri%s]", count, rrdDb, uri);
+                return String.format("RrdEntry [count=%d, rrdDb=%s, uri %s]", count, rrdDb, uri);
             }
         }
     }
@@ -90,7 +90,6 @@ public class RrdDbPool {
      * or returns already existing one. Uses Initialization On Demand Holder idiom.
      *
      * @return Single instance of this class
-     * @throws java.lang.RuntimeException Thrown if the default RRD backend is not derived from the {@link org.rrd4j.core.RrdFileBackendFactory}
      */
     public static RrdDbPool getInstance() {
         return RrdDbPoolSingletonHolder.instance;
@@ -109,7 +108,7 @@ public class RrdDbPool {
     private RrdBackendFactory defaultFactory;
 
     /**
-     * Constructor for RrdDbPool.
+     * Constructor for RrdDbPool. It will use the default backend factory.
      * @since 3.5
      */
     public RrdDbPool() {
@@ -118,7 +117,7 @@ public class RrdDbPool {
 
     /**
      * Constructor for RrdDbPool.
-     * @param defaultFactory the default factory used when given simple path of a rrdDb.
+     * @param defaultFactory the default factory used when given a simple path of a RRD.
      * @since 3.6
      */
     public RrdDbPool(RrdBackendFactory defaultFactory) {
@@ -130,36 +129,40 @@ public class RrdDbPool {
     }
 
     /**
-     * Returns the number of open RRD files.
+     * Returns the number of open RRD.
      *
-     * @return Number of currently open RRD files held in the pool.
+     * @return Number of currently open RRD held in the pool.
      */
     public int getOpenFileCount() {
         return pool.size();
     }
 
     /**
-     * Returns an array of open file URI.
+     * Returns an array of open RRD URI.
      *
-     * @return Array with {@link URI} to open RRD files held in the pool.
+     * @return Array with {@link URI} to open RRD held in the pool.
      */
     public URI[] getOpenUri() {
-        //Direct toarray from keySet can fail
-        Set<URI> uris = new HashSet<>(pool.size());
-        pool.forEach((k,v) -> uris.add(k));
-        return uris.toArray(new URI[uris.size()]);
+        return pool.keySet().stream().toArray(URI[]::new);
     }
 
     /**
-     * Returns an array of open file path.
+     * Returns an stream open RRD.
      *
-     * @return Array with canonical path to open RRD files held in the pool.
+     * @return Stream with canonical URI to open RRD path held in the pool.
+     * @since 3.7
+     */
+    public Stream<URI> getOpenUriStream() {
+        return pool.keySet().stream();
+    }
+
+    /**
+     * Returns an array of open RRD.
+     *
+     * @return Array with canonical path to open RRD path held in the pool.
      */
     public String[] getOpenFiles() {
-        //Direct toarray from keySet can fail
-        Set<String> uris = new HashSet<>(pool.size());
-        pool.forEach((k,v) -> uris.add(k.getPath()));
-        return uris.toArray(new String[uris.size()]);
+        return pool.keySet().stream().map(URI::getPath).toArray(String[]::new);
     }
 
     private RrdEntry getEntry(URI uri, boolean cancreate) throws InterruptedException {
@@ -260,8 +263,9 @@ public class RrdDbPool {
                 try {
                     usageWLock.lockInterruptibly();
                     fullCondition.signalAll();
-                } catch (InterruptedException ex) {
-                    throw new UndeclaredThrowableException(ex);
+                } catch (InterruptedException e1) {
+                    // Lost slot available notification
+                    Thread.currentThread().interrupt();
                 } finally {
                     if (usageWLock.isHeldByCurrentThread()) {
                         usageWLock.unlock();
@@ -278,11 +282,12 @@ public class RrdDbPool {
 
     /**
      * Releases RrdDb reference previously obtained from the pool. When a reference is released, its usage
-     * count is decremented by one. If usage count drops to zero, the underlying RRD file will be closed.
+     * count is decremented by one. If usage count drops to zero, the underlying RRD will be closed.
      *
      * @param rrdDb RrdDb reference to be returned to the pool
      * @throws java.io.IOException Thrown in case of I/O error
-     * @deprecated a db remember if it was open directly or from the pool, no need to manage it manually any more
+     * @throws java.lang.IllegalStateException if the thread was interrupted
+     * @deprecated A RrdDb remember if it was open directly or from a pool, no need to manage it manually any more
      */
     @Deprecated
     public void release(RrdDb rrdDb) throws IOException {
@@ -291,8 +296,7 @@ public class RrdDbPool {
         if (rrdDb == null) {
             return;
         }
-
-        URI dburi = rrdDb.getUri();
+        URI dburi = rrdDb.getCanonicalUri();
         RrdEntry ref = null;
         try {
             ref = getEntry(dburi, false);
@@ -326,20 +330,19 @@ public class RrdDbPool {
     }
 
     /**
-     * <p>Requests a RrdDb reference for the given RRD file path.</p>
+     * <p>Requests a RrdDb reference for the given RRD path.</p>
      * <ul>
-     * <li>If the file is already open, previously returned RrdDb reference will be returned. Its usage count
+     * <li>If the RRD is already open, previously returned RrdDb reference will be returned. Its usage count
      * will be incremented by one.
-     * <li>If the file is not already open and the number of already open RRD files is less than
-     * {@link #INITIAL_CAPACITY}, the file will be open and a new RrdDb reference will be returned.
-     * If the file is not already open and the number of already open RRD files is equal to
-     * {@link #INITIAL_CAPACITY}, the method blocks until some RRD file is closed.
+     * <li>If the RRD is not already open and the number of already open RRD is less than
+     * {@link #getCapacity()}, it will be opened and a new RrdDb reference will be returned.
+     * If the RRD is not already open and the number of already open RRD is equal to
+     * {@link #getCapacity()}, the method blocks until some RRD are closed.
      * </ul>
-     * <p>The path is transformed internally to URI using the default factory, that is the reference that will
-     * be used elsewhere.</p>
+     * <p>The path is transformed to an URI using the default factory defined at the creation of the pool.</p>
      *
-     * @param path Path to existing RRD file
-     * @return reference for the give RRD file
+     * @param path Path to existing RRD.
+     * @return reference for the given RRD.
      * @throws java.io.IOException Thrown in case of I/O error
      */
     public RrdDb requestRrdDb(String path) throws IOException {
@@ -347,23 +350,25 @@ public class RrdDbPool {
     }
 
     /**
-     * <p>Requests a RrdDb reference for the given RRD file path.</p>
+     * <p>Requests a RrdDb reference for the given RRD URI.</p>
      * <ul>
-     * <li>If the file is already open, previously returned RrdDb reference will be returned. Its usage count
+     * <li>If the RRD is already open, previously returned RrdDb reference will be returned. Its usage count
      * will be incremented by one.
-     * <li>If the file is not already open and the number of already open RRD files is less than
-     * {@link #INITIAL_CAPACITY}, the file will be open and a new RrdDb reference will be returned.
-     * If the file is not already open and the number of already open RRD files is equal to
-     * {@link #INITIAL_CAPACITY}, the method blocks until some RRD file is closed.
+     * <li>If the RRD is not already open and the number of already open RRD is less than
+     * {@link #getCapacity()}, it will be opened and a new RrdDb reference will be returned.
+     * If the RRD is not already open and the number of already open RRD is equal to
+     * {@link #getCapacity()}, the method blocks until some RRD are closed.
      * </ul>
+     * <p>
+     * If the default backend factory for the pool can handle this URI, it will be used, 
+     * or else {@link RrdBackendFactory#findFactory(URI)} will be used to find the backend factory used.
      *
      * @param uri {@link URI} to existing RRD file
      * @return reference for the give RRD file
      * @throws java.io.IOException Thrown in case of I/O error
      */
     public RrdDb requestRrdDb(URI uri) throws IOException {
-        RrdBackendFactory factory = RrdBackendFactory.findFactory(uri);
-        return requestRrdDb(uri, factory);
+        return requestRrdDb(uri, checkFactory(uri));
     }
 
     /**
@@ -412,7 +417,7 @@ public class RrdDbPool {
             // Someone might have already open it, rechecks
             if (ref.count == 0) {
                 try {
-                    ref.rrdDb = RrdDb.getBuilder().setPath(factory.getPath(uri)).setBackendFactory(factory).setPool(this).build();
+                    ref.rrdDb = RrdDb.getBuilder().setPath(factory.getPath(uri)).setBackendFactory(factory).setPoolInternal(this).build();
                 } catch (IOException | RuntimeException e) {
                     passNext(ACTION.DROP, ref);
                     throw e;
@@ -427,17 +432,17 @@ public class RrdDbPool {
         }
     }
 
-    RrdDb requestRrdDb(RrdDef rrdDef, RrdBackendFactory backend) throws IOException {
+    RrdDb requestRrdDb(RrdDef rrdDef, RrdBackendFactory factory) throws IOException {
         RrdEntry ref = null;
         try {
-            URI uri = backend.getCanonicalUri(rrdDef.getUri());
+            URI uri = factory.getCanonicalUri(rrdDef.getUri());
             ref = requestEmpty(uri);
-            ref.rrdDb = RrdDb.getBuilder().setRrdDef(rrdDef).setBackendFactory(backend).setPool(this).build();
+            ref.rrdDb = RrdDb.getBuilder().setRrdDef(rrdDef).setBackendFactory(factory).setPoolInternal(this).build();
             ref.count = 1;
             return ref.rrdDb;
         } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
-            throw new RuntimeException("request interrupted for new rrdDef " + rrdDef.getPath(), e);
+            throw new IllegalStateException("request interrupted for new rrdDef " + rrdDef.getPath(), e);
         } catch (RuntimeException e) {
             passNext(ACTION.DROP, ref);
             ref = null;
@@ -447,18 +452,18 @@ public class RrdDbPool {
         }
     }
 
-    private RrdDb requestRrdDb(RrdDb.Builder builder, URI uri, RrdBackendFactory backend)
+    private RrdDb requestRrdDb(RrdDb.Builder builder, URI uri, RrdBackendFactory factory)
             throws IOException {
         RrdEntry ref = null;
-        uri = backend.getCanonicalUri(uri);
+        uri = factory.getCanonicalUri(uri);
         try {
             ref = requestEmpty(uri);
-            ref.rrdDb = builder.setPath(uri).setBackendFactory(backend).setPool(this).build();
+            ref.rrdDb = builder.setPath(uri).setBackendFactory(factory).setPoolInternal(this).build();
             ref.count = 1;
             return ref.rrdDb;
         } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
-            throw new RuntimeException("request interrupted for new rrd " + uri, e);
+            throw new IllegalStateException("request interrupted for new rrd " + uri, e);
         } catch (RuntimeException e) {
             passNext(ACTION.DROP, ref);
             ref = null;
@@ -468,46 +473,51 @@ public class RrdDbPool {
         }
     }
 
-    RrdDb requestRrdDb(URI uri, RrdBackendFactory backend, DataImporter importer) throws IOException {
-        return requestRrdDb(RrdDb.getBuilder().setImporter(importer), uri, backend);
+    RrdDb requestRrdDb(URI uri, RrdBackendFactory factory, DataImporter importer) throws IOException {
+        return requestRrdDb(RrdDb.getBuilder().setImporter(importer), uri, factory);
     }
 
     /**
-     * <p>Requests a RrdDb reference for the given RRD file definition object.</p>
+     * <p>Requests a RrdDb reference for the given RRD definition object.</p>
      * <ul>
-     * <li>If the file with the path specified in the RrdDef object is already open,
+     * <li>If the RRD with the path specified in the RrdDef object is already open,
      * the method blocks until the file is closed.
-     * <li>If the file is not already open and the number of already open RRD files is less than
-     * {@link #INITIAL_CAPACITY}, a new RRD file will be created and a its RrdDb reference will be returned.
-     * If the file is not already open and the number of already open RRD files is equal to
-     * {@link #INITIAL_CAPACITY}, the method blocks until some RRD file is closed.
+     * <li>If the RRD is not already open and the number of already open RRD is less than
+     * {@link #getCapacity()}, a new RRD will be created and it's RrdDb reference will be returned.
+     * If the RRD is not already open and the number of already open RRD is equal to
+     * {@link #getCapacity()}, the method blocks until some RrdDb references are closed.
      * </ul>
+     * <p>
+     * If the factory defined when creating the pool can handle the URI, it will be used, 
+     * or else {@link RrdBackendFactory#findFactory(URI)} will be used.
      *
-     * @param rrdDef Definition of the RRD file to be created
-     * @return Reference to the newly created RRD file
+     * @param rrdDef Definition of the RRD file to be created.
+     * @return Reference to the newly created RRD file.
      * @throws java.io.IOException Thrown in case of I/O error
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public RrdDb requestRrdDb(RrdDef rrdDef) throws IOException {
-        return requestRrdDb(rrdDef, RrdBackendFactory.findFactory(rrdDef.getUri()));
+        return requestRrdDb(rrdDef, checkFactory(rrdDef.getUri()));
     }
 
     /**
-     * <p>Requests a RrdDb reference for the given path. The file will be created from
+     * <p>Requests a RrdDb reference for the given path. The RRD will be created from
      * external data (from XML dump or RRDTool's binary RRD file).</p>
      * <ul>
-     * <li>If the file with the path specified is already open,
+     * <li>If the RRD with the path specified is already open,
      * the method blocks until the file is closed.
-     * <li>If the file is not already open and the number of already open RRD files is less than
-     * {@link #INITIAL_CAPACITY}, a new RRD file will be created and a its RrdDb reference will be returned.
-     * If the file is not already open and the number of already open RRD files is equal to
-     * {@link #INITIAL_CAPACITY}, the method blocks until some RRD file is closed.
+     * <li>If the RRD is not already open and the number of already open RRD is less than
+     * {@link #getCapacity()}, a new RRD will be created and it's RrdDb reference will be returned.
+     * If the RRD is not already open and the number of already open RRD is equal to
+     * {@link #getCapacity()}, the method blocks until some RrdDb references are closed.
      * </ul>
-     * <p>The path is transformed internally to an URI using the default factory of the pool.</p>
+     * <p>The path is transformed to an URI using the default factory of the pool.</p>
      *
-     * @param path       Path to RRD file which should be created
-     * @param sourcePath Path to external data which is to be converted to Rrd4j's native RRD file format
-     * @return Reference to the newly created RRD file
+     * @param path       Path to the RRD that should be created.
+     * @param sourcePath Path to external data which is to be converted to Rrd4j's native RRD file format.
+     * @return Reference to the newly created RRD.
      * @throws java.io.IOException Thrown in case of I/O error
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public RrdDb requestRrdDb(String path, String sourcePath)
             throws IOException {
@@ -516,35 +526,39 @@ public class RrdDbPool {
     }
 
     /**
-     * <p>Requests a RrdDb reference for the given path. The file will be created from
+     * <p>Requests a RrdDb reference for the given URI. The RRD will be created from
      * external data (from XML dump or RRDTool's binary RRD file).</p>
      * <ul>
-     * <li>If the file with the path specified is already open,
+     * <li>If the RRD with the URI specified is already open,
      * the method blocks until the file is closed.
-     * <li>If the file is not already open and the number of already open RRD files is less than
-     * {@link #INITIAL_CAPACITY}, a new RRD file will be created and a its RrdDb reference will be returned.
-     * If the file is not already open and the number of already open RRD files is equal to
-     * {@link #INITIAL_CAPACITY}, the method blocks until some RRD file is closed.
+     * <li>If the RRD is not already open and the number of already open RRD is less than
+     * {@link #getCapacity()}, a new RRD will be created and it's RrdDb reference will be returned.
+     * If the RRD is not already open and the number of already open RRD is equal to
+     * {@link #getCapacity()}, the method blocks until some RrdDb references are closed.
      * </ul>
-     * <p>The path is transformed internally to URI using the default factory, that is the reference that will
-     * be used elsewhere.</p>
+     * If the factory defined when creating the pool can handle the URI, it will be used, 
+     * or else {@link RrdBackendFactory#findFactory(URI)} will be used to choose the factory.
      *
-     * @param uri       Path to RRD file which should be created
+     * @param uri        URI to the RRD that should be created
      * @param sourcePath Path to external data which is to be converted to Rrd4j's native RRD file format
-     * @return Reference to the newly created RRD file
+     * @return Reference to the newly created RRD
      * @throws java.io.IOException Thrown in case of I/O error
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public RrdDb requestRrdDb(URI uri, String sourcePath)
             throws IOException {
-        return requestRrdDb(RrdDb.getBuilder().setExternalPath(sourcePath), uri, RrdBackendFactory.findFactory(uri));
+        return requestRrdDb(RrdDb.getBuilder().setExternalPath(sourcePath), uri, checkFactory(uri));
     }
 
     /**
-     * Sets the default factory to use when obtaining rrdDb from simple path and not URI.
+     * Sets the default factory to use when obtaining RrdDb reference from simple path and not URI.
      *
-     * @param defaultFactory The factory to used.
-     * @throws IllegalStateException if done will the pool is not empty or the thread was interrupted.
+     * @param defaultFactory The factory to use.
+     * @throws IllegalStateException if called while the pool is not empty or the thread was interrupted
+     * @throws java.lang.IllegalStateException if the thread was interrupted
+     * @deprecated the pool is no longer a singleton, create a new pool instead of changing it.
      */
+    @Deprecated
     public void setDefaultFactory(RrdBackendFactory defaultFactory) {
         try {
             usageWLock.lockInterruptibly();
@@ -563,10 +577,10 @@ public class RrdDbPool {
     }
 
     /**
-     * Sets the maximum number of simultaneously open RRD files.
+     * Sets the maximum number of simultaneously open RRD.
      *
-     * @param newCapacity Maximum number of simultaneously open RRD files.
-     * @throws IllegalStateException if done will the pool is not empty or the thread was interrupted.
+     * @param newCapacity Maximum number of simultaneously open RRD.
+     * @throws IllegalStateException if called while the pool is not empty or the thread was interrupted.
      */
     public void setCapacity(int newCapacity) {
         try {
@@ -587,9 +601,10 @@ public class RrdDbPool {
     }
 
     /**
-     * Returns the maximum number of simultaneously open RRD files.
+     * Returns the maximum number of simultaneously open RRD.
      *
-     * @return maximum number of simultaneously open RRD files
+     * @return maximum number of simultaneously open RRD
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public int getCapacity() {
         try {
@@ -609,42 +624,76 @@ public class RrdDbPool {
      * Returns the number of usage for a RRD.
      *
      * @param rrdDb RrdDb reference for which informations is needed.
-     * @return the number of request for this rrd
-     * @throws java.io.IOException if any.
+     * @return the number of request for this RRD.
+     * @throws java.io.IOException if any
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public int getOpenCount(RrdDb rrdDb) throws IOException {
-        return getOpenCount(rrdDb.getUri());
+        return getCanonicalUriUsage(rrdDb.getCanonicalUri());
     }
 
     /**
      * Returns the number of usage for a RRD.
+     * <p>The path is transformed to an URI using the default factory.</p>
      *
      * @param path RRD's path for which informations is needed.
-     * @return the number of request for this file
-     * @throws java.io.IOException if any.
+     * @return the number of request for this RRD.
+     * @throws java.io.IOException if any
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public int getOpenCount(String path) throws IOException {
-        return getOpenCount(defaultFactory.getUri(path));
+        return getCanonicalUriUsage(defaultFactory.getCanonicalUri(defaultFactory.getUri(path)));
     }
 
     /**
      * Returns the number of usage for a RRD.
      *
-     * @param uri RRD's uri for which informations is needed.
-     * @return the number of request for this file
-     * @throws java.io.IOException if any.
+     * @param uri RRD's URI for which informations is needed.
+     * @return the number of request for this RRD.
+     * @throws java.io.IOException if any
+     * @throws java.lang.IllegalStateException if the thread was interrupted
      */
     public int getOpenCount(URI uri) throws IOException {
+        return getCanonicalUriUsage(checkFactory(uri).getCanonicalUri(uri));
+    }
+
+    private int getCanonicalUriUsage(URI uri) {
         RrdEntry ref = null;
         try {
             ref = getEntry(uri, false);
             return Optional.ofNullable(ref).map(e -> e.count).orElse(0);
         } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
-            throw new RuntimeException("getOpenCount interrupted", e);
+            throw new IllegalStateException("getOpenCount interrupted", e);
         } finally {
             passNext(ACTION.SWAP, ref);
         }
     }
 
+    /**
+     * Wait until the pool is empty and return a lock that prevent any additions of new RrdDb references until it's released.
+     * 
+     * @since 3.7
+     *
+     * @param timeout the time to wait for the write lock
+     * @param unit the time unit of the timeout argument
+     * @return a lock to release when operations on this pool are finished.
+     * @throws InterruptedException if interrupted whole waiting for the lock
+     */
+    public Lock lockEmpty(long timeout, TimeUnit unit) throws InterruptedException {
+        usageWLock.tryLock(timeout, unit);
+        try {
+            usage.acquire(maxCapacity);
+        } catch (InterruptedException e) {
+            usageWLock.unlock();
+            Thread.currentThread().interrupt();
+            throw e;
+        }
+        return usageWLock;
+    }
+
+    private RrdBackendFactory checkFactory(URI uri) {
+        return defaultFactory.canStore(uri) ? defaultFactory : RrdBackendFactory.findFactory(uri);
+    }
+
 }
