@@ -10,6 +10,7 @@
 #
 # zzz - public domain
 #
+cd `dirname $0`
 CLASS=net.i2p.router.web.messages
 TMPFILE=build/javafiles.txt
 export TZ=UTC
@@ -38,7 +39,9 @@ fi
 # router/ now has its own bundle for some files
 ROUTERFILES="\
    ../../../router/java/src/net/i2p/router/Blocklist.java \
+   ../../../router/java/src/net/i2p/router/Router.java \
    ../../../router/java/src/net/i2p/router/networkdb/reseed/Reseeder.java \
+   ../../../router/java/src/net/i2p/router/sybil/Analysis.java \
    ../../../router/java/src/net/i2p/router/tasks/CoalesceStatsEvent.java \
    ../../../router/java/src/net/i2p/router/transport/CommSystemFacadeImpl.java \
    ../../../router/java/src/net/i2p/router/transport/GetBidsJob.java \
@@ -113,9 +116,11 @@ do
 			RC=1
 			break
 		fi
-		rm -f ${i}t
-		# so we don't do this again
-		touch $i
+		# the string with '75%' causes it to add a java-printf-format directive,
+		# and then the testscript fails if the translated
+		# string doesn't have a '%' in it; strip out the directive
+		grep -v java-printf-format $i > ${i}t
+		mv ${i}t ${i}
 	fi
 
     if [ "$LG" != "en" ]
