@@ -1,7 +1,7 @@
 package net.i2p.router.news;
 
 import java.io.File;
-import java.io.FileInputStream;
+import java.io.InputStream;
 import java.io.IOException;
 import java.io.Reader;
 import java.text.DateFormat;
@@ -20,6 +20,7 @@ import net.i2p.data.DataHelper;
 import net.i2p.util.FileUtil;
 import net.i2p.util.Log;
 import net.i2p.util.SystemVersion;
+import net.i2p.util.Translate;
 import net.i2p.util.TranslateReader;
 
 import org.cybergarage.xml.Node;
@@ -48,6 +49,7 @@ public class NewsManager implements ClientApp {
 
     public static final String APP_NAME = "news";
     private static final String BUNDLE_NAME = "net.i2p.router.news.messages";
+    private static final String WELCOME_AUTHOR = "I2P Development Team";
 
     /**
      *  @param args ignored
@@ -88,7 +90,7 @@ public class NewsManager implements ClientApp {
      *  Does NOT update the NewsEntry list.
      *
      *  @param entries each one should be "entry" at the root
-     *  @return success
+     *  @return true if any new entry was written (not if changed)
      */
     public synchronized boolean storeEntries(List<Node> entries) {
         return PersistNews.store(_context, entries);
@@ -190,12 +192,17 @@ public class NewsManager implements ClientApp {
     }
 
     private List<NewsEntry> parseInitialNews() {
-        File file = new File(_context.getBaseDir(), "docs/initialNews/initialNews.xml");
+        InputStream is = NewsManager.class.getResourceAsStream("/net/i2p/router/news/resources/docs/initialNews/initialNews.xml");
+        if (is == null) {
+            if (_log.shouldWarn())
+                _log.warn("failed to load initial news");
+            return Collections.emptyList();
+        }
         Reader reader = null;
         try {
             char[] buf = new char[512];
             StringBuilder out = new StringBuilder(2048);
-            reader = new TranslateReader(_context, BUNDLE_NAME, new FileInputStream(file));
+            reader = new TranslateReader(_context, BUNDLE_NAME, is);
             int len;
             while((len = reader.read(buf)) > 0) {
                 out.append(buf, 0, len);
@@ -204,16 +211,21 @@ public class NewsManager implements ClientApp {
             if (!rv.isEmpty()) {
                 //rv.get(0).updated = RFC3339Date.parse3339Date("2015-01-01");
                 rv.get(0).updated = _context.clock().now();
+                // Tagged in initialNews.xml inside a comment
+                rv.get(0).authorName = Translate.getString(WELCOME_AUTHOR, _context, BUNDLE_NAME);
             } else {
                 if (_log.shouldWarn())
-                    _log.warn("failed to load " + file);
+                    _log.warn("failed to load initial news");
             }
             return rv;
         } catch (IOException ioe) {
             if (_log.shouldWarn())
-                _log.warn("failed to load " + file, ioe);
+                _log.warn("failed to load initial news", ioe);
             return Collections.emptyList();
         } finally {
+            try {
+                is.close();
+            } catch (IOException foo) {}
             try {
                 if (reader != null)
                     reader.close();
