@@ -25,6 +25,8 @@ public class NewsHelper extends ContentHelper {
     public static final String PROP_LAST_CHECKED = "routerconsole.newsLastChecked";
     /** @since 0.9.4 */
     public static final String PROP_LAST_UPDATED = "routerconsole.newsLastUpdated";
+    /** @since 0.9.55 */
+    public static final String PROP_LAST_NEW_ENTRY = "routerconsole.newsLastNewEntry";
     /**
      * Default true
      * @since 0.9.21
@@ -69,6 +71,18 @@ public class NewsHelper extends ContentHelper {
                mgr.getUpdateAvailable(ROUTER_SIGNED_SU3) != null;
     }
 
+    /**
+     *  A router update had been downloaded and handled by an UpdatePostProcessor.
+     *  It will provide wrapper-like function to install the update and restart after shutdown.
+     *
+     *  @since 0.9.51
+     */
+    public static boolean isExternalRestartPending() {
+        ConsoleUpdateManager mgr = ConsoleUpdateManager.getInstance();
+        if (mgr == null) return false;
+        return mgr.isExternalRestartPending();
+    }
+
     /**
      *  Release update only.
      *  Available version, will be null if already downloaded.
@@ -244,7 +258,7 @@ public class NewsHelper extends ContentHelper {
      *  @since 0.9.4
      */
     public static boolean shouldShowNews(RouterContext ctx) {
-        long lastUpdated = lastUpdated(ctx);
+        long lastUpdated = lastNewEntry(ctx);
         if (lastUpdated <= 0)
             return true;
         long last = ctx.getProperty(PROP_LAST_HIDDEN, 0L);
@@ -264,7 +278,7 @@ public class NewsHelper extends ContentHelper {
      *  @since 0.9.4
      */
     public static void showNews(RouterContext ctx, boolean yes) {
-        long stamp = yes ? 0 : lastUpdated(ctx);
+        long stamp = yes ? 0 : lastNewEntry(ctx);
         ctx.router().saveConfig(PROP_LAST_HIDDEN, Long.toString(stamp));
     }
 
@@ -358,4 +372,17 @@ public class NewsHelper extends ContentHelper {
         ctx.router().saveConfig(PROP_LAST_UPDATED, Long.toString(rv));
         return rv;
     }
+
+    /**
+     *  When we last got a new news entry
+     *  @since 0.9.55
+     */
+    private static long lastNewEntry(RouterContext ctx) {
+        long rv = ctx.getProperty(PROP_LAST_NEW_ENTRY, 0L);
+        if (rv > 0)
+            return rv;
+        rv = lastUpdated(ctx);
+        ctx.router().saveConfig(PROP_LAST_NEW_ENTRY, Long.toString(rv));
+        return rv;
+    }
 }
