@@ -434,7 +434,7 @@ public class PluginStarter implements Runnable {
         if(fullprop != null && fullprop.length() > 1){
             byte[] decoded = Base64.decode(fullprop);
             if(decoded != null) {
-                NavHelper.setBinary(appName, decoded);
+                NavHelper.getInstance(ctx).setBinary(appName, decoded);
                 iconfile = "/Plugins/pluginicon?plugin=" + appName;
             } else {
                 iconfile = "/themes/console/images/plugin.png";
@@ -475,7 +475,7 @@ public class PluginStarter implements Runnable {
                             if (log.shouldLog(Log.INFO))
                                 log.info("Starting webapp: " + warName);
                             String path = files[i].getCanonicalPath();
-                            WebAppStarter.startWebApp(ctx, server, warName, path);
+                            WebAppStarter.startWebApp(ctx, server, warName, path, appName);
                             pluginWars.get(appName).add(warName);
                         }
                     } catch (IOException ioe) {
@@ -486,9 +486,23 @@ public class PluginStarter implements Runnable {
                 String icfile = stripHTML(props, "console-icon");
                 if (icfile != null && !icfile.contains("..")) {
                     StringBuilder buf = new StringBuilder(32);
-                    buf.append('/').append(appName);
-                    if (!icfile.startsWith("/"))
+                    String url = stripHTML(props, "consoleLinkURL");
+                    if (url != null) {
+                        if (url.endsWith(".html") || url.endsWith(".htm") || url.endsWith(".jsp") || url.endsWith("/home")) {
+                            int slash = url.lastIndexOf('/');
+                            if (slash > 0)
+                                url = url.substring(0, slash);
+                        }
+                    } else {
+                        url = appName;
+                    }
+                    if (!url.startsWith("/"))
                         buf.append('/');
+                    buf.append(url);
+                    if (!url.endsWith("/") && !icfile.startsWith("/"))
+                        buf.append('/');
+                    else if (url.endsWith("/") && icfile.startsWith("/"))
+                        icfile = icfile.substring(1);
                     buf.append(icfile);
                     iconfile = buf.toString();
                 }
@@ -532,7 +546,7 @@ public class PluginStarter implements Runnable {
             String tip = stripHTML(props, "consoleLinkTooltip_" + Messages.getLanguage(ctx));
             if (tip == null)
                 tip = stripHTML(props, "consoleLinkTooltip");
-            NavHelper.registerApp(appName, name, url, tip, iconfile);
+            NavHelper.getInstance(ctx).registerApp(appName, name, url, tip, iconfile);
         }
 
         return true;
@@ -562,13 +576,23 @@ public class PluginStarter implements Runnable {
         if (log.shouldLog(Log.WARN))
             log.warn("Stopping plugin: " + appName);
 
-        // stop things in clients.config
-        File clientConfig = new File(pluginDir, "clients.config");
-        if (clientConfig.exists()) {
-            Properties props = new Properties();
-            DataHelper.loadProps(props, clientConfig);
-            List<ClientAppConfig> clients = ClientAppConfig.getClientApps(clientConfig);
-            runClientApps(ctx, pluginDir, clients, "stop");
+        ClientApp client = ctx.clientAppManager().getRegisteredApp(appName);
+        if (client != null) {
+            try{
+                client.shutdown(null);
+            }catch(Throwable t){
+                if (log.shouldLog(Log.ERROR))
+                    log.error("Error stopping client app: " + appName, t);
+            }
+        } else {
+            // stop things in clients.config
+            File clientConfig = new File(pluginDir, "clients.config");
+            if (clientConfig.exists()) {
+                Properties props = new Properties();
+                DataHelper.loadProps(props, clientConfig);
+                List<ClientAppConfig> clients = ClientAppConfig.getClientApps(clientConfig);
+                runClientApps(ctx, pluginDir, clients, "stop");
+            }
         }
 
         // stop console webapps in console/webapps
@@ -603,7 +627,7 @@ public class PluginStarter implements Runnable {
         //}
 
         // remove summary bar link
-        NavHelper.unregisterApp(appName);
+        NavHelper.getInstance(ctx).unregisterApp(appName);
 
         return true;
     }
@@ -681,7 +705,7 @@ public class PluginStarter implements Runnable {
         File dir = I2PAppContext.getGlobalContext().getConfigDir();
         Properties rv = new Properties();
         File cfgFile = new File(dir, CONFIG_FILE);
-        
+
         try {
             DataHelper.loadProps(rv, cfgFile);
         } catch (IOException ioe) {}
@@ -792,7 +816,7 @@ public class PluginStarter implements Runnable {
      */
     private static void runClientApps(RouterContext ctx, File pluginDir, List<ClientAppConfig> apps, String action) throws Exception {
         Log log = ctx.logManager().getLog(PluginStarter.class);
-        
+
         // initialize pluginThreadGroup and _pendingPluginClients
         String pluginName = pluginDir.getName();
         if (!pluginThreadGroups.containsKey(pluginName))
@@ -800,7 +824,7 @@ public class PluginStarter implements Runnable {
         ThreadGroup pluginThreadGroup = pluginThreadGroups.get(pluginName);
         if (action.equals("start"))
             _pendingPluginClients.put(pluginName, new ConcurrentHashSet<SimpleTimer2.TimedEvent>());
-        
+
         for(ClientAppConfig app : apps) {
             // If the client is a running ClientApp that we want to stop,
             // bypass all the logic below.
@@ -814,6 +838,8 @@ public class PluginStarter implements Runnable {
                         argVal[i] = argVal[i].replace("$I2P", ctx.getBaseDir().getAbsolutePath());
                         argVal[i] = argVal[i].replace("$CONFIG", ctx.getConfigDir().getAbsolutePath());
                         argVal[i] = argVal[i].replace("$PLUGIN", pluginDir.getAbsolutePath());
+                        argVal[i] = argVal[i].replace("$OS", SystemVersion.getOS());
+                        argVal[i] = argVal[i].replace("$ARCH", SystemVersion.getArch());
                     }
                 }
                 ClientApp ca = ctx.routerAppManager().getClientApp(app.className, argVal);
@@ -853,6 +879,8 @@ public class PluginStarter implements Runnable {
                     argVal[i] = argVal[i].replace("$I2P", ctx.getBaseDir().getAbsolutePath());
                     argVal[i] = argVal[i].replace("$CONFIG", ctx.getConfigDir().getAbsolutePath());
                     argVal[i] = argVal[i].replace("$PLUGIN", pluginDir.getAbsolutePath());
+                    argVal[i] = argVal[i].replace("$OS", SystemVersion.getOS());
+                    argVal[i] = argVal[i].replace("$ARCH", SystemVersion.getArch());
                 }
             }
 
@@ -863,6 +891,8 @@ public class PluginStarter implements Runnable {
                     cp = cp.replace("$I2P", ctx.getBaseDir().getAbsolutePath());
                     cp = cp.replace("$CONFIG", ctx.getConfigDir().getAbsolutePath());
                     cp = cp.replace("$PLUGIN", pluginDir.getAbsolutePath());
+                    cp = cp.replace("$OS", SystemVersion.getOS());
+                    cp = cp.replace("$ARCH", SystemVersion.getArch());
                 }
 
                 // Old way - add for the whole JVM
@@ -903,7 +933,7 @@ public class PluginStarter implements Runnable {
                     // quick check
                     LoadClientAppsJob.testClient(app.className, cl);
                 } catch (ClassNotFoundException ex) {
-                    // Try again 1 or 2 seconds later. 
+                    // Try again 1 or 2 seconds later.
                     // This should be enough time. Although it is a lousy hack
                     // it should work for most cases.
                     // Perhaps it may be even better to delay a percentage
@@ -966,7 +996,7 @@ public class PluginStarter implements Runnable {
      */
     protected static boolean isPluginRunning(String pluginName, RouterContext ctx, Server s) {
         Log log = ctx.logManager().getLog(PluginStarter.class);
-        
+
         boolean isJobRunning = false;
         Collection<SimpleTimer2.TimedEvent> pending = _pendingPluginClients.get(pluginName);
         if (pending != null && !pending.isEmpty()) {
@@ -987,17 +1017,35 @@ public class PluginStarter implements Runnable {
             }
         }
 
+        // load and check for ShellServices.
+        boolean isProcessRunning = false;
+        ClientApp client = ctx.clientAppManager().getRegisteredApp(pluginName);
+        if (client != null) {
+            if (log.shouldLog(Log.DEBUG))
+                log.debug("Checking state of client " + pluginName + client.getState());
+            if (client.getState() == ClientAppState.RUNNING) {
+                isProcessRunning = true;
+            }
+        } else {
+            if (log.shouldLog(Log.DEBUG))
+                log.debug("No client found for plugin " + pluginName);
+        }
+
         boolean isClientThreadRunning = isClientThreadRunning(pluginName, ctx);
         if (log.shouldLog(Log.DEBUG))
-            log.debug("plugin name = <" + pluginName + ">; threads running? " + isClientThreadRunning + "; webapp running? " + isWarRunning + "; jobs running? " + isJobRunning);
-        return isClientThreadRunning || isWarRunning || isJobRunning;
+            log.debug("plugin name = <" + pluginName +
+            ">; threads running? " + isClientThreadRunning +
+            "; webapp running? " + isWarRunning +
+            "; jobs running? " + isJobRunning +
+            "; process running? " + isProcessRunning);
+        return isClientThreadRunning || isWarRunning || isJobRunning || isProcessRunning;
         //
         //if (log.shouldLog(Log.DEBUG))
         //    log.debug("plugin name = <" + pluginName + ">; threads running? " + isClientThreadRunning(pluginName) + "; webapp running? " + WebAppStarter.isWebAppRunning(pluginName) + "; jobs running? " + isJobRunning);
         //return isClientThreadRunning(pluginName) || WebAppStarter.isWebAppRunning(pluginName) || isJobRunning;
         //
     }
-    
+
     /**
      * Returns <code>true</code> if one or more client threads are running in a given plugin.
      * @param pluginName
@@ -1008,8 +1056,8 @@ public class PluginStarter implements Runnable {
         if (group == null)
             return false;
         boolean rv = group.activeCount() > 0;
-        
-        // Plugins start before the eepsite, and will create the static Timer thread
+
+        // Plugins start before the I2P Site, and will create the static Timer thread
         // in RolloverFileOutputStream, which never stops. Don't count it.
         // Ditto HSQLDB Timer (jwebcache)
         if (rv) {
@@ -1032,7 +1080,7 @@ public class PluginStarter implements Runnable {
 
         return rv;
     }
-    
+
     /**
      *  Perhaps there's an easy way to use Thread.setContextClassLoader()
      *  but I don't see how to make it magically get used for everything.
