@@ -15,7 +15,6 @@ import net.i2p.router.web.CSSHelper;
 import net.i2p.router.web.FormHandler;
 import static net.i2p.router.web.GraphConstants.*;
 import net.i2p.router.web.HelperBase;
-import net.i2p.router.web.NavHelper;
 import net.i2p.router.web.StatSummarizer;
 import net.i2p.router.web.SummaryListener;
 import net.i2p.stat.Rate;
@@ -32,6 +31,7 @@ public class GraphHelper extends FormHandler {
     private int _refreshDelaySeconds;
     private boolean _persistent;
     private boolean _graphHideLegend;
+    private boolean _utc;
     private String _stat;
     private int _end;
 
@@ -131,6 +131,9 @@ public class GraphHelper extends FormHandler {
     /** @since 0.9.32 */
     public void setHideLegend(String foo) { _graphHideLegend = true; }
 
+    /** @since 0.9.49 */
+    public void setUtc(String foo) { _utc = true; }
+
     /**
      *  For single stat page
      *  @since 0.9
@@ -157,6 +160,8 @@ public class GraphHelper extends FormHandler {
                 else if (title.equals("bw.recvRate")) hasRx = true;
             }
             boolean hideLegend = _context.getProperty(PROP_LEGEND, DEFAULT_LEGEND);
+            // param is ignored, we get it from the property, but add it to defeat caching on change
+            boolean utc = _context.getBooleanProperty(PROP_UTC);
 
             if (hasTx && hasRx && !_showEvents) {
                 // remove individual tx/rx graphs if displaying combined
@@ -170,6 +175,7 @@ public class GraphHelper extends FormHandler {
                            + "&amp;c=" + (3 * _periodCount )
                            + "&amp;w=" + (3 * _width)
                            + "&amp;h=" + (3 * _height)
+                           + (utc ? "&amp;utc" : "")
                            + "\">");
                 String title = _t("Combined bandwidth graph");
                 _out.write("<img class=\"statimage\""
@@ -184,6 +190,8 @@ public class GraphHelper extends FormHandler {
                     // no legend, no height difference needed
                     _out.write("&amp;height=" + (_height));
                 }
+                if (utc)
+                    _out.write("&amp;utc");
                 _out.write("&amp;hideLegend=" + hideLegend
                            + "\" alt=\"" + title + "\" title=\"" + title + "\"></a>\n");
             }
@@ -199,6 +207,7 @@ public class GraphHelper extends FormHandler {
                            + "&amp;w=" + (3 * _width)
                            + "&amp;h=" + (3 * _height)
                            + (_showEvents ? "&amp;showEvents=1" : "")
+                           + (utc ? "&amp;utc" : "")
                            + "\">");
                 _out.write("<img class=\"statimage\" border=\"0\""
                            + " src=\"viewstat.jsp?stat="
@@ -209,6 +218,7 @@ public class GraphHelper extends FormHandler {
                            + "&amp;width=" + _width
                            + "&amp;height=" + _height
                            + "&amp;hideLegend=" + hideLegend
+                           + (utc ? "&amp;utc" : "")
                            + "\" alt=\"" + title
                            + "\" title=\"" + title + "\"></a>\n");
             }
@@ -344,9 +354,7 @@ public class GraphHelper extends FormHandler {
             _out.write(link(_stat, !_showEvents, _periodCount, _end, _width, _height));
             if (!_stat.equals("bw.combined"))
                 _out.write(_showEvents ? _t("Plot averages") : _t("plot events"));
-            _out.write("</a>");
-
-            _out.write("</p><p><i>" + _t("All times are UTC.") + "</i></p>\n");
+            _out.write("</a></p>\n");
         } catch (IOException ioe) {
             ioe.printStackTrace();
         }
@@ -389,7 +397,7 @@ public class GraphHelper extends FormHandler {
                        + "\">" + _t("pixels wide") + "&nbsp;&nbsp;&nbsp;<input size=\"4\" style=\"text-align: right;\" type=\"text\" name=\"height\" value=\"" + _height
                        + "\">" + _t("pixels high") + "</span><br><span class=\"nowrap\">\n<b>");
 
-            _out.write(_t("Display period") + ":</b> <input size=\"5\" style=\"text-align: right;\" type=\"text\" name=\"periodCount\" value=\"" + _periodCount + "\">" + _t("minutes") + "</span><br><span class=\"nowrap\">\n<b>");
+            _out.write(_t("Display period") + ":</b> <input size=\"4\" style=\"text-align: right;\" type=\"text\" name=\"periodCount\" value=\"" + _periodCount + "\">" + _t("minutes") + "</span><br><span class=\"nowrap\">\n<b>");
 
 
             _out.write(_t("Refresh delay") + ":</b> <select name=\"refreshDelay\">");
@@ -417,6 +425,12 @@ public class GraphHelper extends FormHandler {
             if (hideLegend)
                 _out.write(HelperBase.CHECKED);
             _out.write(">" + _t("Do not show legend on graphs") + "</label></span><br><span class=\"nowrap\">\n<b>");
+            _out.write(_t("UTC") + ":</b> ");
+            _out.write("<label><input type=\"checkbox\" class=\"optbox\" value=\"true\" name=\"utc\"");
+            boolean utc = _context.getBooleanProperty(PROP_UTC);
+            if (utc)
+                _out.write(HelperBase.CHECKED);
+            _out.write(">" + _t("Use UTC time on graphs") + "</label></span><br><span class=\"nowrap\">\n<b>");
             _out.write(_t("Persistence") +
                        ":</b> <label><input type=\"checkbox\" class=\"optbox\" value=\"true\" name=\"persistent\"");
             boolean persistent = _context.getBooleanPropertyDefaultTrue(SummaryListener.PROP_PERSISTENT);
@@ -477,6 +491,7 @@ public class GraphHelper extends FormHandler {
             _refreshDelaySeconds != _context.getProperty(PROP_REFRESH, DEFAULT_REFRESH) ||
             _showEvents != _context.getBooleanProperty(PROP_EVENTS) ||
             _graphHideLegend != _context.getProperty(PROP_LEGEND, DEFAULT_LEGEND) ||
+            _utc != _context.getBooleanProperty(PROP_UTC) ||
             _persistent != _context.getBooleanPropertyDefaultTrue(SummaryListener.PROP_PERSISTENT)) {
             Map<String, String> changes = new HashMap<String, String>();
             changes.put(PROP_X, Integer.toString(_width));
@@ -486,6 +501,7 @@ public class GraphHelper extends FormHandler {
             changes.put(PROP_EVENTS, Boolean.toString(_showEvents));
             changes.put(PROP_LEGEND, Boolean.toString(_graphHideLegend));
             changes.put(SummaryListener.PROP_PERSISTENT, Boolean.toString(_persistent));
+            changes.put(PROP_UTC, Boolean.toString(_utc));
             boolean warn = _persistent != _context.getBooleanPropertyDefaultTrue(SummaryListener.PROP_PERSISTENT);
             _context.router().saveConfig(changes, null);
             addFormNotice(_t("Graph settings saved"));
