@@ -39,18 +39,99 @@ public class HomeHelper extends HelperBase {
                 "</div>\n" + 
                 "</div>\n";
 
+/*
+
+Policy adapted from http://zzz.i2p/topics/236 last updated there 2016-08.
+Subject to change without notice.
+
+How to get my Eepsite added to the Router Console home page
+
+If your site is:
+
+- Broadly useful, in good taste, and of general interest to the I2P community
+- Is not an image board (chan)
+- Is not a general image host or file host unless it has strict editorial control and TOS (but even so we probably won't accept it)
+- If it contains user-generated content, or is a forum, tracker, file host, wiki, or anything allowing user comments,
+  it has English (and native language if different) terms of service posted that prohibit extremely inappropriate things.
+- If it is an index of i2p sites, it does not link to sites that are extremely inappropriate.
+- Reliably up for a few months at least
+- Will be up 24/7 except for maintenance, updates, etc
+- Decently fast (home DSL/cable is fine)
+- On a router running the current release
+
+...then contact a dev.
+
+** You MUST include the following info ** :
+
+- Affirm that you are the owner of the site in question. We do not add links to sites without permission.
+- The hostname
+- The registration authentication string, or the registration site or feed where it is available
+- An email address
+- The URL to link to
+- The URL of your English terms of service if available or necessary
+- If the site is not in English, a brief description of the site in English
+- A URL to a 32x32 transparent png icon to display. We will copy this icon into the router console source and serve it locally.
+  64x64 is ok also but it will be scaled to 32x32.
+- The license of the icon.
+- (Optional) A one or two-word label in English. If not provided we will use example.i2p
+- (Optional) a few words or a sentence in English for a popup (tooltip)
+- Affirm that you will regularly update the router to the latest release
+
+Translations will happen through our normal translation process.
+
+Other criteria:
+
+Your site should not require browsers to load clearnet resources.
+Please test your site to ensure that clearnet Javascript, CSS, fonts and images are not embedded.
+Your submission may be rejected for this reason.
+
+All decisions on inclusion will be made at a dev meeting, generally in IRC #i2p-dev
+on a Tuesday at 8 PM UTC about 2-4 weeks before a scheduled release.
+The group decision at the meeting is final.
+
+Space is limited. Not all requests will be accepted, even if they meet all the above criteria.
+
+(end of content adapted from zzz.i2p)
+
+Steps for the devs after receiving a submission:
+
+- Verify hostname was not previously used or registered anywhere by another site
+- Verify compatibility with all criteria
+- Verify icon is consistent with the console home page style and will look fine in both themes
+- Verify icon license compatibility with our licenses
+- Validate authentication string unless submitted to a registration site that does that
+- Pick the right category or discuss with submitter
+- Ensure display name is short and reasonable and translatable
+- Ensure description is short and reasonable and translatable
+- Resolve any issues with submitter
+- Add to agenda for a monthly meeting before the scheduled tag freeze
+
+Steps for the devs after approval at a meeting:
+
+- Add to the bottom of the correct category below
+- Ensure display name is tagged and does not contain commas unless the hostname alone
+- Ensure description is tagged and does not contain commas
+- Check in the icon in the right place with a description of the license in the checkin comment
+- Add to installer/resources/hosts.txt WITHOUT the authentication string and check in
+- Check in this file HomeHelper.java
+- Test
+- Add to i2p.www i2p2www/static/hosts.txt WITH the authentication string and check in
+  (don't forget this step!)
+
+*/
+
     // No commas allowed in text strings!
     static final String DEFAULT_SERVICES =
         _x("Email") + S + _x("Anonymous webmail client") + S + "/webmail" + S + I + "email.png" + S +
         _x("Hidden Services Manager") + S + _x("Control your client and server tunnels") + S + "/i2ptunnelmgr" + S + I + "server_32x32.png" + S +
         _x("Torrents") + S + _x("Built-in anonymous BitTorrent Client") + S + "/torrents" + S + I + "i2psnark.png" + S +
         _x("Web Server") + S + _x("Local web server for hosting your own content on I2P") + S + "http://127.0.0.1:7658/" + S + I + "server_32x32.png" + S +
-        _x("Addressbook") + S + _x("Manage your I2P hosts file here (I2P domain name resolution)") + S + "/dns" + S + I + "book_addresses.png" + S +
+        _x("Address Book") + S + _x("Manage your I2P hosts file here (I2P domain name resolution)") + S + "/dns" + S + I + "book_addresses.png" + S +
         "";
 
     /** @since 0.9.44 */
     static final String DEFAULT_CONFIG =
-        _x("Configure Homepage") + S + _x("Configure the contents of this page") + S + "/confighome" + S + I + "info/home.png" + S +
+        //_x("Configure Homepage") + S + _x("Configure the contents of this page") + S + "/confighome" + S + I + "info/home.png" + S +
         _x("Configure Bandwidth") + S + _x("I2P Bandwidth Configuration") + S + "/config" + S + I + "info/bandwidth.png" + S +
         // FIXME wasn't escaped
         //_x("Configure UI") + S + _x("Select console theme & language & set optional console password").replace("&", "&amp;") + S + "/configui" + S + I + "info/ui.png" + S +
@@ -66,9 +147,9 @@ public class HomeHelper extends HelperBase {
         //"anoncoin.i2p" + S + _x("The Anoncoin project") + S + "http://anoncoin.i2p/" + S + I + "anoncoin_32.png" + S +
         //"colombo-bt.i2p" + S + _x("The Italian Bittorrent Resource") + S + "http://colombo-bt.i2p/" + S + I + "colomboicon.png" + S +
         //_x("Dev Builds") + S + _x("Development builds of I2P") + S + "http://bobthebuilder.i2p/" + S + I + "script_gear.png" + S +
-        _x("Dev Forum") + S + _x("Development forum") + S + "http://zzz.i2p/" + S + I + "group_gear.png" + S +
+        //_x("Dev Forum") + S + _x("Development forum") + S + "http://zzz.i2p/" + S + I + "group_gear.png" + S +
         //_x("diftracker") + S + _x("Bittorrent tracker") + S + "http://diftracker.i2p/" + S + I + "i2psnark.png" + S +
-        "echelon.i2p" + S + _x("I2P Applications") + S + "http://echelon.i2p/" + S + I + "echelon.png" + S +
+        //"echelon.i2p" + S + _x("I2P Applications") + S + "http://echelon.i2p/" + S + I + "echelon.png" + S +
         //"exchanged.i2p" + S + _x("Anonymous cryptocurrency exchange") + S + "http://exchanged.i2p/" + S + I + "exchanged.png" + S +
         _x("I2P FAQ") + S + _x("Frequently Asked Questions") + S + "http://i2p-projekt.i2p/faq" + S + I + "question.png" + S +
         _x("I2P Forum") + S + _x("Community forum") + S + "http://i2pforum.i2p/" + S + I + "group.png" + S +
@@ -82,8 +163,8 @@ public class HomeHelper extends HelperBase {
         //"jisko.i2p" + S + _x("Simple and fast microblogging website") + S + "http://jisko.i2p/" + S + I + "jisko_console_icon.png" + S +
         //_x("Key Server") + S + _x("OpenPGP Keyserver") + S + "http://keys.i2p/" + S + I + "education.png" + S +
         //"killyourtv.i2p" + S + _x("Debian and Tahoe-LAFS repositories") + S + "http://killyourtv.i2p/" + S + I + "television_delete.png" + S +
-        _x("MuWire") + S + _x("Easy anonymous file sharing") + S + "http://muwire.i2p/" + S + I + "muwire.png" + S +
-        //_x("Open4You") + S + _x("Free eepsite hosting with PHP and MySQL") + S + "http://open4you.i2p/" + S + I + "open4you-logo.png" + S +
+        //_x("MuWire") + S + _x("Easy anonymous file sharing") + S + "http://muwire.i2p/" + S + I + "muwire.png" + S +
+        //_x("Open4You") + S + _x("Free I2P Site hosting with PHP and MySQL") + S + "http://open4you.i2p/" + S + I + "open4you-logo.png" + S +
         //_x("Pastebin") + S + _x("Encrypted I2P Pastebin") + S + "http://zerobin.i2p/" + S + I + "paste_plain.png" + S +
         _x("Planet I2P") + S + _x("I2P News") + S + "http://planet.i2p/" + S + I + "world.png" + S +
         //_x("I2P Plugins") + S + _x("Add-on directory") + S + "http://i2pwiki.i2p/index.php?title=Plugins" + S + I + "info/plugin_link.png" + S +
@@ -92,9 +173,12 @@ public class HomeHelper extends HelperBase {
         _x("Project Website") + S + _x("I2P home page") + S + "http://i2p-projekt.i2p/" + S + I + "glass.png" + S +
         //_x("lenta news [ru]") + S + _x("Russian News Feed") + S + "http://lenta.i2p/" + S + I + "lenta_main_logo.png" + S +
         //"Salt" + S + "salt.i2p" + S + "http://salt.i2p/" + S + I + "salt_console.png" + S +
-        _x("The Tin Hat") + S + _x("Privacy guides and tutorials") + S + "http://secure.thetinhat.i2p/" + S + I + "thetinhat.png" + S +
+        //_x("The Tin Hat") + S + _x("Privacy guides and tutorials") + S + "http://secure.thetinhat.i2p/" + S + I + "thetinhat.png" + S +
         //_x("Ugha's Wiki") + S + S + "http://ugha.i2p/" + S + I + "billiard_marker.png" + S +
         //"sponge.i2p" + S + _x("Seedless and the Robert BitTorrent applications") + S + "http://sponge.i2p/" + S + I + "user_astronaut.png" + S +
+        "notbob.i2p" + S + _x("Not Bob's Address Services") + S + "http://notbob.i2p/" + S + I + "notblob.png" + S +
+        "[Ramble]" + S + _x("Ramble user-moderated forum aggregator") + S + "http://ramble.i2p/" + S + I + "ramble.png" + S +
+        "StormyCloud" + S + _x("StormyCloud Outproxy Services") + S + "http://stormycloud.i2p/" + S + I + "stormycloud.png" + S +
         "";
 
     // No commas allowed in text strings!
@@ -103,8 +187,8 @@ public class HomeHelper extends HelperBase {
         _x("Logs") + S + _x("View the logs") + S + "/logs" + S + I + "info/logs.png" + S +
         _x("Graphs") + S + _x("Visualize information about the router") + S + "/graphs" + S + I + "chart_line.png" + S +
         _x("I2P Technical Docs") + S + _x("Technical documentation") + S + "http://i2p-projekt.i2p/how" + S + I + "education.png" + S +
-        _x("Trac Wiki") + S + S + "http://trac.i2p2.i2p/" + S + I + "trac_wiki.png" + S +
-        _x("I2P Bug Reports") + S + _x("Bug tracker") + S + "http://trac.i2p2.i2p/report/1" + S + I + "bug.png" + S +
+        _x("I2P Wiki") + S + S + "http://wiki.i2p-projekt.i2p/" + S + I + "trac_wiki.png" + S +
+        _x("I2P Bug Reports") + S + _x("Bug tracker") + S + "http://git.idk.i2p/i2p-hackers/i2p.i2p/-/issues" + S + I + "bug.png" + S +
         "stats.i2p" + S + _x("I2P Network Statistics") + S + "http://stats.i2p/cgi-bin/dashboard.cgi" + S + I + "chart_bar.png" + S +
         "";
 
@@ -140,7 +224,7 @@ public class HomeHelper extends HelperBase {
     
     /** @since 0.9.47 */
     public String getPlugins() {
-        List<App> plugins = NavHelper.getClientApps(_context);
+        List<App> plugins = NavHelper.getInstance(_context).getClientApps(_context);
         String table = pluginTable(plugins);
         if (table.length() == 0) {
             return "";
@@ -172,7 +256,7 @@ public class HomeHelper extends HelperBase {
             return "";
         }
         StringBuilder buf = new StringBuilder(1380);
-        buf.append(topWrap(_t("Network Information and Developer Information")));
+        buf.append(topWrap(_t("Network and Developer Information")));
         buf.append(table);
         buf.append(bottomWrap);
         return buf.toString();
@@ -307,7 +391,7 @@ public class HomeHelper extends HelperBase {
         for (App app : apps) {
             String url;
             if (app.name.equals(website) && app.url.equals("http://127.0.0.1:7658/")) {
-                // fixup eepsite link
+                // fixup I2P Site link
                 url = SummaryBarRenderer.getEepsiteURL(pm);
                 if (url == null)
                     continue;
@@ -315,17 +399,22 @@ public class HomeHelper extends HelperBase {
                 url = app.url;
                 // check for disabled webapps and other things
                 if (url.equals("/dns")) {
-                    if (!pm.isRegistered("susidns"))
+                    if (!pm.isRegistered(PortMapper.SVC_SUSIDNS))
                         continue;
                 } else if (url.equals("/webmail")) {
-                    if (!pm.isRegistered("susimail"))
+                    if (!pm.isRegistered(PortMapper.SVC_SUSIMAIL))
                         continue;
                 } else if (url.equals("/torrents")) {
-                    if (!pm.isRegistered("i2psnark"))
+                    if (!pm.isRegistered(PortMapper.SVC_I2PSNARK))
                         continue;
                 } else if (url.equals("/i2ptunnelmgr")) {
-                    if (!pm.isRegistered("i2ptunnel"))
+                    if (!pm.isRegistered(PortMapper.SVC_I2PTUNNEL))
                         continue;
+                    // need both webapp and TCG, but we aren't refreshing
+                    // the icons, so let's not do this
+                    //ClientAppManager cmgr = _context.clientAppManager();
+                    //if (cmgr != null && cmgr.getRegisteredApp("i2ptunnel") == null)
+                    //    continue;
                 } else if (url.equals("/configplugins")) {
                     if (!PluginStarter.pluginsEnabled(_context))
                         continue;
@@ -393,8 +482,8 @@ public class HomeHelper extends HelperBase {
         }
         buf.append("<tr id=\"addnew\"><td colspan=\"2\" align=\"center\"><b>")
            .append(_t("Add")).append(":</b>" +
-                   "</td><td align=\"left\"><input type=\"text\" name=\"nofilter_name\"></td>" +
-                   "<td align=\"left\"><input type=\"text\" size=\"40\" name=\"nofilter_url\"></td></tr>");
+                   "</td><td align=\"left\"><input type=\"text\" name=\"nofilter_name\" required></td>" +
+                   "<td align=\"left\"><input type=\"text\" size=\"40\" name=\"nofilter_url\" required></td></tr>");
         buf.append("</table>\n");
         return buf.toString();
     }
