@@ -1,15 +1,21 @@
 package net.i2p.router.web.helpers;
 
 import java.io.IOException;
+import java.text.Collator;
+import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
+import java.util.TreeMap;
 
+import net.i2p.crypto.EncType;
 import net.i2p.crypto.SigType;
 import net.i2p.data.DataHelper;
+import net.i2p.data.Hash;
 import net.i2p.util.SystemVersion;
 import net.i2p.router.sybil.Analysis;
 import net.i2p.router.web.FormHandler;
+import net.i2p.router.web.Messages;
 
 /**
  *  /netdb
@@ -20,14 +26,16 @@ public class NetDbHelper extends FormHandler {
     private String _routerPrefix;
     private String _version;
     private String _country;
-    private String _family, _caps, _ip, _sybil, _mtu, _ssucaps, _ipv6, _transport;
-    private int _full, _port, _cost, _page, _mode;
+    private String _family, _caps, _ip, _sybil, _mtu, _ssucaps, _ipv6, _transport, _hostname, _sort;
+    private int _full, _port, _cost, _page, _mode, _highPort, _icount;
     private long _date;
     private int _limit = DEFAULT_LIMIT;
     private boolean _lease;
+    private boolean _clientOnly;
     private boolean _debug;
     private boolean _graphical;
     private SigType _type;
+    private EncType _etype;
     private String _newNonce;
     private boolean _postOK;
 
@@ -44,7 +52,10 @@ public class NetDbHelper extends FormHandler {
                                            _x("LeaseSets"),                     // 5
                                            "LeaseSet Debug",                    // 6
                                            "Sybil",                             // 7
-                                           "Advanced Lookup"   };               // 8
+                                           "Advanced Lookup",                   // 8
+                                           "LeaseSet Lookup",                   // 9
+                                           "LeaseSets (Client DBs)"             // 10
+                                          };
 
     private static final String links[] =
                                           {"",                                  // 0
@@ -55,7 +66,11 @@ public class NetDbHelper extends FormHandler {
                                            "?l=1",                              // 5
                                            "?l=2",                              // 6
                                            "?f=3",                              // 7
-                                           "?f=4" };                            // 8
+                                           "?f=4",                              // 8
+                                           "",                                  // 9
+                                           "?l=7",                              // 10
+                                          };
+                                           
 
     public void setRouter(String r) {
         if (r != null && r.length() > 0)
@@ -108,8 +123,16 @@ public class NetDbHelper extends FormHandler {
 
     /** @since 0.9.28 */
     public void setPort(String f) {
+        if (f == null)
+            return;
         try {
-            _port = Integer.parseInt(f);
+            int dash = f.indexOf('-');
+            if (dash > 0) {
+                _port = Integer.parseInt(f.substring(0, dash).trim());
+                _highPort = Integer.parseInt(f.substring(dash + 1).trim());
+            } else {
+                _port = Integer.parseInt(f.trim());
+            }
         } catch (NumberFormatException nfe) {}
     }
 
@@ -119,6 +142,12 @@ public class NetDbHelper extends FormHandler {
             _type = SigType.parseSigType(f);
     }
 
+    /** @since 0.9.49 */
+    public void setEtype(String f) {
+        if (f != null && f.length() > 0)
+            _etype = EncType.parseEncType(f);
+    }
+
     /** @since 0.9.28 */
     public void setMtu(String f) {
         if (f != null && f.length() > 0)
@@ -174,10 +203,17 @@ public class NetDbHelper extends FormHandler {
     }
 
     public void setLease(String l) {
+        _clientOnly = "7".equals(l);
         _debug = "2".equals(l);
         _lease = _debug || "1".equals(l);
     }
 
+    /** @since 0.9.57 */
+    public void setLeaseset(String f) {
+        if (f != null && f.length() > 0)
+            _hostname = DataHelper.stripHTML(f);
+    }
+
     /** @since 0.9.36 */
     public void setLimit(String f) {
         try {
@@ -197,6 +233,18 @@ public class NetDbHelper extends FormHandler {
                 _page = 0;
         } catch (NumberFormatException nfe) {}
     }
+
+    /** @since 0.9.57 */
+    public void setSort(String f) {
+        _sort = f;
+    }
+    
+    /** @since 0.9.58 */
+    public void setIntros(String f) {
+        try {
+            _icount = Integer.parseInt(f);
+        } catch (NumberFormatException nfe) {}
+    }
     
     /**
      *  call for non-text-mode browsers
@@ -271,13 +319,16 @@ public class NetDbHelper extends FormHandler {
             if (_routerPrefix != null || _version != null || _country != null ||
                 _family != null || _caps != null || _ip != null || _sybil != null ||
                 _port != 0 || _type != null || _mtu != null || _ipv6 != null ||
-                _ssucaps != null || _transport != null || _cost != 0) {
+                _ssucaps != null || _transport != null || _cost != 0 || _etype != null ||
+                _icount > 0) {
                 renderer.renderRouterInfoHTML(_out, _limit, _page,
                                               _routerPrefix, _version, _country,
-                                              _family, _caps, _ip, _sybil, _port, _type,
-                                              _mtu, _ipv6, _ssucaps, _transport, _cost);
+                                              _family, _caps, _ip, _sybil, _port, _highPort, _type, _etype,
+                                              _mtu, _ipv6, _ssucaps, _transport, _cost, _icount);
             } else if (_lease) {
-                renderer.renderLeaseSetHTML(_out, _debug);
+                renderer.renderLeaseSetHTML(_out, _debug, null);
+            } else if (_hostname != null) {
+                renderer.renderLeaseSet(_out, _hostname, true);
             } else if (_full == 3) {
                 if (_mode == 12 && !_postOK)
                     _mode = 0;
@@ -286,7 +337,13 @@ public class NetDbHelper extends FormHandler {
                 (new SybilRenderer(_context)).getNetDbSummary(_out, _newNonce, _mode, _date);
             } else if (_full == 4) {
                 renderLookupForm();
+            } else if (_clientOnly) {
+                for (Hash client : _context.clientManager().getPrimaryHashes()) {
+                    renderer.renderLeaseSetHTML(_out, false, client);
+                }
             } else {
+                if (_full == 0 && _sort != null)
+                    _full = 3;
                 renderer.renderStatusHTML(_out, _limit, _page, _full);
             }
         } catch (IOException ioe) {
@@ -308,7 +365,7 @@ public class NetDbHelper extends FormHandler {
         if (_routerPrefix != null || _version != null || _country != null ||
             _family != null || _caps != null || _ip != null || _sybil != null ||
             _port != 0 || _type != null || _mtu != null || _ipv6 != null ||
-            _ssucaps != null || _transport != null || _cost != 0)
+            _ssucaps != null || _transport != null || _cost != 0 || _etype != null)
             return 2;
         if (_full == 2)
             return 3;
@@ -318,6 +375,10 @@ public class NetDbHelper extends FormHandler {
             return 7;
         if (_full == 4)
             return 8;
+        if (_hostname != null)
+            return 9;
+        if (_clientOnly)
+            return 10;
         return 0;
     }
 
@@ -334,8 +395,15 @@ public class NetDbHelper extends FormHandler {
         for (int i = 0; i < titles.length; i++) {
             if (i == 2 && tab != 2)
                 continue;   // can't nav to lookup
+            if (i == 9 && tab != 9)
+                continue;   // can't nav to lookup
             if (i > 2 && i != tab && !isAdvanced())
                 continue;
+            if (i == 10) {
+                // skip if no clients
+                if (_context.clientManager().getPrimaryHashes().isEmpty() && i != tab)
+                    continue;
+            }
             if (i == tab) {
                 // we are there
                 if (span)
@@ -347,9 +415,9 @@ public class NetDbHelper extends FormHandler {
                     buf.append("<span class=\"tab\">");
                 buf.append("<a href=\"netdb").append(links[i]).append("\">").append(_t(titles[i])).append("</a>");
             }
-            if (span)
+            if (span) {
                 buf.append("</span>\n");
-            else if (i != titles.length - 1)
+            } else if (i != titles.length - 1)
                 buf.append("&nbsp;&nbsp;\n");
         }
         if (!span)
@@ -366,23 +434,54 @@ public class NetDbHelper extends FormHandler {
                    "<input type=\"hidden\" name=\"nonce\" value=\"" + _newNonce + "\" >\n" +
                    "<table id=\"netdblookup\"><tr><th colspan=\"3\">Network Database Search</th></tr>\n" +
                    "<tr><td colspan=\"3\" class=\"subheading\"><b>Enter one search field <i>only</i>:</b></td></tr>\n" +
-                   "<tr><td>Capabilities:</td><td><input type=\"text\" name=\"caps\"></td><td>e.g. f or XOfR</td></tr>\n" +
+                   "<tr><td>Capabilities:</td><td><input type=\"text\" name=\"caps\"></td><td>e.g. f or XfR</td></tr>\n" +
                    "<tr><td>Cost:</td><td><input type=\"text\" name=\"cost\"></td><td></td></tr>\n" +
-                   "<tr><td>Country Code:</td><td><input type=\"text\" name=\"c\"></td><td>e.g. ru</td></tr>\n" +
-                   "<tr><td>Router Family:</td><td><input type=\"text\" name=\"fam\"></td><td></td></tr>\n" +
+                   "<tr><td>" + _t("Country") + ":</td><td><select name=\"c\"><option value=\"\" selected=\"selected\"></option>");
+        Map<String, String> sorted = new TreeMap<String, String>(Collator.getInstance());
+        for (Map.Entry<String, String> e : _context.commSystem().getCountries().entrySet()) {
+            String tr = Messages.getString(e.getValue(), _context, Messages.COUNTRY_BUNDLE_NAME);
+            sorted.put(tr, e.getKey());
+        }
+        for (Map.Entry<String, String> e : sorted.entrySet()) {
+            _out.write("<option value=\"" + e.getValue() + "\">" + e.getKey() + "</option>\n");
+        }
+        _out.write("</select></td><td></td></tr>\n" +
+                   "<tr><td>Encryption Type:</td><td><select name=\"etype\"><option value=\"\" selected=\"selected\"></option>");
+        for (EncType type : EnumSet.allOf(EncType.class)) {
+            _out.write("<option value=\"" + type + "\">" + type + "</option>\n");
+        }
+        _out.write("</select></td><td></td></tr>\n" +
+                   "<tr><td>" + _t("Router Family") + ":</td><td><input type=\"text\" name=\"fam\"></td><td></td></tr>\n" +
                    "<tr><td>Hash Prefix:</td><td><input type=\"text\" name=\"r\"></td><td></td></tr>\n" +
-                   "<tr><td>IP or Hostname:</td><td><input type=\"text\" name=\"ip\"></td><td>host name, IPv4, or IPv6, /24,/16,/8 suffixes optional for IPv4, prefix ok for IPv6</td></tr>\n" +
+                   "<tr><td>" + _t("Full destination, name, Base32, or hash") + ":</td><td><input type=\"text\" name=\"ls\"></td><td></td></tr>\n" +
+                   "<tr><td>Min. Introducer Count:</td><td><input type=\"text\" name=\"i\"></td><td></td></tr>\n" +
+                   "<tr><td>IP:</td><td><input type=\"text\" name=\"ip\"></td><td>IPv4 or IPv6, /24,/16,/8 suffixes optional for IPv4, prefix ok for IPv6</td></tr>\n" +
                    "<tr><td>IPv6 Prefix:</td><td><input type=\"text\" name=\"ipv6\"></td><td></td></tr>\n" +
-                   "<tr><td>MTU:</td><td><input type=\"text\" name=\"mtu\"></td><td></td></tr>\n" +
-                   "<tr><td>Port Number:</td><td><input type=\"text\" name=\"port\"></td><td></td></tr>\n" +
-                   "<tr><td>Signature Type:</td><td><input type=\"text\" name=\"type\"></td><td></td></tr>\n" +
-                   "<tr><td>SSU Capabilities:</td><td><input type=\"text\" name=\"ssucaps\"></td><td></td></tr>\n" +
-                   "<tr><td>Transport:</td><td><input type=\"text\" name=\"tr\"></td><td></td></tr>\n" +
+                   "<tr><td>" + _t("MTU") + ":</td><td><input type=\"text\" name=\"mtu\"></td><td></td></tr>\n" +
+                   "<tr><td>" + _t("Port") + " or Port Range:</td><td><input type=\"text\" name=\"port\"></td><td>e.g. 1024-1028</td></tr>\n" +
+                   "<tr><td>Signature Type:</td><td><select name=\"type\"><option value=\"\" selected=\"selected\"></option>");
+        for (SigType type : EnumSet.allOf(SigType.class)) {
+            _out.write("<option value=\"" + type + "\">" + type + "</option>\n");
+        }
+        _out.write("</select></td><td></td></tr>\n" +
+                   "<tr><td>Transport:</td><td><select name=\"tr\"><option value=\"\" selected=\"selected\">" +
+                   "<option value=\"NTCP\">NTCP</option>\n" +
+                   "<option value=\"NTCP_1\">NTCP (v1 only)</option>\n" +
+                   "<option value=\"NTCP_2\">NTCP (v2 supported)</option>\n" +
+                   "<option value=\"NTCP2\">NTCP2</option>\n" +
+                   "<option value=\"SSU\">SSU</option>\n" +
+                   "<option value=\"SSU_1\">SSU (v1 only)</option>\n" +
+                   "<option value=\"SSU_2\">SSU (v2 supported)</option>\n" +
+                   "<option value=\"SSU2\">SSU2</option>\n" +
+                   "</select></td><td></td></tr>\n" +
+                   "<tr><td>Transport Capabilities:</td><td><input type=\"text\" name=\"ssucaps\"></td><td></td></tr>\n" +
                    "<tr><td>Router Version:</td><td><input type=\"text\" name=\"v\"></td><td></td></tr>\n" +
                    "<tr><td colspan=\"3\" class=\"subheading\"><b>Add Sybil analysis (must pick one above):</b></td></tr>\n" +
-                   "<tr><td>Sybil close to:</td><td><input type=\"text\" name=\"sybil2\"></td><td>Router hash, dest hash, b32, or from address book</td>\n" +
+                   "<tr><td>Sybil close to:</td><td><input type=\"text\" name=\"sybil2\"></td><td>Router hash, destination hash, b32, or from address book</td>\n" +
                    "<tr><td><label for=\"closetorouter\">or Sybil close to this router:</label></td><td><input type=\"checkbox\" class=\"optbox\" value=\"1\" name=\"sybil\" id=\"closetorouter\"></td><td></td></tr>\n" +
-                   "<tr><td colspan=\"3\" class=\"optionsave\"><button type=\"submit\" class=\"search\" value=\"Lookup\">Lookup</button></td></tr>\n" +
+                   "<tr><td colspan=\"3\" class=\"optionsave\">" +
+                   "<button type=\"reset\" class=\"cancel\" value=\"Cancel\">" + _t("Cancel") + "</button> " +
+                   "<button type=\"submit\" class=\"search\" value=\"Lookup\">Lookup</button></td></tr>\n" +
                    "</table>\n</form>\n");
     }
 }
