@@ -3,12 +3,14 @@ package net.i2p.router.web.helpers;
 import java.io.IOException;
 import java.io.Serializable;
 import java.io.Writer;
+import java.math.RoundingMode;
 import java.util.Comparator;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Set;
 import java.util.SortedMap;
 import java.util.TreeMap;
 import java.util.TreeSet;
@@ -17,12 +19,15 @@ import net.i2p.data.DataHelper;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.router.transport.Transport;
 import net.i2p.router.transport.TransportManager;
+import net.i2p.router.transport.TransportUtil;
 import net.i2p.router.transport.ntcp.NTCPConnection;
 import net.i2p.router.transport.ntcp.NTCPTransport;
 import net.i2p.router.transport.udp.PeerState;
 import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.router.web.HelperBase;
 import static net.i2p.router.web.helpers.UDPSorters.*;
+import net.i2p.util.Addresses;
+import net.i2p.util.AddressType;
 import net.i2p.util.SystemVersion;
 
 
@@ -34,16 +39,20 @@ public class PeerHelper extends HelperBase {
     private boolean _graphical;
 
     private static final String titles[] = {
+                                            _x("Status"),
                                             _x("Addresses"),
                                              "NTCP",
                                              "SSU",
+                                             _x("Banned"),
                                              _x("UPnP Status")
                                            };
 
     private static final String links[] = {
                                             "",
+                                             "?tx=addresses",
                                              "?tx=ntcp",
                                              "?tx=ssu",
+                                             "?tx=banned",
                                              "?tx=upnp"
                                            };
 
@@ -101,7 +110,7 @@ public class PeerHelper extends HelperBase {
             return;
         }
         renderNavBar(out);
-        if (isAdvanced() && _transport == null) {
+        if (_transport == null) {
             out.write("<p id=\"upnpstatus\"><b>");
             out.write(_t("Status"));
             out.write(": ");
@@ -110,37 +119,7 @@ public class PeerHelper extends HelperBase {
         }
 
         SortedMap<String, Transport> transports = _context.commSystem().getTransports();
-        if (_transport != null && !_transport.equals("upnp")) {
-            boolean rendered = false;
-            for (Map.Entry<String, Transport> e : transports.entrySet()) {
-                String style = e.getKey();
-                Transport t = e.getValue();
-                if (style.equals("NTCP") && "ntcp".equals(_transport)) {
-                    NTCPTransport nt = (NTCPTransport) t;
-                    render(nt, out, urlBase, sortFlags);
-                    rendered = true;
-                    break;
-                } else if (style.equals("SSU") && "ssu".equals(_transport)) {
-                    UDPTransport ut = (UDPTransport) t;
-                    render(ut, out, urlBase, sortFlags);
-                    rendered = true;
-                    break;
-                } else if (style.equals(_transport)) {
-                    // pluggable (none yet)
-                    t.renderStatusHTML(out, urlBase, sortFlags);
-                    rendered = true;
-                    break;
-                }
-            }
-            if (rendered) {
-                out.write(getTransportsLegend());
-            } else {
-                out.write("<p class=\"infohelp\">" +
-                          "Transport not enabled: " +
-                          DataHelper.escapeHTML(_transport) +
-                          "</p>");
-            }
-        } else if (_transport == null) {
+        if ("addresses".equals(_transport)) {
             StringBuilder buf = new StringBuilder(4*1024);
             buf.append("<h3 id=\"transports\">").append(_t("Router Transport Addresses")).append("</h3><pre id=\"transports\">\n");
             if (!transports.isEmpty()) {
@@ -179,6 +158,128 @@ public class PeerHelper extends HelperBase {
                .append("</p></div>");
             out.write(buf.toString());
             _context.commSystem().renderStatusHTML(_out, _urlBase, _sortFlags);
+        } else if ("banned".equals(_transport)) {
+            BanlistRenderer br = new BanlistRenderer(_context);
+            br.renderStatusHTML(_out);
+        } else if (_transport != null) {
+            boolean rendered = false;
+            for (Map.Entry<String, Transport> e : transports.entrySet()) {
+                String style = e.getKey();
+                Transport t = e.getValue();
+                if (style.equals("NTCP") && "ntcp".equals(_transport)) {
+                    NTCPTransport nt = (NTCPTransport) t;
+                    render(nt, out, urlBase, sortFlags);
+                    rendered = true;
+                    break;
+                } else if (style.equals("SSU") && "ssu".equals(_transport)) {
+                    UDPTransport ut = (UDPTransport) t;
+                    render(ut, out, urlBase, sortFlags);
+                    rendered = true;
+                    break;
+                } else if (style.equals(_transport)) {
+                    // pluggable (none yet)
+                    t.renderStatusHTML(out, urlBase, sortFlags);
+                    rendered = true;
+                    break;
+                }
+            }
+            if (rendered) {
+                out.write(getTransportsLegend());
+            } else {
+                out.write("<p class=\"infohelp\">" +
+                          "Transport not enabled: " +
+                          DataHelper.escapeHTML(_transport) +
+                          "</p>");
+            }
+        } else {
+            // summary
+            Set<AddressType> connected = Addresses.getConnectedAddressTypes();
+            TransportUtil.IPv6Config ntcpConfig = TransportUtil.getIPv6Config(_context, "NTCP");
+            TransportUtil.IPv6Config ssuConfig = TransportUtil.getIPv6Config(_context, "SSU");
+            boolean showIPv4 = connected.contains(AddressType.IPV4) &&
+                               (ntcpConfig != TransportUtil.IPv6Config.IPV6_ONLY ||
+                                ssuConfig != TransportUtil.IPv6Config.IPV6_ONLY);
+            boolean showIPv6 = connected.contains(AddressType.IPV6) &&
+                               (ntcpConfig != TransportUtil.IPv6Config.IPV6_DISABLED ||
+                                ssuConfig != TransportUtil.IPv6Config.IPV6_DISABLED);
+            StringBuilder buf = new StringBuilder(512);
+            buf.append("<h3 id=\"transports\">").append(_t("Peer Connections")).append("</h3><table><tr><th>")
+               .append(_t("Transport")).append("</th><th>")
+               .append(_t("Total")).append("</th>");
+            if (showIPv4) {
+                buf.append("<th>")
+                   .append(_t("IPv4")).append(" <img src=\"/themes/console/images/inbound.png\" alt=\"").append(_t("Inbound")).append("\" title=\"").append(_t("Inbound")).append("\"/></th><th>")
+                   .append(_t("IPv4")).append(" <img src=\"/themes/console/images/outbound.png\" alt=\"").append(_t("Outbound")).append("\" title=\"").append(_t("Outbound")).append("\"/></th>");
+            }
+            if (showIPv6) {
+                buf.append("<th>")
+                   .append(_t("IPv6")).append(" <img src=\"/themes/console/images/inbound.png\" alt=\"").append(_t("Inbound")).append("\" title=\"").append(_t("Inbound")).append("\"/></th><th>")
+                   .append(_t("IPv6")).append(" <img src=\"/themes/console/images/outbound.png\" alt=\"").append(_t("Outbound")).append("\" title=\"").append(_t("Outbound")).append("\"/></th>");
+            }
+            buf.append("</tr>\n");
+            boolean warnInbound = !_context.router().isHidden() && _context.router().getUptime() > 15*60*1000;
+            int[] totals = new int[5];
+            int rows = 0;
+            for (Map.Entry<String, Transport> e : transports.entrySet()) {
+                String style = e.getKey();
+                Transport t = e.getValue();
+                int[] counts = t.getPeerCounts();
+                for (int idx = 0; idx < 8; idx += 4) {
+                    if (style.equals("NTCP") && idx == 0)
+                        continue;
+                    if (style.equals("SSU") && idx == 0 && !_context.getBooleanProperty(TransportManager.PROP_ENABLE_SSU1))
+                        continue;
+                    rows++;
+                    buf.append("<tr><td align=\"center\"><b>")
+                       .append(style)
+                       .append(1 + (idx / 4))
+                       .append("</b></td><td align=\"center\">");
+                    int total = 0;
+                    for (int i = 0; i < 4; i++) {
+                        total += counts[idx + i];
+                    }
+                    if (total <= 0)
+                        buf.append("<b>**</b> ");
+                    else
+                        totals[0] += total;
+                    buf.append(total);
+                    for (int i = 0; i < 4; i++) {
+                        if (!showIPv4 && i < 2)
+                            continue;
+                        if (!showIPv6 && i >= 2)
+                            break;
+                        int cnt = counts[idx + i];
+                        buf.append("</td><td align=\"center\">");
+                        if (cnt <= 0) {
+                            if ((i & 0x01) != 0 || warnInbound)
+                                buf.append("<b>**</b> ");
+                        } else {
+                            totals[i + 1] += cnt;
+                        }
+                        buf.append(cnt);
+                    }
+                    buf.append("</td></tr>\n");
+                }
+            }
+            if (rows > 1) {
+                buf.append("<tr><th align=\"center\"><b>").append(_t("Total")).append("</b>");
+                for (int i = 0; i < 5; i++) {
+                    if (!showIPv4 && i > 0 && i < 3)
+                        continue;
+                    if (!showIPv6 && i >= 3)
+                        break;
+                    int cnt = totals[i];
+                    buf.append("</th><th align=\"center\">");
+                    if (cnt <= 0) {
+                        if ((i & 0x01) == 0 || warnInbound)
+                            buf.append("<b>**</b> ");
+                    }
+                    buf.append("<b>").append(cnt).append("</b");
+                }
+                buf.append("</th></tr>");
+            }
+            buf.append("</table>\n");
+            out.write(buf.toString());
         }
         out.flush();
     }
@@ -187,12 +288,16 @@ public class PeerHelper extends HelperBase {
      *  @since 0.9.38
      */
     private int getTab() {
-        if ("ntcp".equals(_transport))
+        if ("addresses".equals(_transport))
             return 1;
-        if ("ssu".equals(_transport))
+        if ("ntcp".equals(_transport))
             return 2;
-        if ("upnp".equals(_transport))
+        if ("ssu".equals(_transport))
             return 3;
+        if ("banned".equals(_transport))
+            return 4;
+        if ("upnp".equals(_transport))
+            return 5;
         return 0;
     }
 
@@ -213,10 +318,10 @@ public class PeerHelper extends HelperBase {
                     buf.append("<span class=\"tab2\">");
                 buf.append(_t(titles[i]));
             } else {
-                if (i == 1) {
+                if (i == 2) {
                     if (!_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_NTCP))
                         continue;
-                } else if (i == 2) {
+                } else if (i == 3) {
                     if (!_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP))
                         continue;
                 }
@@ -258,20 +363,20 @@ public class PeerHelper extends HelperBase {
            .append("<tr><td><b id=\"def.up\">").append(_t("Up")).append("</b></td><td>").append(_t("How long ago this connection was established")).append("</td></tr>\n")
            .append("<tr><td><b id=\"def.skew\">").append(_t("Skew")).append("</b></td><td>").append(_t("The difference between the peer's clock and your own")).append("</td></tr>\n");
         if (isAdvanced()) {
-           buf.append("<tr><td><b id=\"def.cwnd\">CWND</b></td><td>").append(_t("The congestion window, which is how many bytes can be sent without an acknowledgement")).append(" /<br>\n")
-           .append(_t("The number of sent messages awaiting acknowledgement")).append(" /<br>\n")
+           buf.append("<tr><td><b id=\"def.cwnd\">CWND</b></td><td>").append(_t("The congestion window, which is how many bytes can be sent without an acknowledgment")).append(" /<br>\n")
+           .append(_t("The number of sent messages awaiting acknowledgment")).append(" /<br>\n")
            .append(_t("The maximum number of concurrent messages to send")).append(" /<br>\n")
            .append(_t("The number of pending sends which exceed congestion window")).append("</td></tr>\n")
            .append("<tr><td><b id=\"def.ssthresh\">SST</b></td><td>").append(_t("The slow start threshold")).append("</td></tr>\n")
            .append("<tr><td><b id=\"def.rtt\">RTT</b></td><td>").append(_t("The round trip time in milliseconds")).append("</td></tr>\n")
            //.append("<tr><td><b id=\"def.dev\">").append(_t("Dev")).append("</b></td><td>").append(_t("The standard deviation of the round trip time in milliseconds")).append("</td></tr>\n")
            .append("<tr><td><b id=\"def.rto\">RTO</b></td><td>").append(_t("The retransmit timeout in milliseconds")).append("</td></tr>\n")
-           .append("<tr><td><b id=\"def.mtu\">MTU</b></td><td>").append(_t("Current maximum send packet size / estimated maximum receive packet size (bytes)")).append("</td></tr>\n");
+           .append("<tr><td><b id=\"def.mtu\">MTU</b></td><td>").append(_t("Maximum receive / send packet size (bytes)")).append("</td></tr>\n");
         }
-        buf.append("<tr><td><b id=\"def.send\">").append(_t("TX")).append("</b></td><td>").append(_t("The total number of messages sent to the peer")).append("</td></tr>\n")
-           .append("<tr><td><b id=\"def.recv\">").append(_t("RX")).append("</b></td><td>").append(_t("The total number of messages received from the peer")).append("</td></tr>\n")
-           .append("<tr><td><b id=\"def.resent\">").append(_t("Dup TX")).append("</b></td><td>").append(_t("The total number of packets retransmitted to the peer")).append("</td></tr>\n")
+        buf.append("<tr><td><b id=\"def.recv\">").append(_t("RX")).append("</b></td><td>").append(_t("The total number of messages received from the peer")).append("</td></tr>\n")
+           .append("<tr><td><b id=\"def.send\">").append(_t("TX")).append("</b></td><td>").append(_t("The total number of messages sent to the peer")).append("</td></tr>\n")
            .append("<tr><td><b id=\"def.dupRecv\">").append(_t("Dup RX")).append("</b></td><td>").append(_t("The total number of duplicate packets received from the peer")).append("</td></tr>\n")
+           .append("<tr><td><b id=\"def.resent\">").append(_t("Dup TX")).append("</b></td><td>").append(_t("The total number of packets retransmitted to the peer")).append("</td></tr>\n")
            .append("</table>");
         return buf.toString();
     }
@@ -300,7 +405,7 @@ public class PeerHelper extends HelperBase {
             }
         }
 
-        StringBuilder buf = new StringBuilder(512);
+        StringBuilder buf = new StringBuilder(1024);
         buf.append("<h3 id=\"ntcpcon\">").append(_t("NTCP connections")).append(": ").append(peers.size());
         buf.append(". ").append(_t("Limit")).append(": ").append(nt.getMaxConnections());
         //buf.append(". ").append(_t("Timeout")).append(": ").append(DataHelper.formatDuration2(_pumper.getIdleTimeout()));
@@ -314,12 +419,12 @@ public class PeerHelper extends HelperBase {
                    .append("\">").append(_t("Dir")).append("</a></th>" +
                    "<th>").append(_t("IPv6")).append("</th>" +
                    "<th>").append(_t("Version")).append("</th>" +
-                   "<th align=\"right\"><a href=\"#def.idle\">").append(_t("Idle")).append("</a></th>" +
-                   "<th align=\"right\"><a href=\"#def.rate\">").append(_t("In/Out")).append("</a></th>" +
+                   "<th><a href=\"#def.idle\">").append(_t("Idle")).append("</a></th>" +
+                   "<th><a href=\"#def.rate\">").append(_t("In/Out")).append("</a></th>" +
                    "<th align=\"right\"><a href=\"#def.up\">").append(_t("Up")).append("</a></th>" +
                    "<th align=\"right\"><a href=\"#def.skew\">").append(_t("Skew")).append("</a></th>" +
-                   "<th align=\"right\"><a href=\"#def.send\">").append(_t("TX")).append("</a></th>" +
                    "<th align=\"right\"><a href=\"#def.recv\">").append(_t("RX")).append("</a></th>" +
+                   "<th align=\"right\"><a href=\"#def.send\">").append(_t("TX")).append("</a></th>" +
                    "<th>").append(_t("Out Queue")).append("</th>" +
                    "<th title=\"").append(_t("Is peer backlogged?")).append("\">").append(_t("Backlogged?")).append("</th>" +
                    //"<th>").append(_t("Reading?")).append("</th>" +
@@ -334,11 +439,17 @@ public class PeerHelper extends HelperBase {
             //if (ip != null)
             //    buf.append(' ').append(_context.blocklist().toStr(ip));
             buf.append("</td><td class=\"cells\" align=\"center\">");
-            if (con.isInbound())
-                buf.append("<img src=\"/themes/console/images/inbound.png\" alt=\"Inbound\" title=\"").append(_t("Inbound")).append("\"/>");
-            else
-                buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound")).append("\"/>");
-            buf.append("</td><td class=\"cells peeripv6\" align=\"center\">");
+            if (con.isInbound()) {
+                buf.append("<img src=\"/themes/console/images/inbound.png\" alt=\"Inbound\" title=\"").append(_t("Inbound"));
+                if (isAdvanced())
+                    buf.append(' ').append(Addresses.toString(con.getRemoteIP(), con.getChannel().socket().getPort()));
+            } else {
+                buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound"));
+                if (isAdvanced())
+                    buf.append(' ').append(Addresses.toString(con.getRemoteIP(), con.getRemoteAddress().getPort()));
+            }
+            buf.append("\"/>" +
+                       "</td><td class=\"cells peeripv6\" align=\"center\">");
             if (con.isIPv6())
                 buf.append("<span class=\"backlogged\">&#x2713;</span>");
             else
@@ -368,10 +479,10 @@ public class PeerHelper extends HelperBase {
             totalUptime += con.getUptime();
             offsetTotal = offsetTotal + con.getClockSkew();
             buf.append("</td><td class=\"cells\" align=\"right\">").append(DataHelper.formatDuration2(1000 * con.getClockSkew()));
-            buf.append("</td><td class=\"cells\" align=\"right\">").append(con.getMessagesSent());
-            totalSend += con.getMessagesSent();
             buf.append("</td><td class=\"cells\" align=\"right\">").append(con.getMessagesReceived());
             totalRecv += con.getMessagesReceived();
+            buf.append("</td><td class=\"cells\" align=\"right\">").append(con.getMessagesSent());
+            totalSend += con.getMessagesSent();
             long outQueue = con.getOutboundQueueSize();
             buf.append("</td><td class=\"cells\" align=\"center\">").append(outQueue);
             buf.append("</td><td class=\"cells\" align=\"center\">");
@@ -398,7 +509,7 @@ public class PeerHelper extends HelperBase {
             buf.append(THINSP).append("<span class=\"left\"><b>").append(formatRate(bpsSend/1000)).append("</b></span>");
             buf.append("</td><td align=\"right\"><b>").append(DataHelper.formatDuration2(totalUptime/peers.size()));
             buf.append("</b></td><td align=\"right\"><b>").append(DataHelper.formatDuration2(offsetTotal*1000/peers.size()));
-            buf.append("</b></td><td align=\"right\"><b>").append(totalSend).append("</b></td><td align=\"right\"><b>").append(totalRecv);
+            buf.append("</b></td><td align=\"right\"><b>").append(totalRecv).append("</b></td><td align=\"right\"><b>").append(totalSend);
             buf.append("</b></td><td>&nbsp;</td><td>&nbsp;</td></tr>\n");
         }
 
@@ -408,8 +519,13 @@ public class PeerHelper extends HelperBase {
     }
 
     private static final NumberFormat _rateFmt = new DecimalFormat("#,##0.00");
+    static {
+        _rateFmt.setRoundingMode(RoundingMode.HALF_UP);
+    }
 
     private static String formatRate(float rate) {
+        if (rate < 0.005f)
+            return "0";
         synchronized (_rateFmt) { return _rateFmt.format(rate); }
     }
 
@@ -463,7 +579,7 @@ public class PeerHelper extends HelperBase {
         int numPeers = 0;
         int numRTTPeers = 0;
 
-        StringBuilder buf = new StringBuilder(512);
+        StringBuilder buf = new StringBuilder(4096);
         buf.append("<h3 id=\"udpcon\">").append(_t("UDP connections")).append(": ").append(peers.size());
         buf.append(". ").append(_t("Limit")).append(": ").append(ut.getMaxConnections());
         //buf.append(". ").append(_t("Timeout")).append(": ").append(DataHelper.formatDuration2(_expireTimeout));
@@ -479,6 +595,7 @@ public class PeerHelper extends HelperBase {
         buf.append("</th><th nowrap><a href=\"#def.dir\" title=\"")
            .append(_t("Direction/Introduction")).append("\">").append(_t("Dir"))
            .append("</a></th><th nowrap>").append(_t("IPv6"))
+           .append("</th><th nowrap>").append(_t("Version"))
            .append("</th><th nowrap><a href=\"#def.idle\">").append(_t("Idle")).append("</a><br>");
         appendSortLinks(buf, urlBase, sortFlags, _t("Sort by idle inbound"), FLAG_IDLE_IN);
         buf.append(" / ");
@@ -511,15 +628,15 @@ public class PeerHelper extends HelperBase {
             appendSortLinks(buf, urlBase, sortFlags, _t("Sort by outbound maximum transmit unit"), FLAG_MTU);
             buf.append("</th>");
         }
-        buf.append("<th nowrap><span class=\"peersort\"><a href=\"#def.send\">").append(_t("TX")).append("</a><br>");
-        appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets sent"), FLAG_SEND);
         buf.append("</span></th><th nowrap><span class=\"peersort\"><a href=\"#def.recv\">").append(_t("RX")).append("</a><br>");
         appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets received"), FLAG_RECV);
+        buf.append("<th nowrap><span class=\"peersort\"><a href=\"#def.send\">").append(_t("TX")).append("</a><br>");
+        appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets sent"), FLAG_SEND);
         buf.append("</span></th>\n");
-        buf.append("<th nowrap><span class=\"peersort\"><a href=\"#def.resent\">").append(_t("Dup TX")).append("</a><br>");
-        appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets retransmitted"), FLAG_RESEND);
         buf.append("</span></th><th nowrap><span class=\"peersort\"><a href=\"#def.dupRecv\">").append(_t("Dup RX")).append("</a><br>");
         appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets received more than once"), FLAG_DUP);
+        buf.append("<th nowrap><span class=\"peersort\"><a href=\"#def.resent\">").append(_t("Dup TX")).append("</a><br>");
+        appendSortLinks(buf, urlBase, sortFlags, _t("Sort by packets retransmitted"), FLAG_RESEND);
         buf.append("</span></th></tr>\n");
         out.write(buf.toString());
         buf.setLength(0);
@@ -534,11 +651,16 @@ public class PeerHelper extends HelperBase {
             //if (ip != null)
             //    buf.append(' ').append(_context.blocklist().toStr(ip));
             buf.append("</td><td class=\"cells\" nowrap align=\"left\">");
-            if (peer.isInbound())
+            if (peer.isInbound()) {
                 buf.append("<img src=\"/themes/console/images/inbound.png\" alt=\"Inbound\" title=\"").append(_t("Inbound"));
-            else
+                if (isAdvanced())
+                    buf.append(' ').append(Addresses.toString(peer.getRemoteIP(), peer.getRemotePort()));
+            } else {
                 buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound"));
-            buf.append("\">");
+                if (isAdvanced())
+                    buf.append(' ').append(Addresses.toString(peer.getRemoteIP(), peer.getRemotePort()));
+            }
+            buf.append("\"/>");
             if (peer.getWeRelayToThemAs() > 0)
                 buf.append("&nbsp;&nbsp;<img src=\"/themes/console/images/outbound.png\" height=\"8\" width=\"12\" alt=\"^\" title=\"").append(_t("We offered to introduce them")).append("\">");
             if (peer.getTheyRelayToUsAs() > 0)
@@ -575,6 +697,10 @@ public class PeerHelper extends HelperBase {
                 buf.append("");
             buf.append("</td>");
 
+            buf.append("<td class=\"cells peeripv6\" align=\"center\">");
+            buf.append(peer.getVersion());
+            buf.append("</td>");
+
             long idleIn = Math.max(now-peer.getLastReceiveTime(), 0);
             long idleOut = Math.max(now-peer.getLastSendTime(), 0);
 
@@ -584,8 +710,8 @@ public class PeerHelper extends HelperBase {
             buf.append("<span class=\"left\">").append(DataHelper.formatDuration2(idleOut));
             buf.append("</span></td>");
  
-            int recvBps = (idleIn > 15*1000 ? 0 : peer.getReceiveBps());
-            int sendBps = (idleOut > 15*1000 ? 0 : peer.getSendBps());
+            int recvBps = peer.getReceiveBps(now);
+            int sendBps = peer.getSendBps(now);
 
             buf.append("<td class=\"cells\" align=\"center\" nowrap><span class=\"right\">");
             buf.append(formatKBps(recvBps));
@@ -610,24 +736,25 @@ public class PeerHelper extends HelperBase {
             buf.append("</td>");
             offsetTotal = offsetTotal + skew;
 
-            long sendWindow = peer.getSendWindowBytes();
-
             int rtt = 0;
             int rto = 0;
             if (isAdvanced) {
+                int sendWindow = peer.getSendWindowBytes();
+                cwinTotal += sendWindow;
                 buf.append("<td class=\"cells cwnd\" align=\"center\"><span class=\"right\">");
-                buf.append(sendWindow/1024);
-                buf.append("K");
+                buf.append(DataHelper.formatSize2(sendWindow));
+                buf.append("B");
                 buf.append("</span>").append(THINSP).append("<span class=\"right\">").append(peer.getConcurrentSends());
                 buf.append("</span>").append(THINSP).append("<span class=\"right\">").append(peer.getConcurrentSendWindow());
                 buf.append("</span>").append(THINSP).append("<span class=\"left\">").append(peer.getConsecutiveSendRejections());
+                buf.append("</span>");
                 if (peer.isBacklogged())
-                    buf.append(' ').append(_t("backlogged"));
-                buf.append("</span></td>");
+                    buf.append("<br><i>").append(_t("backlogged")).append("</i>");
+                buf.append("</td>");
 
                 buf.append("<td class=\"cells\" align=\"right\">");
-                buf.append(peer.getSlowStartThreshold()/1024);
-                buf.append("K</td>");
+                buf.append(DataHelper.formatSize2(peer.getSlowStartThreshold()));
+                buf.append("B</td>");
 
                 rtt = peer.getRTT();
                 rto = peer.getRTO();
@@ -648,12 +775,9 @@ public class PeerHelper extends HelperBase {
                 buf.append("</td>");
 
                 buf.append("<td class=\"cells\" align=\"center\"><span class=\"right\">");
-                buf.append(peer.getMTU()).append("</span>").append(THINSP);
-                buf.append("<span class=\"left\">").append(peer.getReceiveMTU());
+                buf.append(peer.getReceiveMTU()).append("</span>").append(THINSP);
+                buf.append("<span class=\"left\">").append(peer.getMTU());
 
-                //.append('/');
-                //buf.append(peer.getMTUIncreases()).append('/');
-                //buf.append(peer.getMTUDecreases());
                 buf.append("</span></td>");
             }
 
@@ -661,11 +785,11 @@ public class PeerHelper extends HelperBase {
             long recv = peer.getMessagesReceived();
 
             buf.append("<td class=\"cells\" align=\"right\">");
-            buf.append(sent);
+            buf.append(recv);
             buf.append("</td>");
 
             buf.append("<td class=\"cells\" align=\"right\">");
-            buf.append(recv);
+            buf.append(sent);
             buf.append("</td>");
 
             //double sent = (double)peer.getPacketsPeriodTransmitted();
@@ -677,16 +801,14 @@ public class PeerHelper extends HelperBase {
             long dupRecv = peer.getPacketsReceivedDuplicate();
 
             buf.append("<td class=\"cells\" align=\"right\">");
-            //buf.append(formatPct(sendLostPct));
-            buf.append(resent); // + "/" + peer.getPacketsPeriodRetransmitted() + "/" + sent);
-            //buf.append(peer.getPacketRetransmissionRate());
-            buf.append("</td>");
+            buf.append(dupRecv);
+            buf.append("</td>" +
 
-            buf.append("<td class=\"cells\" align=\"right\">");
-            buf.append(dupRecv); //formatPct(recvDupPct));
-            buf.append("</td>");
+                       "<td class=\"cells\" align=\"right\">");
+            buf.append(resent);
+            buf.append("</td>" +
 
-            buf.append("</tr>\n");
+                       "</tr>\n");
             out.write(buf.toString());
             buf.setLength(0);
 
@@ -694,7 +816,6 @@ public class PeerHelper extends HelperBase {
             bpsOut += sendBps;
 
             uptimeMsTotal += uptime;
-            cwinTotal += sendWindow;
             if (rtt > 0) {
                 rttTotal += rtt;
                 numRTTPeers++;
@@ -711,7 +832,7 @@ public class PeerHelper extends HelperBase {
 
       if (numPeers > 0) {
 //        buf.append("<tr><td colspan=\"16\"><hr></td></tr>\n");
-        buf.append("<tr class=\"tablefooter\"><td colspan=\"4\" align=\"left\"><b>")
+        buf.append("<tr class=\"tablefooter\"><td colspan=\"5\" align=\"left\"><b>")
            .append(ngettext("{0} peer", "{0} peers", peers.size()))
            .append("</b></td>" +
                    "<td align=\"center\" nowrap><span class=\"right\"><b>");
@@ -734,12 +855,12 @@ public class PeerHelper extends HelperBase {
             //buf.append("</b></td><td>&nbsp;</td><td align=\"center\"><b>");
             buf.append("</b></td><td align=\"right\"><b>");
             buf.append(DataHelper.formatDuration2(rtoTotal/numPeers));
-            buf.append("</b></td><td align=\"center\"><b>").append(ut.getMTU(false)).append("</b></td>");
+            buf.append("</b></td><td></td>"); // MTU
         }
         buf.append("<td align=\"right\"><b>");
-        buf.append(sendTotal).append("</b></td><td align=\"right\"><b>").append(recvTotal).append("</b></td>\n" +
-                   "<td align=\"right\"><b>").append(resentTotal);
-        buf.append("</b></td><td align=\"right\"><b>").append(dupRecvTotal).append("</b></td></tr>\n");
+        buf.append(recvTotal).append("</b></td><td align=\"right\"><b>").append(sendTotal).append("</b></td>\n" +
+                   "<td align=\"right\"><b>").append(dupRecvTotal);
+        buf.append("</b></td><td align=\"right\"><b>").append(resentTotal).append("</b></td></tr>\n");
 /****
         if (sortFlags == FLAG_DEBUG) {
             buf.append("<tr><td colspan=\"16\">");
@@ -777,8 +898,13 @@ public class PeerHelper extends HelperBase {
     }
 
     private static final DecimalFormat _fmt = new DecimalFormat("#,##0.00");
+    static {
+        _fmt.setRoundingMode(RoundingMode.HALF_UP);
+    }
 
     private static final String formatKBps(int bps) {
+        if (bps < 5)
+            return "0";
         synchronized (_fmt) {
             return _fmt.format((float)bps/1000);
         }
