@@ -44,7 +44,7 @@ class ProfileOrganizerRenderer {
         Set<Hash> peers = _organizer.selectAllPeers();
 
         long now = _context.clock().now();
-        long hideBefore = now - 90*60*1000;
+        long hideBefore = now - 45*60*1000;
 
         Set<PeerProfile> order = new TreeSet<PeerProfile>(mode == 2 ? new HashComparator() : new ProfileComparator());
         int older = 0;
@@ -55,7 +55,7 @@ class ProfileOrganizerRenderer {
             if (prof == null)
                 continue;
             if (mode == 2) {
-                RouterInfo info = _context.netDb().lookupRouterInfoLocally(peer);
+                RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(peer);
                 if (info != null && info.getCapabilities().indexOf('f') >= 0)
                     order.add(prof);
                 continue;
@@ -114,7 +114,6 @@ class ProfileOrganizerRenderer {
             } else if (_organizer.isHighCapacity(peer)) {
                 tier = 2;
                 reliable++;
-            } else if (_organizer.isFailing(peer)) {
             } else {
                 tier = 3;
             }
@@ -138,16 +137,21 @@ class ProfileOrganizerRenderer {
             switch (tier) {
                 case 1: buf.append(_t("Fast, High Capacity")); break;
                 case 2: buf.append(_t("High Capacity")); break;
-                case 3: buf.append(_t("Standard")); break;
+                case 3: /* buf.append(_t("Standard")); */ break;
                 default: buf.append(_t("Failing")); break;
             }
-            if (isIntegrated) buf.append(", ").append(_t("Integrated"));
-            RouterInfo info = _context.netDb().lookupRouterInfoLocally(peer);
+            if (isIntegrated) {
+                if (tier != 3)
+                    buf.append(", ");
+                buf.append(_t("Integrated"));
+            }
+            RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(peer);
             if (info != null) {
                 // prevent HTML injection in the caps and version
                 buf.append("<td align=\"right\">").append(DataHelper.stripHTML(info.getCapabilities()));
             } else {
-                buf.append("<td align=\"right\"><i>").append(_t("unknown")).append("</i></td>");
+                //buf.append("<td align=\"right\"><i>").append(_t("unknown")).append("</i></td>");
+                buf.append("<td></td>");
             }
             buf.append("<td align=\"right\">");
             String v = info != null ? info.getOption("router.version") : null;
@@ -178,21 +182,17 @@ class ProfileOrganizerRenderer {
                 buf.append(_t("Banned"));
                 ok = false;
             }
-            if (prof.getIsFailing()) {
-                buf.append(' ').append(_t("Failing"));
-                ok = false;
-            }
             if (_context.commSystem().wasUnreachable(peer)) {
                 buf.append(' ').append(_t("Unreachable"));
                 ok = false;
             }
             RateAverages ra = RateAverages.getTemp();
-            Rate failed = prof.getTunnelHistory().getFailedRate().getRate(30*60*1000);
+            Rate failed = prof.getTunnelHistory().getFailedRate().getRate(60*60*1000);
             long fails = failed.computeAverages(ra, false).getTotalEventCount();
             if (ok && fails == 0) {
                 buf.append(_t("OK"));
             } else if (fails > 0) {
-                Rate accepted = prof.getTunnelCreateResponseTime().getRate(30*60*1000);
+                Rate accepted = prof.getTunnelCreateResponseTime().getRate(60*60*1000);
                 long total = fails + accepted.computeAverages(ra, false).getTotalEventCount();
                 if (total / fails <= 10)   // hide if < 10%
                     buf.append(' ').append(fails).append('/').append(total).append(' ').append(_t("Test Fails"));
@@ -244,7 +244,7 @@ class ProfileOrganizerRenderer {
             buf.append("<tr><td align=\"center\" nowrap>");
             buf.append(_context.commSystem().renderPeerHTML(peer));
             buf.append("</td>");
-            RouterInfo info = _context.netDb().lookupRouterInfoLocally(peer);
+            RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(peer);
             if (info != null)
                 buf.append("<td align=\"center\">").append(DataHelper.stripHTML(info.getCapabilities())).append("</td>");
             else
@@ -267,7 +267,7 @@ class ProfileOrganizerRenderer {
                 buf.append("<td align=\"right\">").append(davg(dbh, 24*60*60*1000l, ra)).append("</td>");
             } else {
                 for (int i = 0; i < 6; i++)
-                    buf.append("<td align=\"right\">").append(_t(NA));
+                    buf.append("<td align=\"right\">").append(NA);
             }
             buf.append("</tr>\n");
         }
@@ -282,11 +282,11 @@ class ProfileOrganizerRenderer {
         buf.append("<h3 class=\"tabletitle\">").append(_t("Thresholds")).append("</h3>\n")
            .append("<table id=\"thresholds\"><tbody>")
            .append("<tr><th><b>")
-           .append(_t("Speed")).append(": </b>").append(num(_organizer.getSpeedThreshold()))
+           .append(_t("Speed")).append(": </b>").append(num(Math.max(1, _organizer.getSpeedThreshold())))
            .append("</th><th><b>")
-           .append(_t("Capacity")).append(": </b>").append(num(_organizer.getCapacityThreshold()))
+           .append(_t("Capacity")).append(": </b>").append(num(Math.max(1, _organizer.getCapacityThreshold())))
            .append("</th><th><b>")
-           .append(_t("Integration")).append(": </b>").append(num(_organizer.getIntegrationThreshold()))
+           .append(_t("Integration")).append(": </b>").append(num(Math.max(1, _organizer.getIntegrationThreshold())))
            .append("</th></tr><tr><td>")
            .append(ngettext("{0} fast peer", "{0} fast peers", fast))
            .append("</td><td>")
@@ -302,34 +302,34 @@ class ProfileOrganizerRenderer {
         buf.append("<tr><td><b>")
            .append("Caps").append(":</b></td><td>").append(_t("capabilities in the netDb, not used to determine profiles"))
            .append("</td></tr>");
-        buf.append("<tr id=\"capabilities_key\"><td colspan=\"2\"><table><tbody>");
-        buf.append("<tr><td>&nbsp;</td>")
-           .append("<td><b>B</b></td><td>").append(_t("SSU Testing")).append("</td>")
-           .append("<td><b>C</b></td><td>").append(_t("SSU Introducer")).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
+        buf.append("<tr><td colspan=\"2\"><table id=\"capabilities_key\"><tbody>");
         buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>D</b></td><td>").append(_t("Congested")).append("</td>")
+           .append("<td><b>E</b></td><td>").append(_t("Severely congested")).append("</td>" +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>f</b></td><td>").append(_t("Floodfill")).append("</td>")
-           .append("<td><b>H</b></td><td>").append(_t("Hidden")).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
-        buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>G</b></td><td>").append(_t("Not accepting tunnels")).append("</td>" +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>K</b></td><td>").append(_t("Under {0} shared bandwidth", Router.MIN_BW_L + " KBps")).append("</td>")
-           .append("<td><b>L</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_L, Router.MIN_BW_M))).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
-        buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>L</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_L, Router.MIN_BW_M))).append("</td>" +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>M</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_M, Router.MIN_BW_N))).append("</td>")
-           .append("<td><b>N</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_N, Router.MIN_BW_O))).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
-        buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>N</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_N, Router.MIN_BW_O))).append("</td>" +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>O</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_O, Router.MIN_BW_P))).append("</td>")
-           .append("<td><b>P</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_P, Router.MIN_BW_X))).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
-        buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>P</b></td><td>").append(_t("{0} shared bandwidth", range(Router.MIN_BW_P, Router.MIN_BW_X))).append("</td>"  +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>R</b></td><td>").append(_t("Reachable")).append("</td>")
-           .append("<td><b>U</b></td><td>").append(_t("Unreachable")).append("</td>")
-           .append("<td>&nbsp;</td></tr>");
-        buf.append("<tr><td>&nbsp;</td>")
+           .append("<td><b>U</b></td><td>").append(_t("Unreachable")).append("</td>" +
+                   "<td>&nbsp;</td></tr>" +
+                   "<tr><td>&nbsp;</td>")
            .append("<td><b>X</b></td><td>").append(_t("Over {0} shared bandwidth", Math.round(Router.MIN_BW_X * 1.024f) + " KBps")).append("</td>")
-           .append("<td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>");
+           .append("<td>&nbsp;</td><td>&nbsp;</td></tr>");
         buf.append("</tbody></table></td></tr>"); // profile_defs
         buf.append("<tr><td><b>")
            .append(_t("speed"))
@@ -378,20 +378,11 @@ class ProfileOrganizerRenderer {
                 } else {
                     return -1;
                 }
-            } else if (_context.profileOrganizer().isFailing(left.getPeer())) {
-                if (_context.profileOrganizer().isFailing(right.getPeer())) {
-                    return super.compare(left, right);
-                } else {
-                    return 1;
-                }
             } else {
-                // left is not failing
                 if (_context.profileOrganizer().isFast(right.getPeer())) {
                     return 1;
                 } else if (_context.profileOrganizer().isHighCapacity(right.getPeer())) {
                     return 1;
-                } else if (_context.profileOrganizer().isFailing(right.getPeer())) {
-                    return -1;
                 } else {
                     return super.compare(left, right);
                 }
@@ -412,40 +403,50 @@ class ProfileOrganizerRenderer {
     }
 
     private final static DecimalFormat _fmt = new DecimalFormat("###,##0.00");
-    private final static String num(double num) { synchronized (_fmt) { return _fmt.format(num); } }
-    private final static String NA = HelperBase._x("n/a");
+
+    private final static String num(double num) {
+        if (num >= 1.0d) {
+            synchronized (_fmt) { return _fmt.format(num); }
+        } else {
+            return NA;
+        }
+    }
+
+    private final static String NA = "--";
 
     private String avg (PeerProfile prof, long rate, RateAverages ra) {
             RateStat rs = prof.getDbResponseTime();
             if (rs == null)
-                return _t(NA);
+                return NA;
             Rate r = rs.getRate(rate);
             if (r == null)
-                return _t(NA);
+                return NA;
             r.computeAverages(ra, false);
             if (ra.getTotalEventCount() == 0)
-                return _t(NA);
+                return NA;
             return DataHelper.formatDuration2(Math.round(ra.getAverage()));
     }
 
     private String davg (DBHistory dbh, long rate, RateAverages ra) {
             RateStat rs = dbh.getFailedLookupRate();
             if (rs == null)
-                return "0%";
+                return NA;
             Rate r = rs.getRate(rate);
             if (r == null)
-                return "0%";
+                return NA;
             r.computeAverages(ra, false);
             if (ra.getTotalEventCount() <= 0)
-                return "0%";
+                return NA;
             double avg = 0.5 + 100 * ra.getAverage();
+            if (avg < 1.0d)
+                return NA;
             return ((int) avg) + "%";
     }
 
     /** @since 0.9.21 */
     private String formatInterval(long now, long then) {
         if (then <= 0)
-            return _t(NA);
+            return NA;
         // avoid 0 or negative
         if (now <= then)
             return DataHelper.formatDuration2(1);
