@@ -33,7 +33,7 @@ import net.i2p.util.SystemVersion;
 class SummaryBarRenderer {
 
     static final String ALL_SECTIONS[] =
-        {"HelpAndFAQ", "I2PServices", "I2PInternals", "RouterInfo", "ShortRouterInfo", "AdvancedRouterInfo", "MemoryBar", "NetworkReachability",
+        {"HelpAndFAQ", "I2PServices", "I2PInternals", "I2PDiagnostics", "RouterInfo", "ShortRouterInfo", "AdvancedRouterInfo", "MemoryBar", "NetworkReachability",
         "UpdateStatus", "RestartStatus", "Peers", "PeersAdvanced", "FirewallAndReseedStatus", "Bandwidth", "BandwidthGraph", "Tunnels",
         "Congestion", "TunnelStatus", "Destinations", "NewsHeadings", "Advanced" };
     static final Map<String, String> SECTION_NAMES;
@@ -42,7 +42,8 @@ class SummaryBarRenderer {
         Map<String, String> aMap = new HashMap<String, String>();
         aMap.put("HelpAndFAQ", _x("Help &amp; FAQ"));
         aMap.put("I2PServices", _x("I2P Services"));
-        aMap.put("I2PInternals", _x("I2P Internals"));
+        aMap.put("I2PInternals", _x("Configuration"));
+        aMap.put("I2PDiagnostics", _x("Diagnostics"));
         aMap.put("RouterInfo", _x("Router Information"));
         aMap.put("ShortRouterInfo", _x("Router Information (brief)"));
         aMap.put("AdvancedRouterInfo", _x("Router Information (advanced)"));
@@ -96,6 +97,8 @@ class SummaryBarRenderer {
                 buf.append(renderI2PServicesHTML());
             else if ("I2PInternals".equals(section))
                 buf.append(renderI2PInternalsHTML());
+            else if ("I2PDiagnostics".equals(section))
+                buf.append(renderI2PDiagnosticsHTML());
             else if ("Advanced".equals(section))
                 buf.append(renderAdvancedHTML());
             else if ("RouterInfo".equals(section) || "General".equals(section)) // Backwards-compatibility
@@ -169,7 +172,7 @@ class SummaryBarRenderer {
            .append("</a>\n");
         svcs.put(tx, rbuf.toString());
 
-        tx = _t("Legal");
+        tx = _t("Licenses");
         rbuf.setLength(0);
         rbuf.append("<a href=\"/viewlicense\" target=\"_top\" title=\"")
            .append(_t("Information regarding software and licenses used by I2P"))
@@ -178,19 +181,10 @@ class SummaryBarRenderer {
            .append("</a>\n");
         svcs.put(tx, rbuf.toString());
 
-        tx = _t("Reachability");
+        tx = _t("Network");
         rbuf.setLength(0);
-        rbuf.append("<a href=\"/help#reachability\" target=\"_top\" title=\"")
-           .append(_t("A short guide to the sidebar's network reachability notification"))
-           .append("\">")
-           .append(nbsp(tx))
-           .append("</a>\n");
-        svcs.put(tx, rbuf.toString());
-
-        tx = _t("Setup");
-        rbuf.setLength(0);
-        rbuf.append("<a href=\"/welcome\" target=\"_top\" title=\"")
-           .append(_t("New Install Wizard"))
+        rbuf.append("<a href=\"/help#reachabilityhelp\" target=\"_top\" title=\"")
+           .append(_t("Information about the network status"))
            .append("\">")
            .append(nbsp(tx))
            .append("</a>\n");
@@ -261,22 +255,30 @@ class SummaryBarRenderer {
             svcs.put(tx, rbuf.toString());
         }
 
-        String url = getEepsiteURL(pm);
-        if (url != null) {
+        List<String> urls = pm.getEepsiteURLs();
+        if (urls != null) {
             String tx = _t("Web Server");
-            rbuf.setLength(0);
-            rbuf.append("<tr><td><img src=\"/themes/console/images/server.png\" height=\"16\" width=\"16\" alt=\"\"></td><td align=\"left\">" +
-                        "<a href=\"")
-                .append(url)
-                .append("\" target=\"_blank\" title=\"")
-                .append(_t("Local web server"))
-               . append("\">")
-                .append(nbsp(tx))
-                .append("</a></td></tr>\n");
-            svcs.put(tx, rbuf.toString());
+            String txtt = _t("Local web Server");
+            int sz = urls.size();
+            if (sz > 1)
+                Collections.sort(urls);
+            for (int i = 0; i < sz; i++) { 
+                String url = urls.get(i);
+                String txp = sz > 1 ? tx + ' ' + (i + 1) : tx;
+                rbuf.setLength(0);
+                rbuf.append("<tr><td><img src=\"/themes/console/images/server.png\" height=\"16\" width=\"16\" alt=\"\"></td><td align=\"left\">" +
+                            "<a href=\"")
+                    .append(url)
+                    .append("\" target=\"_blank\" title=\"")
+                    .append(txtt)
+                    .append("\">")
+                    .append(nbsp(txp))
+                    .append("</a></td></tr>\n");
+                 svcs.put(txp, rbuf.toString());
+            }
         }
 
-        Map<String, String> apps = NavHelper.getClientAppLinks();
+        Map<String, String> apps = NavHelper.getInstance(_context).getClientAppLinks();
         if (apps != null)
             svcs.putAll(apps);
         if (!svcs.isEmpty()) {
@@ -333,7 +335,7 @@ class SummaryBarRenderer {
         buf.append("<h3><a href=\"/config\" target=\"_top\" title=\"")
            .append(_t("Configure I2P Router"))
            .append("\">")
-           .append(_t("I2P Internals"))
+           .append(_t("Configuration"))
            .append("</a></h3><hr class=\"b\">\n" +
 
                    "<table id=\"sb_internals\"><tr><td>\n");
@@ -343,7 +345,7 @@ class SummaryBarRenderer {
         StringBuilder rbuf = new StringBuilder(128);
         PortMapper pm = _context.portMapper();
         if (pm.isRegistered(PortMapper.SVC_SUSIDNS)) {
-            String tx = _t("Addressbook");
+            String tx = _t("Address Book");
             rbuf.append("<a href=\"/dns\" target=\"_top\" title=\"")
                 .append(_t("Manage your I2P hosts file here (I2P domain name resolution)"))
                 .append("\">")
@@ -352,11 +354,11 @@ class SummaryBarRenderer {
             svcs.put(tx, rbuf.toString());
         }
 
-        if (!StatSummarizer.isDisabled(_context)) {
-            String tx = _t("Graphs");
+        if (_helper.isI2PTunnelRunning()) {
+            String tx = _t("Hidden Services Manager");
             rbuf.setLength(0);
-            rbuf.append("<a href=\"/graphs\" target=\"_top\" title=\"")
-                .append(_t("Graph router performance"))
+            rbuf.append("<a href=\"/i2ptunnelmgr\" target=\"_top\" title=\"")
+                .append(_t("Local Tunnels"))
                 .append("\">")
                 .append(nbsp(tx))
                 .append("</a>\n");
@@ -372,18 +374,62 @@ class SummaryBarRenderer {
            .append("</a>\n");
         svcs.put(tx, rbuf.toString());
 
-        if (pm.isRegistered(PortMapper.SVC_I2PTUNNEL)) {
-            tx = _t("Hidden Services Manager");
+        tx = _t("Settings");
+        rbuf.setLength(0);
+        rbuf.append("<a href=\"/config\" target=\"_top\" title=\"")
+           .append(_t("Configure I2P Router"))
+           .append("\">")
+           .append(nbsp(tx))
+           .append("</a>\n");
+        svcs.put(tx, rbuf.toString());
+
+        tx = _t("Setup");
+        rbuf.setLength(0);
+        rbuf.append("<a href=\"/welcome\" target=\"_top\" title=\"")
+           .append(_t("New Install Wizard"))
+           .append("\">")
+           .append(nbsp(tx))
+           .append("</a>\n");
+        svcs.put(tx, rbuf.toString());
+
+        for (String row : svcs.values()) {
+             buf.append(row);
+        }
+        buf.append("</td></tr></table>\n");
+        return buf.toString();
+    }
+
+
+    /**
+     *  Renders html for section containing logging, peer information, non-configuration "Admin" elements.
+     *  @since 0.9.50 separates config from non config elements in "I2P Internals"
+     */
+    public String renderI2PDiagnosticsHTML() {
+        StringBuilder buf = new StringBuilder(512);
+        buf.append("<h3><a href=\"/config\" target=\"_top\" title=\"")
+           .append(_t("Configure I2P Router"))
+           .append("\">")
+           .append(_t("Diagnostics"))
+           .append("</a></h3><hr class=\"b\">\n" +
+
+                   "<table id=\"sb_internals\"><tr><td>\n");
+
+        // Store all items in map so they are sorted by translated name, then output
+        Map<String, String> svcs = new TreeMap<String, String>(Collator.getInstance());
+        StringBuilder rbuf = new StringBuilder(128);
+
+        if (!StatSummarizer.isDisabled(_context)) {
+            String tx = _t("Graphs");
             rbuf.setLength(0);
-            rbuf.append("<a href=\"/i2ptunnelmgr\" target=\"_top\" title=\"")
-                .append(_t("Local Tunnels"))
+            rbuf.append("<a href=\"/graphs\" target=\"_top\" title=\"")
+                .append(_t("Graph router performance"))
                 .append("\">")
                 .append(nbsp(tx))
                 .append("</a>\n");
             svcs.put(tx, rbuf.toString());
         }
 
-        tx = _t("Logs");
+        String tx = _t("Logs");
         rbuf.setLength(0);
         rbuf.append("<a href=\"/logs\" target=\"_top\" title=\"")
             .append(_t("Health Report"))
@@ -395,7 +441,7 @@ class SummaryBarRenderer {
         tx = _t("NetDB");
         rbuf.setLength(0);
         rbuf.append("<a href=\"/netdb\" target=\"_top\" title=\"")
-            .append(_t("Show list of all known I2P routers"))
+            .append(_t("Network Database Router Statistics"))
             .append("\">")
             .append(nbsp(tx))
             .append("</a>\n");
@@ -531,15 +577,6 @@ class SummaryBarRenderer {
            .append("</a>\n");
         svcs.put(tx, rbuf.toString());
 
-        tx = _t("Proof");
-        rbuf.setLength(0);
-        rbuf.append("<a title=\"")
-           .append(_t("Signed proof of ownership of this router"))
-           .append("\" href=\"/proof\" target=\"_top\">")
-           .append(nbsp(tx))
-           .append("</a>\n");
-        svcs.put(tx, rbuf.toString());
-
         tx = _t("Stats");
         rbuf.setLength(0);
         rbuf.append("<a href=\"/stats\" target=\"_top\" title=\"")
@@ -715,8 +752,8 @@ class SummaryBarRenderer {
             default:
                 buf.append("testing");
         }
-        buf.append("\"><a href=\"/help#confignet\" target=\"_top\" title=\"")
-           .append(_t("Help with configuring your firewall and router for optimal I2P performance"))
+        buf.append("\"><a href=\"/help#reachabilityhelp\" target=\"_top\" title=\"")
+           .append(_t("Information about the network status"))
            .append("\">")
            .append(_t("Network"))
            .append(": ")
@@ -806,7 +843,7 @@ class SummaryBarRenderer {
            .append(_t("The number of peers available for network database inquiries"))
            .append("\">" +
                    "<td align=\"left\"><b>")
-           .append(_t("Integrated"))
+           .append(_t("Floodfill"))
            .append(":</b></td><td align=\"right\">")
            .append(_helper.getWellIntegratedPeers())
            .append("</td></tr>\n" +
@@ -870,7 +907,7 @@ class SummaryBarRenderer {
            .append(_t("The number of peers available for network database inquiries"))
            .append("\">" +
                    "<td align=\"left\"><b>")
-           .append(_t("Integrated"))
+           .append(_t("Floodfill"))
            .append(":</b></td><td align=\"right\">")
            .append(_helper.getWellIntegratedPeers())
            .append("</td></tr>\n" +
@@ -882,17 +919,6 @@ class SummaryBarRenderer {
            .append(_t("Known"))
            .append(":</b></td><td align=\"right\">")
            .append(_helper.getAllPeers())
-           .append("</td></tr>\n" +
-
-                   "<tr class=\"separator\"><td colspan=\"2\"><hr></td></tr>" +
-
-                   "<tr title=\"")
-           .append(_t("The number of peers failing network tests"))
-           .append("\">" +
-                   "<td align=\"left\"><a href=\"/profiles\"><b>")
-           .append(_t("Failing"))
-           .append(":</b></a></td><td align=\"right\">")
-           .append(_helper.getFailingPeers())
            .append("</td></tr>\n" +
 
                    "<tr title=\"")
