@@ -11,6 +11,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
 
+import net.i2p.app.ClientAppManager;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.Hash;
@@ -26,6 +27,7 @@ import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import net.i2p.router.networkdb.reseed.ReseedChecker;
 import net.i2p.router.transport.TransportUtil;
 import net.i2p.router.web.CSSHelper;
+import net.i2p.router.web.DeadlockDetector;
 import net.i2p.router.web.HelperBase;
 import net.i2p.router.web.NewsHelper;
 import net.i2p.router.web.WebAppStarter;
@@ -57,6 +59,7 @@ public class SummaryHelper extends HelperBase {
         "FirewallAndReseedStatus" + S +
         "I2PServices" + S +
         "I2PInternals" + S +
+        "I2PDiagnostics" + S +
         "HelpAndFAQ" + S +
         "Peers" + S +
         "RestartStatus" + S +
@@ -74,6 +77,7 @@ public class SummaryHelper extends HelperBase {
         "FirewallAndReseedStatus" + S +
         "I2PServices" + S +
         "I2PInternals" + S +
+        "I2PDiagnostics" + S +
         "Advanced" + S +
         "Peers" + S +
         "RestartStatus" + S +
@@ -230,10 +234,10 @@ public class SummaryHelper extends HelperBase {
         // Warn based on actual skew from peers, not update status, so if we successfully offset
         // the clock, we don't complain.
         //if (!_context.clock().getUpdatedSuccessfully())
-        long skew = _context.commSystem().getFramedAveragePeerClockSkew(33);
+        long skew = _context.commSystem().getFramedAveragePeerClockSkew(10);
         // Display the actual skew, not the offset
         if (Math.abs(skew) > 30*1000)
-            return new NetworkStateMessage(NetworkState.CLOCKSKEW, _t("ERR-Clock Skew of {0}", DataHelper.formatDuration2(Math.abs(skew))));
+            return new NetworkStateMessage(NetworkState.CLOCKSKEW, fixup(_t("ERR-Clock Skew of {0}", DataHelper.formatDuration2(Math.abs(skew)))));
         if (_context.router().isHidden())
             return new NetworkStateMessage(NetworkState.HIDDEN, _t("Hidden"));
         RouterInfo routerInfo = _context.router().getRouterInfo();
@@ -268,22 +272,22 @@ public class SummaryHelper extends HelperBase {
                 // TODO set IPv6 arg based on configuration?
                 if (TransportUtil.isPubliclyRoutable(ip, true))
                     return new NetworkStateMessage(NetworkState.RUNNING, txstatus);
-                return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-Private TCP Address"));
+                return new NetworkStateMessage(NetworkState.ERROR, fixup(_t("ERR-Private TCP Address")));
 
             case IPV4_SNAT_IPV6_UNKNOWN:
             case DIFFERENT:
-                return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-SymmetricNAT"));
+                return new NetworkStateMessage(NetworkState.ERROR, fixup(_t("ERR-SymmetricNAT")));
 
             case REJECT_UNSOLICITED:
                 state = NetworkState.FIREWALLED;
             case IPV4_DISABLED_IPV6_FIREWALLED:
                 if (routerInfo.getTargetAddress("NTCP") != null)
-                    return new NetworkStateMessage(NetworkState.WARN, _t("WARN-Firewalled with Inbound TCP Enabled"));
+                    return new NetworkStateMessage(NetworkState.WARN, fixup(_t("WARN-Firewalled with Inbound TCP Enabled")));
                 // fall through...
             case IPV4_FIREWALLED_IPV6_OK:
             case IPV4_FIREWALLED_IPV6_UNKNOWN:
-                if (((FloodfillNetworkDatabaseFacade)_context.netDb()).floodfillEnabled())
-                    return new NetworkStateMessage(NetworkState.WARN, _t("WARN-Firewalled and Floodfill"));
+                if ((_context.netDb()).floodfillEnabled())
+                    return new NetworkStateMessage(NetworkState.WARN, fixup(_t("WARN-Firewalled and Floodfill")));
                 //if (_context.router().getRouterInfo().getCapabilities().indexOf('O') >= 0)
                 //    return new NetworkStateMessage(NetworkState.WARN, _t("WARN-Firewalled and Fast"));
                 return new NetworkStateMessage(state, txstatus);
@@ -292,27 +296,49 @@ public class SummaryHelper extends HelperBase {
                 return new NetworkStateMessage(NetworkState.TESTING, _t("Disconnected - check network connection"));
 
             case HOSED:
-                return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-UDP Port In Use - Set i2np.udp.internalPort=xxxx in advanced config and restart"));
+                return new NetworkStateMessage(NetworkState.ERROR, fixup(_t("ERR-UDP Port In Use - Set i2np.udp.internalPort=xxxx in advanced config and restart")));
 
             case UNKNOWN:
                 state = NetworkState.TESTING;
             case IPV4_UNKNOWN_IPV6_FIREWALLED:
             case IPV4_DISABLED_IPV6_UNKNOWN:
             default:
-                RouterAddress ra = routerInfo.getTargetAddress("SSU");
-                if (ra == null && _context.router().getUptime() > 5*60*1000) {
+                List<RouterAddress> ra = routerInfo.getTargetAddresses("SSU", "SSU2");
+                if (ra.isEmpty() && _context.router().getUptime() > 5*60*1000) {
                     if (getActivePeers() <= 0)
-                        return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-No Active Peers, Check Network Connection and Firewall"));
+                        return new NetworkStateMessage(NetworkState.ERROR, fixup(_t("ERR-No Active Peers, Check Network Connection and Firewall")));
                     else if (_context.getProperty(ConfigNetHelper.PROP_I2NP_NTCP_HOSTNAME) == null ||
                         _context.getProperty(ConfigNetHelper.PROP_I2NP_NTCP_PORT) == null)
-                        return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-UDP Disabled and Inbound TCP host/port not set"));
+                        return new NetworkStateMessage(NetworkState.ERROR, fixup(_t("ERR-UDP Disabled and Inbound TCP host/port not set")));
                     else
-                        return new NetworkStateMessage(NetworkState.WARN, _t("WARN-Firewalled with UDP Disabled"));
+                        return new NetworkStateMessage(NetworkState.WARN, fixup(_t("WARN-Firewalled with UDP Disabled")));
                 }
                 return new NetworkStateMessage(state, txstatus);
         }
     }
 
+    /**
+     *  Make the already-translated ERR- and WARN- strings a little prettier
+     *  @since 0.9.58
+     */
+    private static String fixup(String s) {
+        if (s.startsWith("ERR-")) {
+            s = s.substring(4);
+        } else if (s.startsWith("ERR -")) {
+            s = s.substring(5);
+        } else if (s.startsWith("WARN-")) {
+            s = s.substring(5);
+        } else if (s.startsWith("WARN -")) {
+            s = s.substring(6);
+        } else {
+            // translated
+            int d = s.indexOf('-');
+            if (d > 0 && d < 10)
+                s = s.substring(d + 1);
+        }
+        return s;
+    }
+
     /**
      * Retrieve amount of used memory.
      * @since 0.9.32 uncommented
@@ -433,17 +459,6 @@ public class SummaryHelper extends HelperBase {
         return _context.peerManager().getPeersByCapability(FloodfillNetworkDatabaseFacade.CAPABILITY_FLOODFILL).size();
     }
 
-    /**
-     * How many peers the router ranks as failing.
-     * @since 0.9.32 uncommented
-     */
-    public int getFailingPeers() {
-        if (_context == null)
-            return 0;
-        else
-            return _context.profileOrganizer().countFailingPeers();
-    }
-
     /**
      * How many peers are banned.
      * @since 0.9.32 uncommented
@@ -562,6 +577,18 @@ public class SummaryHelper extends HelperBase {
         return DataHelper.formatSize2Decimal(sent) + 'B';
     }
 
+    /**
+     * Are both the webapp and TCG running?
+     *
+     * @since 0.9.58
+     */
+    public boolean isI2PTunnelRunning() {
+        if (!_context.portMapper().isRegistered(PortMapper.SVC_I2PTUNNEL))
+            return false;
+        ClientAppManager cmgr = _context.clientAppManager();
+        return cmgr != null && cmgr.getRegisteredApp("i2ptunnel") != null;
+    }
+
     /**
      * Client destinations connected locally.
      *
@@ -572,7 +599,7 @@ public class SummaryHelper extends HelperBase {
         List<Destination> clients = new ArrayList<Destination>(_context.clientManager().listClients());
 
         StringBuilder buf = new StringBuilder(512);
-        boolean link = _context.portMapper().isRegistered("i2ptunnel");
+        boolean link = isI2PTunnelRunning();
         buf.append("<h3>");
         if (link) {
             buf.append("<a href=\"/i2ptunnelmgr\" target=\"_top\" title=\"")
@@ -585,7 +612,7 @@ public class SummaryHelper extends HelperBase {
         }
         buf.append("</h3><hr class=\"b\">");
         if (!clients.isEmpty()) {
-            Collections.sort(clients, new AlphaComparator());
+            DataHelper.sort(clients, new AlphaComparator());
             buf.append("<table id=\"sb_localtunnels\">");
 
             for (Destination client : clients) {
@@ -597,7 +624,7 @@ public class SummaryHelper extends HelperBase {
                     buf.append("server.png\" alt=\"Server\" title=\"").append(_t("Hidden Service")).append("\">");
                 else
                     buf.append("client.png\" alt=\"Client\" title=\"").append(_t("Client")).append("\">");
-                buf.append("</td><td align=\"left\"><b><a href=\"tunnels#").append(h.toBase64().substring(0,4));
+                buf.append("</td><td align=\"left\"><b><a href=\"tunnels#").append(h.toBase64(), 0, 4);
                 buf.append("\" target=\"_top\" title=\"").append(_t("Show tunnels")).append("\">");
                 // Increase permitted max length of tunnel name & handle overflow with css
                 if (name.length() <= 32)
@@ -605,7 +632,7 @@ public class SummaryHelper extends HelperBase {
                 else
                     buf.append(DataHelper.escapeHTML(ServletUtil.truncate(name, 29))).append("&hellip;");
                 buf.append("</a></b></td>\n");
-                LeaseSet ls = _context.netDb().lookupLeaseSetLocally(h);
+                LeaseSet ls = _context.clientNetDb(client.calculateHash()).lookupLeaseSetLocally(h);
                 if (ls != null && _context.tunnelManager().getOutboundClientTunnelCount(h) > 0) {
                     if (!ls.isCurrent(0)) {
                         // yellow light
@@ -835,7 +862,7 @@ public class SummaryHelper extends HelperBase {
             else
                 needSpace = true;
             buf.append("<h4 class=\"sb_info sb_update\"><b>").append(_t("Update downloaded")).append("<br>");
-            if (_context.hasWrapper())
+            if (_context.hasWrapper() || NewsHelper.isExternalRestartPending())
                 buf.append(_t("Click Restart to install"));
             else
                 buf.append(_t("Click Shutdown and restart to install"));
@@ -949,13 +976,23 @@ public class SummaryHelper extends HelperBase {
                .append("</a></h4>");
         }
 
+        if (DeadlockDetector.isDeadlocked()) {
+            buf.append("<div class=\"sb_notice\"><b>")
+               .append(_t("Deadlock detected"))
+               .append(" - <a href=\"/logs\">")
+               .append(_t("Please report"))
+               .append("</a> - ").append(_t("After reporting, please restart your router"))
+               .append("</b></div>");
+        }
+
+        // checker will be null for DummyNetworkDatabaseFacade
         ReseedChecker checker = _context.netDb().reseedChecker();
-        String status = checker.getStatus();
+        String status = checker != null ? checker.getStatus() : "";
         if (status.length() > 0) {
             // Show status message even if not running, timer in ReseedChecker should remove after 20 minutes
             buf.append("<div class=\"sb_notice\"><i>").append(status).append("</i></div>");
         }
-        if (!checker.inProgress()) {
+        if (checker != null && !checker.inProgress()) {
             // If a new reseed isn't running, and the last reseed had errors, show error message
             String reseedErrorMessage = checker.getError();
             if (reseedErrorMessage.length() > 0) {
