@@ -4,6 +4,7 @@ import java.io.Serializable;
 import java.util.Collections;
 import java.util.Comparator;
 
+import net.i2p.I2PAppContext;
 import net.i2p.data.DataHelper;
 import net.i2p.router.transport.udp.PeerState;
 
@@ -120,9 +121,11 @@ class UDPSorters {
     }
 
     static class RateInComparator extends PeerComparator {
+        private final long now = I2PAppContext.getGlobalContext().clock().now();
+
         @Override
         public int compare(PeerState l, PeerState r) {
-            int rv = l.getReceiveBps() - r.getReceiveBps();
+            int rv = l.getReceiveBps(now) - r.getReceiveBps(now);
             if (rv == 0) // fallback on alpha
                 return super.compare(l, r);
             else
@@ -131,9 +134,11 @@ class UDPSorters {
     }
 
     static class RateOutComparator extends PeerComparator {
+        private final long now = I2PAppContext.getGlobalContext().clock().now();
+
         @Override
         public int compare(PeerState l, PeerState r) {
-            int rv = l.getSendBps() - r.getSendBps();
+            int rv = l.getSendBps(now) - r.getSendBps(now);
             if (rv == 0) // fallback on alpha
                 return super.compare(l, r);
             else
@@ -155,7 +160,7 @@ class UDPSorters {
     static class SkewComparator extends PeerComparator {
         @Override
         public int compare(PeerState l, PeerState r) {
-            long rv = Math.abs(l.getClockSkew()) - Math.abs(r.getClockSkew());
+            long rv = l.getClockSkew() - r.getClockSkew();
             if (rv == 0) // fallback on alpha
                 return super.compare(l, r);
             else
@@ -227,17 +232,19 @@ class UDPSorters {
         @Override
         public int compare(PeerState l, PeerState r) {
             int rv = l.getMTU() - r.getMTU();
-            if (rv == 0) // fallback on alpha
-                return super.compare(l, r);
-            else
-                return rv;
+            if (rv == 0) {
+                rv = l.getReceiveMTU() - r.getReceiveMTU();
+                if (rv == 0) // fallback on alpha
+                    return super.compare(l, r);
+            }
+            return rv;
         }
     }
 
     static class SendCountComparator extends PeerComparator {
         @Override
         public int compare(PeerState l, PeerState r) {
-            long rv = l.getPacketsTransmitted() - r.getPacketsTransmitted();
+            long rv = l.getMessagesSent() - r.getMessagesSent();
             if (rv == 0) // fallback on alpha
                 return super.compare(l, r);
             else
@@ -248,7 +255,7 @@ class UDPSorters {
     static class RecvCountComparator extends PeerComparator {
         @Override
         public int compare(PeerState l, PeerState r) {
-            long rv = l.getPacketsReceived() - r.getPacketsReceived();
+            long rv = l.getMessagesReceived() - r.getMessagesReceived();
             if (rv == 0) // fallback on alpha
                 return super.compare(l, r);
             else
