@@ -16,6 +16,7 @@ import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.Hash;
 import net.i2p.data.SessionKey;
+import net.i2p.util.ByteCache;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.ConvertToHash;
 import net.i2p.util.LHMCache;
@@ -54,6 +55,7 @@ class ConnectionManager {
     /** since 0.9, each manager instantiates its own timer */
     private final SimpleTimer2 _timer;
     private final Map<Long, Object> _recentlyClosed;
+    private final ByteCache _cache = ByteCache.getInstance(32, 4*1024);
     private static final Object DUMMY = new Object();
 
     /** cache of the property to detect changes */
@@ -67,13 +69,14 @@ class ConnectionManager {
     /** once over throttle limits, respond this many times before just dropping */
     private static final int DROP_OVER_LIMIT = 3;
 
-    // TODO https://stackoverflow.com/questions/16022624/examples-of-http-api-rate-limiting-http-response-headers
+    // https://stackoverflow.com/questions/16022624/examples-of-http-api-rate-limiting-http-response-headers
+    // RFC 6585
     private static final String LIMIT_HTTP_RESPONSE =
          "HTTP/1.1 429 Denied\r\n"+
          "Content-Type: text/html; charset=iso-8859-1\r\n"+
          "Cache-Control: no-cache\r\n"+
          "Connection: close\r\n"+
-         "Proxy-Connection: close\r\n"+
+         "Retry-After: 900\r\n"+
          "\r\n"+
          "<html><head><title>429 Denied</title></head>"+
          "<body><h3>429 Denied</h3>" +
@@ -82,6 +85,8 @@ class ConnectionManager {
 
     /**
      *  Manage all conns for this session
+     *
+     *  @param session the primary session, packets may come in on subsessions also
      */
     public ConnectionManager(I2PAppContext context, 
                              I2PSession session, 
@@ -126,6 +131,7 @@ class ConnectionManager {
         _context.statManager().createRateStat("stream.receiveActive", "How many streams are active when a new one is received (period being not yet dropped)", "Stream", new long[] { 60*60*1000, 24*60*60*1000 });
         // Stats for Connection
         _context.statManager().createRateStat("stream.con.windowSizeAtCongestion", "How large was our send window when we send a dup?", "Stream", new long[] { 60*60*1000 });
+        _context.statManager().createRateStat("stream.connectionReceived", "How stream connections were received", "Stream", new long[] { 60*60*1000 });
         _context.statManager().createRateStat("stream.chokeSizeBegin", "How many messages were outstanding when we started to choke?", "Stream", new long[] { 60*60*1000 });
         _context.statManager().createRateStat("stream.chokeSizeEnd", "How many messages were outstanding when we stopped being choked?", "Stream", new long[] { 60*60*1000 });
         _context.statManager().createRateStat("stream.fastRetransmit", "How long a packet has been around for if it has been resent per the fast retransmit timer?", "Stream", new long[] { 10*60*1000 });
@@ -235,39 +241,77 @@ class ConnectionManager {
      *         or if the connection was rejected
      */
     public Connection receiveConnection(Packet synPacket) {
+        Destination from = synPacket.getOptionalFrom();
+        if (from == null) {
+            if (_log.shouldWarn())
+                _log.warn("SYN w/o FROM: " + synPacket);
+            return null;
+        }
+        ByteArray ba = _cache.acquire();
+        boolean sigOk = synPacket.verifySignature(_context, ba.getData());
+        _cache.release(ba);
+        if (sigOk) {
+            long[] nacks = synPacket.getNacks();
+            if (nacks != null && nacks.length == 8) {
+                // we use the packet's session because it may be a subsession
+                Hash hash = synPacket.getSession().getMyDestination().calculateHash();
+                byte[] h = hash.getData();
+                for (int i = 0; i < 8; i++) {
+                    if (nacks[i] != DataHelper.fromLong(h, i << 2, 4)) {
+                        if (_log.shouldWarn()) {
+                            // glue it back together for logging only
+                            byte[] g = new byte[32];
+                            for (int j = 0; j < 8; j++) {
+                                DataHelper.toLong(g, j << 2, 4, nacks[j]);
+                            }
+                            Hash ghash = new Hash(g);
+                            _log.warn("Sig passed but hash failed, expected: " + hash.toBase32() + " got: " + ghash.toBase32());
+                        }
+                        sigOk = false;
+                        break;
+                    }
+                }
+                if (sigOk && _log.shouldWarn())
+                    _log.warn("Validated SYN NACKS from: " + from.toBase32());
+            }
+        }
+        if (!sigOk) {
+            if (_log.shouldWarn())
+                _log.warn("Received unsigned / forged SYN apparently from " + from.toBase32() + ": " + synPacket);
+            return null;
+        }
+
         boolean reject = false;
+        int retryAfter = 0;
 
             if (locked_tooManyStreams()) {
                 if ((!_defaultOptions.getDisableRejectLogging()) || _log.shouldLog(Log.WARN))
                     _log.logAlways(Log.WARN, "Refusing connection since we have exceeded our max of " 
                               + _defaultOptions.getMaxConns() + " connections");
                 reject = true;
+                retryAfter = 120;
             } else {
                 // this may not be right if more than one is enabled
-                String why = shouldRejectConnection(synPacket);
+                Reason why = shouldRejectConnection(synPacket);
                 if (why != null) {
                     if ((!_defaultOptions.getDisableRejectLogging()) || _log.shouldLog(Log.WARN))
                         _log.logAlways(Log.WARN, "Refusing connection since peer is " + why +
                            (synPacket.getOptionalFrom() == null ? "" : ": " + synPacket.getOptionalFrom().toBase32()));
                     reject = true;
+                    retryAfter = why.getSeconds();
                 }
             }
         
         _context.statManager().addRateData("stream.receiveActive", 1);
         
         if (reject) {
-            Destination from = synPacket.getOptionalFrom();
-            if (from == null)
-                return null;
             String resp = _defaultOptions.getLimitAction();
             if ("drop".equals(resp)) {
                 // always drop
                 return null;
             }
             Hash h = from.calculateHash();
-            if (_globalBlacklist.contains(h) ||
-                (_defaultOptions.isAccessListEnabled() && !_defaultOptions.getAccessList().contains(h)) ||
-                (_defaultOptions.isBlacklistEnabled() && _defaultOptions.getBlacklist().contains(h))) {
+            if (retryAfter >= MAX_TIME) {
                 // always drop these regardless of setting
                 return null;
             }
@@ -290,7 +334,10 @@ class ConnectionManager {
             boolean custom = !(reset || http);
             String sendResponse;
             if (http) {
-                sendResponse = LIMIT_HTTP_RESPONSE;
+                if (retryAfter > 0)
+                    sendResponse = LIMIT_HTTP_RESPONSE.replace("900", Integer.toString(retryAfter));
+                else
+                    sendResponse = LIMIT_HTTP_RESPONSE.replace("Retry-After: 900\r\n", "");
             } else if (custom) {
                 sendResponse = resp.replace("\\r", "\r").replace("\\n", "\n");
             } else {
@@ -391,7 +438,7 @@ class ConnectionManager {
             return false;
         if (con == null) {
             // Use the same throttling as for connections
-            String why = shouldRejectConnection(ping);
+            Reason why = shouldRejectConnection(ping);
             if (why != null) {
                 if ((!_defaultOptions.getDisableRejectLogging()) || _log.shouldLog(Log.WARN))
                     _log.logAlways(Log.WARN, "Dropping ping since peer is " + why + ": " + dest.calculateHash());
@@ -554,7 +601,6 @@ class ConnectionManager {
                 break;
         }
         
-        _context.statManager().addRateData("stream.connectionCreated", 1);
         return con;
     }
 
@@ -591,14 +637,35 @@ class ConnectionManager {
     }
     
     /**
+     *  @since 0.9.49
+     */
+    private static class Reason {
+        private final String txt;
+        private final int seconds;
+
+        public Reason(String text, int secs) {
+            txt = text; seconds = secs;
+        }
+
+        @Override
+        public String toString() { return txt; }
+
+        public int getSeconds() { return seconds; }
+    }
+
+    private static final int MAX_TIME = 9999999;
+
+
+    /**
+     *  Reason time is in seconds for Retry-After header; MAX_TIME for drop, 0 if unknown
      *  @return reason string or null if not rejected
      */
-    private String shouldRejectConnection(Packet syn) {
+    private Reason shouldRejectConnection(Packet syn) {
         // unfortunately we don't have access to the router client manager here,
         // so we can't whitelist local access
         Destination from = syn.getOptionalFrom();
         if (from == null)
-            return "null";
+            return new Reason("null", MAX_TIME);
         Hash h = from.calculateHash();
 
         // As of 0.9.9, run the blacklist checks BEFORE the port counters,
@@ -631,61 +698,61 @@ class ConnectionManager {
             }
         }
         if (hashes.length() > 0 && _globalBlacklist.contains(h))
-            return "blacklisted globally";
+            return new Reason("blacklisted globally", MAX_TIME);
 
         if (_defaultOptions.isAccessListEnabled() &&
             !_defaultOptions.getAccessList().contains(h))
-            return "not whitelisted";
+            return new Reason("not whitelisted", MAX_TIME);
         if (_defaultOptions.isBlacklistEnabled() &&
             _defaultOptions.getBlacklist().contains(h))
-            return "blacklisted";
+            return new Reason("blacklisted", MAX_TIME);
 
 
         if (_dayThrottler != null && _dayThrottler.shouldThrottle(h)) {
             _context.statManager().addRateData("stream.con.throttledDay", 1);
             if (_defaultOptions.getMaxConnsPerDay() <= 0)
-                return "throttled by" +
+                return new Reason("throttled by" +
                         " total limit of " + _defaultOptions.getMaxTotalConnsPerDay() +
-                        " per day";
+                        " per day", 86400);
             else if (_defaultOptions.getMaxTotalConnsPerDay() <= 0)
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerDay() +
-                        " per day";
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerDay() +
+                        " per day", 86400);
             else
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerDay() +
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerDay() +
                         " or total limit of " + _defaultOptions.getMaxTotalConnsPerDay() +
-                        " per day";
+                        " per day", 86400);
         }
         if (_hourThrottler != null && _hourThrottler.shouldThrottle(h)) {
             _context.statManager().addRateData("stream.con.throttledHour", 1);
             if (_defaultOptions.getMaxConnsPerHour() <= 0)
-                return "throttled by" +
+                return new Reason("throttled by" +
                         " total limit of " + _defaultOptions.getMaxTotalConnsPerHour() +
-                        " per hour";
+                        " per hour", 3600);
             else if (_defaultOptions.getMaxTotalConnsPerHour() <= 0)
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerHour() +
-                        " per hour";
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerHour() +
+                        " per hour", 3600);
             else
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerHour() +
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerHour() +
                         " or total limit of " + _defaultOptions.getMaxTotalConnsPerHour() +
-                        " per hour";
+                        " per hour", 3600);
         }
         if (_minuteThrottler != null && _minuteThrottler.shouldThrottle(h)) {
             _context.statManager().addRateData("stream.con.throttledMinute", 1);
             if (_defaultOptions.getMaxConnsPerMinute() <= 0)
-                return "throttled by" +
+                return new Reason("throttled by" +
                         " total limit of " + _defaultOptions.getMaxTotalConnsPerMinute() +
-                        " per minute";
+                        " per minute", 60);
             else if (_defaultOptions.getMaxTotalConnsPerMinute() <= 0)
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerMinute() +
-                        " per minute";
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerMinute() +
+                        " per minute", 60);
             else
-                return "throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerMinute() +
+                return new Reason("throttled by per-peer limit of " + _defaultOptions.getMaxConnsPerMinute() +
                         " or total limit of " + _defaultOptions.getMaxTotalConnsPerMinute() +
-                        " per minute";
+                        " per minute", 60);
         }
 
         if (!_connectionFilter.allowDestination(from)) {
-            return "not allowed by filter";
+            return new Reason("not allowed by filter", 0);
         }
 
         return null;
