@@ -119,7 +119,7 @@ public class AddressBean
 
 		boolean needsIDN = false;
 		// Here we do easy checks and throw translated exceptions.
-		// We do checks on the whole host name, not on each "label", so
+		// We do checks on the whole hostname, not on each "label", so
 		// we allow '.', and some untranslated errors will be thrown by IDN.toASCII()
 		for (int i = 0; i < host.length(); i++) {
 			char c = host.charAt(i);
@@ -205,7 +205,7 @@ public class AddressBean
 
 	/** @since 0.9.26 */
 	public boolean isValidated() {
-		return Boolean.valueOf(getProp("v"));
+		return Boolean.parseBoolean(getProp("v"));
 	}
 
 	/** @since 0.8.7 */
