@@ -207,7 +207,7 @@ public class AddressbookBean extends BaseBean
 		} else {
 			if (resultCount <= 0)
 				// covered in jsp
-				//message = _t("This addressbook is empty.");
+				//message = _t("This address book is empty.");
 				message = "";
 			else
 				message = ngettext("Address book contains 1 entry.",
@@ -229,7 +229,7 @@ public class AddressbookBean extends BaseBean
 			message += ' ' + _t("Showing {0} of {1}", "" + (getBeginInt()+1) + '-' + (getEndInt()+1), Integer.valueOf(resultCount));
 			if (getEndInt() < resultCount - 1) {
 				int newBegin = Math.min(resultCount - 1, getEndInt() + 1);
-				int newEnd = Math.min(resultCount, getEndInt() + DISPLAY_SIZE);
+				int newEnd = Math.min(resultCount - 1, getEndInt() + DISPLAY_SIZE);
 		       		message += " | <a href=\"addressbook?book=" + getBook() + filterArg +
 				           "&amp;begin=" + newBegin + "&amp;end=" + newEnd + "\">" + (newBegin+1) +
 				           '-' + (newEnd+1) + "</a>";
@@ -310,14 +310,14 @@ public class AddressbookBean extends BaseBean
 									else
 										message = _t("Destination changed for {0}.", displayHost);
 									if (!host.endsWith(".i2p"))
-										message += "<br>" + _t("Warning - host name does not end with \".i2p\"");
+										message += "<br>" + _t("Warning - hostname does not end with \".i2p\"");
 									// clear form
 									hostname = null;
 									destination = null;
 								} else {
 									if (message.length() <= 0) {
 										if (wasB32)
-											message = _t("Invalid Base 32 host name.");
+											message = _t("Invalid Base 32 hostname.");
 										else
 											message = _t("Invalid Base 64 destination.");
 									}
@@ -326,10 +326,10 @@ public class AddressbookBean extends BaseBean
 						} catch (IllegalArgumentException iae) {
 							message = iae.getMessage();
 							if (message == null)
-								message = _t("Invalid host name \"{0}\".", hostname);
+								message = _t("Invalid hostname \"{0}\".", hostname);
 						}
 					} else {
-						message = _t("Please enter a host name and destination");
+						message = _t("Please enter a hostname and destination");
 					}
 					// clear search when adding
 					search = null;
@@ -366,7 +366,7 @@ public class AddressbookBean extends BaseBean
 						message += "<br>" + _t("Address book saved.");
 					} catch (IOException e) {
 						warn(e);
-						message += "<br>" + _t("ERROR: Could not write addressbook file.");
+						message += "<br>" + _t("ERROR: Could not write address book file.");
 					}
 				}
 			}			
