@@ -11,6 +11,7 @@ package net.i2p.client.impl;
 
 import java.io.BufferedInputStream;
 import java.io.EOFException;
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -55,6 +56,7 @@ import net.i2p.data.SigningPublicKey;
 import net.i2p.data.i2cp.BlindingInfoMessage;
 import net.i2p.data.i2cp.DestLookupMessage;
 import net.i2p.data.i2cp.DestReplyMessage;
+import net.i2p.data.i2cp.DestroySessionMessage;
 import net.i2p.data.i2cp.GetBandwidthLimitsMessage;
 import net.i2p.data.i2cp.GetDateMessage;
 import net.i2p.data.i2cp.HostLookupMessage;
@@ -769,7 +771,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                 auth.setProperty(I2PClient.PROP_USER, _options.getProperty(I2PClient.PROP_USER));
                 auth.setProperty(I2PClient.PROP_PW, _options.getProperty(I2PClient.PROP_PW));
             }
-            sendMessage_unchecked(new GetDateMessage(CoreVersion.VERSION, auth));
+            sendMessage_unchecked(new GetDateMessage(CoreVersion.PUBLISHED_VERSION, auth));
             waitForDate();
 
             if (_log.shouldLog(Log.DEBUG)) _log.debug(getPrefix() + "Before producer.connect()");
@@ -834,6 +836,8 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                 msg = "Failed to bind to the router on " + _options.getProperty(PROP_DOMAIN_SOCKET) + " and build tunnels";
             else
                 msg = "Cannot connect to the router on " + _hostname + ':' + _portNum + " and build tunnels";
+            if (ioe.getMessage() != null)
+                msg += " - " + ioe.getMessage();
             throw new I2PSessionException(getPrefix() + msg, ioe);
         } finally {
             if (success) {
@@ -1306,13 +1310,24 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         }
         
         if (_log.shouldLog(Log.INFO)) _log.info(getPrefix() + "Destroy the session", new Exception("DestroySession()"));
-        if (sendDisconnect && _producer != null) {    // only null if overridden by I2PSimpleSession
-            try {
-                _producer.disconnect(this);
-            } catch (I2PSessionException ipe) {
-                //propogateError("Error destroying the session", ipe);
-                if (_log.shouldLog(Log.WARN))
-                    _log.warn("Error destroying the session", ipe);
+        if (sendDisconnect) {
+            if (_producer != null) {    // only null if overridden by I2PSimpleSession
+                try {
+                    _producer.disconnect(this);
+                } catch (I2PSessionException ipe) {
+                    //propogateError("Error destroying the session", ipe);
+                    if (_log.shouldLog(Log.WARN))
+                        _log.warn("Error destroying the session", ipe);
+                }
+            } else if (!_context.isRouterContext()) {
+                // Simple session, prevent error on router side in I2CP reader
+                DestroySessionMessage dmsg = new DestroySessionMessage();
+                dmsg.setSessionId(DUMMY_SESSION);
+                try {
+                    sendMessage_unchecked(dmsg);
+                    // give it a chance to get there
+                    try { Thread.sleep(20); } catch (InterruptedException ie) {}
+                } catch (I2PSessionException ise) {}
             }
         }
         // SimpleSession does not initialize
@@ -1707,7 +1722,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     }
 
     /**
-     *  Ask the router to lookup a Destination by host name.
+     *  Ask the router to lookup a Destination by hostname.
      *  Blocking. Waits a max of 10 seconds by default.
      *
      *  This only makes sense for a b32 hostname, OR outside router context.
@@ -1729,7 +1744,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     }
 
     /**
-     *  Ask the router to lookup a Destination by host name.
+     *  Ask the router to lookup a Destination by hostname.
      *  Blocking. See above for details.
      *  @param maxWait ms
      *  @since 0.9.11
@@ -1745,7 +1760,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     }
 
     /**
-     *  Ask the router to lookup a Destination by host name.
+     *  Ask the router to lookup a Destination by hostname.
      *  Blocking. See above for details.
      *  Same as lookupDest() but with a failure code in the return value
      *
@@ -1767,7 +1782,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     }
 
     /**
-     *  Ask the router to lookup a Destination by host name.
+     *  Ask the router to lookup a Destination by hostname.
      *  Blocking. See above for details.
      *  @param maxWait ms
      *  @since 0.9.11
@@ -1880,7 +1895,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
      */
     public String getRouterVersion() {
         if (_context.isRouterContext())
-            return CoreVersion.VERSION;
+            return CoreVersion.PUBLISHED_VERSION;
         return _routerVersion;
     }
 
@@ -1921,7 +1936,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         StringBuilder buf = new StringBuilder(32);
         buf.append("Session: ");
         if (_myDestination != null)
-            buf.append(_myDestination.calculateHash().toBase64().substring(0, 4));
+            buf.append(_myDestination.calculateHash().toBase64(), 0, 4);
         else
             buf.append("[null dest]");
         buf.append(getPrefix());
