@@ -38,9 +38,11 @@ import java.util.Set;
 import javax.security.auth.x500.X500Principal;
 
 import net.i2p.I2PAppContext;
+import net.i2p.crypto.provider.I2PProvider;
 import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
 import net.i2p.data.SigningPrivateKey;
+import net.i2p.data.SigningPublicKey;
 import net.i2p.util.Log;
 import net.i2p.util.FileSuffixFilter;
 import net.i2p.util.SecureFileOutputStream;
@@ -57,6 +59,10 @@ public final class CertUtil {
     private static final String REVOCATION_DIR = "revocations";
     private static final int LINE_LENGTH = 64;
 
+    static {
+        I2PProvider.addProvider();
+    }
+
     /**
      *  Write a certificate to a file in base64 format.
      *
@@ -284,6 +290,10 @@ public final class CertUtil {
      *  Throws if the certificate is invalid (e.g. expired).
      *
      *  This does NOT check for revocation.
+     *  This verifies the signature, assuming it is self-signed,
+     *  but will not throw an exception,
+     *  will log a warning only, as of 0.9.54.
+     *  We do NOT fetch additional certs or attempt to validate a cert up the chain.
      *
      *  @return non-null, throws on all errors including certificate invalid
      *  @since 0.9.24 adapted from SU3File private method
@@ -295,6 +305,11 @@ public final class CertUtil {
             CertificateFactory cf = CertificateFactory.getInstance("X.509");
             X509Certificate cert = (X509Certificate)cf.generateCertificate(fis);
             cert.checkValidity();
+            try {
+                cert.verify(cert.getPublicKey());
+            } catch (Exception e) {
+                System.out.println("Warning: Cert is not self-signed or has a bad signature: " + kd + " - " + e);
+            }
             return cert;
         } catch (IllegalArgumentException iae) {
             // java 1.8.0_40-b10, openSUSE
@@ -335,9 +350,9 @@ public final class CertUtil {
                 throw new CertificateEncodingException("bad base64 cert");
             PrivateKey rv = null;
             // try all the types
+            KeySpec ks = new PKCS8EncodedKeySpec(data);
             for (SigAlgo algo : EnumSet.allOf(SigAlgo.class)) {
                 try {
-                    KeySpec ks = new PKCS8EncodedKeySpec(data);
                     String alg = algo.getName();
                     KeyFactory kf = KeyFactory.getInstance(alg);
                     rv = kf.generatePrivate(ks);
@@ -569,7 +584,9 @@ public final class CertUtil {
             File f = new File(args[1]);
             if (args[0].equals("loadcert")) {
                 X509Certificate cert = loadCert(f);
-                System.out.println(net.i2p.util.HexDump.dump(cert.getEncoded()));
+                PublicKey pub = cert.getPublicKey();
+                SigningPublicKey spk = SigUtil.fromJavaKey(pub);
+                System.out.println("Loaded " + spk + ' ' + spk.toBase64());
             } else if (args[0].equals("loadcrl")) {
                 loadCRL(f);
             } else if (args[0].equals("loadcrldir")) {
