@@ -82,6 +82,8 @@ public class DataHelper {
             "ihost0", "iport0", "ikey0", "itag0", "iexp0",
             "ihost1", "iport1", "ikey1", "itag1", "iexp1",
             "ihost2", "iport2", "ikey2", "itag2", "iexp2",
+            "ihost3", "iport3", "ikey3", "itag3", "iexp3",
+            "ih0", "ih1", "ih2", "ih3",
             // RouterInfo options
             "caps", "coreVersion", "netId", "router.version",
             "netdb.knownLeaseSets", "netdb.knownRouters",
@@ -108,8 +110,8 @@ public class DataHelper {
         }
     }
 
-    private static final Pattern ILLEGAL_KEY =  Pattern.compile("[#=\r\n;]");
-    private static final Pattern ILLEGAL_VALUE =  Pattern.compile("[#\r\n]");
+    private static final Pattern ILLEGAL_KEY =  Pattern.compile("[#=\\r\\n;]");
+    private static final Pattern ILLEGAL_VALUE =  Pattern.compile("[#\\r\\n]");
 
     /**
      *  The default formatting for date/time, current locale, local time zone
@@ -526,18 +528,20 @@ public class DataHelper {
         try {
             fos = new SecureFileOutputStream(tmpFile);
             out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(fos, "UTF-8")));
+            if (file.getName().equals("router.config"))
+                out.println("# WARNING: Do NOT copy this file from one router to another, this may cause serious problems.");
             out.println("# NOTE: This I2P config file must use UTF-8 encoding");
             out.println("# Last saved: " + formatTime(System.currentTimeMillis()));
             for (Map.Entry<Object, Object> entry : props.entrySet()) {
                 String name = (String) entry.getKey();
                 String val = (String) entry.getValue();
-                if (ILLEGAL_KEY.matcher(name).matches()) {
+                if (ILLEGAL_KEY.matcher(name).find()) {
                     if (iae == null)
                         iae = new IllegalArgumentException("Invalid character (one of \"#;=\\r\\n\") in key: \"" +
                                                            name + "\" = \"" + val + '\"');
                     continue;
                 }
-                if (ILLEGAL_VALUE.matcher(val).matches()) {
+                if (ILLEGAL_VALUE.matcher(val).find()) {
                     if (iae == null)
                         iae = new IllegalArgumentException("Invalid character (one of \"#\\r\\n\") in value: \"" +
                                                            name + "\" = \"" + val + '\"');
@@ -629,6 +633,7 @@ public class DataHelper {
      *  @return (new BigInteger(1, buf)).toString()
      *  @deprecated unused
      */
+    @Deprecated
     public static String toDecimalString(byte buf[], int len) {
         if (buf == null)
             return "0";
@@ -652,6 +657,7 @@ public class DataHelper {
      *  @return minimum-length representation (with possible leading 0 byte)
      *  @deprecated unused
      */
+    @Deprecated
     public final static byte[] fromHexString(String val) {
         BigInteger bv = new BigInteger(val, 16);
         return bv.toByteArray();
@@ -2011,6 +2017,7 @@ public class DataHelper {
      *  @throws RuntimeException
      *  @deprecated unused
      */
+    @Deprecated
     public static byte[] getUTF8(StringBuffer orig) {
         if (orig == null) return null;
         return getUTF8(orig.toString());
@@ -2101,7 +2108,7 @@ public class DataHelper {
         Pattern p = patterns.get(regex);
         if (p == null) {
             // catches easy mistake, and also swapping the args by mistake
-            if (regex.length() > 1 && !regex.startsWith("[") && !regex.equals("\r\n")) {
+            if (regex.length() > 1 && !regex.startsWith("[") && !regex.equals("\r\n") && !regex.startsWith("\\")) {
                 //(new Exception("Warning: Split on regex: \"" + regex + "\" should probably be enclosed with []")).printStackTrace();
                 System.out.println("Warning: Split on regex: \"" + regex + "\" should probably be enclosed with []");
             }
