@@ -62,10 +62,31 @@ public abstract class DatabaseEntry extends DataStructureImpl {
     // synch: this
     private Hash _currentRoutingKey;
     private long _routingKeyGenMod;
+    private boolean _receivedAsPublished;
+    private boolean _receivedAsReply;
+
+    /**
+     * Hash of the client receiving the routerinfo, or null if it was sent directly.
+     */
+    private Hash _receivedBy;
+
+    /**
+     * The Hash of the local client that received this LS,
+     * null if the router or unknown.
+     *
+     * @since 0.9.47
+     */
+    public Hash getReceivedBy() {
+        return _receivedBy;
+    }
+
+    public void setReceivedBy(Hash receivedBy) {
+        this._receivedBy = receivedBy;
+    }
 
     /**
      * A common interface to the timestamp of the two subclasses.
-     * Identical to getEarliestLeaseData() in LeaseSet,
+     * Identical to getEarliestLeaseDate() in LeaseSet,
      * and getPublished() in RouterInfo.
      * Note that for a LeaseSet this will be in the future,
      * and for a RouterInfo it will be in the past.
@@ -253,4 +274,45 @@ public abstract class DatabaseEntry extends DataStructureImpl {
             return false;
         return DSAEngine.getInstance().verifySignature(_signature, data, spk);
     }
+    
+    /**
+     * If true, we received this LeaseSet by a remote peer publishing it to
+     * us, rather than by searching for it ourselves or locally creating it.
+     * Default false.
+     *
+     * @since 0.9.58 moved up from LeaseSet
+     */
+    public boolean getReceivedAsPublished() { 
+        return _receivedAsPublished;
+    }
+
+    /**
+     * @since 0.9.58 moved up from LeaseSet
+     * 
+     * use this carefully, when updating the flags make sure the old and new
+     * leaseSet are actually equivalent, or simply copy over the reply value,
+     * see KademliaNetworkDatabaseFacade.java line 997 for more information.
+     */
+    public void setReceivedAsPublished() {
+        _receivedAsPublished = true; 
+    }
+
+    /**
+     * If true, we received this LeaseSet by searching for it
+     * Default false.
+     *
+     * @since 0.7.14, moved up from LeaseSet in 0.9.58
+     */
+    public boolean getReceivedAsReply() {
+        return _receivedAsReply; 
+    }
+
+    /**
+     * set to true
+     *
+     * @since 0.7.14, moved up from LeaseSet in 0.9.58
+     */
+    public void setReceivedAsReply() {
+        _receivedAsReply = true;
+    }
 }
