@@ -19,6 +19,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
+import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -26,8 +27,9 @@ import java.util.Map;
 import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
+import java.util.regex.Pattern;
 
-import org.apache.http.conn.util.InetAddressUtils;
+import net.i2p.apache.http.conn.util.InetAddressUtils;
 
 import net.i2p.I2PAppContext;
 import net.i2p.data.DataHelper;
@@ -40,7 +42,7 @@ import net.i2p.data.DataHelper;
  * @author zzz
  */
 public abstract class Addresses {
-    
+
     private static final File IF_INET6_FILE = new File("/proc/net/if_inet6");
     private static final long INET6_CACHE_EXPIRE = 10*60*1000;
     private static final boolean INET6_CACHE_ENABLED = !SystemVersion.isMac() && !SystemVersion.isWindows() &&
@@ -54,9 +56,36 @@ public abstract class Addresses {
     private static final Set<String> _macCache = new HashSet<String>();
     private static final boolean TEST_IPV6_ONLY = false;
 
+    // Convert IPv6 to canonical RFC 5952
+    // https://stackoverflow.com/questions/7043983/ipv6-address-into-compressed-form-in-java
+    private static final String RFC5952_MATCH = "((?:(?:^|:)0+\\b){2,}):?(?!\\S*\\b\\1:0+\\b)(\\S*)";
+    private static final Pattern RFC5952_PATTERN = Pattern.compile(RFC5952_MATCH);
+
+    /**
+     *  Do we have any address of this type?
+     *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
+     *  Use getConnectedAddressTypes() to get all types at once.
+     *
+     *  @since 0.9.54
+     */
+    public static boolean isConnected(AddressType type) {
+        switch(type) {
+            case IPV6:
+                return isConnectedIPv6();
+
+            case YGG:
+                return getYggdrasilAddress() != null;
+
+            case IPV4:
+            default:
+                return isConnected();
+        }
+    }
+
     /**
      *  Do we have any non-loop, non-wildcard IPv4 address at all?
      *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
+     *  Use getConnectedAddressTypes() to get all types at once.
      *
      *  @since 0.9.4
      */
@@ -68,6 +97,7 @@ public abstract class Addresses {
     /**
      *  Do we have any non-loop, non-wildcard IPv6 address at all?
      *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
+     *  Use getConnectedAddressTypes() to get all types at once.
      *
      *  @since 0.9.29
      */
@@ -122,10 +152,10 @@ public abstract class Addresses {
      *
      *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
      *
-     *  @return a sorted set of all addresses including wildcard
-     *  @param includeLocal whether to include local
+     *  @param includeLocal whether to include local addresses
+     *                      and deprecated IPv6 addresses
      *  @param includeIPv6 whether to include IPV6
-     *  @return a Set of all addresses
+     *  @return a sorted set of all addresses including wildcard
      *  @since 0.8.3
      */
     public static SortedSet<String> getAddresses(boolean includeLocal, boolean includeIPv6) {
@@ -141,11 +171,11 @@ public abstract class Addresses {
      *
      *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
      *
-     *  @return a sorted set of all addresses
      *  @param includeSiteLocal whether to include private like 192.168.x.x
+     *                          and deprecated IPv6 addresses
      *  @param includeLoopbackAndWildcard whether to include 127.x.x.x and 0.0.0.0
      *  @param includeIPv6 whether to include IPV6
-     *  @return a Set of all addresses
+     *  @return a sorted set of all addresses
      *  @since 0.9.4
      */
     public static SortedSet<String> getAddresses(boolean includeSiteLocal,
@@ -163,12 +193,12 @@ public abstract class Addresses {
      *
      *  Warning, very slow on Windows, appx. 200ms + 50ms/interface
      *
-     *  @return a sorted set of all addresses
      *  @param includeSiteLocal whether to include private like 192.168.x.x
+     *                          and deprecated IPv6 addresses
      *  @param includeLoopbackAndWildcard whether to include 127.x.x.x and 0.0.0.0
      *  @param includeIPv6 whether to include IPV6
      *  @param includeIPv6Temporary whether to include IPV6 temporary addresses
-     *  @return a Set of all addresses
+     *  @return a sorted set of all addresses
      *  @since 0.9.46
      */
     public static SortedSet<String> getAddresses(boolean includeSiteLocal,
@@ -263,6 +293,69 @@ public abstract class Addresses {
         return rv;
     }
 
+    /**
+     *  Warning, very slow on Windows. Caller should cache.
+     *
+     *  @return the IPv6 address with prefix 02xx: or 03xx:, or null
+     *  @since 0.9.49
+     */
+    public static byte[] getYggdrasilAddress() {
+        if (SystemVersion.isAndroid())
+            return null;
+        try {
+            Enumeration<NetworkInterface> ifcs = NetworkInterface.getNetworkInterfaces();
+            if (ifcs != null) {
+                while (ifcs.hasMoreElements()) {
+                    NetworkInterface ifc = ifcs.nextElement();
+                    if (!ifc.isUp())
+                        continue;
+                    for(Enumeration<InetAddress> addrs =  ifc.getInetAddresses(); addrs.hasMoreElements();) {
+                        InetAddress addr = addrs.nextElement();
+                        byte[] ip = addr.getAddress();
+                        if (ip.length == 16 && (ip[0] & 0xfe) == 0x02)
+                            return ip;
+                    }
+                }
+            }
+        } catch (SocketException e) {
+        } catch (java.lang.Error e) {
+        }
+        return null;
+    }
+
+    /**
+     *  Efficiently get all connected address types in one pass.
+     *  Warning, very slow on Windows. Caller should cache.
+     *
+     *  @return the set of connected address types, non-null
+     *  @since 0.9.54
+     */
+    public static Set<AddressType> getConnectedAddressTypes() {
+        Set<AddressType> rv = EnumSet.noneOf(AddressType.class);
+        try {
+            Enumeration<NetworkInterface> ifcs = NetworkInterface.getNetworkInterfaces();
+            if (ifcs != null) {
+                while (ifcs.hasMoreElements()) {
+                    NetworkInterface ifc = ifcs.nextElement();
+                    if (!ifc.isUp())
+                        continue;
+                    for(Enumeration<InetAddress> addrs =  ifc.getInetAddresses(); addrs.hasMoreElements();) {
+                        InetAddress addr = addrs.nextElement();
+                        byte[] ip = addr.getAddress();
+                        if (ip.length == 16 && (ip[0] & 0xfe) == 0x02) {
+                            rv.add(AddressType.YGG);
+                        } else if (shouldInclude(addr, true, false, true)) {
+                            rv.add(ip.length == 16 ? AddressType.IPV6 : AddressType.IPV4);
+                        }
+                    }
+                }
+            }
+        } catch (SocketException e) {
+        } catch (java.lang.Error e) {
+        }
+        return rv;
+    }
+
     /**
      *  Strip the trailing "%nn" from Inet6Address.getHostAddress()
      *  @since IPv6
@@ -276,8 +369,8 @@ public abstract class Addresses {
 
     private static boolean shouldInclude(InetAddress ia, boolean includeSiteLocal,
                                          boolean includeLoopbackAndWildcard, boolean includeIPv6) {
+        byte[] ip = ia.getAddress();
         if (TEST_IPV6_ONLY) {
-            byte[] ip = ia.getAddress();
             if (ip.length == 4) {
                 int i = ip[0] & 0xff;
                 if (i != 127 &&
@@ -297,8 +390,10 @@ public abstract class Addresses {
             (includeSiteLocal ||
              ((!ia.isSiteLocalAddress()) &&
               // disallow fc00::/8 and fd00::/8 (Unique local addresses RFC 4193)
+              // and yggdrasil
               // not recognized as local by InetAddress
-              (ia.getAddress().length != 16 || (ia.getAddress()[0] & 0xfe) != 0xfc))) &&
+              (ip.length != 16 ||
+               ((ip[0] & 0xff) >= 0x20 && (ip[0] & 0xff) <= 0x3f)))) &&
             // Hamachi 5/8 allocated to RIPE (30 November 2010)
             // Removed from TransportImpl.isPubliclyRoutable()
             // Check moved to here, for now, but will eventually need to
@@ -306,7 +401,7 @@ public abstract class Addresses {
             //(includeLocal ||
             //(!ia.getHostAddress().startsWith("5."))) &&
             (includeIPv6 ||
-             (ia instanceof Inet4Address));
+             ip.length == 4);
     }
 
     /**
@@ -343,7 +438,66 @@ public abstract class Addresses {
             return "(bad IP length " + addr.length + "):" + port;
         }
     }
-    
+
+    /**
+     *  Same as toString() but returns IPv6 in compressed form, ref. RFC 5952
+     *  @since 0.9.57
+     */
+    public static String toCanonicalString(byte[] addr) {
+        if (addr == null)
+            return "null";
+        try {
+            String rv = InetAddress.getByAddress(addr).getHostAddress();
+            if (addr.length == 16)
+                rv = x_toCanonicalString(rv);
+            return rv;
+        } catch (UnknownHostException uhe) {
+            return "bad IP length " + addr.length;
+        }
+    }
+
+    /**
+     *  Same as toString() but returns IPv6 in compressed form, ref. RFC 5952
+     *  @since 0.9.57
+     */
+    public static String toCanonicalString(byte[] addr, int port) {
+        if (addr == null)
+            return "null:" + port;
+        try {
+            String ip = InetAddress.getByAddress(addr).getHostAddress();
+            if (addr.length != 16)
+                return ip + ':' + port;
+            ip = x_toCanonicalString(ip);
+            return '[' + ip + "]:" + port;
+        } catch (UnknownHostException uhe) {
+            return "(bad IP length " + addr.length + "):" + port;
+        }
+    }
+
+    /**
+     *  Converts IPv6 to compressed form, ref. RFC 5952. IPv4 returned unchanged.
+     *  @since 0.9.57
+     */
+    public static String toCanonicalString(String host) {
+        if (host == null)
+            return "null";
+        if (host.indexOf(':') < 0)
+            return host;
+        return x_toCanonicalString(host);
+    }
+
+    /**
+     *  Internal
+     *  @param host non-null
+     *  @since 0.9.57
+     */
+    private static String x_toCanonicalString(String host) {
+        String rv =  RFC5952_PATTERN.matcher(host).replaceAll("::$2");
+        if (rv.startsWith("0::"))
+            rv = rv.substring(1);
+        return rv;
+    }
+
     /**
      *  Convenience method to convert and validate a port String
      *  without throwing an exception.
@@ -391,18 +545,27 @@ public abstract class Addresses {
 
     /**
      *  Caching version of InetAddress.getByName(host).getAddress(), which is slow.
-     *  Caches numeric host names only.
-     *  Will resolve but not cache DNS host names.
+     *  Caches numeric addresses only.
+     *  Will resolve but not cache DNS addresses.
+     *
+     *  DEPRECATED for IPs in RouterAddresses, blocklists, etc.,
+     *  use getI2POnly() which avoids getByName() in most cases.
      *
      *  Unlike InetAddress.getByName(), we do NOT allow numeric IPs
      *  of the form d.d.d, d.d, or d, as these are almost certainly mistakes.
      *
-     *  @param host DNS or IPv4 or IPv6 host name; if null returns null
+     *  InetAddress.getByName() is documented to return 127.0.0.1 for a null host;
+     *  here we return null.
+     *
+     *  InetAddress.getByName() also returns 127.0.0.1 for a host "",
+     *  but this is undocumented; as of 0.9.49, here we return null.
+     *
+     *  @param host DNS or IPv4 or IPv6 address; if null or empty returns null
      *  @return IP or null
      *  @since 0.9.3
      */
     public static byte[] getIP(String host) {
-        if (host == null)
+        if (host == null || host.isEmpty())
             return null;
         byte[] rv;
         synchronized (_IPAddress) {
@@ -417,7 +580,6 @@ public abstract class Addresses {
                     _negativeCache.remove(host);
                 }
             }
-            //I2PAppContext.getGlobalContext().logManager().getLog(Addresses.class).error("lookup of " + host, new Exception("I did it"));
             try {
                 rv = InetAddress.getByName(host).getAddress();
                 if (isIPAddress(host)) {
@@ -438,17 +600,23 @@ public abstract class Addresses {
     /**
      *  Caching version of InetAddress.getByName(host).getAddress(), which is slow.
      *  Resolves literal IP addresses only, will not cause a DNS lookup.
-     *  Will return null for host names.
+     *  Will return null for hostnames.
+     *
+     *  PREFERRED for IPs in RouterAddresses, blocklists, etc. over getI2P()
+     *  because it avoids getByName() in most cases.
      *
      *  Unlike InetAddress.getByName(), we do NOT allow numeric IPs
      *  of the form d.d.d, d.d, or d, as these are almost certainly mistakes.
      *
+     *  InetAddress.getByName() also returns 127.0.0.1 for a host "",
+     *  but this is undocumented; as of 0.9.50, here we return null.
+     *
      *  @param host literal IPv4 or IPv6 address; if null returns null
      *  @return IP or null
      *  @since 0.9.32
      */
     public static byte[] getIPOnly(String host) {
-        if (host == null)
+        if (host == null || host.isEmpty())
             return null;
         byte[] rv;
         synchronized (_IPAddress) {
@@ -457,13 +625,21 @@ public abstract class Addresses {
         if (rv == null) {
             if (isIPAddress(host)) {
                 try {
-                    rv = InetAddress.getByName(host).getAddress();
+                    if (host.indexOf('.') > 0) {
+                        rv = getIPv4(host);
+                        if (rv == null)
+                            return null;
+                    } else if (host.indexOf(':') >= 0 && !host.contains("::")) {
+                        rv = getIPv6(host);
+                        if (rv == null)
+                            return null;
+                    } else {
+                        rv = InetAddress.getByName(host).getAddress();
+                    }
                     synchronized (_IPAddress) {
                         _IPAddress.put(host, rv);
                     }
                 } catch (UnknownHostException uhe) {}
-            //} else {
-            //    I2PAppContext.getGlobalContext().logManager().getLog(Addresses.class).warn("Not looking up " + host, new Exception("I did it"));
             }
         }
         return rv;
@@ -471,16 +647,19 @@ public abstract class Addresses {
 
     /**
      *  For literal IP addresses, this is the same as getIP(String).
-     *  For host names, will return the preferred type (IPv4/v6) if available,
+     *  For hostnames, will return the preferred type (IPv4/v6) if available,
      *  else the other type if available.
-     *  Will resolve but not cache DNS host names.
+     *  Will resolve but not cache DNS hostnames.
      *
-     *  @param host DNS or IPv4 or IPv6 host name; if null returns null
+     *  InetAddress.getByName() also returns 127.0.0.1 for a host "",
+     *  but this is undocumented; as of 0.9.50, here we return null.
+     *
+     *  @param host DNS or IPv4 or IPv6 address; if null returns null
      *  @return IP or null
      *  @since 0.9.28
      */
     public static byte[] getIP(String host, boolean preferIPv6) {
-        if (host == null)
+        if (host == null || host.isEmpty())
             return null;
         if (isIPAddress(host))
             return getIP(host);
@@ -517,9 +696,9 @@ public abstract class Addresses {
 
     /**
      *  For literal IP addresses, this is the same as getIP(String).
-     *  For host names, may return multiple addresses, both IPv4 and IPv6,
+     *  For hostnames, may return multiple addresses, both IPv4 and IPv6,
      *  even if those addresses are not reachable due to configuration or available interfaces.
-     *  Will resolve but not cache DNS host names.
+     *  Will resolve but not cache DNS hostnames.
      *
      *  Note that order of returned results, and whether
      *  multiple results for either IPv4 or IPv6 or both are actually
@@ -528,12 +707,15 @@ public abstract class Addresses {
      *  Number of results may also change based on caching at various layers,
      *  even if the ultimate name server results did not change.
      *
-     *  @param host DNS or IPv4 or IPv6 host name; if null returns null
+     *  InetAddress.getByName() also returns 127.0.0.1 for a host "",
+     *  but this is undocumented; as of 0.9.50, here we return null.
+     *
+     *  @param host DNS or IPv4 or IPv6 address; if null returns null
      *  @return non-empty list IPs, or null if none
      *  @since 0.9.28
      */
     public static List<byte[]> getIPs(String host) {
-        if (host == null)
+        if (host == null || host.isEmpty())
             return null;
         if (isIPAddress(host)) {
             byte[] brv = getIP(host);
@@ -588,8 +770,121 @@ public abstract class Addresses {
         return InetAddressUtils.isIPv4Address(host) || InetAddressUtils.isIPv6Address(host);
     }
 
+/*
+    private static byte[] oldGetIPv4(String host) {
+        String[] s = DataHelper.split(host, "\\.", 4);
+        if (s.length != 4)
+            return null;
+        byte[] rv = new byte[4];
+        try {
+            for (int i = 0; i < 4; i++) {
+                int b = Integer.parseInt(s[i]);
+                if (b < 0 || b > 255)
+                    return null;
+                rv[i] = (byte) b;
+            }
+        } catch (NumberFormatException nfe) {
+            return null;
+        }
+        return rv;
+    }
 
+    private static byte[] oldGetIPv6(String host) {
+        String[] s = DataHelper.split(host, ":", 8);
+        if (s.length != 8)
+            return null;
+        byte[] rv = new byte[16];
+        try {
+            int j = 0;
+            for (int i = 0; i < 8; i++) {
+                int b = Integer.parseInt(s[i], 16);
+                if (b < 0 || b > 65535)
+                    return null;
+                rv[j++] = (byte) (b >> 8);
+                rv[j++] = (byte) b;
+            }
+        } catch (NumberFormatException nfe) {
+            return null;
+        }
+        return rv;
+    }
+*/
 
+    /**
+     *  Because InetAddress.getByName() is slow, esp. on Windows.
+     *  Also avoids split(), Integer.parseInt(), and object churn.
+     *
+     *  @param host w.x.y.z only
+     *  @return 4 bytes or null
+     *  @since 0.9.50
+     */
+    private static byte[] getIPv4(String host) {
+        byte[] rv = new byte[4];
+        int b = 0;
+        int dots = 0;
+        int len = host.length();
+        for (int i = 0; i < len; i++) {
+            char c = host.charAt(i);
+            if (c == '.') {
+               if (i == 0 || i == len - 1 || dots == 3 || b > 255 || host.charAt(i - 1) == '.')
+                   return null;
+               rv[dots++] = (byte) b;
+               b = 0;
+            } else if (c >= '0' && c <= '9') {
+               b *= 10;
+               b += c - '0';
+            } else {
+               return null;
+            }
+        }
+        if (dots != 3 || b > 255)
+            return null;
+        rv[3] = (byte) b;
+        return rv;
+    }
+
+    /**
+     *  Because InetAddress.getByName() is slow, esp. on Windows.
+     *  Also avoids split(), Integer.parseInt(), and object churn.
+     *
+     *  @param host full 0:1:2:3:4:5:6:7 only, no ::
+     *  @return 16 bytes or null
+     *  @since 0.9.50
+     */
+    private static byte[] getIPv6(String host) {
+        byte[] rv = new byte[16];
+        int b = 0;
+        int j = 0;
+        int colons = 0;
+        int len = host.length();
+        for (int i = 0; i < len; i++) {
+            char c = host.charAt(i);
+            if (c == ':') {
+               if (i == 0 || i == len - 1 || colons == 7 || b > 65535 || host.charAt(i - 1) == ':')
+                   return null;
+               rv[j++] = (byte) (b >> 8);
+               rv[j++] = (byte) b;
+               colons++;
+               b = 0;
+            } else if (c >= '0' && c <= '9') {
+               b <<= 4;
+               b |= c - '0';
+            } else if (c >= 'a' && c <= 'f') {
+               b <<= 4;
+               b |= 10 + c - 'a';
+            } else if (c >= 'A' && c <= 'F') {
+               b <<= 4;
+               b |= 10 + c - 'A';
+            } else {
+               return null;
+            }
+        }
+        if (colons != 7 || b > 65535)
+            return null;
+        rv[14] = (byte) (b >> 8);
+        rv[15] = (byte) b;
+        return rv;
+    }
 
     //////// IPv6 Cache Utils ///////
 
@@ -640,7 +935,7 @@ public abstract class Addresses {
                 buf.setLength(0);
                 int i = 0;
                 while(true) {
-                    buf.append(as.substring(i, i+4));
+                    buf.append(as, i, i+4);
                     i += 4;
                     if (i >= 32)
                         break;
@@ -775,29 +1070,36 @@ public abstract class Addresses {
      *  Print out the local addresses
      */
     public static void main(String[] args) {
+        //test(); if (true) return;
+        System.out.println("Connected Address Types: " + getConnectedAddressTypes() + '\n');
         System.out.println("External IPv4 Addresses:");
         Set<String> a = getAddresses(false, false, false);
         print(a);
         System.out.println("\nExternal and Local IPv4 Addresses:");
         a = getAddresses(true, false, false);
         print(a);
-        System.out.println("\nAll External Addresses:");
+        System.out.println("\nAll External Addresses (except deprecated IPv6):");
         a = getAddresses(false, false, true);
-        print(a);
-        System.out.println("\nAll External and Local Addresses:");
+        printCanonical(a);
+        byte[] ygg = getYggdrasilAddress();
+        if (ygg != null) {
+            System.out.println("\nYggdrasil Address:");
+            System.out.println(toCanonicalString(ygg));
+        }
+        System.out.println("\nAll External and Local Addresses (may include deprecated IPv6):");
         a = getAddresses(true, false, true);
-        print(a);
+        printCanonical(a);
         System.out.println("\nAll addresses:");
         long time = System.currentTimeMillis();
         a = getAddresses(true, true, true);
         time = System.currentTimeMillis() - time;
-        print(a);
+        printCanonical(a);
         System.out.println("\nIPv6 address flags:");
+        StringBuilder buf = new StringBuilder(64);
         for (String s : a) {
             if (!s.contains(":"))
                 continue;
-            StringBuilder buf = new StringBuilder(64);
-            buf.append(s);
+            buf.append(toCanonicalString(s));
             Inet6Address addr;
             try {
                 addr = (Inet6Address) InetAddress.getByName(buf.toString());
@@ -815,37 +1117,119 @@ public abstract class Addresses {
                         buf.append(" temporary");
                     if (isDynamic(addr))
                         buf.append(" dynamic");
+                    else
+                        buf.append(" permanent");
                     if (isDeprecated(addr))
                         buf.append(" deprecated");
                 }
             } catch (UnknownHostException uhe) {}
             System.out.println(buf.toString());
+            buf.setLength(0);
         }
 
         System.out.println("\nMac addresses:");
         Set<String> macs = new TreeSet<String>();
-        StringBuilder buf = new StringBuilder(17);
         for (String m : _macCache) {
-            buf.setLength(0);
             int i = 0;
             while(true) {
-                buf.append(m.substring(i, i+2));
+                buf.append(m, i, i+2);
                 i += 2;
                 if (i >= 12)
                     break;
                 buf.append(':');
             }
             macs.add(buf.toString());
+            buf.setLength(0);
         }
         print(macs);
-        System.out.println("\nIs connected? " + isConnected() +
-                           "\nIs conn IPv6? " + isConnectedIPv6());
+        System.out.println("\nHas IPv4?     " + isConnected() +
+                           "\nHas IPv6?     " + isConnectedIPv6() +
+                           "\nHas Ygg?      " + (ygg != null));
         System.out.println("Has v6 flags? " + INET6_CACHE_ENABLED);
         System.out.println("Uses v6 temp? " + getPrivacyStatus());
         // Windows 8.1 Java 1.8.0_66 netbook appx. 200ms + 50ms/interface
         System.out.println("scan time:    " + DataHelper.formatDuration(time));
     }
 
+/*
+    // test results (linux):
+    // new is about 10x faster than old
+    // InetAddress about the same as old for IPv4, about 4x slower for IPv6
+    private static void test() {
+        String[] tt = { "1.2.3.4", "0.0.0.0", "255.255.255.255", "", "a", "1", "1.2", "1.2.3",
+                        ".1.2.3", "1.2.3.", "266.1.2.3", "1.266.2.3", "1.2.3.266", "1.2.3.4.5" };
+        for (String t : tt) {
+            byte[] b = getIPv4(t);
+            System.out.println(t + " -> " + toString(b));
+        }
+        tt = new String[] { "a:B:c:D:e:f:1:2", "aaaa:bbbb:CCC:dd:e:f:111:2222", "a", "1", "1:2", "1::2:3:4:5:6:7:8",
+                            ":1:2:3:4:5:6:7", "2:3:4:5:6:7:8:", "x:2:3:4:5:6:7:8", "::1", "::", "",
+                            "99999:2:3:4:5:6:7:8" };
+        for (String t : tt) {
+            byte[] b = getIPv6(t);
+            System.out.println(t + " -> " + toString(b));
+        }
+        int runs = 1000;
+        for (int i = 0; i < runs; i++) {
+            try {
+                InetAddress.getByName("192.168.142.117").getAddress();
+                InetAddress.getByName("aaaa:bbbb:cccc:dddd:eeee:ffff:1111:2222").getAddress();
+            } catch (Exception e) {}
+            oldGetIPv4("192.168.142.117");
+            oldGetIPv6("aaaa:bbbb:cccc:dddd:eeee:ffff:1111:2222");
+            getIPv4("192.168.142.117");
+            getIPv6("aaaa:bbbb:cccc:dddd:eeee:ffff:1111:2222");
+        }
+        runs = 10*1000*1000;
+        long start = System.currentTimeMillis();
+        for (int i = 0; i < runs; i++) {
+            oldGetIPv4("192.168.142.117");
+        }
+        long end = System.currentTimeMillis();
+        System.out.println("old ipv4 took " + (end - start));
+        start = end;
+
+        for (int i = 0; i < runs; i++) {
+            getIPv4("192.168.142.117");
+        }
+        end = System.currentTimeMillis();
+        System.out.println("new ipv4 took " + (end - start));
+        start = end;
+
+        for (int i = 0; i < runs; i++) {
+            try {
+                InetAddress.getByName("192.168.142.117").getAddress();
+                InetAddress.getByName("aaaa:bbbb:cccc:dddd:eeee:ffff:1111:2222").getAddress();
+            } catch (Exception e) {}
+        }
+        end = System.currentTimeMillis();
+        System.out.println("INA ipv4 took " + (end - start));
+        start = end;
+
+        for (int i = 0; i < runs; i++) {
+            oldGetIPv6("192.168.142.117");
+        }
+        end = System.currentTimeMillis();
+        System.out.println("old ipv6 took " + (end - start));
+        start = end;
+
+        for (int i = 0; i < runs; i++) {
+            getIPv6("192.168.142.117");
+        }
+        end = System.currentTimeMillis();
+        System.out.println("new ipv6 took " + (end - start));
+
+        for (int i = 0; i < runs; i++) {
+            try {
+                InetAddress.getByName("aaaa:bbbb:cccc:dddd:eeee:ffff:1111:2222").getAddress();
+            } catch (Exception e) {}
+        }
+        end = System.currentTimeMillis();
+        System.out.println("INA ipv6 took " + (end - start));
+        start = end;
+    }
+*/
+
     /** @since 0.9.34 */
     private static void print(Set<String> a) {
         if (a.isEmpty()) {
@@ -857,18 +1241,27 @@ public abstract class Addresses {
         }
     }
 
+    /** @since 0.9.57 */
+    private static void printCanonical(Set<String> a) {
+        if (a.isEmpty()) {
+            System.out.println("none");
+        } else {
+            for (String s : a) {
+                System.out.println(toCanonicalString(s));
+            }
+        }
+    }
+
     /**
-     * RFC 4941
-     * @since 0.9.34
+     * @return "true", "false", or "unknown"
+     * @since 0.9.50
      */
-    private static String getPrivacyStatus() {
+    public static String useIPv6TempAddresses() {
         // Windows: netsh interface ipv6 show privacy
         // Mac: sysctl net.inet6.ip6.use_tempaddr (1 is enabled)
         if (SystemVersion.isMac() || SystemVersion.isWindows())
             return "unknown";
         long t = getLong("/proc/sys/net/ipv6/conf/all/use_tempaddr");
-        if (t < 0)
-            return "unknown";
         String rv;
         if (t == 0)
             rv = "false";
@@ -876,7 +1269,16 @@ public abstract class Addresses {
             rv = "true";
         else
             rv = "unknown";
-        if (t == 2) {
+        return rv;
+    }
+
+    /**
+     * RFC 4941
+     * @since 0.9.34
+     */
+    private static String getPrivacyStatus() {
+        String rv = useIPv6TempAddresses();
+        if (Boolean.parseBoolean(rv)) {
             long pref = getLong("/proc/sys/net/ipv6/conf/all/temp_prefered_lft");
             if (pref > 0)
                 rv += ", preferred lifetime " + DataHelper.formatDuration(pref * 1000);
