@@ -83,9 +83,9 @@ public class EepGet {
     protected boolean _notModified;
     protected String _contentType;
     protected boolean _transferFailed;
-    protected boolean _aborted;
+    protected volatile boolean _aborted;
     protected int _fetchHeaderTimeout;
-    private long _fetchEndTime;
+    protected int _fetchTotalTimeout;
     protected int _fetchInactivityTimeout;
     protected int _redirects;
     protected String _redirectLocation;
@@ -476,6 +476,7 @@ public class EepGet {
 
         public void attempting(String url);
     }
+
     protected class CLIStatusListener implements StatusListener {
         private final int _markSize;
         private final int _lineSize;
@@ -657,8 +658,11 @@ public class EepGet {
      * @return success
      */
     public boolean fetch(long fetchHeaderTimeout, long totalTimeout, long inactivityTimeout) {
+        // we need a SocketTimeout if we have a totalTimeout
+        if (totalTimeout > 0 && fetchHeaderTimeout <= 0)
+            fetchHeaderTimeout = totalTimeout;
         _fetchHeaderTimeout = (int) Math.min(fetchHeaderTimeout, Integer.MAX_VALUE);
-        _fetchEndTime = (totalTimeout > 0 ? System.currentTimeMillis() + totalTimeout : -1);
+        _fetchTotalTimeout = (int) Math.min(totalTimeout, Integer.MAX_VALUE);
         _fetchInactivityTimeout = (int) Math.min(inactivityTimeout, Integer.MAX_VALUE);
         _keepFetching = true;
 
@@ -667,16 +671,21 @@ public class EepGet {
         while (_keepFetching) {
             SocketTimeout timeout = null;
             if (_fetchHeaderTimeout > 0) {
+                // We create the SocketTimeout with an inactivity time of the header timeout.
+                // After fetchHeaders(), we must call either resetTimer() or cancel()
                 timeout = new SocketTimeout(_fetchHeaderTimeout);
-                final SocketTimeout stimeout = timeout; // ugly - why not use sotimeout?
+                final SocketTimeout stimeout = timeout;
+                final Thread thread = Thread.currentThread();
                 timeout.setTimeoutCommand(new Runnable() {
                     public void run() {
                         if (_log.shouldLog(Log.DEBUG))
                             _log.debug("timeout reached on " + _url + ": " + stimeout);
                         _aborted = true;
+                        thread.interrupt();
                     }
                 });
-                timeout.setTotalTimeoutPeriod(_fetchEndTime);
+                if (_fetchTotalTimeout > 0)
+                    timeout.setTotalTimeoutPeriod(_fetchTotalTimeout);
             }
             try {
                 for (int i = 0; i < _listeners.size(); i++) 
@@ -685,14 +694,10 @@ public class EepGet {
                 if (timeout != null)
                     timeout.resetTimer();
                 doFetch(timeout);
-                if (timeout != null)
-                    timeout.cancel();
                 if (!_transferFailed)
                     return true;
                 break;
             } catch (IOException ioe) {
-                if (timeout != null)
-                    timeout.cancel();
                 for (int i = 0; i < _listeners.size(); i++) 
                     _listeners.get(i).attemptFailed(_url, _bytesTransferred, _bytesRemaining, _currentAttempt, _numRetries, ioe);
                 if (_log.shouldLog(Log.WARN))
@@ -702,6 +707,8 @@ public class EepGet {
                     ioe instanceof ConnectException) // proxy or nonproxied host Connection Refused
                     _keepFetching = false;
             } finally {
+                if (timeout != null)
+                    timeout.cancel();
                 if (_out != null) {
                     try {
                         _out.close();
@@ -736,7 +743,8 @@ public class EepGet {
     }
 
     /**
-     *  single fetch
+     *  This reads the response to a single fetch.
+     *  Call after sendRequest()
      *  @param timeout may be null
      */
     protected void doFetch(SocketTimeout timeout) throws IOException {
@@ -745,20 +753,28 @@ public class EepGet {
         if (_aborted)
             throw new IOException("Timed out reading the HTTP headers");
         
-        if (timeout != null) {
-            timeout.resetTimer();
-            if (_fetchInactivityTimeout > 0)
-                timeout.setInactivityTimeout(_fetchInactivityTimeout);
-            else
-                timeout.setInactivityTimeout(INACTIVITY_TIMEOUT);
-        }
         // _proxy is null when extended by I2PSocketEepGet
-        if (_proxy != null && !_shouldProxy) {
+        if (_proxy != null) {
+            if (timeout != null) {
+                if (_fetchTotalTimeout > 0) {
+                    timeout.resetTimer();
+                } else {
+                    // we don't need the timeout any more, we'll use soTimeout
+                    timeout.cancel();
+                    timeout = null;
+                }
+            }
             // we only set the soTimeout before the headers if not proxied
             if (_fetchInactivityTimeout > 0)
                 _proxy.setSoTimeout(_fetchInactivityTimeout);
             else
                 _proxy.setSoTimeout(INACTIVITY_TIMEOUT);
+        } else if (timeout != null) {
+            timeout.resetTimer();
+            if (_fetchInactivityTimeout > 0)
+                timeout.setInactivityTimeout(_fetchInactivityTimeout);
+            else
+                timeout.setInactivityTimeout(INACTIVITY_TIMEOUT);
         }
         
         if (_redirectLocation != null) {
@@ -1238,7 +1254,17 @@ public class EepGet {
                 throw new IOException("Header line too long: " + buf.toString());
         }
     }
-    
+
+    /**
+     *  Should we read the body of the response?
+     *  @return true always, overridden in EepHead
+     *  @since 0.9.50
+     */
+    protected boolean shouldReadBody() { return true; }
+
+    /**
+     *  TODO this does not skip over chunk extensions (RFC 2616 sec. 3.6.1)
+     */
     protected long readChunkLength() throws IOException {
         StringBuilder buf = new StringBuilder(8);
         int nl = 0;
@@ -1322,7 +1348,8 @@ public class EepGet {
         } else if (key.equals("last-modified")) {
             _lastModified = val;
         } else if (key.equals("transfer-encoding")) {
-            _encodingChunked = val.toLowerCase(Locale.US).contains("chunked");
+            // don't read chunk header in readHeaders() for EepHead
+            _encodingChunked = shouldReadBody() && val.toLowerCase(Locale.US).contains("chunked");
         } else if (key.equals("content-encoding")) {
             // This is kindof a hack, but if we are downloading a gzip file
             // we don't want to transparently gunzip it and save it as a .gz file.
@@ -1382,7 +1409,7 @@ public class EepGet {
                 if ("http".equals(url.getScheme())) {
                     String host = url.getHost();
                     if (host == null)
-                        throw new MalformedURLException("URL is not supported:" + _actualURL);
+                        throw new MalformedURLException("URL is not supported: " + _actualURL);
                     String hostlc = host.toLowerCase(Locale.US);
                     if (hostlc.endsWith(".i2p"))
                         throw new UnknownHostException("I2P addresses must be proxied");
@@ -1399,7 +1426,7 @@ public class EepGet {
                         _proxy = new Socket(host, port);
                     }
                 } else {
-                    throw new MalformedURLException("URL is not supported:" + _actualURL);
+                    throw new MalformedURLException("URL is not supported: " + _actualURL);
                 }
             } catch (URISyntaxException use) {
                 IOException ioe = new MalformedURLException("Request URL is invalid");
@@ -1758,20 +1785,38 @@ public class EepGet {
                     // use standard alphabet
                     authMode = AUTH_MODE.BASIC;
                     authChallenge = auth.substring(6);
+                    args = parseAuthArgs(authChallenge);
                 }
             } else if (authLC.startsWith("digest ")) {
-                // better than anything
-                authMode = AUTH_MODE.DIGEST;
-                authChallenge = auth.substring(7);
+                // RFC 7616 take the first one that we support
+                if (authMode != AUTH_MODE.DIGEST) {
+                    String ac = auth.substring(7);
+                    Map<String, String> aargs = parseAuthArgs(ac);
+                    String algo = aargs.get("algorithm");
+                    if (algo != null) {
+                        algo = algo.toLowerCase(Locale.US);
+                        if (!algo.equals("sha-256") && !algo.equals("md5")) {
+                            if (_log.shouldWarn())
+                                _log.warn("Unsupported auth algorithm " + algo);
+                            return;
+                        }
+                    }
+                    authMode = AUTH_MODE.DIGEST;
+                    authChallenge = ac;
+                    args = aargs;
+                } else {
+                    if (_log.shouldDebug())
+                        _log.debug("Ignoring auth algorithm " + auth);
+                }
             } else {
                 // better than NONE only
                 if (authMode == AUTH_MODE.NONE) {
                     authMode = AUTH_MODE.UNKNOWN;
                     authChallenge = null;
+                    args = null;
                 }
             }
             nonceCount = 0;
-            args = null;
         }
 
         public String getAuthHeader(String method, String uri) throws IOException {
@@ -1785,8 +1830,6 @@ public class EepGet {
                 case DIGEST:
                     if (authChallenge == null)
                         throw new IOException("Bad proxy auth response");
-                    if (args == null)
-                        args = parseAuthArgs(authChallenge);
                     Map<String, String> outArgs = generateAuthArgs(method, uri);
                     if (outArgs == null)
                         throw new IOException("Bad proxy auth response");
@@ -1815,7 +1858,14 @@ public class EepGet {
             String nonce = args.get("nonce");
             String qop = args.get("qop");
             String opaque = args.get("opaque");
-            //String algorithm = args.get("algorithm");
+            // RFC 7616
+            String algorithm = args.get("algorithm");
+            boolean isSHA256 = false;
+            if (algorithm == null)
+                algorithm = "MD5";
+            else
+                isSHA256 = algorithm.toLowerCase(Locale.US).equals("sha-256");
+            rv.put("algorithm", algorithm);
             //String stale = args.get("stale");
             if (realm == null || nonce == null) {
                 if (_log.shouldLog(Log.INFO))
@@ -1845,13 +1895,14 @@ public class EepGet {
             }
 
             // get H(A1)
-            String ha1 = PasswordManager.md5Hex(username + ':' + realm + ':' + password);
+            String a1 = username + ':' + realm + ':' + password;
+            String ha1 = isSHA256 ? PasswordManager.sha256Hex(a1) : PasswordManager.md5Hex(a1);
             // get H(A2)
             String a2 = method + ':' + uri;
-            String ha2 = PasswordManager.md5Hex(a2);
+            String ha2 = isSHA256 ? PasswordManager.sha256Hex(a2) : PasswordManager.md5Hex(a2);
             // response
             String kd = ha1 + ':' + nonce + kdMiddle + ':' + ha2;
-            rv.put("response", '"' + PasswordManager.md5Hex(kd) + '"');
+            rv.put("response", '"' + (isSHA256 ? PasswordManager.sha256Hex(kd) : PasswordManager.md5Hex(kd)) + '"');
             return rv;
         }
 
