@@ -103,7 +103,7 @@ public class SSLEepGet extends EepGet {
 
     private static final String CERT_DIR = "certificates/ssl";
     private static final String PROP_USE_DNS_OVER_HTTPS = "eepget.useDNSOverHTTPS";
-    private static final boolean DEFAULT_USE_DNS_OVER_HTTPS = false;
+    private static final boolean DEFAULT_USE_DNS_OVER_HTTPS = true;
 
     /**
      *  Not all may be supported.
@@ -586,12 +586,16 @@ public class SSLEepGet extends EepGet {
         if (_aborted)
             throw new IOException("Timed out reading the HTTP headers");
         
+        // _proxy is the socket, even if not proxied.
+        // We never use the timeout inactivity timer; only socket SoTimeout.
         if (timeout != null) {
-            timeout.resetTimer();
-            if (_fetchInactivityTimeout > 0)
-                timeout.setInactivityTimeout(_fetchInactivityTimeout);
-            else
-                timeout.setInactivityTimeout(60*1000);
+            if (_fetchTotalTimeout > 0) {
+                timeout.resetTimer();
+            } else {
+                // we don't need the timeout any more, we'll use soTimeout
+                timeout.cancel();
+                timeout = null;
+            }
         }        
         if (_fetchInactivityTimeout > 0)
             _proxy.setSoTimeout(_fetchInactivityTimeout);
@@ -762,19 +766,26 @@ public class SSLEepGet extends EepGet {
 
                 String originalHost = host;
                 boolean useDNSOverHTTPS;
-                if (_forceDoH == 2)
-                    useDNSOverHTTPS = true;
-                else if (_forceDoH == 1)
+                if (_forceDoH == 1 || _shouldProxy)
                     useDNSOverHTTPS = false;
+                else if (_forceDoH == 2)
+                    useDNSOverHTTPS = true;
                 else
                     useDNSOverHTTPS = _context.getProperty(PROP_USE_DNS_OVER_HTTPS, DEFAULT_USE_DNS_OVER_HTTPS);
                 // This duplicates checks in DNSOverHTTPS.lookup() but do it here too so
                 // we don't even construct it if we don't need it
+                String ip = null;
                 if (useDNSOverHTTPS && !host.equals("dns.google") && !Addresses.isIPAddress(host)) {
                     DNSOverHTTPS doh = new DNSOverHTTPS(_context, getSSLState());
-                    String ip = doh.lookup(host);
-                    if (ip != null)
-                        host = ip;
+                    ip = doh.lookup(host);
+                    if (ip != null) {
+                        // will be used below
+                        if (_log.shouldDebug())
+                            _log.debug("DoH success: " + host + ' ' + ip);
+                    } else {
+                        if (_log.shouldWarn())
+                            _log.debug("DoH fail: " + host);
+                    }
                 }
 
                 if (_shouldProxy) {
@@ -810,6 +821,20 @@ public class SSLEepGet extends EepGet {
                         _proxy = ((SSLSocketFactory) SSLSocketFactory.getDefault()).createSocket(_proxy, host, port, true);
                     if (_log.shouldLog(Log.DEBUG))
                         _log.debug(_proxyType + " proxy headers read completely");
+                } else if (ip != null) {
+                    // DoH, create the socket with the IP, then create the SSL socket with the host
+                    // So that SNI and cert validation works
+                    if (_fetchHeaderTimeout > 0) {
+                        _proxy = new Socket();
+                        _proxy.setSoTimeout(_fetchHeaderTimeout);
+                        _proxy.connect(new InetSocketAddress(ip, port), _fetchHeaderTimeout);
+                     } else {
+                        _proxy = new Socket(ip, port);
+                     }
+                    if (_sslContext != null)
+                        _proxy = _sslContext.getSocketFactory().createSocket(_proxy, host, port, true);
+                    else
+                        _proxy = ((SSLSocketFactory) SSLSocketFactory.getDefault()).createSocket(_proxy, host, port, true);
                 } else {
                     // Warning, createSocket() followed by connect(InetSocketAddress)
                     // disables SNI, at least on Java 7.
@@ -823,6 +848,8 @@ public class SSLEepGet extends EepGet {
                         _proxy.setSoTimeout(_fetchHeaderTimeout);
                     }
                 }
+                if (timeout != null)
+                    timeout.setSocket(_proxy);
 
                 SSLSocket socket = (SSLSocket) _proxy;
                 I2PSSLSocketFactory.setProtocolsAndCiphers(socket);
@@ -930,7 +957,8 @@ public class SSLEepGet extends EepGet {
         _proxyIn = _proxy.getInputStream();
         _proxyOut = _proxy.getOutputStream();
         StringBuilder buf = new StringBuilder(64);
-        buf.append("CONNECT ").append(host).append(':').append(port).append(" HTTP/1.1\r\n");
+        buf.append("CONNECT ").append(host).append(':').append(port).append(" HTTP/1.1\r\n" +
+                   "Host: ").append(host).append(':').append(port).append("\r\n");
         // TODO if we need extra headers to the proxy, add a new method and list.
         // Standard extra headers go the server, not the proxy
         //if (_extraPHeaders != null) {
