@@ -93,6 +93,9 @@ public class HandshakeState implements Destroyable, Cloneable {
 	 */
 	public static final int COMPLETE = 5;
 	
+	/** I2P for debugging */
+	private static final String STATE_NAMES[] = { "NO_ACTION", "WRITE_MESSAGE", "READ_MESSAGE", "FAILED", "SPLIT", "COMPLETE" };
+
 	/**
 	 * Local static keypair is required for the handshake.
 	 */
@@ -128,14 +131,26 @@ public class HandshakeState implements Destroyable, Cloneable {
 	 */
 	private static final int FALLBACK_POSSIBLE = 0x40;
 
+	/** NTCP2 */
 	public static final String protocolName = "Noise_XKaesobfse+hs2+hs3_25519_ChaChaPoly_SHA256";
+	/** Ratchet */
 	public static final String protocolName2 = "Noise_IKelg2+hs2_25519_ChaChaPoly_SHA256";
+	/** Tunnels */
 	public static final String protocolName3 = "Noise_N_25519_ChaChaPoly_SHA256";
+	/** SSU2 */
+        public static final String protocolName4 = "Noise_XKchaobfse+hs1+hs2+hs3_25519_ChaChaPoly_SHA256";
 	private static final String prefix;
 	private final String patternId;
+	/** NTCP2 */
 	public static final String PATTERN_ID_XK = "XK";
+	/** Ratchet */
 	public static final String PATTERN_ID_IK = "IK";
+	/** Tunnels */
 	public static final String PATTERN_ID_N = "N";
+	/** same as N but no post-mixHash needed */
+	public static final String PATTERN_ID_N_NO_RESPONSE = "N!";
+	/** SSU2 */
+	public static final String PATTERN_ID_XK_SSU2 = "XK-SSU2";
 	private static String dh;
 	private static final String cipher;
 	private static final String hash;
@@ -180,6 +195,11 @@ public class HandshakeState implements Destroyable, Cloneable {
 		PATTERN_N = Pattern.lookup(id);
 		if (PATTERN_N == null)
 			throw new IllegalArgumentException("Handshake pattern is not recognized");
+		// XK-SSU2
+		components = protocolName4.split("_");
+		id = components[1].substring(0, 2);
+		if (!PATTERN_ID_XK.equals(id))
+			throw new IllegalArgumentException();
 	}
 
 	/**
@@ -205,6 +225,10 @@ public class HandshakeState implements Destroyable, Cloneable {
 			pattern = PATTERN_IK;
 		else if (patternId.equals(PATTERN_ID_N))
 			pattern = PATTERN_N;
+		else if (patternId.equals(PATTERN_ID_N_NO_RESPONSE))  // same as N but no post-mixHash needed
+			pattern = PATTERN_N;
+		else if (patternId.equals(PATTERN_ID_XK_SSU2))
+			pattern = PATTERN_XK;
 		else
 			throw new IllegalArgumentException("Handshake pattern is not recognized");
 		short flags = pattern[0];
@@ -506,6 +530,18 @@ public class HandshakeState implements Destroyable, Cloneable {
 	/**
 	 * Writes a message payload during the handshake.
 	 * 
+	 * Payload (plaintext) and message (encrypted) may be in the same buffer if the payload
+	 * if offset enough past the message offset to leave room for the
+	 * key(s) and/or MAC. For 32 byte keys and 16 byte MACs,
+	 * if message == payload, payloadOffset must be at least this much
+	 * greater than messageOffset:
+	 *
+	 * XK: Message 1: 32; message 2: 32; message 3: 48
+	 *
+	 * IK: Message 1: 80; message 2: 48
+	 *
+	 * N: Message 1: 32
+	 *
 	 * @param message The buffer that will be populated with the
 	 * handshake packet to be written to the transport.
 	 * @param messageOffset First offset within the message buffer
@@ -537,7 +573,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 		// Validate the parameters and state.
 		if (action != WRITE_MESSAGE) {
 			throw new IllegalStateException
-				("Handshake state does not allow writing messages");
+				("Handshake state " + STATE_NAMES[action] + " does not allow writing messages");
 		}
 		if (payload == null && (payloadOffset != 0 || payloadLength != 0)) {
 			throw new IllegalArgumentException("Invalid payload argument");
@@ -648,10 +684,19 @@ public class HandshakeState implements Destroyable, Cloneable {
 			}
 			
 			// Add the payload to the message buffer and encrypt it.
-			if (payload != null)
-				messagePosn += symmetric.encryptAndHash(payload, payloadOffset, message, messagePosn, payloadLength);
-			else
-				messagePosn += symmetric.encryptAndHash(message, messagePosn, message, messagePosn, 0);
+			if (payload != null) {
+				// no need to hash for N, we don't split() and no more messages follow
+				if (patternId.equals(PATTERN_ID_N_NO_RESPONSE))
+					messagePosn += symmetric.encryptOnly(payload, payloadOffset, message, messagePosn, payloadLength);
+				else
+					messagePosn += symmetric.encryptAndHash(payload, payloadOffset, message, messagePosn, payloadLength);
+			} else {
+				// no need to hash for N, we don't split() and no more messages follow
+				if (patternId.equals(PATTERN_ID_N_NO_RESPONSE))
+					messagePosn += symmetric.encryptOnly(message, messagePosn, message, messagePosn, 0);
+				else
+					messagePosn += symmetric.encryptAndHash(message, messagePosn, message, messagePosn, 0);
+			}
 			success = true;
 		} finally {
 			// If we failed, then clear any sensitive data that may have
@@ -666,7 +711,10 @@ public class HandshakeState implements Destroyable, Cloneable {
 
 	/**
 	 * Reads a message payload during the handshake.
-	 * 
+	 *
+	 * Payload (plaintext) and message (encrypted) may be in the same buffer
+	 * and have the same offset.
+	 *
 	 * @param message Buffer containing the incoming handshake
 	 * that was read from the transport.
 	 * @param messageOffset Offset of the first message byte.
@@ -697,7 +745,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 		// Validate the parameters.
 		if (action != READ_MESSAGE) {
 			throw new IllegalStateException
-				("Handshake state does not allow reading messages");
+				("Handshake state " + STATE_NAMES[action] + " does not allow reading messages");
 		}
 		if (messageOffset > message.length || payloadOffset > payload.length) {
 			throw new ShortBufferException();
@@ -819,7 +867,12 @@ public class HandshakeState implements Destroyable, Cloneable {
 			}
 			
 			// Decrypt the message payload.
-			int payloadLength = symmetric.decryptAndHash(message, messageOffset, payload, payloadOffset, messageEnd - messageOffset);
+			int payloadLength;
+			// no need to hash for N, we don't split() and no more messages follow
+			if (patternId.equals(PATTERN_ID_N_NO_RESPONSE))
+				payloadLength = symmetric.decryptOnly(message, messageOffset, payload, payloadOffset, messageEnd - messageOffset);
+			else
+				payloadLength = symmetric.decryptAndHash(message, messageOffset, payload, payloadOffset, messageEnd - messageOffset);
 			success = true;
 			return payloadLength;
 		} finally {
@@ -844,7 +897,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 	{
 		if (action != SPLIT) {
 			throw new IllegalStateException
-				("Handshake has not finished");
+				("Handshake has not finished, state: " + STATE_NAMES[action]);
 		}
 		CipherStatePair pair = symmetric.split();
 		if (!isInitiator)
@@ -871,7 +924,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 	{
 		if (action != SPLIT) {
 			throw new IllegalStateException
-				("Handshake has not finished");
+				("Handshake has not finished, state: " + STATE_NAMES[action]);
 		}
 		CipherStatePair pair = symmetric.split(secondaryKey, offset, length);
 		if (!isInitiator) {
@@ -894,7 +947,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 	{
 		if (action != SPLIT && action != COMPLETE) {
 			throw new IllegalStateException
-				("Handshake has not completed");
+				("Handshake has not finished, state: " + STATE_NAMES[action]);
 		}
 		return symmetric.getHandshakeHash();
 	}
@@ -980,6 +1033,7 @@ public class HandshakeState implements Destroyable, Cloneable {
 	public String toString() {
 		StringBuilder buf = new StringBuilder(256);
 		buf.append(patternId);
+		buf.append(' ').append(STATE_NAMES[action]);
 		buf.append(" Handshake State:\n");
 		buf.append(symmetric.toString());
 
