@@ -40,15 +40,18 @@ class SymmetricState implements Destroyable, Cloneable {
 	private static final byte[] INIT_CK_XK;
 	private static final byte[] INIT_CK_IK;
 	private static final byte[] INIT_CK_N;
+	private static final byte[] INIT_CK_XK_SSU2;
 	// precalculated hash of the hash of the Noise name = mixHash(nullPrologue)
 	private static final byte[] INIT_HASH_XK = new byte[32];
 	private static final byte[] INIT_HASH_IK = new byte[32];
 	private static final byte[] INIT_HASH_N = new byte[32];
+	private static final byte[] INIT_HASH_XK_SSU2 = new byte[32];
 
 	static {
 		INIT_CK_XK = initHash(HandshakeState.protocolName);
 		INIT_CK_IK = initHash(HandshakeState.protocolName2);
 		INIT_CK_N = initHash(HandshakeState.protocolName3);
+		INIT_CK_XK_SSU2 = initHash(HandshakeState.protocolName4);
 		try {
 			MessageDigest md = Noise.createHash("SHA256");
 			md.update(INIT_CK_XK, 0, 32);
@@ -57,6 +60,8 @@ class SymmetricState implements Destroyable, Cloneable {
 			md.digest(INIT_HASH_IK, 0, 32);
 			md.update(INIT_CK_N, 0, 32);
 			md.digest(INIT_HASH_N, 0, 32);
+			md.update(INIT_CK_XK_SSU2, 0, 32);
+			md.digest(INIT_HASH_XK_SSU2, 0, 32);
 		} catch (Exception e) {
 			throw new IllegalStateException(e);
 		}
@@ -121,9 +126,13 @@ class SymmetricState implements Destroyable, Cloneable {
 		} else if (patternId.equals(HandshakeState.PATTERN_ID_IK)) {
 			initCK = INIT_CK_IK;
 			initHash = INIT_HASH_IK;
-		} else if (patternId.equals(HandshakeState.PATTERN_ID_N)) {
+		} else if (patternId.equals(HandshakeState.PATTERN_ID_N) ||
+		           patternId.equals(HandshakeState.PATTERN_ID_N_NO_RESPONSE)) {
 			initCK = INIT_CK_N;
 			initHash = INIT_HASH_N;
+		} else if (patternId.equals(HandshakeState.PATTERN_ID_XK_SSU2)) {
+			initCK = INIT_CK_XK_SSU2;
+			initHash = INIT_HASH_XK_SSU2;
 		} else {
 			throw new IllegalArgumentException("Handshake pattern is not recognized");
 		}
@@ -273,6 +282,16 @@ class SymmetricState implements Destroyable, Cloneable {
 		return ciphertextLength;
 	}
 
+	/**
+	 * I2P - Same as encryptAndHash() but without the post-mixHash(), for N only.
+	 * @since 0.9.49
+	 */
+	public int encryptOnly(byte[] plaintext, int plaintextOffset, byte[] ciphertext, int ciphertextOffset, int length) throws ShortBufferException
+	{
+		int ciphertextLength = cipher.encryptWithAd(h, plaintext, plaintextOffset, ciphertext, ciphertextOffset, length);
+		return ciphertextLength;
+	}
+
 	/**
 	 * Decrypts a block of ciphertext and mixes it into the handshake hash.
 	 * 
@@ -302,6 +321,15 @@ class SymmetricState implements Destroyable, Cloneable {
 		return cipher.decryptWithAd(prev_h, ciphertext, ciphertextOffset, plaintext, plaintextOffset, length);
 	}
 
+	/**
+	 * I2P - Same as decryptAndHash() but without the post-mixHash(), for N only.
+	 * @since 0.9.49
+	 */
+	public int decryptOnly(byte[] ciphertext, int ciphertextOffset, byte[] plaintext, int plaintextOffset, int length) throws ShortBufferException, BadPaddingException
+	{
+		return cipher.decryptWithAd(h, ciphertext, ciphertextOffset, plaintext, plaintextOffset, length);
+	}
+
 	/**
 	 * Splits the symmetric state into two ciphers for session encryption.
 	 * 
