@@ -215,11 +215,11 @@ public class RouterAddress extends DataStructureImpl {
     
     /**
      *  Caching version of InetAddress.getByName(getOption("host")).getAddress(), which is slow.
-     *  Caches numeric host names, and negative caches also.
+     *  Caches numeric hostnames, and negative caches also.
      *
-     *  As of 0.9.32, this works for literal IP addresses only, and does NOT resolve host names.
-     *  If a host name is specified in the options, this will return null.
-     *  Use getHost() if you need the host name.
+     *  As of 0.9.32, this works for literal IP addresses only, and does NOT resolve hostnames.
+     *  If a hostname is specified in the options, this will return null.
+     *  Use getHost() if you need the hostname.
      *
      *  @return IP or null
      *  @since 0.9.3
@@ -228,7 +228,7 @@ public class RouterAddress extends DataStructureImpl {
         if (_ip == NOT_LOOKED_UP) {
             // Only look up once, even if it fails, so we don't generate excessive DNS lookups.
             // The lifetime of a RouterAddress object is a few hours at most,
-            // it will get republished or expired, so it's OK even for host names.
+            // it will get republished or expired, so it's OK even for hostnames.
             String host = getHost();
             if (host != null)
                 _ip = Addresses.getIPOnly(host);
@@ -286,10 +286,12 @@ public class RouterAddress extends DataStructureImpl {
         // reduce Object proliferation
         if (_transportStyle.equals("SSU"))
             _transportStyle = "SSU";
-        else if (_transportStyle.equals("NTCP"))
-            _transportStyle = "NTCP";
         else if (_transportStyle.equals("NTCP2"))
             _transportStyle = "NTCP2";
+        else if (_transportStyle.equals("NTCP"))
+            _transportStyle = "NTCP";
+        else if (_transportStyle.equals("SSU2"))
+            _transportStyle = "SSU2";
         DataHelper.readProperties(in, _options);
     }
     
@@ -315,12 +317,16 @@ public class RouterAddress extends DataStructureImpl {
         if (object == this) return true;
         if ((object == null) || !(object instanceof RouterAddress)) return false;
         RouterAddress addr = (RouterAddress) object;
-        return
+        boolean rv =
                getPort() == addr.getPort() &&
                DataHelper.eq(getHost(), addr.getHost()) &&
                DataHelper.eq(_transportStyle, addr._transportStyle);
                //DataHelper.eq(_options, addr._options) &&
                //DataHelper.eq(_expiration, addr._expiration);
+        if (!rv || !_transportStyle.equals("SSU"))
+               return rv;
+        // SSU 4/6 caps
+        return DataHelper.eq(_options.getProperty("caps"), addr._options.getProperty("caps"));
     }
     
     /**
