@@ -17,6 +17,7 @@ import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -32,6 +33,9 @@ import net.i2p.data.Hash;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import net.i2p.router.transport.GeoIP;
+import net.i2p.router.transport.TransportUtil;
+import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.update.UpdateManager;
 import net.i2p.update.UpdateType;
 import net.i2p.util.Addresses;
@@ -39,6 +43,7 @@ import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.LHMCache;
 import net.i2p.util.Log;
 import net.i2p.util.SimpleTimer2;
+import net.i2p.util.SystemVersion;
 import net.i2p.util.Translate;
 
 /**
@@ -83,13 +88,16 @@ public class Blocklist {
     private Entry _wrapSave;
     private final Set<Hash> _inProcess = new HashSet<Hash>(4);
     private final File _blocklistFeedFile;
+    private final boolean _haveIPv6;
     private boolean _started;
+    private long _lastExpired = 0;
     // temp
     private final Map<Hash, String> _peerBlocklist = new HashMap<Hash, String>(4);
     
     private static final String PROP_BLOCKLIST_ENABLED = "router.blocklist.enable";
     private static final String PROP_BLOCKLIST_DETAIL = "router.blocklist.detail";
     private static final String PROP_BLOCKLIST_FILE = "router.blocklist.file";
+    private static final String PROP_BLOCKLIST_EXPIRE_INTERVAL = "router.blocklist.expireInterval";
     public static final String BLOCKLIST_FILE_DEFAULT = "blocklist.txt";
     private static final String BLOCKLIST_FEED_FILE = "docs/feed/blocklist/blocklist.txt";
     /** @since 0.9.48 */
@@ -100,11 +108,11 @@ public class Blocklist {
      *  Note that it's impossible to prevent clogging up
      *  the tables by a determined attacker, esp. on IPv6
      */
-    private static final int MAX_IPV4_SINGLES = 8192;
-    private static final int MAX_IPV6_SINGLES = 4096;
+    private static final int MAX_IPV4_SINGLES = SystemVersion.isSlow() ? 2048 : 8192;
+    private static final int MAX_IPV6_SINGLES = SystemVersion.isSlow() ? 256 : 4096;
 
-    private final Set<Integer> _singleIPBlocklist = new ConcurrentHashSet<Integer>(4);
-    private final Map<BigInteger, Object> _singleIPv6Blocklist = new LHMCache<BigInteger, Object>(MAX_IPV6_SINGLES);
+    private final Map<Integer, Object> _singleIPBlocklist = new LHMCache<Integer, Object>(MAX_IPV4_SINGLES);
+    private final Map<BigInteger, Object> _singleIPv6Blocklist;
 
     private static final Object DUMMY = Integer.valueOf(0);    
 
@@ -117,6 +125,7 @@ public class Blocklist {
     private static final String ID_LOCAL = "local";
     private static final String ID_COUNTRY = "country";
     private static final String ID_USER = "user";
+    public static final String ID_SYBIL = "sybil";
 
 
     /**
@@ -126,6 +135,9 @@ public class Blocklist {
         _context = context;
         _log = context.logManager().getLog(Blocklist.class);
         _blocklistFeedFile = new File(context.getConfigDir(), BLOCKLIST_FEED_FILE);
+        _haveIPv6 = TransportUtil.getIPv6Config(_context, "SSU") != TransportUtil.IPv6Config.IPV6_DISABLED &&
+                    Addresses.isConnectedIPv6();
+        _singleIPv6Blocklist = _haveIPv6 ? new LHMCache<BigInteger, Object>(MAX_IPV6_SINGLES) : null;
     }
     
     /** only for testing with main() */
@@ -133,6 +145,40 @@ public class Blocklist {
         _context = null;
         _log = new Log(Blocklist.class);
         _blocklistFeedFile = new File(BLOCKLIST_FEED_FILE);
+        _haveIPv6 = TransportUtil.getIPv6Config(_context, "SSU") != TransportUtil.IPv6Config.IPV6_DISABLED &&
+                    Addresses.isConnectedIPv6();
+        _singleIPv6Blocklist = _haveIPv6 ? new LHMCache<BigInteger, Object>(MAX_IPV6_SINGLES) : null;
+    }
+
+
+    private int expireInterval(){
+        String expireIntervalValue = _context.getProperty(PROP_BLOCKLIST_EXPIRE_INTERVAL, "0");
+        try{
+            Integer expireIntervalInt = 0;
+            if (expireIntervalValue.endsWith("s")) {
+                expireIntervalValue = expireIntervalValue.substring(0, expireIntervalValue.length() - 1);
+                expireIntervalInt = Integer.parseInt(expireIntervalValue) * 1000;
+            }else if(expireIntervalValue.endsWith("m")){
+                expireIntervalValue = expireIntervalValue.substring(0, expireIntervalValue.length() - 1);
+                expireIntervalInt = Integer.parseInt(expireIntervalValue) * 60000;
+            }else if(expireIntervalValue.endsWith("h")){
+                expireIntervalValue = expireIntervalValue.substring(0, expireIntervalValue.length() - 1);
+                expireIntervalInt = Integer.parseInt(expireIntervalValue) * 3600000;
+            }else if (expireIntervalValue.endsWith("d")) {
+                expireIntervalValue = expireIntervalValue.substring(0, expireIntervalValue.length() - 1);
+                expireIntervalInt = Integer.parseInt(expireIntervalValue) * 86400000;
+            }else{
+                expireIntervalInt = Integer.parseInt(expireIntervalValue);
+            }
+            if (expireIntervalInt < 0)
+                expireIntervalInt = 0;
+            return expireIntervalInt;
+        }catch(NumberFormatException nfe){
+            if (_log.shouldLog(_log.ERROR))
+                _log.error("format error in "+PROP_BLOCKLIST_EXPIRE_INTERVAL, nfe);
+        }
+        // if we don't have a valid value in this field, return 0 which is the same as disabling it.
+        return 0;
     }
 
     /**
@@ -160,8 +206,11 @@ public class Blocklist {
             files.add(new BLFile(blFile, ID_LOCAL));
         }
         files.add(new BLFile(_blocklistFeedFile, ID_FEED));
-        blFile = new File(_context.getConfigDir(), BLOCKLIST_COUNTRY_FILE);
-        files.add(new BLFile(blFile, ID_COUNTRY));
+        if (_context.router().isHidden() ||
+            _context.getBooleanProperty(GeoIP.PROP_BLOCK_MY_COUNTRY)) {
+            blFile = new File(_context.getConfigDir(), BLOCKLIST_COUNTRY_FILE);
+            files.add(new BLFile(blFile, ID_COUNTRY));
+        }
         // user specified
         String file = _context.getProperty(PROP_BLOCKLIST_FILE);
         if (file != null && !file.equals(BLOCKLIST_FILE_DEFAULT)) {
@@ -178,6 +227,11 @@ public class Blocklist {
         // but it's important to have this initialized before we read in the netdb.
         //job.getTiming().setStartAfter(_context.clock().now() + 30*1000);
         _context.jobQueue().addJob(job);
+        if (expireInterval() > 0) {
+            Job cleanupJob = new CleanupJob();
+            cleanupJob.getTiming().setStartAfter(_context.clock().now() + expireInterval());
+            _context.jobQueue().addJob(cleanupJob);
+        }
     }
 
     /**
@@ -217,6 +271,34 @@ public class Blocklist {
             }
         }
     }
+    
+    private class CleanupJob extends JobImpl {
+        public CleanupJob() {
+            super(_context);
+        }
+        public String getName(){
+            return "Expire blocklist at user-defined interval of " + expireInterval();
+        }
+        public void runJob() {
+            clear();
+            _lastExpired = System.currentTimeMillis();
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("Expiring blocklist entrys at" + _lastExpired);
+            // schedule the next one
+            super.requeue(expireInterval());
+        }
+    }
+
+    private void clear(){
+        synchronized(_singleIPBlocklist) {
+            _singleIPBlocklist.clear();
+        }
+        if (_singleIPv6Blocklist != null) {
+            synchronized(_singleIPv6Blocklist) {
+                _singleIPv6Blocklist.clear();
+            }
+        }
+    }
 
     private class ReadinJob extends JobImpl {
         private final List<BLFile> _files;
@@ -270,13 +352,20 @@ public class Blocklist {
                     reason = _x("Banned by router hash: {0}");
                 else
                     reason = _x("Banned by router hash");
-                _context.banlist().banlistRouterForever(peer, reason, comment);
+                banlistRouter(peer, reason, comment);
             }
             _peerBlocklist.clear();
             return count;
         }
     }
 
+    private void banlistRouter(Hash peer, String reason, String comment) {
+        if (expireInterval() > 0)
+            _context.banlist().banlistRouter(peer, reason, comment, null, expireInterval());
+        else
+            _context.banlist().banlistRouterForever(peer, reason, comment);
+    }
+
     /**
      *  The blocklist-country.txt file was created or updated.
      *  Read it in. Not required normally, as the country file
@@ -381,6 +470,7 @@ public class Blocklist {
         try {
             br = new BufferedReader(new InputStreamReader(
                     new FileInputStream(blFile), "UTF-8"));
+            String source = blFile.toString();
             String buf = null;
             while ((buf = br.readLine()) != null) {
                 Entry e = parse(buf, true);
@@ -395,18 +485,18 @@ public class Blocklist {
                 }
                 byte[] ip1 = e.ip1;
                 if (ip1.length == 4) {
-                    if (isFeedFile) {
-                        // temporary
-                        add(ip1);
-                        feedcount++;
-                    } else {
+                    //if (isFeedFile) {
+                    //    // temporary
+                    //    add(ip1, source);
+                    //    feedcount++;
+                    //} else {
                         byte[] ip2 = e.ip2;
                         store(ip1, ip2, blocklist, count++);
                         ipcount += 1 + toInt(ip2) - toInt(ip1); // includes dups, oh well
-                    }
+                    //}
                 } else {
                     // IPv6
-                    add(ip1);
+                    add(ip1, source);
                 }
             }
         } catch (IOException ioe) {
@@ -436,7 +526,8 @@ public class Blocklist {
             _log.info("Stats for " + blFile);
             _log.info("Removed " + badcount + " bad entries and comment lines");
             _log.info("Read " + read + " valid entries from the blocklist " + blFile);
-            _log.info("Blocking " + (isFeedFile ? feedcount : ipcount) + " IPs and " + peercount + " hashes");
+            //_log.info("Blocking " + (isFeedFile ? feedcount : ipcount) + " IPs and " + peercount + " hashes");
+            _log.info("Blocking " + ipcount + " IPs and " + peercount + " hashes");
             _log.info("Blocklist processing finished, time: " + (_context.clock().now() - start));
         }
         return count;
@@ -663,9 +754,28 @@ public class Blocklist {
      * @param ip IPv4 or IPv6
      */
     public void add(String ip) {
-        byte[] pib = Addresses.getIP(ip);
+        if (!_haveIPv6 && ip.indexOf(':') >= 0)
+            return;
+        byte[] pib = Addresses.getIPOnly(ip);
         if (pib == null) return;
-        add(pib);
+        add(pib, null);
+    }
+
+    /**
+     * Maintain a simple in-memory single-IP blocklist
+     * This is used for new additions, NOT for the main list
+     * of IP ranges read in from the file.
+     *
+     * @param ip IPv4 or IPv6
+     * @param source for logging only, may be null
+     * @since 0.9.57
+     */
+    public void add(String ip, String source) {
+        if (!_haveIPv6 && ip.indexOf(':') >= 0)
+            return;
+        byte[] pib = Addresses.getIPOnly(ip);
+        if (pib == null) return;
+        add(pib, source);
     }
 
     /**
@@ -676,15 +786,60 @@ public class Blocklist {
      * @param ip IPv4 or IPv6
      */
     public void add(byte ip[]) {
+        add(ip, null);
+    }
+
+    /**
+     * Maintain a simple in-memory single-IP blocklist
+     * This is used for new additions, NOT for the main list
+     * of IP ranges read in from the file.
+     *
+     * @param ip IPv4 or IPv6
+     * @param source for logging only, may be null
+     * @since 0.9.57
+     */
+    public void add(byte ip[], String source) {
         boolean rv;
-        if (ip.length == 4)
+        if (ip.length == 4) {
+            // don't ever block ourselves
+            String us = _context.getProperty(UDPTransport.PROP_IP);
+            if (us != null) {
+                byte[] usb = Addresses.getIP(us);
+                if (usb != null && DataHelper.eq(usb, ip)) {
+                    if (_log.shouldWarn())
+                        _log.warn("Not adding our own IP " + us, new Exception());
+                    return;
+                }
+            }
             rv = add(toInt(ip));
-        else if (ip.length == 16)
+            if (rv)
+                _context.commSystem().removeExemption(Addresses.toString(ip));
+        } else if (ip.length == 16) {
+            if (!_haveIPv6)
+                return;
+            // don't ever block ourselves
+            String us = _context.getProperty(UDPTransport.PROP_IPV6);
+            if (us != null) {
+                byte[] usb = Addresses.getIP(us);
+                if (usb != null && DataHelper.eq(usb, ip)) {
+                    if (_log.shouldWarn())
+                        _log.warn("Not adding our own IP " + us, new Exception());
+                    return;
+                }
+            }
             rv = add(new BigInteger(1, ip));
-        else
-            rv = false;
-        if (rv && _log.shouldInfo())
-            _log.info("Adding IP to blocklist: " + Addresses.toString(ip));
+            if (rv)
+                _context.commSystem().removeExemption(Addresses.toCanonicalString(ip));
+        } else {
+            return;
+        }
+        if (rv) {
+            // lower log level at startup when initializing from blocklist files
+            if (source == null && _log.shouldWarn())
+                _log.warn("Added: " + Addresses.toString(ip), new Exception("source"));
+            else if (_log.shouldDebug())
+                _log.debug("Added: " + Addresses.toString(ip) + " source: " + source);
+        }
     }
 
     /**
@@ -696,37 +851,57 @@ public class Blocklist {
      * @since 0.9.28
      */
     public void remove(byte ip[]) {
-        if (ip.length == 4)
+        if (ip.length == 4) {
             remove(toInt(ip));
-        else if (ip.length == 16)
+        } else if (ip.length == 16) {
+            if (!_haveIPv6)
+                return;
             remove(new BigInteger(1, ip));
+        }
     }
 
+    /**
+     * @return true if it was NOT previously on the list
+     */
     private boolean add(int ip) {
-        if (_singleIPBlocklist.size() >= MAX_IPV4_SINGLES)
+        // save space, don't put in both
+        if (isPermanentlyBlocklisted(ip))
             return false;
-        return _singleIPBlocklist.add(Integer.valueOf(ip));
+        Integer iip = Integer.valueOf(ip);
+        synchronized(_singleIPBlocklist) {
+            return _singleIPBlocklist.put(iip, DUMMY) == null;
+        }
     }
 
     /**
      * @since 0.9.28
      */
     private void remove(int ip) {
-        _singleIPBlocklist.remove(Integer.valueOf(ip));
+        Integer iip = Integer.valueOf(ip);
+        synchronized(_singleIPBlocklist) {
+            _singleIPBlocklist.remove(iip);
+        }
     }
 
     private boolean isOnSingleList(int ip) {
-        return _singleIPBlocklist.contains(Integer.valueOf(ip));
+        Integer iip = Integer.valueOf(ip);
+        synchronized(_singleIPBlocklist) {
+            return _singleIPBlocklist.get(iip) != null;
+        }
     }
 
     /**
      * @param ip IPv6 non-negative
+     * @return true if it was NOT previously on the list
      * @since IPv6
      */
     private boolean add(BigInteger ip) {
-        synchronized(_singleIPv6Blocklist) {
-            return _singleIPv6Blocklist.put(ip, DUMMY) == null;
+        if (_singleIPv6Blocklist != null) {
+            synchronized(_singleIPv6Blocklist) {
+                return _singleIPv6Blocklist.put(ip, DUMMY) == null;
+            }
         }
+        return false;
     }
 
     /**
@@ -734,8 +909,10 @@ public class Blocklist {
      * @since 0.9.28
      */
     private void remove(BigInteger ip) {
-        synchronized(_singleIPv6Blocklist) {
-            _singleIPv6Blocklist.remove(ip);
+        if (_singleIPv6Blocklist != null) {
+            synchronized(_singleIPv6Blocklist) {
+                _singleIPv6Blocklist.remove(ip);
+            }
         }
     }
 
@@ -744,9 +921,12 @@ public class Blocklist {
      * @since IPv6
      */
     private boolean isOnSingleList(BigInteger ip) {
-        synchronized(_singleIPv6Blocklist) {
-            return _singleIPv6Blocklist.get(ip) != null;
+        if (_singleIPv6Blocklist != null) {
+            synchronized(_singleIPv6Blocklist) {
+                return _singleIPv6Blocklist.get(ip) != null;
+            }
         }
+        return false;
     }
 
     /**
@@ -763,12 +943,14 @@ public class Blocklist {
      * Will not contain duplicates.
      * @since 0.9.29
      */
-    private static List<byte[]> getAddresses(RouterInfo pinfo) {
+    private List<byte[]> getAddresses(RouterInfo pinfo) {
         List<byte[]> rv = new ArrayList<byte[]>(4);
         // for each peer address
         for (RouterAddress pa : pinfo.getAddresses()) {
             byte[] pib = pa.getIP();
             if (pib == null) continue;
+            if (!_haveIPv6 && pib.length == 16)
+                continue;
             // O(n**2)
             boolean dup = false;
             for (int i = 0; i < rv.size(); i++) {
@@ -786,6 +968,9 @@ public class Blocklist {
     /**
      * Does the peer's IP address appear in the blocklist?
      * If so, and it isn't banlisted, banlist it forever...
+     * or, if the user configured an override, ban it for the 
+     * override period.
+     * @since 0.9.29
      */
     public boolean isBlocklisted(Hash peer) {
         List<byte[]> ips = getAddresses(peer);
@@ -805,6 +990,8 @@ public class Blocklist {
     /**
      * Does the peer's IP address appear in the blocklist?
      * If so, and it isn't banlisted, banlist it forever...
+     * or, if the user configured an override, ban it for the 
+     * override period.
      * @since 0.9.29
      */
     public boolean isBlocklisted(RouterInfo pinfo) {
@@ -829,7 +1016,9 @@ public class Blocklist {
      * @param ip IPv4 or IPv6
      */
     public boolean isBlocklisted(String ip) {
-        byte[] pib = Addresses.getIP(ip);
+        if (!_haveIPv6 && ip.indexOf(':') >= 0)
+            return false;
+        byte[] pib = Addresses.getIPOnly(ip);
         if (pib == null) return false;
         return isBlocklisted(pib);
     }
@@ -842,8 +1031,11 @@ public class Blocklist {
     public boolean isBlocklisted(byte ip[]) {
         if (ip.length == 4)
             return isBlocklisted(toInt(ip));
-        if (ip.length == 16)
+        if (ip.length == 16) {
+            if (!_haveIPv6)
+                return false;
             return isOnSingleList(new BigInteger(1, ip));
+        }
         return false;
     }
 
@@ -913,10 +1105,12 @@ public class Blocklist {
         return match(ip, blocklist[cur]);
     }
 
+/*
     // Is the IP included in the entry _blocklist[cur] ?
     private boolean match(int ip, int cur) {
         return match(ip, _blocklist[cur]);
     }
+*/
 
     // Is the IP included in the compressed entry?
     private static boolean match(int ip, long entry) {
@@ -1020,9 +1214,21 @@ public class Blocklist {
      *
      */
     private void banlist(Hash peer, byte[] ip) {
+        // Don't bother unless we have IPv6
+        if (!_haveIPv6 && ip.length == 16)
+            return;
         // Temporary reason, until the job finishes
         String reason = _x("IP banned by blocklist.txt entry {0}");
-        _context.banlist().banlistRouterForever(peer, reason, Addresses.toString(ip));
+        String sip = Addresses.toString(ip);
+        if ("127.0.0.1".equals(sip) ||
+            "0:0:0:0:0:0:0:1".equals(sip) ||
+            sip.startsWith("192.168.")) {
+            // i2pd bug, possibly at startup, don't ban forever
+            _context.banlist().banlistRouter(peer, reason, sip, null,
+                                             _context.clock().now() + Banlist.BANLIST_DURATION_LOCALHOST);
+            return;
+        }
+        banlistRouter(peer, reason, sip);
         if (!  _context.getBooleanPropertyDefaultTrue(PROP_BLOCKLIST_DETAIL))
             return;
         boolean shouldRunJob;
@@ -1050,7 +1256,7 @@ public class Blocklist {
         }
         public String getName() { return "Ban Peer by IP"; }
         public void runJob() {
-            banlistForever(_peer, _ips);
+            banlistRouter(_peer, _ips, expireInterval());
             synchronized (_inProcess) {
                 _inProcess.remove(_peer);
             }
@@ -1066,7 +1272,13 @@ public class Blocklist {
      * So we also stagger these jobs.
      *
      */
-    private synchronized void banlistForever(Hash peer, List<byte[]> ips) {
+    private void banlistRouter( Hash peer, String reason, String reasonCode, long duration) {
+        if (duration > 0)
+            _context.banlist().banlistRouter(peer, reason, reasonCode, null, System.currentTimeMillis()+expireInterval());
+        else
+            _context.banlist().banlistRouterForever(peer, reason, reasonCode);
+    }
+    private synchronized void banlistRouter(Hash peer, List<byte[]> ips, long duration) {
         // This only checks one file for now, pick the best one
         // user specified
         File blFile = null;
@@ -1086,7 +1298,7 @@ public class Blocklist {
             // just ban it and be done
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Banlisting " + peer);
-            _context.banlist().banlistRouterForever(peer, "Banned");
+            banlistRouter(peer, "Banned", "Banned", expireInterval());
             return;
         }
 
@@ -1117,7 +1329,7 @@ public class Blocklist {
                         //reason = reason + " banned by " + BLOCKLIST_FILE_DEFAULT + " entry \"" + buf + "\"";
                         if (_log.shouldLog(Log.WARN))
                             _log.warn("Banlisting " + peer + " " + reason);
-                        _context.banlist().banlistRouterForever(peer, reason, buf.toString());
+                        banlistRouter(peer, reason, buf.toString(), expireInterval());
                         return;
                     }
                 }
@@ -1135,12 +1347,16 @@ public class Blocklist {
      *  Single IPs blocked until restart. Unsorted.
      *
      *  Public for console only, not a public API
+     *  As of 0.9.57, will not contain IPs permanently banned,
+     *  except for ones banned permanently after being added to the transient list.
      *
      *  @return a copy, unsorted
      *  @since 0.9.48
      */
     public List<Integer> getTransientIPv4Blocks() {
-        return new ArrayList<Integer>(_singleIPBlocklist);
+        synchronized(_singleIPBlocklist) {
+            return new ArrayList<Integer>(_singleIPBlocklist.keySet());
+        }
     }
 
     /**
@@ -1152,9 +1368,14 @@ public class Blocklist {
      *  @since 0.9.48
      */
     public List<BigInteger> getTransientIPv6Blocks() {
-        synchronized(_singleIPv6Blocklist) {
-            return new ArrayList<BigInteger>(_singleIPv6Blocklist.keySet());
+        if (!_haveIPv6)
+            return Collections.<BigInteger>emptyList();
+        if (_singleIPv6Blocklist != null) {
+            synchronized(_singleIPv6Blocklist) {
+                    return new ArrayList<BigInteger>(_singleIPv6Blocklist.keySet());
+            }
         }
+        return Collections.<BigInteger>emptyList();
     }
 
     /**
