@@ -29,6 +29,7 @@ import gnu.getopt.Getopt;
 
 import freenet.support.CPUInformation.CPUID;
 import freenet.support.CPUInformation.UnknownCPUException;
+import net.i2p.CoreVersion;
 import net.i2p.client.impl.I2PSessionImpl;
 import net.i2p.crypto.SigUtil;
 import net.i2p.data.Base64;
@@ -41,6 +42,7 @@ import net.i2p.data.PublicKey;
 import net.i2p.data.SigningPrivateKey;
 import net.i2p.data.SigningPublicKey;
 import net.i2p.data.i2np.GarlicMessage;
+import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.CommSystemFacade.Status;
 import net.i2p.router.crypto.FamilyKeyCrypto;
@@ -50,12 +52,15 @@ import net.i2p.router.startup.CreateRouterInfoJob;
 import net.i2p.router.startup.PortableWorkingDir;
 import net.i2p.router.startup.StartupJob;
 import net.i2p.router.startup.WorkingDir;
+import net.i2p.router.sybil.Analysis;
 import net.i2p.router.tasks.*;
 import net.i2p.router.transport.FIFOBandwidthLimiter;
 import net.i2p.router.transport.UPnPScannerCallback;
 import net.i2p.router.transport.ntcp.NTCPTransport;
 import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.router.util.EventLog;
+import net.i2p.stat.Rate;
+import net.i2p.stat.RateAverages;
 import net.i2p.stat.RateStat;
 import net.i2p.stat.StatManager;
 import net.i2p.util.ByteCache;
@@ -72,6 +77,7 @@ import net.i2p.util.SecureFileOutputStream;
 import net.i2p.util.SimpleByteCache;
 import net.i2p.util.SystemVersion;
 import net.i2p.util.Translate;
+import net.i2p.util.VersionComparator;
 
 /**
  * Main driver for the router.
@@ -107,6 +113,7 @@ public class Router implements RouterClock.ClockShiftListener {
     private UPnPScannerCallback _upnpScannerCallback;
     private long _downtime = -1;
     
+    private static final String BUNDLE_NAME = "net.i2p.router.web.messages";
     public final static String PROP_CONFIG_FILE = "router.configLocation";
     
     /** let clocks be off by 1 minute */
@@ -139,6 +146,8 @@ public class Router implements RouterClock.ClockShiftListener {
     private static final String PROP_JBIGI = "jbigi.loadedResource";
     private static final String PROP_JBIGI_PROCESSOR = "jbigi.lastProcessor";
     public static final String UPDATE_FILE = "i2pupdate.zip";
+    //// remove after release ////
+    private static final boolean CONGESTION_CAPS = VersionComparator.comp(CoreVersion.PUBLISHED_VERSION, "0.9.60") >= 0;
         
     private static final int SHUTDOWN_WAIT_SECS = 60;
 
@@ -675,11 +684,12 @@ public class Router implements RouterClock.ClockShiftListener {
             changeState(State.STARTING_1);
         }
         String last = _config.get("router.previousFullVersion");
-        if (last != null) {
+        if (last != null && !_context.getBooleanProperty("i2p.vmCommSystem")) {
             _eventLog.addEvent(EventLog.UPDATED, "from " + last + " to " + RouterVersion.FULL_VERSION);
             saveConfig("router.previousFullVersion", null);
         }
-        _eventLog.addEvent(EventLog.STARTED, RouterVersion.FULL_VERSION);
+        if (!_context.getBooleanProperty("i2p.vmCommSystem"))
+            _eventLog.addEvent(EventLog.STARTED, RouterVersion.FULL_VERSION);
         startupStuff();
         changeState(State.STARTING_2);
         _started = System.currentTimeMillis();
@@ -915,11 +925,11 @@ public class Router implements RouterClock.ClockShiftListener {
             } else if (_state == State.EXPL_TUNNELS_READY) {
                 changeState(State.RUNNING);
                 changed = true;
-            } else {
-                _log.warn("Invalid state " + _state + " for setNetDbReady()");
             }
         }
-        if (changed) {
+        if (_context.netDb().isInitialized()) {
+            if (_log.shouldWarn())
+                _log.warn("NetDB ready, publishing RI");
             // any previous calls to netdb().publish() did not
             // actually publish, because netdb init was not complete
             Republish r = new Republish(_context);
@@ -927,7 +937,16 @@ public class Router implements RouterClock.ClockShiftListener {
             // so we probably don't need to throw it to the timer queue,
             // but just to be safe
             _context.simpleTimer2().addEvent(r, 0);
+        }
+        if (changed) {
             _context.commSystem().initGeoIP();
+
+            if (!SystemVersion.isSlow() &&
+                !_context.getBooleanProperty("i2np.allowLocal") &&
+                _context.getProperty(Analysis.PROP_FREQUENCY, Analysis.DEFAULT_FREQUENCY) > 0) {
+                // registers and starts itself
+                Analysis.getInstance(_context);
+            }
         }
     }
 
@@ -988,15 +1007,16 @@ public class Router implements RouterClock.ClockShiftListener {
      *
      *  Warning - risk of deadlock - do not call while holding locks
      *
-     * @param blockingRebuild If true, netdb publish will happen in-line.
-     *                        This may take a long time.
+     * @param blockingRebuild ignored, always nonblocking
      */
     public void rebuildRouterInfo(boolean blockingRebuild) {
         if (_log.shouldLog(Log.INFO))
             _log.info("Rebuilding new routerInfo, publish inline? " + blockingRebuild, new Exception("I did it"));
+        // deadlock thru createAddresses() thru SSU REA... moved outside lock
+        List<RouterAddress> addresses = _context.commSystem().createAddresses();
         _routerInfoLock.writeLock().lock();
         try {
-            locked_rebuildRouterInfo(blockingRebuild);
+            locked_rebuildRouterInfo(addresses);
         } finally {
             _routerInfoLock.writeLock().unlock();
         }
@@ -1006,8 +1026,9 @@ public class Router implements RouterClock.ClockShiftListener {
     /**
      * Rebuild and republish our routerInfo since something significant 
      * has changed.
+     *
      */
-    private void locked_rebuildRouterInfo(boolean blockingRebuild) {
+    private void locked_rebuildRouterInfo(List<RouterAddress> addresses) {
         RouterInfo ri;
         if (_routerInfo != null)
             ri = new RouterInfo(_routerInfo);
@@ -1019,8 +1040,7 @@ public class Router implements RouterClock.ClockShiftListener {
             Properties stats = _context.statPublisher().publishStatistics();
             
             ri.setOptions(stats);
-            // deadlock thru createAddresses() thru SSU REA... move outside lock?
-            ri.setAddresses(_context.commSystem().createAddresses());
+            ri.setAddresses(addresses);
 
             SigningPrivateKey key = _context.keyManager().getSigningPrivateKey();
             if (key == null) {
@@ -1032,9 +1052,10 @@ public class Router implements RouterClock.ClockShiftListener {
             if (!ri.isValid())
                 throw new DataFormatException("Our RouterInfo has a bad signature");
             Republish r = new Republish(_context);
-            if (blockingRebuild)
-                r.timeReached();
-            else
+            // Avoid deadlocks part 1000
+            //if (blockingRebuild)
+            //    r.timeReached();
+            //else
                 _context.simpleTimer2().addEvent(r, 0);
         } catch (DataFormatException dfe) {
             _log.log(Log.CRIT, "Internal error - unable to sign our own address?!", dfe);
@@ -1057,8 +1078,9 @@ public class Router implements RouterClock.ClockShiftListener {
                 if (!_familyKeyCryptoFail) {
                     try {
                         _familyKeyCrypto = new FamilyKeyCrypto(_context);
-                    } catch (GeneralSecurityException gse) {
-                        _log.error("Failed to initialize family key crypto", gse);
+                    } catch (Exception e) {
+                        // Could be IllegalArgumentException from key problems
+                        _log.error("Failed to initialize family key crypto", e);
                         _familyKeyCryptoFail = true;
                     }
                 }
@@ -1082,6 +1104,14 @@ public class Router implements RouterClock.ClockShiftListener {
     
     public static final char CAPABILITY_REACHABLE = 'R';
     public static final char CAPABILITY_UNREACHABLE = 'U';
+
+    /** @since 0.9.58, proposal 162 */
+    public static final char CAPABILITY_CONGESTION_MODERATE = 'D';
+    /** @since 0.9.58, proposal 162 */
+    public static final char CAPABILITY_CONGESTION_SEVERE = 'E';
+    /** @since 0.9.58, proposal 162 */
+    public static final char CAPABILITY_NO_TUNNELS = 'G';
+
     /** for testing */
     public static final String PROP_FORCE_UNREACHABLE = "router.forceUnreachable";
 
@@ -1152,7 +1182,8 @@ public class Router implements RouterClock.ClockShiftListener {
      */
     public String getCapabilities() {
         StringBuilder rv = new StringBuilder(4);
-        char bw = getBandwidthClass();
+        boolean hidden = isHidden();
+        char bw = hidden ? CAPABILITY_BW32 : getBandwidthClass();
         rv.append(bw);
         // 512 and unlimited supported as of 0.9.18;
         // Add 256 as well for compatibility
@@ -1167,8 +1198,10 @@ public class Router implements RouterClock.ClockShiftListener {
         if(_context.getBooleanProperty(PROP_HIDDEN))
             rv.append(RouterInfo.CAPABILITY_HIDDEN);
         
-        if (_context.getBooleanProperty(PROP_FORCE_UNREACHABLE)) {
+        if (hidden || _context.getBooleanProperty(PROP_FORCE_UNREACHABLE)) {
             rv.append(CAPABILITY_UNREACHABLE);
+            //if (CONGESTION_CAPS)
+            //    rv.append(CAPABILITY_NO_TUNNELS);
             return rv.toString();
         }
         switch (_context.commSystem().getStatus()) {
@@ -1199,6 +1232,64 @@ public class Router implements RouterClock.ClockShiftListener {
                 // no explicit capability
                 break;
         }
+
+        char cong = 0;
+        int maxTunnels = _context.getProperty(RouterThrottleImpl.PROP_MAX_TUNNELS, RouterThrottleImpl.DEFAULT_MAX_TUNNELS);
+        if (maxTunnels <= 0) {
+            cong = CAPABILITY_NO_TUNNELS;
+        } else if (maxTunnels <= 50 || SystemVersion.isSlow()) {
+            cong = CAPABILITY_CONGESTION_MODERATE;
+        } else {
+            int numTunnels = _context.tunnelManager().getParticipatingCount();
+            if (numTunnels > 9 * maxTunnels / 10) {
+                cong = CAPABILITY_CONGESTION_SEVERE;
+            } else if (numTunnels > 8 * maxTunnels / 10) {
+                cong = CAPABILITY_CONGESTION_MODERATE;
+            } else {
+                // this is a greatly simplified version of RouterThrottleImpl.acceptTunnelRequest()
+                long lag = _context.jobQueue().getMaxLag();
+                if (lag > 300 && getUptime() > 10*60*1000) {
+                    if (lag > 500)
+                        cong = CAPABILITY_CONGESTION_SEVERE;
+                    else
+                        cong = CAPABILITY_CONGESTION_MODERATE;
+                } else {
+                    double bwLim = getSharePercentage() * 1024 *
+                                   Math.min(_context.bandwidthLimiter().getInboundKBytesPerSecond(),
+                                            _context.bandwidthLimiter().getOutboundKBytesPerSecond());
+                    if (bwLim < 4*1024) {
+                        cong = CAPABILITY_NO_TUNNELS;
+                    } else {
+                        RateStat rs = _context.statManager().getRate("tunnel.participatingMessageCountAvgPerTunnel");
+                        double messagesPerTunnel = 0;
+                        if (rs != null) {
+                            Rate r = rs.getRate(20*60*1000);
+                            if (r != null) {
+                                RateAverages ra = RateAverages.getTemp();
+                                messagesPerTunnel = r.computeAverages(ra, true).getAverage();
+                            }
+                        }
+                        if (messagesPerTunnel < RouterThrottleImpl.DEFAULT_MESSAGES_PER_TUNNEL_ESTIMATE)
+                            messagesPerTunnel = RouterThrottleImpl.DEFAULT_MESSAGES_PER_TUNNEL_ESTIMATE;
+                        double bpsAllocated = messagesPerTunnel * numTunnels * 1024 / (10 * 60);
+                        if (_log.shouldInfo())
+                            _log.info("bps allocated: " + bpsAllocated + " bw limit: " + bwLim);
+                        if (bpsAllocated > 0.9 * bwLim)
+                            cong = CAPABILITY_CONGESTION_SEVERE;
+                        else if (bpsAllocated > 0.8 * bwLim)
+                            cong = CAPABILITY_CONGESTION_MODERATE;
+                    }
+                }
+            }
+        }
+        if (cong != 0) {
+            if (CONGESTION_CAPS) {
+                rv.append(cong);
+            } else {
+                if (_log.shouldWarn())
+                    _log.warn("Congestion cap: " + cong);
+            }
+        }
         return rv.toString();
     }
     
@@ -1268,6 +1359,8 @@ public class Router implements RouterClock.ClockShiftListener {
             removeConfigSetting(UDPTransport.PROP_INTERNAL_PORT);
             removeConfigSetting(UDPTransport.PROP_EXTERNAL_PORT);
             removeConfigSetting(UDPTransport.PROP_INTRO_KEY);
+            removeConfigSetting(UDPTransport.PROP_SSU2_SP);
+            removeConfigSetting(UDPTransport.PROP_SSU2_IKEY);
             removeConfigSetting(NTCPTransport.PROP_I2NP_NTCP_PORT);
             removeConfigSetting(NTCPTransport.PROP_NTCP2_SP);
             removeConfigSetting(NTCPTransport.PROP_NTCP2_IV);
@@ -1337,7 +1430,7 @@ public class Router implements RouterClock.ClockShiftListener {
                     // only do this on these OSes
                     boolean goodOS = isWin || isMac ||
                                      osName.contains("linux") || osName.contains("freebsd");
-                    File jbigiJar = new File(_context.getBaseDir(), "lib/jbigi.jar");
+                    File jbigiJar = new File(_context.getLibDir(), "jbigi.jar");
                     if (goodOS && jbigiJar.exists() && _context.getBaseDir().canWrite()) {
                         String libPrefix = isWin ? "" : "lib";
                         String libSuffix = isWin ? ".dll" : isMac ? ".jnilib" : ".so";
@@ -1431,7 +1524,10 @@ public class Router implements RouterClock.ClockShiftListener {
         // help us shut down esp. after OOM
         int priority = (exitCode == EXIT_OOM) ? Thread.MAX_PRIORITY - 1 : Thread.NORM_PRIORITY + 2;
         Thread.currentThread().setPriority(priority);
-        _log.log(Log.CRIT, "Starting final shutdown(" + exitCode + ')');
+        if (exitCode == EXIT_HARD_RESTART || exitCode == EXIT_GRACEFUL_RESTART)
+            _log.log(Log.CRIT, _t("Restart imminent"));
+        else
+            _log.log(Log.CRIT, _t("Shutdown imminent"));
         // So we can get all the way to the end
         // No, you can't do Thread.currentThread.setDaemon(false)
         if (_killVMOnEnd) {
@@ -1510,7 +1606,7 @@ public class Router implements RouterClock.ClockShiftListener {
         try { _context.jobQueue().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the job queue", t); }
         try { _context.tunnelManager().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the tunnel manager", t); }
         try { _context.tunnelDispatcher().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the tunnel dispatcher", t); }
-        try { _context.netDb().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the networkDb", t); }
+        try { _context.netDbSegmentor().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the networkDb", t); }
         try { _context.commSystem().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the comm system", t); }
         try { _context.bandwidthLimiter().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the comm system", t); }
         try { _context.peerManager().shutdown(); } catch (Throwable t) { _log.error("Error shutting down the peer manager", t); }
@@ -1545,7 +1641,8 @@ public class Router implements RouterClock.ClockShiftListener {
         // logManager shut down in finalShutdown()
         _watchdog.shutdown();
         _watchdogThread.interrupt();
-        _eventLog.addEvent(EventLog.STOPPED, Integer.toString(exitCode));
+        if (!_context.commSystem().isDummy())
+            _eventLog.addEvent(EventLog.STOPPED, Integer.toString(exitCode));
         finalShutdown(exitCode);
     }
 
@@ -1810,8 +1907,9 @@ public class Router implements RouterClock.ClockShiftListener {
             changeState(State.RESTARTING);
         }
         ((RouterClock) _context.clock()).removeShiftListener(this);
-        // Let's not stop accepting tunnels, etc
-        //_started = _context.clock().now();
+        // Stop accepting tunnels, etc.
+        // This also prevents netdb from immediately expiring all the RIs
+        _started = System.currentTimeMillis();
         synchronized(_configFileLock) {
             _downtime = 1;
         }
@@ -1929,6 +2027,9 @@ public class Router implements RouterClock.ClockShiftListener {
             if (downtime > LIVELINESS_DELAY) {
                 System.err.println("WARN: Old router was not shut down gracefully, deleting " + f);
                 f.delete();
+                if (lastWritten > 0)
+                    _eventLog.addEvent(EventLog.CRASHED,
+                                       Translate.getString("{0} ago", DataHelper.formatDuration2(downtime), _context, BUNDLE_NAME));
             } else {
                 return false;
             }
@@ -1956,7 +2057,7 @@ public class Router implements RouterClock.ClockShiftListener {
      *  May take a while to run the first time, if it has to go through the event log.
      *  Once called, the result is cached.
      *
-     *  @since 0.0.47
+     *  @since 0.9.47
      */
     public long getEstimatedDowntime() {
         synchronized(_configFileLock) {
@@ -1978,7 +2079,7 @@ public class Router implements RouterClock.ClockShiftListener {
     /** 
      *  Only for soft restart. Not for external use.
      *
-     *  @since 0.0.47
+     *  @since 0.9.47
      */
     public void setEstimatedDowntime(long downtime) {
         if (downtime <= 0)
@@ -2122,4 +2223,12 @@ public class Router implements RouterClock.ClockShiftListener {
             recv = (int)rs.getRate(5*60*1000).getAverageValue();
         return Math.max(send, recv);
     }
+
+    /**
+     *  Translate with console bundle
+     *  @since 0.9.53
+     */
+    private final String _t(String s) {
+        return Translate.getString(s, _context, BUNDLE_NAME);
+    }
 }
