@@ -25,6 +25,8 @@ import net.i2p.router.crypto.TransientSessionKeyManager;
 import net.i2p.router.dummy.*;
 import net.i2p.router.message.GarlicMessageParser;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseSegmentor;
+import net.i2p.router.networkdb.kademlia.SegmentedNetworkDatabaseFacade;
 import net.i2p.router.peermanager.PeerManagerFacadeImpl;
 import net.i2p.router.peermanager.ProfileManagerImpl;
 import net.i2p.router.peermanager.ProfileOrganizer;
@@ -56,7 +58,7 @@ public class RouterContext extends I2PAppContext {
     private OutNetMessagePool _outNetMessagePool;
     private MessageHistory _messageHistory;
     private OutboundMessageRegistry _messageRegistry;
-    private NetworkDatabaseFacade _netDb;
+    private SegmentedNetworkDatabaseFacade _netDb;
     private KeyManager _keyManager;
     private CommSystemFacade _commSystem;
     private ProfileOrganizer _profileOrganizer;
@@ -242,9 +244,9 @@ public class RouterContext extends I2PAppContext {
         //_messageStateMonitor = new MessageStateMonitor(this);
         _routingKeyGenerator = new RouterKeyGenerator(this);
         if (!getBooleanProperty("i2p.dummyNetDb"))
-            _netDb = new FloodfillNetworkDatabaseFacade(this); // new KademliaNetworkDatabaseFacade(this);
+            _netDb = new FloodfillNetworkDatabaseSegmentor(this);
         else
-            _netDb = new DummyNetworkDatabaseFacade(this);
+            _netDb = new DummyNetworkDatabaseSegmentor(this);
         _keyManager = new KeyManager(this);
         if (!getBooleanProperty("i2p.vmCommSystem"))
             _commSystem = new CommSystemFacadeImpl(this);
@@ -368,8 +370,23 @@ public class RouterContext extends I2PAppContext {
 
     /**
      * Our db cache
+     * @since 0.9.61
      */
-    public NetworkDatabaseFacade netDb() { return _netDb; }
+    public SegmentedNetworkDatabaseFacade netDbSegmentor() { return _netDb; }
+
+    public NetworkDatabaseFacade netDb() { return _netDb.mainNetDB(); }
+
+    /**
+     * Get the client netDb for the given id.
+     * Will return the main netDb if
+     * the dbid is null or the client db is not found.
+     * 
+     * @since 0.9.61
+     * @param id may be null
+     * @return non-null
+     */
+    public NetworkDatabaseFacade clientNetDb(Hash id) { return _netDb.clientNetDB(id); }
+
     /**
      * The actual driver of the router, where all jobs are enqueued and processed.
      */
@@ -732,7 +749,7 @@ public class RouterContext extends I2PAppContext {
      *  Once called, the result is cached.
      *
      *  @return downtime in ms or 0 if unknown
-     *  @since 0.0.47
+     *  @since 0.9.47
      */
     public long getEstimatedDowntime() {
         if (_router == null)
