@@ -34,9 +34,9 @@ public class RouterThrottleImpl implements RouterThrottle {
     public static final String PROP_MAX_TUNNELS = "router.maxParticipatingTunnels";
     public static final int DEFAULT_MAX_TUNNELS = 10*1000;
     private static final String PROP_MAX_PROCESSINGTIME = "router.defaultProcessingTimeThrottle";
-    private static final long DEFAULT_REJECT_STARTUP_TIME = 10*60*1000;
+    public static final long DEFAULT_REJECT_STARTUP_TIME = 10*60*1000;
     private static final long MIN_REJECT_STARTUP_TIME = 90*1000;
-    private static final String PROP_REJECT_STARTUP_TIME = "router.rejectStartupTime";
+    public static final String PROP_REJECT_STARTUP_TIME = "router.rejectStartupTime";
     private static final int DEFAULT_MIN_THROTTLE_TUNNELS = SystemVersion.isAndroid() ? 100 :
                                                             SystemVersion.isARM() ? 500 : 1000;
 
@@ -293,7 +293,7 @@ public class RouterThrottleImpl implements RouterThrottle {
         // of another tunnel?
         rs = _context.statManager().getRate("tunnel.participatingMessageCountAvgPerTunnel");
         r = null;
-        double messagesPerTunnel = DEFAULT_MESSAGES_PER_TUNNEL_ESTIMATE;
+        double messagesPerTunnel = 0;
         if (rs != null) {
             r = rs.getRate(60*1000);
             if (r != null) 
@@ -345,7 +345,7 @@ public class RouterThrottleImpl implements RouterThrottle {
         return TUNNEL_ACCEPT;
     }
 
-    private static final int DEFAULT_MESSAGES_PER_TUNNEL_ESTIMATE = 40; // .067KBps
+    static final int DEFAULT_MESSAGES_PER_TUNNEL_ESTIMATE = 40; // .067KBps
     /** also limited to 90% - see below */
     private static final int MIN_AVAILABLE_BPS = 4*1024; // always leave at least 4KBps free when allowing
     private static final String LIMIT_STR = _x("Rejecting tunnels: Bandwidth limit");
@@ -363,8 +363,6 @@ public class RouterThrottleImpl implements RouterThrottle {
         int usedIn = Math.min(_context.router().get1sRateIn(), _context.router().get15sRateIn());
         int usedOut = Math.min(_context.router().get1sRate(true), _context.router().get15sRate(true));
         int used = Math.max(usedIn, usedOut);
-        int used1mIn = _context.router().get1mRateIn();
-        int used1mOut = _context.router().get1mRate(true);
 
         // Check the inbound and outbound total bw available (separately)
         // We block all tunnels when share bw is over (max * 0.9) - 4KB
@@ -391,6 +389,8 @@ public class RouterThrottleImpl implements RouterThrottle {
         _context.statManager().addRateData("router.throttleTunnelBytesAllowed", availBps, (long)bytesAllocated);
 
         // Now see if 1m rates are too high
+        int used1mIn = _context.router().get1mRateIn();
+        int used1mOut = _context.router().get1mRate(true);
         long overage = Math.max(used1mIn - (maxKBpsIn*1024), used1mOut - (maxKBpsOut*1024));
         if ( (overage > 0) && 
              ((overage/(maxKBps*1024f)) > _context.random().nextFloat()) ) {
@@ -399,20 +399,42 @@ public class RouterThrottleImpl implements RouterThrottle {
             return false;
         }
 
+        double probReject;
+        boolean reject;
+        if (availBps <= 0) {
+            probReject = 1;
+            reject = true;
+            if (_log.shouldWarn())
+                _log.warn("Reject avail/maxK/used " + availBps + "/" + maxKBps + "/" 
+                          + used + " pReject = 1 numTunnels = " + numTunnels 
+                          + " est = " + bytesAllocated);
+        } else {
             // limit at 90% - 4KBps (see above)
             float maxBps = (maxKBps * 1024f * 0.9f) - MIN_AVAILABLE_BPS;
             float pctFull = (maxBps - availBps) / (maxBps);
-            double probReject = Math.pow(pctFull, 16); // steep curve 
-            double rand = _context.random().nextFloat();
-            boolean reject = rand <= probReject;
-            if (reject && _log.shouldLog(Log.WARN))
-                _log.warn("Reject avail/maxK/used " + availBps + "/" + maxKBps + "/" 
+            if (pctFull < 0.83f) {
+                // probReject < ~5%
+                probReject = 0;
+                reject = false;
+                if (_log.shouldDebug())
+                    _log.debug("Accept avail/maxK/used " + availBps + "/" + maxKBps + "/" 
+                               + used + " pReject = 0 numTunnels = " + numTunnels 
+                               + " est = " + bytesAllocated);
+            } else {
+                probReject = Math.pow(pctFull, 16); // steep curve 
+                double rand = _context.random().nextFloat();
+                reject = rand <= probReject;
+                if (reject && _log.shouldWarn())
+                    _log.warn("Reject avail/maxK/used " + availBps + "/" + maxKBps + "/" 
                           + used + " pReject = " + probReject + " pFull = " + pctFull + " numTunnels = " + numTunnels 
                           + " rand = " + rand + " est = " + bytesAllocated);
-            else if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Accept avail/maxK/used " + availBps + "/" + maxKBps + "/" 
+                else if (_log.shouldDebug())
+                    _log.debug("Accept avail/maxK/used " + availBps + "/" + maxKBps + "/" 
                            + used + " pReject = " + probReject + " pFull = " + pctFull + " numTunnels = " + numTunnels 
                            + " rand = " + rand + " est = " + bytesAllocated);
+            }
+        }
+
             if (probReject >= 0.9)
                 setTunnelStatus(LIMIT_STR);
             else if (probReject >= 0.5)
@@ -477,17 +499,23 @@ public class RouterThrottleImpl implements RouterThrottle {
     
     private double getTunnelGrowthFactor() {
         try {
-            return Double.parseDouble(_context.getProperty("router.tunnelGrowthFactor", "1.3"));
+            String p = _context.getProperty("router.tunnelGrowthFactor");
+            if (p == null)
+                return 1.3d;
+            return Double.parseDouble(p);
         } catch (NumberFormatException nfe) {
-            return 1.3;
+            return 1.3d;
         }
     }
 
     private double getTunnelTestTimeGrowthFactor() {
         try {
-            return Double.parseDouble(_context.getProperty("router.tunnelTestTimeGrowthFactor", "1.3"));
+            String p = _context.getProperty("router.tunnelTestTimeGrowthFactor");
+            if (p == null)
+                return 1.3d;
+            return Double.parseDouble(p);
         } catch (NumberFormatException nfe) {
-            return 1.3;
+            return 1.3d;
         }
     }
     
@@ -504,28 +532,6 @@ public class RouterThrottleImpl implements RouterThrottle {
         return (long)lagRate.getAverageValue();
     }
     
-    public double getInboundRateDelta() {
-        RateStat receiveRate = _context.statManager().getRate("transport.sendMessageSize");
-        if (receiveRate == null)
-            return 0;
-        double nowBps = getBps(receiveRate.getRate(60*1000));
-        double fiveMinBps = getBps(receiveRate.getRate(5*60*1000));
-        double hourBps = getBps(receiveRate.getRate(60*60*1000));
-        double dailyBps = getBps(receiveRate.getRate(24*60*60*1000));
-        
-        if (nowBps < 0) return 0;
-        if (dailyBps > 0) return nowBps - dailyBps;
-        if (hourBps > 0) return nowBps - hourBps;
-        if (fiveMinBps > 0) return nowBps - fiveMinBps;
-        return 0;
-    }
-
-    private static double getBps(Rate rate) {
-        if (rate == null) return -1;
-        double bytes = rate.getLastTotalValue();
-        return (bytes*1000.0d)/rate.getPeriod(); 
-    }
-    
     public String getTunnelStatus() {
         return _tunnelStatus;
     }
