@@ -16,13 +16,15 @@ import net.i2p.CoreVersion;
  */
 public class RouterVersion {
     /** deprecated */
-    public final static String ID = "Monotone";
+    public final static String ID = "Git";
     public final static String VERSION = CoreVersion.VERSION;
+    /** for example: "beta", "alpha", "rc" */
+    public final static String QUALIFIER = "";
     public final static long BUILD = 0;
-
     /** for example "-test" */
     public final static String EXTRA = "";
-    public final static String FULL_VERSION = VERSION + "-" + BUILD + EXTRA;
+    public final static String FULL_VERSION = VERSION + "-" + BUILD + QUALIFIER + EXTRA;
+
     public static void main(String args[]) {
         System.out.println("I2P Router version: " + FULL_VERSION);
         System.out.println("Router ID: " + RouterVersion.ID);
