@@ -50,10 +50,11 @@ import net.i2p.router.RouterContext;
 import net.i2p.router.crypto.TransientSessionKeyManager;
 import net.i2p.router.crypto.ratchet.RatchetSKM;
 import net.i2p.router.crypto.ratchet.MuxedSKM;
+import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.I2PThread;
 import net.i2p.util.Log;
-import net.i2p.util.SimpleTimer;
+import net.i2p.util.SimpleTimer2;
 
 /**
  * Bridge the router and the client - managing state for a client.
@@ -90,6 +91,8 @@ class ClientConnectionRunner {
     protected I2CPMessageReader _reader;
     /** Used for all sessions, which must all have the same crypto keys */
     private SessionKeyManager _sessionKeyManager;
+    /** Used for leaseSets sent to and recieved from this client */
+    private FloodfillNetworkDatabaseFacade _floodfillNetworkDatabaseFacade;
     /** 
      * This contains the last 10 MessageIds that have had their (non-ack) status 
      * delivered to the client (so that we can be sure only to update when necessary)
@@ -216,21 +219,20 @@ class ClientConnectionRunner {
             // removeSession() was called just before this, and
             // _sessions will be empty.
             for (SessionParams sp : _sessions.values()) {
-                LeaseSet ls = sp.currentLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
-                // unpublish encrypted LS also
-                ls = sp.currentEncryptedLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
+                // we don't need to unpublish(),
+                // as we shut down our subdb below.
                 if (!sp.isPrimary)
                     _context.tunnelManager().removeAlias(sp.dest);
             }
             for (SessionParams sp : _sessions.values()) {
                 if (sp.isPrimary)
                     _context.tunnelManager().removeTunnels(sp.dest);
+                if (sp.rerequestTimer != null)
+                    sp.rerequestTimer.cancel();
             }
         }
+        if (_floodfillNetworkDatabaseFacade != null)
+            _floodfillNetworkDatabaseFacade.shutdown();
         synchronized (_alreadyProcessed) {
             _alreadyProcessed.clear();
         }
@@ -446,6 +448,7 @@ class ClientConnectionRunner {
         if (id == null)
             return;
         boolean isPrimary = false;
+        Hash dbid = getDestHash();
         for (Iterator<SessionParams> iter = _sessions.values().iterator(); iter.hasNext(); ) {
             SessionParams sp = iter.next();
             if (id.equals(sp.sessionId)) {
@@ -455,17 +458,21 @@ class ClientConnectionRunner {
                 // Tell client manger
                 _manager.unregisterSession(id, sp.dest);
                 LeaseSet ls = sp.currentLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
+                if (ls != null && _floodfillNetworkDatabaseFacade != null)
+                    _floodfillNetworkDatabaseFacade.unpublish(ls);
                 // unpublish encrypted LS also
                 ls = sp.currentEncryptedLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
+                if (ls != null && _floodfillNetworkDatabaseFacade != null)
+                    _floodfillNetworkDatabaseFacade.unpublish(ls);
                 isPrimary = sp.isPrimary;
                 if (isPrimary)
                     _context.tunnelManager().removeTunnels(sp.dest);
                 else
                     _context.tunnelManager().removeAlias(sp.dest);
+                synchronized(this) {
+                    if (sp.rerequestTimer != null)
+                        sp.rerequestTimer.cancel();
+                }
                 break;
             }
         }
@@ -476,13 +483,17 @@ class ClientConnectionRunner {
                     _log.info("Destroying remaining client subsession " + sp.sessionId);
                 _manager.unregisterSession(sp.sessionId, sp.dest);
                 LeaseSet ls = sp.currentLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
+                if (ls != null && _floodfillNetworkDatabaseFacade != null)
+                    _floodfillNetworkDatabaseFacade.unpublish(ls);
                 // unpublish encrypted LS also
                 ls = sp.currentEncryptedLeaseSet;
-                if (ls != null)
-                    _context.netDb().unpublish(ls);
+                if (ls != null && _floodfillNetworkDatabaseFacade != null)
+                    _floodfillNetworkDatabaseFacade.unpublish(ls);
                 _context.tunnelManager().removeAlias(sp.dest);
+                synchronized(this) {
+                    if (sp.rerequestTimer != null)
+                        sp.rerequestTimer.cancel();
+                }
             }
             _sessions.clear();
         }
@@ -630,6 +641,12 @@ class ClientConnectionRunner {
                 }
             }
         }
+        if (isPrimary && _floodfillNetworkDatabaseFacade == null) {
+            if (_log.shouldDebug())
+                _log.debug("Initializing subDb for client" + destHash);
+            _floodfillNetworkDatabaseFacade = new FloodfillNetworkDatabaseFacade(_context, destHash);
+            _floodfillNetworkDatabaseFacade.startup();
+        }
         return _manager.destinationEstablished(this, dest);
     }
     
@@ -690,6 +707,10 @@ class ClientConnectionRunner {
                     _log.debug("LeaseSet created fully: " + state + '\n' + ls);
                 sp.leaseRequest = null;
                 _consecutiveLeaseRequestFails = 0;
+                if (sp.rerequestTimer != null) {
+                    sp.rerequestTimer.cancel();
+                    sp.rerequestTimer = null;
+                }
             }
         }
         if ( (state != null) && (state.getOnGranted() != null) )
@@ -888,7 +909,8 @@ class ClientConnectionRunner {
      * @param h the Destination's hash
      * @param set LeaseSet with requested leases - this object must be updated to contain the 
      *            signed version (as well as any changed/added/removed Leases)
-     *            The LeaseSet contains Leases and destination only, it is unsigned.
+     *            The LeaseSet contains Leases only, it is unsigned.
+     *            Must be unique for this hash, do not reuse for subsessions.
      * @param expirationTime ms to wait before failing
      * @param onCreateJob Job to run after the LeaseSet is authorized, null OK
      * @param onFailedJob Job to run after the timeout passes without receiving authorization, null OK
@@ -958,7 +980,7 @@ class ClientConnectionRunner {
                     set.setDestination(dest);
                     Rerequest timer = new Rerequest(set, expirationTime, onCreateJob, onFailedJob);
                     sp.rerequestTimer = timer;
-                    _context.simpleTimer2().addEvent(timer, 3*1000);
+                    timer.schedule(3*1000);
                     if (_log.shouldLog(Log.DEBUG))
                         _log.debug("Already requesting, ours newer, wait 3 sec: " + state);
                 }
@@ -972,13 +994,16 @@ class ClientConnectionRunner {
                     // before we are ready
                     Rerequest timer = new Rerequest(set, expirationTime, onCreateJob, onFailedJob);
                     sp.rerequestTimer = timer;
-                    _context.simpleTimer2().addEvent(timer, 1000);
+                    timer.schedule(1000);
                     if (_log.shouldLog(Log.DEBUG))
                         _log.debug("No current LS but no OB tunnels, wait 1 sec for " + h);
                     return;
                 } else {
                     // so the timer won't fire off with an older LS request
-                    sp.rerequestTimer = null;
+                    if (sp.rerequestTimer != null) {
+                        sp.rerequestTimer.cancel();
+                        sp.rerequestTimer = null;
+                    }
                     long earliest = (current != null) ? current.getEarliestLeaseDate() : 0;
                     sp.leaseRequest = state = new LeaseRequestState(onCreateJob, onFailedJob, earliest,
                                                                 _context.clock().now() + expirationTime, set);
@@ -990,14 +1015,18 @@ class ClientConnectionRunner {
         _context.jobQueue().addJob(new RequestLeaseSetJob(_context, this, state));
     }
 
-    private class Rerequest implements SimpleTimer.TimedEvent {
+    private class Rerequest extends SimpleTimer2.TimedEvent {
         private final LeaseSet _ls;
         private final long _expirationTime;
         private final Job _onCreate;
         private final Job _onFailed;
 
-        /** @param ls dest must be set */
+        /**
+         * Caller must schedule()
+         * @param ls dest must be set
+         */
         public Rerequest(LeaseSet ls, long expirationTime, Job onCreate, Job onFailed) {
+            super(_context.simpleTimer2());
             _ls = ls;
             _expirationTime = expirationTime;
             _onCreate = onCreate;
@@ -1126,6 +1155,19 @@ class ClientConnectionRunner {
      */
     private final static long REQUEUE_DELAY = 500;
     private static final int MAX_REQUEUE = 60;  // 30 sec.
+
+    /**
+     * Get the FloodfillNetworkDatabaseFacade for this runner. This is the client
+     * netDb.
+     * 
+     * If a session has not been created yet, it will return null.
+     * 
+     * @return the client netdb or null if no session was created yet
+     * @since 0.9.61
+     */
+    public FloodfillNetworkDatabaseFacade getFloodfillNetworkDatabaseFacade() {
+        return _floodfillNetworkDatabaseFacade;
+    }
     
     private class MessageDeliveryStatusUpdate extends JobImpl {
         private final SessionId _sessId;
