@@ -43,6 +43,8 @@ import net.i2p.router.ClientMessage;
 import net.i2p.router.Job;
 import net.i2p.router.JobImpl;
 import net.i2p.router.RouterContext;
+import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseSegmentor;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.I2PThread;
 import net.i2p.util.Log;
@@ -89,7 +91,7 @@ class ClientManager {
     /** 2 bytes, save 65535 for unknown */
     private static final int MAX_SESSION_ID = 65534;
     private static final String PROP_MAX_SESSIONS = "i2cp.maxSessions";
-    private static final int DEFAULT_MAX_SESSIONS = 100;
+    private static final int DEFAULT_MAX_SESSIONS = 50;
     /** 65535 */
     public static final SessionId UNKNOWN_SESSION_ID = new SessionId(MAX_SESSION_ID + 1);
 
@@ -431,6 +433,8 @@ class ClientManager {
      *  @since 0.9.12
      */
     private SessionId locked_getNextSessionId() { 
+        if (_ctx.commSystem().isDummy())
+            return null;
         int max = Math.max(1, Math.min(2048, _ctx.getProperty(PROP_MAX_SESSIONS, DEFAULT_MAX_SESSIONS)));
         if (_runnerSessionIds.size() >= max) {
             _log.logAlways(Log.WARN, "Session refused, max is " + max + ", increase " + PROP_MAX_SESSIONS);
@@ -769,6 +773,44 @@ class ClientManager {
         (new HandleJob(msg)).runJob();
     }
 
+    /**
+     * get the FloodfillNetworkDatabaseFacade associated with a particular client destination.
+     * This is inside the runner, so it won't be there if the runner isn't ready.
+     * 
+     * @param destHash destination hash associated with the client who's subDb we're looking for, may be null
+     * @return will be null if desthash is null or client does not exist or its netDb is not initialized
+     * @since 0.9.61
+     */
+    public FloodfillNetworkDatabaseFacade getClientFloodfillNetworkDatabaseFacade(Hash destHash) {
+        if (destHash != null) {
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("Getting subDb for desthash: " + destHash);
+            ClientConnectionRunner runner = getRunner(destHash);
+            if (runner == null){
+                if (_log.shouldLog(Log.WARN))
+                _log.warn("ClientManager got a null runner in getClientFloodfillNetworkDatabaseFacade for " + destHash);
+                return null;
+            }
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("ClientManager got a runner in getClientFloodfillNetworkDatabaseFacade for " + destHash);
+            return runner.getFloodfillNetworkDatabaseFacade();
+        }
+        return null;
+    }
+
+    /**
+     * get all the primary hashes for all the clients and return them as a set
+     * 
+     * @return non-null
+     * @since 0.9.61
+     */
+    public Set<Hash> getPrimaryHashes() {
+        Set<Hash> rv = new HashSet<Hash>();
+        for (ClientConnectionRunner runner : _runners.values())
+            rv.add(runner.getDestHash());
+        return rv;
+    }
+
     private class HandleJob extends JobImpl {
         private final ClientMessage _msg;
 
@@ -835,7 +877,7 @@ class ClientManager {
                 try {
                     // only send version if the client can handle it (0.8.7 or greater)
                     runner.doSend(new SetDateMessage(runner.getClientVersion() != null ?
-                                                     CoreVersion.VERSION : null));
+                                                     CoreVersion.PUBLISHED_VERSION : null));
                 } catch (I2CPMessageException ime) {}
             }
             if (_isStarted)
