@@ -199,7 +199,7 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
             return;
         try {
             // only send version if the client can handle it (0.8.7 or greater)
-            _runner.doSend(new SetDateMessage(clientVersion != null ? CoreVersion.VERSION : null));
+            _runner.doSend(new SetDateMessage(clientVersion != null ? CoreVersion.PUBLISHED_VERSION : null));
         } catch (I2CPMessageException ime) {
             if (_log.shouldLog(Log.ERROR))
                 _log.error("Error writing out the setDate message", ime);
@@ -328,6 +328,15 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
             // force type 3
             props.setProperty("i2cp.leaseSetType", "3");
         }
+        // Ensure we have the nickname properties set
+        String name = props.getProperty("inbound.nickname");
+        if (name == null || name.trim().isEmpty()) {
+            name = dest.toBase32();
+            props.setProperty("inbound.nickname", name);
+        }
+        String name2 = props.getProperty("outbound.nickname");
+        if (name2 == null || name2.trim().isEmpty())
+            props.setProperty("outbound.nickname", name);
         cfg.setOptions(props);
         // this sets the session id
         int status = _runner.sessionEstablished(cfg);
@@ -503,10 +512,6 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
         if (_runner.isDead()) return;
         if (_log.shouldLog(Log.DEBUG))
             _log.debug("Handling receive begin: id = " + message.getMessageId());
-        MessagePayloadMessage msg = new MessagePayloadMessage();
-        msg.setMessageId(message.getMessageId());
-        // TODO validate session id
-        msg.setSessionId(message.getSessionId());
         Payload payload = _runner.getPayload(new MessageId(message.getMessageId()));
         if (payload == null) {
             if (_log.shouldLog(Log.WARN))
@@ -514,7 +519,8 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
                            + "] is null!  Dropped or Unknown message id");
             return;
         }
-        msg.setPayload(payload);
+        // TODO validate session id
+        MessagePayloadMessage msg = new MessagePayloadMessage(message.getSessionId(), message.getMessageId(), payload);
         try {
             _runner.doSend(msg);
         } catch (I2CPMessageException ime) {
@@ -705,13 +711,13 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
             }
             if (_log.shouldDebug())
                 _log.debug("Publishing: " + ls);
-            _context.netDb().publish(ls);
+            _runner.getFloodfillNetworkDatabaseFacade().publish(ls);
             if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
                 // store the decrypted ls also
                 EncryptedLeaseSet encls = (EncryptedLeaseSet) ls;
                 if (_log.shouldDebug())
                     _log.debug("Storing decrypted: " + encls.getDecryptedLeaseSet());
-                _context.netDb().store(dest.getHash(), encls.getDecryptedLeaseSet());
+                _runner.getFloodfillNetworkDatabaseFacade().store(dest.getHash(), encls.getDecryptedLeaseSet());
             }
         } catch (IllegalArgumentException iae) {
             if (_log.shouldLog(Log.ERROR))
@@ -886,6 +892,8 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
                 long nexp = bd.getExpiration();
                 if (nexp > oexp) {
                     obd.setExpiration(nexp);
+                    // to force save at shutdown
+                    _context.netDb().setBlindData(obd);
                     if (_log.shouldWarn())
                         _log.warn("Updated expiration: " + obd);
                 } else {
