@@ -21,12 +21,13 @@ import net.i2p.data.i2cp.I2CPMessage;
 import net.i2p.data.i2cp.I2CPMessageException;
 import net.i2p.data.i2cp.SessionId;
 import net.i2p.router.JobImpl;
+import net.i2p.router.NetworkDatabaseFacade;
 import net.i2p.router.RouterContext;
 import net.i2p.util.Log;
 
 /**
  * Look up the lease of a hash, to convert it to a Destination for the client.
- * Or, since 0.9.11, lookup a host name in the naming service.
+ * Or, since 0.9.11, lookup a hostname in the naming service.
  */
 class LookupDestJob extends JobImpl {
     private final Log _log;
@@ -185,7 +186,10 @@ class LookupDestJob extends JobImpl {
             if (timeout > 1500)
                 timeout -= 500;
             // TODO tell router this is an encrypted lookup, skip 38 or earlier ffs?
-            getContext().netDb().lookupDestination(_hash, done, timeout, _fromLocalDest);
+            NetworkDatabaseFacade db = _runner.getFloodfillNetworkDatabaseFacade();
+            if (db == null)
+                db = getContext().netDb();
+            db.lookupDestination(_hash, done, timeout, _fromLocalDest);
         } else {
             // blinding decode fail
             returnFail(HostReplyMessage.RESULT_DECRYPTION_FAILURE);
@@ -198,10 +202,13 @@ class LookupDestJob extends JobImpl {
         }
         public String getName() { return "LeaseSet Lookup Reply to Client"; }
         public void runJob() {
-            Destination dest = getContext().netDb().lookupDestinationLocally(_hash);
+            NetworkDatabaseFacade db = _runner.getFloodfillNetworkDatabaseFacade();
+            if (db == null)
+                db = getContext().netDb();
+            Destination dest = db.lookupDestinationLocally(_hash);
             if (dest == null && _blindData != null) {
                 // TODO store and lookup original hash instead
-                LeaseSet ls = getContext().netDb().lookupLeaseSetLocally(_hash);
+                LeaseSet ls = db.lookupLeaseSetLocally(_hash);
                 if (ls != null && ls.getType() == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
                     // already decrypted
                     EncryptedLeaseSet encls = (EncryptedLeaseSet) ls;
