@@ -137,6 +137,9 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
     }
 
     /**
+     *  The local destination for this SKM
+     *
+     *  @return our destination or null for the router SKM
      *  @since 0.9.46
      */
     public Destination getDestination() {
@@ -864,7 +867,7 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
                     if (ts.getAcked())
                         buf.append(" acked");
                     buf.append(" created:</b> ").append(DataHelper.formatTime(ts.getCreated()))
-                       .append(" <b>last use:</b> ").append(DataHelper.formatTime(ts.getDate()));
+                       .append(" <b>last used:</b> ").append(DataHelper.formatTime(ts.getDate()));
                     buf.append(" <b>expires in:</b> ").append(DataHelper.formatDuration2(expires)).append(" with ");
                     buf.append(size).append(" tags remaining");
                     if (ts.getNextKey() != null)
@@ -959,12 +962,12 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
         private static final int MIN_RCV_WINDOW_NSR = 12;
         private static final int MAX_RCV_WINDOW_NSR = 12;
         private static final int MIN_RCV_WINDOW_ES = 24;
-        private static final int MAX_RCV_WINDOW_ES = 160;
+        private static final int MAX_RCV_WINDOW_ES = 320;
 
         private static final String INFO_0 = "SessionReplyTags";
         private static final String INFO_7 = "XDHRatchetTagSet";
         private static final int MAX_SEND_ACKS = 16;
-        private static final int MAX_SEND_REVERSE_KEY = 64;
+        private static final int MAX_SEND_REVERSE_KEY = 128;
 
         /**
          * @param d may be null
@@ -1178,8 +1181,11 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
                                                          _context.clock().now(), newtsID, _myOBKeyID);
                     _tagSet = ts;
                     _currentOBTagSetID = newtsID;
-                    if (_log.shouldDebug())
-                        _log.debug("Got nextkey " + key + "\nratchet to new OB ES TS:\n" + ts);
+                    if (_log.shouldWarn())
+                        _log.warn("Got nextkey " + key +
+                                  "from " + (_destination != null ? _destination.toBase32() : "???") +
+                                  "\nold OB TS:\n" + oldts +
+                                  "\nratchet to new OB ES TS:\n" + ts);
                 } else {
                     // this is about my inbound tag set
                     if (key.equals(_hisOBKey)) {
@@ -1273,8 +1279,11 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
                                                          _context.clock().now(), newtsID, _myIBKeyID,
                                                          MAX_RCV_WINDOW_ES, MAX_RCV_WINDOW_ES);
                     _nextIBRootKey = ts.getNextRootKey();
-                    if (_log.shouldDebug())
-                        _log.debug("Got nextkey " + key + "\nratchet to new IB ES TS:\n" + ts);
+                    if (_log.shouldWarn())
+                        _log.warn("Got nextkey " + key +
+                                  "from " + (_destination != null ? _destination.toBase32() : "???") +
+                                  "\nold IB TS ID #" + oldtsID +
+                                  "\nratchet to new IB ES TS:\n" + ts);
                 }
             }
         }
@@ -1435,8 +1444,14 @@ public class RatchetSKM extends SessionKeyManager implements SessionTagListener
                             _tagSet.setDate(now);
                             SessionKeyAndNonce skn = _tagSet.consumeNextKey();
                             // TODO PN
-                            return new RatchetEntry(tag, skn, _tagSet.getID(), 0, _tagSet.getNextKey(),
-                                                    getReverseSendKey(), getAcksToSend());
+                            NextSessionKey fwd = _tagSet.getNextKey();
+                            NextSessionKey rev = getReverseSendKey();
+                            if ((fwd != null || rev != null) && _log.shouldInfo())
+                                _log.info("Sending fwd key: " + fwd +
+                                          " rev key: " + rev +
+                                          " for " + _tagSet);
+                            return new RatchetEntry(tag, skn, _tagSet.getID(), 0, fwd,
+                                                    rev, getAcksToSend());
                         } else if (_log.shouldInfo()) {
                             _log.info("Removing empty " + _tagSet);
                         }
