@@ -209,7 +209,7 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
         _hashPair = new OutboundCache.HashPair(_from.calculateHash(), toHash);
         _toString = toHash.toBase32();
         // we look up here rather than runJob() so we may adjust the timeout
-        _leaseSet = ctx.netDb().lookupLeaseSetLocally(toHash);
+        _leaseSet = ctx.clientNetDb(_from.calculateHash()).lookupLeaseSetLocally(toHash);
         
         // use expiration requested by client if available, otherwise session config,
         // otherwise router config, otherwise default
@@ -260,9 +260,9 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
         ctx.statManager().createFrequencyStat("client.sendMessageFailFrequency", "How often does a client fail to send a message?", "ClientMessages", new long[] { 60*1000l, 60*60*1000l, 24*60*60*1000l });
         ctx.statManager().createRateStat("client.sendMessageSize", "How large are messages sent by the client?", "ClientMessages", new long[] { 60*1000l, 60*60*1000l, 24*60*60*1000l });
         ctx.statManager().createRequiredRateStat("client.sendAckTime", "Message round trip time (ms)", "ClientMessages", new long[] { 60*1000l, 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
-        ctx.statManager().createRateStat("client.timeoutCongestionTunnel", "How lagged our tunnels are when a send times out?", "ClientMessages", new long[] { 60*1000l, 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
-        ctx.statManager().createRateStat("client.timeoutCongestionMessage", "How fast we process messages locally when a send times out?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
-        ctx.statManager().createRateStat("client.timeoutCongestionInbound", "How much faster we are receiving data than our average bps when a send times out?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
+        //ctx.statManager().createRateStat("client.timeoutCongestionTunnel", "How lagged our tunnels are when a send times out?", "ClientMessages", new long[] { 60*1000l, 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
+        //ctx.statManager().createRateStat("client.timeoutCongestionMessage", "How fast we process messages locally when a send times out?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
+        //ctx.statManager().createRateStat("client.timeoutCongestionInbound", "How much faster we are receiving data than our average bps when a send times out?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
         //ctx.statManager().createRateStat("client.leaseSetFoundLocally", "How often we tried to look for a leaseSet and found it locally?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
         ctx.statManager().createRateStat("client.leaseSetFoundRemoteTime", "How long we tried to look for a remote leaseSet (when we succeeded)?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
         ctx.statManager().createRateStat("client.leaseSetFailedRemoteTime", "How long we tried to look for a remote leaseSet (when we failed)?", "ClientMessages", new long[] { 5*60*1000l, 60*60*1000l, 24*60*60*1000l });
@@ -307,7 +307,7 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
                     if (_log.shouldInfo())
                         _log.info(getJobId() + ": RAP LS, firing search: " + _leaseSet.getHash().toBase32());
                     LookupLeaseSetFailedJob failed = new LookupLeaseSetFailedJob(getContext());
-                    getContext().netDb().lookupLeaseSetRemotely(_leaseSet.getHash(), success, failed,
+                    getContext().clientNetDb(_from.calculateHash()).lookupLeaseSetRemotely(_leaseSet.getHash(), success, failed,
                                                                 LS_LOOKUP_TIMEOUT, _from.calculateHash());
                 } else {
                     dieFatal(MessageStatusMessage.STATUS_SEND_FAILURE_NO_LEASESET);
@@ -330,17 +330,17 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
                         long exp = now - _leaseSet.getLatestLeaseDate();
                         _log.info(getJobId() + ": leaseSet expired " + DataHelper.formatDuration(exp) + " ago, firing search: " + _leaseSet.getHash().toBase32());
                     }
-                    getContext().netDb().lookupLeaseSetRemotely(_leaseSet.getHash(), _from.calculateHash());
+                    getContext().clientNetDb(_from.calculateHash()).lookupLeaseSetRemotely(_leaseSet.getHash(), _from.calculateHash());
                 }
             }
             success.runJob();
         } else {
             _leaseSetLookupBegin = getContext().clock().now();
             if (_log.shouldLog(Log.DEBUG))
-                _log.debug(getJobId() + ": Send outbound client message - sending off leaseSet lookup job for " + _toString);
+                _log.debug(getJobId() + ": Send outbound client message - sending off leaseSet lookup job for " + _toString + " from client " + _from.calculateHash().toBase32());
             LookupLeaseSetFailedJob failed = new LookupLeaseSetFailedJob(getContext());
             Hash key = _to.calculateHash();
-            getContext().netDb().lookupLeaseSet(key, success, failed, LS_LOOKUP_TIMEOUT, _from.calculateHash());
+            getContext().clientNetDb(_from.calculateHash()).lookupLeaseSet(key, success, failed, LS_LOOKUP_TIMEOUT, _from.calculateHash());
         }
     }
 
@@ -349,7 +349,7 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
      *  @return lease set or null if we should not send the lease set
      */
     private LeaseSet getReplyLeaseSet(boolean force) {
-        LeaseSet newLS = getContext().netDb().lookupLeaseSetLocally(_from.calculateHash());
+        LeaseSet newLS = getContext().clientNetDb(_from.calculateHash()).lookupLeaseSetLocally(_from.calculateHash());
         if (newLS == null)
             return null;   // punt
 
@@ -423,7 +423,7 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
     private int getNextLease() {
         // set in runJob if found locally
         if (_leaseSet == null || !_leaseSet.getReceivedAsReply()) {
-            _leaseSet = getContext().netDb().lookupLeaseSetLocally(_to.calculateHash());
+            _leaseSet = getContext().clientNetDb(_from.calculateHash()).lookupLeaseSetLocally(_to.calculateHash());
             if (_leaseSet == null) {
                 // shouldn't happen
                 if (_log.shouldLog(Log.WARN))
@@ -587,7 +587,7 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
             
 
             int cause;
-            if (getContext().netDb().isNegativeCachedForever(_to.calculateHash())) {
+            if (getContext().clientNetDb(_from.calculateHash()).isNegativeCachedForever(_to.calculateHash())) {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn("Unable to send to " + _toString + " because the sig type is unsupported");
                 cause = MessageStatusMessage.STATUS_SEND_FAILURE_UNSUPPORTED_ENCRYPTION;
@@ -965,16 +965,8 @@ public class OutboundClientMessageOneShotJob extends JobImpl {
                        " out " + _outTunnel + " in " + _lease + " ack " + _inTunnel +
                        " after " + sendTime + "ms");
         
-        long messageDelay = getContext().throttle().getMessageDelay();
-        long tunnelLag = getContext().throttle().getTunnelLag();
-        long inboundDelta = (long)getContext().throttle().getInboundRateDelta();
-            
-        getContext().statManager().addRateData("client.timeoutCongestionTunnel", tunnelLag, 1);
-        getContext().statManager().addRateData("client.timeoutCongestionMessage", messageDelay, 1);
-        getContext().statManager().addRateData("client.timeoutCongestionInbound", inboundDelta, 1);
-    
         clearCaches();
-        getContext().messageHistory().sendPayloadMessage(_clientMessageId.getMessageId(), false, sendTime);
+        //getContext().messageHistory().sendPayloadMessage(_clientMessageId.getMessageId(), false, sendTime);
         long nonce = _clientMessage.getMessageNonce();
         if (nonce > 0)
             getContext().clientManager().messageDeliveryStatusUpdate(_from, _clientMessageId, nonce, status);
