@@ -56,11 +56,11 @@ class ExploreJob extends SearchJob {
      * @param isRealExplore if true, a standard exploration (no floodfills will be returned)
      *                      if false, a standard lookup (floodfills will be returned, use if low on floodfills)
      */
-    public ExploreJob(RouterContext context, KademliaNetworkDatabaseFacade facade, Hash key, boolean isRealExplore) {
+    public ExploreJob(RouterContext context, KademliaNetworkDatabaseFacade facade, Hash key, boolean isRealExplore, long msgIDBloomXor) {
         // note that we're treating the last param (isLease) as *false* since we're just exploring.
         // if this collides with an actual leaseSet's key, neat, but that wouldn't imply we're actually
         // attempting to send that lease a message!
-        super(context, facade, key, null, null, MAX_EXPLORE_TIME, false, false);
+        super(context, facade, key, null, null, MAX_EXPLORE_TIME, false, false, msgIDBloomXor);
         _peerSelector = (FloodfillPeerSelector) (_facade.getPeerSelector());
         _isRealExplore = isRealExplore;
     }
@@ -119,8 +119,8 @@ class ExploreJob extends SearchJob {
         //    if (len > 0)
         //        msg.getDontIncludePeers().addAll(peers);
         //}
-        
-        available = MAX_CLOSEST - dontIncludePeers.size();
+        //available = MAX_CLOSEST - dontIncludePeers.size();
+
         if (available > 0) {
             // selectNearestExplicit adds our hash to the dontInclude set (3rd param) ...
             // And we end up with MAX_CLOSEST+1 entries.
