@@ -16,13 +16,18 @@ import net.i2p.data.i2np.DatabaseLookupMessage;
 import net.i2p.data.i2np.DatabaseStoreMessage;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.data.router.RouterKeyGenerator;
+import net.i2p.router.peermanager.DBHistory;
+import net.i2p.router.peermanager.PeerProfile;
 import net.i2p.router.Job;
 import net.i2p.router.JobImpl;
 import net.i2p.router.OutNetMessage;
 import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
+import net.i2p.stat.Rate;
+import net.i2p.stat.RateStat;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.Log;
+import net.i2p.util.RandomSource;
 import net.i2p.util.SystemVersion;
 
 /**
@@ -30,6 +35,7 @@ import net.i2p.util.SystemVersion;
  */
 public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacade {
     public static final char CAPABILITY_FLOODFILL = 'f';
+    private static final String MINIMUM_SUBDB_PEERS = "router.subDbMinimumPeers";
     private final Map<Hash, FloodSearchJob> _activeFloodQueries;
     private boolean _floodfillEnabled;
     private final Set<Hash> _verifiesInProgress;
@@ -47,14 +53,27 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
     
     private static final int FLOOD_PRIORITY = OutNetMessage.PRIORITY_NETDB_FLOOD;
     private static final int FLOOD_TIMEOUT = 30*1000;
-    private static final long NEXT_RKEY_RI_ADVANCE_TIME = 45*60*1000;
+    static final long NEXT_RKEY_RI_ADVANCE_TIME = 45*60*1000;
     private static final long NEXT_RKEY_LS_ADVANCE_TIME = 10*60*1000;
     private static final int NEXT_FLOOD_QTY = 2;
     
+    /**
+     *  Main DB
+     */
     public FloodfillNetworkDatabaseFacade(RouterContext context) {
-        super(context);
+        this(context, FloodfillNetworkDatabaseSegmentor.MAIN_DBID);
+    }
+
+    /**
+     *  Sub DBs
+     *
+     *  @param dbid null for main DB
+     *  @since 0.9.61
+     */
+    public FloodfillNetworkDatabaseFacade(RouterContext context, Hash dbid) {
+        super(context, dbid);
         _activeFloodQueries = new HashMap<Hash, FloodSearchJob>();
-         _verifiesInProgress = new ConcurrentHashSet<Hash>(8);
+        _verifiesInProgress = new ConcurrentHashSet<Hash>(8);
 
         long[] rate = new long[] { 60*60*1000L };
         _context.statManager().createRequiredRateStat("netDb.successTime", "Time for successful lookup (ms)", "NetworkDatabase", new long[] { 60*60*1000l, 24*60*60*1000l });
@@ -72,25 +91,43 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
         _context.statManager().createRateStat("netDb.republishQuantity", "How many peers do we need to send a found leaseSet to?", "NetworkDatabase", rate);
         // for ISJ
         _context.statManager().createRateStat("netDb.RILookupDirect", "Was an iterative RI lookup sent directly?", "NetworkDatabase", rate);
-        _ffMonitor = new FloodfillMonitorJob(_context, this);
+        // No need to start the FloodfillMonitorJob for client subDb.
+        if (isClientDb())
+            _ffMonitor = null;
+        else
+            _ffMonitor = new FloodfillMonitorJob(_context, this);
     }
 
     @Override
     public synchronized void startup() {
+        boolean isFF;
         super.startup();
-        _context.jobQueue().addJob(_ffMonitor);
-        _lookupThrottler = new LookupThrottler();
+        if (_ffMonitor != null)
+            _context.jobQueue().addJob(_ffMonitor);
+        if (isClientDb()) {
+            isFF = false;
+        } else {
+            isFF = _context.getBooleanProperty(FloodfillMonitorJob.PROP_FLOODFILL_PARTICIPANT);
+            _lookupThrottler = new LookupThrottler(this);
+        }
 
-        // refresh old routers
-        Job rrj = new RefreshRoutersJob(_context, this);
-        rrj.getTiming().setStartAfter(_context.clock().now() + 5*60*1000);
-        _context.jobQueue().addJob(rrj);
+        long down = _context.router().getEstimatedDowntime();
+        if (!_context.commSystem().isDummy() && !isClientDb() &&
+            (down == 0 || (!isFF && down > 30*60*1000) || (isFF && down > 24*60*60*1000))) {
+            // refresh old routers
+            Job rrj = new RefreshRoutersJob(_context, this);
+            rrj.getTiming().setStartAfter(_context.clock().now() + 5*60*1000);
+            _context.jobQueue().addJob(rrj);
+        }
     }
 
     @Override
     protected void createHandlers() {
-        _context.inNetMessagePool().registerHandlerJobBuilder(DatabaseLookupMessage.MESSAGE_TYPE, new FloodfillDatabaseLookupMessageHandler(_context, this));
-        _context.inNetMessagePool().registerHandlerJobBuilder(DatabaseStoreMessage.MESSAGE_TYPE, new FloodfillDatabaseStoreMessageHandler(_context, this));
+       // Only initialize the handlers for the flooodfill netDb.
+       if (!isClientDb()) {
+            _context.inNetMessagePool().registerHandlerJobBuilder(DatabaseLookupMessage.MESSAGE_TYPE, new FloodfillDatabaseLookupMessageHandler(_context, this));
+            _context.inNetMessagePool().registerHandlerJobBuilder(DatabaseStoreMessage.MESSAGE_TYPE, new FloodfillDatabaseStoreMessageHandler(_context, this));
+        }
     }
     
     /**
@@ -119,7 +156,8 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
                 } catch (InterruptedException ie) {}
             }
         }
-        _context.jobQueue().removeJob(_ffMonitor);
+        if (_ffMonitor != null)
+            _context.jobQueue().removeJob(_ffMonitor);
         super.shutdown();
     }
 
@@ -155,14 +193,24 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
             sendStore(localRouterInfo.getIdentity().calculateHash(), localRouterInfo, null, null, PUBLISH_TIMEOUT, null);
     }
     
+    /**
+     * Send out a store.
+     *
+     * @param key the DatabaseEntry hash
+     * @param onSuccess may be null, always called if we are ff and ds is an RI
+     * @param onFailure may be null, ignored if we are ff and ds is an RI
+     * @param sendTimeout ignored if we are ff and ds is an RI
+     * @param toIgnore may be null, if non-null, all attempted and skipped targets will be added as of 0.9.53,
+     *                 unused if we are ff and ds is an RI
+     */
     @Override
-    public void sendStore(Hash key, DatabaseEntry ds, Job onSuccess, Job onFailure, long sendTimeout, Set<Hash> toIgnore) {
+    void sendStore(Hash key, DatabaseEntry ds, Job onSuccess, Job onFailure, long sendTimeout, Set<Hash> toIgnore) {
         // if we are a part of the floodfill netDb, don't send out our own leaseSets as part 
         // of the flooding - instead, send them to a random floodfill peer so *they* can flood 'em out.
         // perhaps statistically adjust this so we are the source every 1/N times... or something.
         if (floodfillEnabled() && (ds.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO)) {
             flood(ds);
-            if (onSuccess != null) 
+            if (onSuccess != null)
                 _context.jobQueue().addJob(onSuccess);
         } else {
             _context.jobQueue().addJob(new FloodfillStoreJob(_context, this, key, ds, onSuccess, onFailure, sendTimeout, toIgnore));
@@ -196,7 +244,10 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
     public boolean floodConditional(DatabaseEntry ds) {
         if (!floodfillEnabled())
             return false;
-        if (shouldThrottleFlood(ds.getHash())) {
+        Hash h = ds.getHash();
+        if (_context.banlist().isBanlistedForever(h))
+            return false;
+        if (shouldThrottleFlood(h)) {
             _context.statManager().addRateData("netDb.floodThrottled", 1);
             return false;
         }
@@ -304,9 +355,11 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
        if (peer.equals(_context.routerHash()))
            return false;
        // min version checks
+/*
        if (type != DatabaseEntry.KEY_TYPE_ROUTERINFO && type != DatabaseEntry.KEY_TYPE_LEASESET &&
            !StoreJob.shouldStoreLS2To(target))
            return false;
+*/
        if ((type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2 ||
             lsSigType == SigType.RedDSA_SHA512_Ed25519) &&
            !StoreJob.shouldStoreEncLS2To(target))
@@ -347,16 +400,13 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
         }
     }
 
-    @Override
-    protected PeerSelector createPeerSelector() { return new FloodfillPeerSelector(_context); }
-    
     /**
      *  Public, called from console. This wakes up the floodfill monitor,
      *  which will rebuild the RI and log in the event log,
      *  and call setFloodfillEnabledFromMonitor which really sets it.
      */
     public synchronized void setFloodfillEnabled(boolean yes) {
-        if (yes != _floodfillEnabled) {
+        if ((yes != _floodfillEnabled) && (_ffMonitor != null)) {
             _context.jobQueue().removeJob(_ffMonitor);
             _ffMonitor.getTiming().setStartAfter(_context.clock().now() + 1000);
             _context.jobQueue().addJob(_ffMonitor);
@@ -379,7 +429,9 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
     }
 
     @Override
-    public boolean floodfillEnabled() { return _floodfillEnabled; }
+    public boolean floodfillEnabled() { 
+        return _floodfillEnabled;
+    }
     
     /**
      *  @param peer may be null, returns false if null
@@ -387,7 +439,7 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
     public static boolean isFloodfill(RouterInfo peer) {
         if (peer == null) return false;
         String caps = peer.getCapabilities();
-        return caps.indexOf(FloodfillNetworkDatabaseFacade.CAPABILITY_FLOODFILL) >= 0;
+        return caps.indexOf(CAPABILITY_FLOODFILL) >= 0;
     }
 
     public List<RouterInfo> getKnownRouterData() {
@@ -431,6 +483,7 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
                      Hash fromLocalDest) {
         //if (true) return super.search(key, onFindJob, onFailedLookupJob, timeoutMs, isLease);
         if (key == null) throw new IllegalArgumentException("searchin for nothin, eh?");
+        if (fromLocalDest == null && isClientDb()) throw new IllegalArgumentException("client subDbs cannot use exploratory tunnels");
         boolean isNew = false;
         FloodSearchJob searchJob;
         synchronized (_activeFloodQueries) {
@@ -449,12 +502,15 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
         }
         
         if (isNew) {
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug("this is the first search for that key, fire off the FloodSearchJob");
+            if (_log.shouldDebug())
+                _log.debug("[dbid: " + this
+                           + "]: New ISJ ("
+                           + ((fromLocalDest != null) ? "through client tunnels" : "through exploratory tunnels")
+                           + ") for " + key.toBase64());
             _context.jobQueue().addJob(searchJob);
         } else {
-            if (_log.shouldLog(Log.INFO))
-                _log.info("Deferring flood search for " + key.toBase64() + " with " + _activeFloodQueries.size() + " in progress");
+            if (_log.shouldDebug())
+                _log.debug("Wait for pending ISJ for " + key.toBase64());
             searchJob.addDeferred(onFindJob, onFailedLookupJob, timeoutMs, isLease);
             // not necessarily LS
             _context.statManager().addRateData("netDb.lookupDeferred", 1, searchJob.getExpiration()-_context.clock().now());
@@ -564,13 +620,41 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
       */
     @Override
     protected void lookupBeforeDropping(Hash peer, RouterInfo info) {
+        if (_context.commSystem().isEstablished(peer)) {
+            // see DirectLookupJob
+            boolean isNew = false;
+            FloodSearchJob searchJob;
+            Job onFindJob = new DropLookupFoundJob(_context, peer, info);
+            Job onFailedLookupJob = new DropLookupFailedJob(_context, peer, info);
+            synchronized (_activeFloodQueries) {
+                searchJob = _activeFloodQueries.get(peer);
+                if (searchJob == null) {
+                    searchJob = new DirectLookupJob(_context, this, peer, info, onFindJob, onFailedLookupJob);
+                    _activeFloodQueries.put(peer, searchJob);
+                    isNew = true;
+                }
+            }
+            if (isNew) {
+                if (_log.shouldDebug())
+                    _log.debug("Direct RI lookup for " + peer.toBase64());
+                _context.jobQueue().addJob(searchJob);
+            } else {
+                if (_log.shouldDebug())
+                    _log.debug("Pending Direct RI lookup for " + peer.toBase64());
+                searchJob.addDeferred(onFindJob, onFailedLookupJob, 10*1000, false);
+            }
+            return;
+        }
+
         // following are some special situations, we don't want to
         // drop the peer in these cases
         // yikes don't do this - stack overflow //  getFloodfillPeers().size() == 0 ||
         // yikes2 don't do this either - deadlock! // getKnownRouters() < MIN_REMAINING_ROUTERS ||
+        long uptime = _context.router().getUptime();
+        int knownRouters = getKBucketSetSize();
         if (info.getNetworkId() == _networkID &&
-            (getKBucketSetSize() < MIN_REMAINING_ROUTERS ||
-             _context.router().getUptime() < DONT_FAIL_PERIOD ||
+            (knownRouters < MIN_REMAINING_ROUTERS ||
+             uptime < DONT_FAIL_PERIOD ||
              _context.commSystem().countActivePeers() <= MIN_ACTIVE_PEERS)) {
             if (_log.shouldInfo())
                 _log.info("Not failing " + peer.toBase64() + " as we are just starting up or have problems");
@@ -579,9 +663,10 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
 
         // should we skip the search?
         if (_floodfillEnabled ||
+            knownRouters > MAX_DB_BEFORE_SKIPPING_SEARCH ||
             _context.jobQueue().getMaxLag() > 500 ||
-            _context.banlist().isBanlistedForever(peer) ||
-            getKBucketSetSize() > MAX_DB_BEFORE_SKIPPING_SEARCH) {
+            _context.router().gracefulShutdownInProgress() ||
+            _context.banlist().isBanlistedForever(peer)) {
             // don't try to overload ourselves (e.g. failing 3000 router refs at
             // once, and then firing off 3000 netDb lookup tasks)
             // Also don't queue a search if we have plenty of routerinfos
@@ -589,21 +674,95 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
             super.lookupBeforeDropping(peer, info);
             return; 
         }
+
+        // The following doesn't kick in most of the time, because of
+        // the MAX_DB_BEFORE_SKIPPING_SEARCH check above,
+        // that value is pretty small compared to typical netdb sizes.
+
+        String caps = info.getCapabilities();
+        if (caps.indexOf(Router.CAPABILITY_UNREACHABLE) >= 0 ||
+            caps.indexOf(Router.CAPABILITY_BW32) >= 0) {
+            super.lookupBeforeDropping(peer, info);
+            return; 
+        }
+        if (caps.indexOf(CAPABILITY_FLOODFILL) >= 0) {
+            PeerProfile prof = _context.profileOrganizer().getProfile(peer);
+            if (prof == null) {
+                //_log.warn("skip lookup no profile " + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+            // classification similar to that in FloodfillPeerSelector
+            long now = _context.clock().now();
+            long installed = _context.getProperty("router.firstInstalled", 0L);
+            boolean enforceHeard = installed > 0 && (now - installed) > 2*60*60*1000;
+            if (enforceHeard && prof.getFirstHeardAbout() > now - 3*60*60*1000) {
+                //_log.warn("skip lookup new " + DataHelper.formatTime(prof.getFirstHeardAbout()) + ' ' + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+            DBHistory hist = prof.getDBHistory();
+            if (hist == null) {
+                //_log.warn("skip lookup no dbhist " + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+            long cutoff = now - 30*60*1000;
+            long lastLookupSuccess = hist.getLastLookupSuccessful();
+            long lastStoreSuccess = hist.getLastStoreSuccessful();
+            if (uptime > 30*60*1000 &&
+                lastLookupSuccess < cutoff &&
+                lastStoreSuccess < cutoff) {
+                //_log.warn("skip lookup no db success " + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+            cutoff = now - 2*60*60*1000;
+            long lastLookupFailed = hist.getLastLookupFailed();
+            long lastStoreFailed = hist.getLastStoreFailed();
+            if (lastLookupFailed > cutoff ||
+                lastStoreFailed > cutoff ||
+                lastLookupFailed > lastLookupSuccess ||
+                lastStoreFailed > lastStoreSuccess) {
+                //_log.warn("skip lookup dbhist store fail " + DataHelper.formatTime(lastStoreFailed) + " lookup fail " + DataHelper.formatTime(lastLookupFailed) + ' ' + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+            double maxFailRate = 0.95;
+            if (_context.router().getUptime() > 60*60*1000) {
+                RateStat rs = _context.statManager().getRate("peer.failedLookupRate");
+                if (rs != null) {
+                    Rate r = rs.getRate(60*60*1000);
+                    if (r != null) {
+                        double currentFailRate = r.getAverageValue();
+                        maxFailRate = Math.min(0.95d, Math.max(0.20d, 1.25d * currentFailRate));
+                    }
+                }
+            }
+            double failRate = hist.getFailedLookupRate().getRate(60*60*1000).getAverageValue();
+            if (failRate >= 1 || failRate > maxFailRate) {
+                //_log.warn("skip lookup fail rate " + failRate + " max " + maxFailRate + ' ' + peer.toBase64());
+                super.lookupBeforeDropping(peer, info);
+                return; 
+            }
+        }
+
+
         // this sends out the search to the floodfill peers even if we already have the
         // entry locally, firing no job if it gets a reply with an updated value (meaning
         // we shouldn't drop them but instead use the new data), or if they all time out,
         // firing the dropLookupFailedJob, which actually removes out local reference
+        if (_log.shouldDebug())
+            _log.debug("ISJ lookup before dropping for " + peer.toBase64() + ' ' + info.getPublished());
         search(peer, new DropLookupFoundJob(_context, peer, info), new DropLookupFailedJob(_context, peer, info), 10*1000, false);
     }
     
     private class DropLookupFailedJob extends JobImpl {
         private final Hash _peer;
-        private final RouterInfo _info;
     
         public DropLookupFailedJob(RouterContext ctx, Hash peer, RouterInfo info) {
             super(ctx);
             _peer = peer;
-            _info = info;
         }
         public String getName() { return "Lookup on failure of netDb peer timed out"; }
         public void runJob() {
@@ -623,10 +782,8 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
         public String getName() { return "Lookup on failure of netDb peer matched"; }
         public void runJob() {
             RouterInfo updated = lookupRouterInfoLocally(_peer);
-            if ( (updated != null) && (updated.getPublished() > _info.getPublished()) ) {
-                // great, a legitimate update
-            } else {
-                // they just sent us what we already had.  kill 'em both
+            if (updated == null || updated.getPublished() <= _info.getPublished()) {
+                // they just sent us what we already had
                 dropAfterLookupFailed(_peer);
             }
         }
