@@ -178,7 +178,7 @@ class FloodfillPeerSelector extends PeerSelector {
     /** this must be longer than the max streaming timeout (60s) */
     private static final int NO_FAIL_LOOKUP_OK = 75*1000;
     private static final int NO_FAIL_LOOKUP_GOOD = NO_FAIL_LOOKUP_OK * 3;
-    private static final int MAX_GOOD_RESP_TIME = 5*1000;
+    private static final int MAX_GOOD_RESP_TIME = 3500;
     // TODO we need better tracking of floodfill first-heard-about times
     // before we can do this. Old profiles get deleted.
     //private static final long HEARD_AGE = 48*60*60*1000L;
@@ -223,20 +223,20 @@ class FloodfillPeerSelector extends PeerSelector {
         long installed = _context.getProperty("router.firstInstalled", 0L);
         boolean enforceHeard = installed > 0 && (now - installed) > INSTALL_AGE;
 
-        double maxFailRate = 100;
+        double maxFailRate = 0.95;
         if (_context.router().getUptime() > 60*60*1000) {
             RateStat rs = _context.statManager().getRate("peer.failedLookupRate");
             if (rs != null) {
                 Rate r = rs.getRate(60*60*1000);
                 if (r != null) {
                     double currentFailRate = r.getAverageValue();
-                    maxFailRate = Math.max(0.20d, 1.5d * currentFailRate);
+                    maxFailRate = Math.min(0.95d, Math.max(0.20d, 1.25d * currentFailRate));
                 }
             }
         }
 
         // 5 == FNDF.MAX_TO_FLOOD + 1
-        int limit = Math.max(5, howMany + 2);
+        int limit = Math.max(5, howMany * 2);
         limit = Math.min(limit, sorted.size());
         MaskedIPSet maskedIPs = new MaskedIPSet(limit * 3);
         // split sorted list into 3 sorted lists
@@ -248,7 +248,7 @@ class FloodfillPeerSelector extends PeerSelector {
             if (entry == null)
                 break;  // shouldn't happen
             // put anybody in the same /16 at the end
-            RouterInfo info = _context.netDb().lookupRouterInfoLocally(entry);
+            RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(entry);
             MaskedIPSet entryIPs = new MaskedIPSet(_context, entry, info, 2);
             boolean sameIP = false;
             for (String ip : entryIPs) {
@@ -286,7 +286,7 @@ class FloodfillPeerSelector extends PeerSelector {
                             _log.debug("Bad (new): " + entry);
                         badff.add(entry);
                     } else if (prof.getDBHistory() != null) {
-                        if (prof.getDbResponseTime().getRate(10*60*1000).getAverageValue() < maxGoodRespTime
+                        if (prof.getDbResponseTime().getRate(60*60*1000L).getAvgOrLifetimeAvg() < maxGoodRespTime
                             && prof.getDBHistory().getLastStoreFailed() < now - NO_FAIL_STORE_GOOD
                             && prof.getDBHistory().getLastLookupFailed() < now - NO_FAIL_LOOKUP_GOOD
                             && prof.getDBHistory().getFailedLookupRate().getRate(60*60*1000).getAverageValue() < maxFailRate) {
@@ -373,7 +373,7 @@ class FloodfillPeerSelector extends PeerSelector {
             // ... unless they are really bad
             if (_context.banlist().isBanlistedForever(entry))
                 return;
-            RouterInfo info = _context.netDb().lookupRouterInfoLocally(entry);
+            RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(entry);
             //if (info == null)
             //    return;
             
@@ -419,7 +419,7 @@ class FloodfillPeerSelector extends PeerSelector {
             // (Forever banlisted ones are excluded in add() above)
             for (Iterator<Hash> iter = new RandomIterator<Hash>(_floodfillMatches); (found < howMany) && iter.hasNext(); ) {
                 Hash entry = iter.next();
-                RouterInfo info = _context.netDb().lookupRouterInfoLocally(entry);
+                RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(entry);
                 if (info != null && now - info.getPublished() > 3*60*60*1000) {
                     badff.add(entry);
                     if (_log.shouldLog(Log.DEBUG))
