@@ -8,7 +8,6 @@ package net.i2p.router.networkdb.kademlia;
  *
  */
 
-import java.util.NoSuchElementException;
 import java.util.Set;
 
 import net.i2p.data.DatabaseEntry;
@@ -29,6 +28,7 @@ class FloodfillStoreJob extends StoreJob {
     private final FloodfillNetworkDatabaseFacade _facade;
 
     private static final String PROP_RI_VERIFY = "router.verifyRouterInfoStore";
+    private static final long RI_VERIFY_STARTUP_TIME = 3*60*60*1000L;
 
     /**
      * Send a data structure to the floodfills
@@ -69,7 +69,8 @@ class FloodfillStoreJob extends StoreJob {
                     _log.info("Skipping verify, one already in progress for: " + key);
                 return;
             }
-            if (getContext().router().gracefulShutdownInProgress()) {
+            RouterContext ctx = getContext();
+            if (ctx.router().gracefulShutdownInProgress()) {
                 if (shouldLog)
                     _log.info("Skipping verify, shutdown in progress for: " + key);
                 return;
@@ -80,7 +81,9 @@ class FloodfillStoreJob extends StoreJob {
             final int type = data.getType();
             final boolean isRouterInfo = type == DatabaseEntry.KEY_TYPE_ROUTERINFO;
             // default false since 0.9.7.1
-            if (isRouterInfo && !getContext().getBooleanProperty(PROP_RI_VERIFY)) {
+            // verify for a while after startup until we've vetted the floodfills
+            if (isRouterInfo && !ctx.getBooleanProperty(PROP_RI_VERIFY) &&
+                ctx.router().getUptime() > RI_VERIFY_STARTUP_TIME) {
                 _facade.routerInfoPublishSuccessful();
                 return;
             }
@@ -94,10 +97,7 @@ class FloodfillStoreJob extends StoreJob {
                 published = data.getDate();
             }
             // we should always have exactly one successful entry
-            Hash sentTo = null;
-            try {
-                sentTo = _state.getSuccessful().iterator().next();
-            } catch (NoSuchElementException nsee) {}
+            Hash sentTo = _state.getSuccessful();
             Hash client;
             if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
                 // get the real client hash
@@ -105,13 +105,13 @@ class FloodfillStoreJob extends StoreJob {
             } else {
                 client = key;
             }
-            Job fvsj = new FloodfillVerifyStoreJob(getContext(), key, client,
+            Job fvsj = new FloodfillVerifyStoreJob(ctx, key, client,
                                                    published, type,
-                                                   sentTo, _facade);
+                                                   sentTo, _state.getAttempted(), _facade);
             if (shouldLog)
                 _log.info(getJobId() + ": Succeeded sending key " + key +
                           ", queueing verify job " + fvsj.getJobId());
-            getContext().jobQueue().addJob(fvsj);
+            ctx.jobQueue().addJob(fvsj);
     }
     
     @Override
