@@ -14,6 +14,7 @@ import net.i2p.crypto.EncType;
 import net.i2p.crypto.SigType;
 import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
+import net.i2p.data.Destination;
 import net.i2p.data.Hash;
 import net.i2p.data.i2np.DatabaseLookupMessage;
 import net.i2p.data.i2np.I2NPMessage;
@@ -102,7 +103,7 @@ public class IterativeSearchJob extends FloodSearchJob {
      */
     private static final long SINGLE_SEARCH_TIME = 3*1000;
     /** the actual expire time for a search message */
-    private static final long SINGLE_SEARCH_MSG_TIME = 10*1000;
+    private static final long SINGLE_SEARCH_MSG_TIME = 20*1000;
     /**
      *  Use instead of CONCURRENT_SEARCHES in super() which is final.
      *  For now, we don't do concurrent, but we keep SINGLE_SEARCH_TIME very short,
@@ -236,7 +237,8 @@ public class IterativeSearchJob extends FloodSearchJob {
         Job onTimeout = new FloodOnlyLookupTimeoutJob(getContext(), this);
         _out = getContext().messageRegistry().registerPending(replySelector, onReply, onTimeout);
         if (_log.shouldLog(Log.INFO))
-            _log.info(getJobId() + ": New ISJ for " +
+            _log.info("JobId: " + getJobId() + "; dbid: " + _facade
+                      + ": New ISJ for " +
                       (_isLease ? "LS " : "RI ") +
                       _key + " (rkey " + _rkey + ") timeout " +
                       DataHelper.formatDuration(_timeoutMs) + " toTry: "  + DataHelper.toString(_toTry));
@@ -305,14 +307,18 @@ public class IterativeSearchJob extends FloodSearchJob {
                 }
                 _unheardFrom.add(peer);
             }
-            sendQuery(peer);
+            sendQuery(peer, done + pend);
         }
     }
 
     /**
      *  Send a DLM to the peer
+     *
+     *  @param peer who to send to
+     *  @param previouslyTried how many did we send to before this one?
+     *  @since 0.9.53 added previouslyTried param
      */
-    private void sendQuery(Hash peer) {
+    private void sendQuery(Hash peer, int previouslyTried) {
             final RouterContext ctx = getContext();
             TunnelManagerFacade tm = ctx.tunnelManager();
             RouterInfo ri = ctx.netDb().lookupRouterInfoLocally(peer);
@@ -321,8 +327,7 @@ public class IterativeSearchJob extends FloodSearchJob {
                 // querying old floodfills that don't know about those sig types.
                 // This is also more recent than the version that supports encrypted replies,
                 // so we won't request unencrypted replies anymore either.
-                if (!StoreJob.shouldStoreTo(ri) ||
-                    (_isLease && !StoreJob.shouldStoreLS2To(ri))) {
+                if (!StoreJob.shouldStoreTo(ri)) {
                     failed(peer, false);
                     if (_log.shouldInfo())
                         _log.info(getJobId() + ": not sending query to old router: " + ri);
@@ -336,9 +341,19 @@ public class IterativeSearchJob extends FloodSearchJob {
             boolean supportsRatchet = false;
             boolean supportsElGamal = true;
             if (_fromLocalDest != null) {
-                outTunnel = tm.selectOutboundTunnel(_fromLocalDest, peer);
-                if (outTunnel == null)
-                    outTunnel = tm.selectOutboundExploratoryTunnel(peer);
+                // For all tunnel selections, the first time we pick the tunnel with the far-end closest
+                // to the target. After that we pick a random tunnel, or else we'd pick the
+                // same tunnels every time.
+                if (previouslyTried <= 0)
+                    outTunnel = tm.selectOutboundTunnel(_fromLocalDest, peer);
+                else
+                    outTunnel = tm.selectOutboundTunnel(_fromLocalDest);
+                if (outTunnel == null) {
+                    if (previouslyTried <= 0)
+                        outTunnel = tm.selectOutboundExploratoryTunnel(peer);
+                    else
+                        outTunnel = tm.selectOutboundTunnel();
+                }
                 LeaseSetKeys lsk = ctx.keyManager().getKeys(_fromLocalDest);
                 supportsRatchet = lsk != null &&
                                   lsk.isSupported(EncType.ECIES_X25519) &&
@@ -348,15 +363,25 @@ public class IterativeSearchJob extends FloodSearchJob {
                                   lsk.isSupported(EncType.ELGAMAL_2048);
                 if (supportsElGamal || supportsRatchet) {
                     // garlic encrypt to dest SKM
-                    replyTunnel = tm.selectInboundTunnel(_fromLocalDest, peer);
+                    if (previouslyTried <= 0)
+                        replyTunnel = tm.selectInboundTunnel(_fromLocalDest, peer);
+                    else
+                        replyTunnel = tm.selectInboundTunnel(_fromLocalDest);
                     isClientReplyTunnel = replyTunnel != null;
-                    if (!isClientReplyTunnel)
-                        replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                    if (!isClientReplyTunnel) {
+                        if (previouslyTried <= 0)
+                            replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                        else
+                            replyTunnel = tm.selectInboundTunnel();
+                    }
                 } else {
                     // We don't have a way to request/get a ECIES-tagged reply,
                     // so send it to the router SKM
                     isClientReplyTunnel = false;
-                    replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                    if (previouslyTried <= 0)
+                        replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                    else
+                        replyTunnel = tm.selectInboundTunnel();
                 }
                 isDirect = false;
             } else if ((!_isLease) && ri != null && ctx.commSystem().isEstablished(peer)) {
@@ -369,10 +394,18 @@ public class IterativeSearchJob extends FloodSearchJob {
                 replyTunnel = null;
                 isClientReplyTunnel = false;
                 isDirect = true;
+                if (_facade.isClientDb() && _log.shouldLog(Log.WARN))
+                    _log.warn("[JobId: " + getJobId() + "; dbid: " + _facade
+                              + "]: Warning! Direct search selected in a client netDb context!");
                 ctx.statManager().addRateData("netDb.RILookupDirect", 1);
             } else {
-                outTunnel = tm.selectOutboundExploratoryTunnel(peer);
-                replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                if (previouslyTried <= 0) {
+                    outTunnel = tm.selectOutboundExploratoryTunnel(peer);
+                    replyTunnel = tm.selectInboundExploratoryTunnel(peer);
+                } else {
+                    outTunnel = tm.selectOutboundTunnel();
+                    replyTunnel = tm.selectInboundTunnel();
+                }
                 isClientReplyTunnel = false;
                 isDirect = false;
                 ctx.statManager().addRateData("netDb.RILookupDirect", 0);
@@ -497,6 +530,10 @@ public class IterativeSearchJob extends FloodSearchJob {
             if (outMsg == null)
                 outMsg = dlm;
             if (isDirect) {
+                if (_facade.isClientDb() && _log.shouldLog(Log.WARN))
+                    _log.warn("[JobId: " + getJobId() + "; dbid: " + _facade
+                              + "]: Warning! Sending direct search message in a client netDb context! "
+                              + outMsg);
                 OutNetMessage m = new OutNetMessage(ctx, outMsg, outMsg.getMessageExpiration(),
                                                     OutNetMessage.PRIORITY_MY_NETDB_LOOKUP, ri);
                 // Should always succeed, we are connected already
@@ -665,6 +702,7 @@ public class IterativeSearchJob extends FloodSearchJob {
         // we will credit the wrong one.
         int tries;
         Hash peer = null;
+
         synchronized(this) {
             if (_dead) return;
             _dead = true;
@@ -675,6 +713,7 @@ public class IterativeSearchJob extends FloodSearchJob {
                 _unheardFrom.clear();
             }
         }
+
         _facade.complete(_key);
         if (peer != null) {
             Long timeSent = _sentTime.get(peer);
