@@ -23,6 +23,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Set;
 import java.util.NoSuchElementException;
 import java.util.concurrent.ConcurrentHashMap;
 
@@ -34,6 +35,7 @@ import net.i2p.data.router.RouterInfo;
 import net.i2p.router.JobImpl;
 import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
+import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.FileSuffixFilter;
 import net.i2p.util.FileUtil;
 import net.i2p.util.I2PThread;
@@ -62,6 +64,7 @@ public class PersistentDataStore extends TransientDataStore {
     private static final String PROP_FLAT = "router.networkDatabase.flat";
     static final String DIR_PREFIX = "r";
     private static final String B64 = Base64.ALPHABET_I2P;
+    private static final int MAX_ROUTERS_INIT = SystemVersion.isSlow() ? 1000 : 4000;
 
     /**
      *  @param dbDir relative path
@@ -96,11 +99,6 @@ public class PersistentDataStore extends TransientDataStore {
         _writer.flush();
     }
     
-    @Override
-    public void restart() {
-        super.restart();
-    }
-    
     @Override
     public void rescan() {
         if (_initialized)
@@ -144,7 +142,6 @@ public class PersistentDataStore extends TransientDataStore {
     public DatabaseEntry remove(Hash key, boolean persist) {
         if (persist) {
             _writer.remove(key);
-            _context.jobQueue().addJob(new RemoveJob(key));
         }
         return super.remove(key);
     }
@@ -168,24 +165,6 @@ public class PersistentDataStore extends TransientDataStore {
         return rv;
     }
     
-    private class RemoveJob extends JobImpl {
-        private final Hash _key;
-        public RemoveJob(Hash key) {
-            super(PersistentDataStore.this._context);
-            _key = key;
-        }
-        public String getName() { return "Delete RI file"; }
-        public void runJob() {
-            if (_log.shouldLog(Log.INFO))
-                _log.info("Removing key " + _key /* , getAddedBy() */);
-            try {
-                removeFile(_key, _dbDir);
-            } catch (IOException ioe) {
-                _log.error("Error removing key " + _key, ioe);
-            }
-        }
-    }
-    
     /** How many files to write every 10 minutes. Doesn't make sense to limit it,
      *  they just back up in the queue hogging memory.
      */
@@ -202,16 +181,19 @@ public class PersistentDataStore extends TransientDataStore {
      */
     private class Writer implements Runnable, Flushable {
         private final Map<Hash, DatabaseEntry>_keys;
+        private final Set<Hash> _keysToRemove;
         private final Object _waitLock;
         private volatile boolean _quit;
 
         public Writer() { 
             _keys = new ConcurrentHashMap<Hash, DatabaseEntry>(64);
+            _keysToRemove = new ConcurrentHashSet<Hash>();
             _waitLock = new Object();
         }
 
         public void queue(Hash key, DatabaseEntry data) {
             int pending = _keys.size();
+            _keysToRemove.remove(key);
             boolean exists = (null != _keys.put(key, data));
             if (exists)
                 _context.statManager().addRateData("netDb.writeClobber", pending);
@@ -220,6 +202,25 @@ public class PersistentDataStore extends TransientDataStore {
 
         public void remove(Hash key) {
             _keys.remove(key);
+            _keysToRemove.add(key);
+        }
+
+        /*
+         *  @since 0.9.50 was in separate RemoveJob
+         */
+        private void removeQueued() {
+            if (_keysToRemove.isEmpty())
+                return;
+            for (Iterator<Hash> iter = _keysToRemove.iterator(); iter.hasNext(); ) {
+                Hash key = iter.next();
+                iter.remove();
+                try {
+                    removeFile(key, _dbDir);
+                } catch (IOException ioe) {
+                    if (_log.shouldWarn())
+                        _log.warn("Error removing key " + key, ioe);
+                }
+            }
         }
 
         public void run() {
@@ -260,6 +261,7 @@ public class PersistentDataStore extends TransientDataStore {
                 if (count >= WRITE_LIMIT)
                     count = 0;
                 if (count == 0) {
+                    removeQueued();
                     if (lastCount > 0) {
                         long time = _context.clock().now() - startTime;
                         if (_log.shouldLog(Log.INFO))
@@ -360,7 +362,7 @@ public class PersistentDataStore extends TransientDataStore {
                 requeue(READ_DELAY);
                 return;
             }
-            long now = getContext().clock().now();
+            long now = System.currentTimeMillis();
             // check directory mod time to save a lot of object churn in scanning all the file names
             long lastMod = _dbDir.lastModified();
             // if size() (= RI + LS) is too low, call anyway to check for reseed
@@ -402,7 +404,7 @@ public class PersistentDataStore extends TransientDataStore {
                         if (!_context.router().isAlive())
                             break;
                         Hash key = getRouterInfoHash(routerInfoFiles[i].getName());
-                        if ( (key != null) && (!isKnown(key)) ) {
+                        if (key != null) {
                             // Run it inline so we don't clog up the job queue, esp. at startup
                             // Also this allows us to wait until it is really done to call checkReseed() and set _initialized
                             //PersistentDataStore.this._context.jobQueue().addJob(new ReadRouterJob(routerInfoFiles[i], key));
@@ -438,9 +440,16 @@ public class PersistentDataStore extends TransientDataStore {
                 Collections.shuffle(toRead, _context.random());
                 int i = 0;
                 for (File file : toRead) {
+                    // Take the first 4000 good ones, delete the rest
+                    if (i >= MAX_ROUTERS_INIT && !_initialized) {
+                        file.delete();
+                        continue;
+                    }
                     Hash key = getRouterInfoHash(file.getName());
-                    if (key != null && !isKnown(key)) {
-                        (new ReadRouterJob(file, key)).runJob();
+                    if (key != null) {
+                        ReadRouterJob rrj = new ReadRouterJob(file, key);
+                        if (!rrj.read())
+                            continue;
                         if (i++ == 150 && SystemVersion.isSlow() && !_initialized) {
                             // Can take 2 minutes to load them all on Android,
                             // after we have already built expl. tunnels.
@@ -524,7 +533,15 @@ public class PersistentDataStore extends TransientDataStore {
         }
 
         public void runJob() {
-            if (!shouldRead()) return;
+            read();
+        }
+
+        /**
+         *  @return success
+         *  @since 0.9.58
+         */
+        public boolean read() {
+            if (!shouldRead()) return false;
             if (_log.shouldLog(Log.DEBUG))
                 _log.debug("Reading " + _routerFile);
 
@@ -535,18 +552,19 @@ public class PersistentDataStore extends TransientDataStore {
                     fis = new BufferedInputStream(fis);
                     RouterInfo ri = new RouterInfo();
                     ri.readBytes(fis, true);  // true = verify sig on read
+                    Hash h = ri.getIdentity().calculateHash();
                     if (ri.getNetworkId() != _networkID) {
                         corrupt = true;
                         if (_log.shouldLog(Log.ERROR))
                             _log.error("The router "
-                                       + ri.getIdentity().calculateHash().toBase64() 
+                                       + h.toBase64() 
                                        + " is from a different network");
-                    } else if (!ri.getIdentity().calculateHash().equals(_key)) {
+                    } else if (!h.equals(_key)) {
                         // prevent injection from reseeding
                         // this is checked in KNDF.validate() but catch it sooner and log as error.
                         corrupt = true;
                         if (_log.shouldLog(Log.WARN))
-                            _log.warn(ri.getIdentity().calculateHash() + " does not match " + _key + " from " + _routerFile);
+                            _log.warn(h + " does not match " + _key + " from " + _routerFile);
                     } else if (ri.getPublished() <= _knownDate) {
                         // Don't store but don't delete
                         if (_log.shouldLog(Log.WARN))
@@ -554,15 +572,16 @@ public class PersistentDataStore extends TransientDataStore {
                     } else if (getContext().blocklist().isBlocklisted(ri)) {
                         corrupt = true;
                         if (_log.shouldLog(Log.WARN))
-                            _log.warn(ri.getHash() + " is blocklisted");
+                            _log.warn(h + " is blocklisted");
                     } else {
                         try {
                             // persist = false so we don't write what we just read
-                            _facade.store(ri.getIdentity().getHash(), ri, false);
+                            _facade.store(h, ri, false);
                             // when heardAbout() was removed from TransientDataStore, it broke
                             // profile bootstrapping for new routers,
                             // so add it here.
-                            getContext().profileManager().heardAbout(ri.getIdentity().getHash(), ri.getPublished());
+                            if (ri.getCapabilities().indexOf(Router.CAPABILITY_REACHABLE) >= 0)
+                                getContext().profileManager().heardAbout(h, ri.getPublished());
                         } catch (IllegalArgumentException iae) {
                             if (_log.shouldLog(Log.INFO))
                                 _log.info("Refused locally loaded routerInfo - deleting", iae);
@@ -586,6 +605,7 @@ public class PersistentDataStore extends TransientDataStore {
                     if (fis != null) try { fis.close(); } catch (IOException ioe) {}
                 }
                 if (corrupt) _routerFile.delete();
+                return !corrupt;
         }
     }
     
