@@ -4,11 +4,13 @@ import java.io.File;
 import java.io.InputStream;
 import java.io.IOException;
 import java.net.URI;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import net.i2p.data.DataHelper;
 import net.i2p.router.RouterContext;
 import net.i2p.util.Addresses;
+import net.i2p.util.AddressType;
 import net.i2p.util.Log;
 import net.i2p.util.SimpleTimer;
 
@@ -41,7 +43,7 @@ public class ReseedChecker {
 
     /**
      *  All reseeding must be done through this instance.
-     *  Access through context.netDb().reseedChecker(), others should not instantiate
+     *  Access through context.netDbSegmentor().reseedChecker(), others should not instantiate
      *
      *  @since 0.9
      */
@@ -66,7 +68,8 @@ public class ReseedChecker {
                 return false;
         }
 
-        if (_context.getBooleanProperty(Reseeder.PROP_DISABLE)) {
+        if (_context.getBooleanProperty(Reseeder.PROP_DISABLE) ||
+            _context.getBooleanProperty("i2p.vmCommSystem")) {
             int x = count - 1;  // us
             // no ngettext, this is rare
             String s;
@@ -74,8 +77,10 @@ public class ReseedChecker {
                 s = "Only " + x + " peers remaining but reseed disabled by configuration";
             else
                 s = "No peers remaining but reseed disabled by configuration";
-            _lastError = s;
-            _log.logAlways(Log.WARN, s);
+            if (!s.equals(_lastError)) {
+                _lastError = s;
+                _log.logAlways(Log.WARN, s);
+            }
             return false;
         }
 
@@ -87,8 +92,10 @@ public class ReseedChecker {
                 s = "Only " + x + " peers remaining but reseed disabled by shutdown in progress";
             else
                 s = "No peers remaining but reseed disabled by shutdown in progress";
-            _lastError = s;
-            _log.logAlways(Log.WARN, s);
+            if (!s.equals(_lastError)) {
+                _lastError = s;
+                _log.logAlways(Log.WARN, s);
+            }
             return false;
         }
 
@@ -100,7 +107,8 @@ public class ReseedChecker {
         File noReseedFileAlt2 = new File(_context.getConfigDir(), ".i2pnoreseed");
         File noReseedFileAlt3 = new File(_context.getConfigDir(), "noreseed.i2p");
         if (!noReseedFile.exists() && !noReseedFileAlt1.exists() && !noReseedFileAlt2.exists() && !noReseedFileAlt3.exists()) {
-            if (!Addresses.isConnected()) {
+            Set<AddressType> addrs = Addresses.getConnectedAddressTypes();
+            if (!addrs.contains(AddressType.IPV4) && !addrs.contains(AddressType.IPV6)) {
                 if (!_networkLogged) {
                     _log.logAlways(Log.WARN, "Cannot reseed, no network connection");
                     _networkLogged = true;
@@ -121,8 +129,10 @@ public class ReseedChecker {
                 s = "Only " + x + " peers remaining but reseed disabled by config file";
             else
                 s = "No peers remaining but reseed disabled by config file";
-            _lastError = s;
-            _log.logAlways(Log.WARN, s);
+            if (!s.equals(_lastError)) {
+                _lastError = s;
+                _log.logAlways(Log.WARN, s);
+            }
             return false;
         }
     }
