@@ -41,12 +41,14 @@ public class PeerProfile {
     private long _lastSentToSuccessfully;
     private long _lastFailedSend;
     private long _lastHeardFrom;
+    // unused
     private float _tunnelTestResponseTimeAvg;
     // periodic rates
     //private RateStat _sendSuccessSize = null;
     //private RateStat _receiveSize = null;
     private RateStat _dbResponseTime;
     private RateStat _tunnelCreateResponseTime;
+    // unused
     private RateStat _tunnelTestResponseTime;
     private RateStat _dbIntroduction;
     // calculation bonuses
@@ -95,6 +97,7 @@ public class PeerProfile {
     // x**4 = .5; x = 4th root of .5,  x = .5**(1/4), x ~= 0.84
     private static final float DEGRADE_FACTOR = (float) Math.pow(TOTAL_DEGRADE_PER_DAY, 1.0d / DEGRADES_PER_DAY);
     //static { System.out.println("Degrade factor is " + DEGRADE_FACTOR); }
+    static final boolean ENABLE_TUNNEL_TEST_RESPONSE_TIME = false;
 
     private long _lastCoalesceDate = System.currentTimeMillis();
     
@@ -163,7 +166,17 @@ public class PeerProfile {
      * 5 minutes)
      */
     public boolean getIsActive() {
-        return getIsActive(5*60*1000);
+        return getIsActive(5*60*1000, _context.clock().now());
+    }
+    
+    /**
+     * Is this peer active at the moment (sending/receiving messages within the last
+     * 5 minutes)
+     *
+     * @since 0.9.58
+     */
+    public boolean getIsActive(long now) {
+        return getIsActive(5*60*1000, now);
     }
     
     /** @since 0.8.11 */
@@ -219,17 +232,11 @@ public class PeerProfile {
      *
      * @param period must be one of the periods in the RateStat constructors below
      *        (5*60*1000 or 60*60*1000)
+     *
+     * @since 0.9.58
      */
-    public boolean getIsActive(long period) {
-        //if ( (getSendSuccessSize().getRate(period).getCurrentEventCount() > 0) ||
-        //     (getSendSuccessSize().getRate(period).getLastEventCount() > 0) ||
-        //     (getReceiveSize().getRate(period).getCurrentEventCount() > 0) ||
-        //     (getReceiveSize().getRate(period).getLastEventCount() > 0) ||
-        //     _context.commSystem().isEstablished(_peer) )
-        //    return true;
-        //else
-        //    return false;
-        long before = _context.clock().now() - period;
+    public boolean getIsActive(long period, long now) {
+        long before = now - period;
         return getLastHeardFrom() < before ||
                getLastSendSuccessful() < before ||
                isEstablished();
@@ -238,7 +245,7 @@ public class PeerProfile {
     
     /**
      *  When did we first hear about this peer?
-     *  @return greater than zero, set to now in consturctor
+     *  @return greater than zero, set to now in constructor
      */
     public synchronized long getFirstHeardAbout() { return _firstHeardAbout; }
 
@@ -301,9 +308,18 @@ public class PeerProfile {
     /** how long it takes to get a tunnel create response from the peer (in milliseconds), calculated over a 1 minute, 1 hour, and 1 day period
         Warning - may return null if !getIsExpanded() */
     public RateStat getTunnelCreateResponseTime() { return _tunnelCreateResponseTime; }
-    /** how long it takes to successfully test a tunnel this peer participates in (in milliseconds), calculated over a 10 minute, 1 hour, and 1 day period
-        Warning - may return null if !getIsExpanded() */
+
+    /**
+     *  How long it takes to successfully test a tunnel this peer participates in (in milliseconds),
+     *  calculated over a 10 minute, 1 hour, and 1 day period
+     *  Warning - may return null if !getIsExpanded()
+     *
+     *  @deprecated unused
+     *  @return null always
+     */
+    @Deprecated
     public RateStat getTunnelTestResponseTime() { return _tunnelTestResponseTime; }
+
     /** how many new peers we get from dbSearchReplyMessages or dbStore messages, calculated over a 1 hour, 1 day, and 1 week period
         Warning - may return null if !getIsExpandedDB() */
     public RateStat getDbIntroduction() { return _dbIntroduction; }
@@ -352,14 +368,30 @@ public class PeerProfile {
     public float getIntegrationValue() { return _integrationValue; }
     /**
      * is this peer actively failing (aka not worth touching)?
-     * deprecated - unused - always false
+     * @deprecated - unused - always false
      */
+    @Deprecated
     public boolean getIsFailing() { return false; }
 
+    /**
+     *  @deprecated unused
+     *  @return 0 always
+     */
+    @Deprecated
     public float getTunnelTestTimeAverage() { return _tunnelTestResponseTimeAvg; }
-    void setTunnelTestTimeAverage(float avg) { _tunnelTestResponseTimeAvg = avg; }
+
+    /**
+     *  @deprecated unused
+     */
+    @Deprecated
+    void setTunnelTestTimeAverage(float avg) { /* _tunnelTestResponseTimeAvg = avg; */ }
     
+    /**
+     *  @deprecated unused
+     */
+    @Deprecated
     void updateTunnelTestTimeAverage(long ms) {
+/*
         if (_tunnelTestResponseTimeAvg <= 0) 
             _tunnelTestResponseTimeAvg = 30*1000; // should we instead start at $ms?
         
@@ -372,6 +404,7 @@ public class PeerProfile {
         if (_log.shouldLog(Log.INFO))
             _log.info("Updating tunnel test time for " + _peer.toBase64().substring(0,6) 
                       + " to " + _tunnelTestResponseTimeAvg + " via " + ms);
+*/
     }
 
     public float getPeakThroughputKBps() { 
@@ -514,13 +547,10 @@ public class PeerProfile {
      */
     public synchronized void expandProfile() {
         String group = (null == _peer ? "profileUnknown" : _peer.toBase64().substring(0,6));
-        //if (_sendSuccessSize == null)
-        //    _sendSuccessSize = new RateStat("sendSuccessSize", "How large successfully sent messages are", group, new long[] { 5*60*1000l, 60*60*1000l });
-        //if (_receiveSize == null)
-        //    _receiveSize = new RateStat("receiveSize", "How large received messages are", group, new long[] { 5*60*1000l, 60*60*1000l } );
         if (_tunnelCreateResponseTime == null)
-            _tunnelCreateResponseTime = new RateStat("tunnelCreateResponseTime", "how long it takes to get a tunnel create response from the peer (in milliseconds)", group, new long[] { 10*60*1000l, 30*60*1000l, 60*60*1000l, 24*60*60*1000 } );
-        if (_tunnelTestResponseTime == null)
+            _tunnelCreateResponseTime = new RateStat("tunnelCreateResponseTime", "how long it takes to get a tunnel create response from the peer (ms)", group, TunnelHistory.RATES);
+
+        if (ENABLE_TUNNEL_TEST_RESPONSE_TIME && _tunnelTestResponseTime == null)
             _tunnelTestResponseTime = new RateStat("tunnelTestResponseTime", "how long it takes to successfully test a tunnel this peer participates in (in milliseconds)", group, new long[] { 10*60*1000l, 30*60*1000l, 60*60*1000l, 3*60*60*1000l, 24*60*60*1000 } );
 
         if (_tunnelHistory == null)
@@ -605,7 +635,8 @@ public class PeerProfile {
     	//_receiveSize.coalesceStats();
     	//_sendSuccessSize.coalesceStats();
     	_tunnelCreateResponseTime.coalesceStats();
-    	_tunnelTestResponseTime.coalesceStats();
+        if (_tunnelTestResponseTime != null)
+            _tunnelTestResponseTime.coalesceStats();
     	_tunnelHistory.coalesceStats();
     	if (_expandedDB) {
     		_dbIntroduction.coalesceStats();
@@ -640,9 +671,13 @@ public class PeerProfile {
     private float calculateSpeed() { return (float) SpeedCalculator.calc(this); }
     private float calculateCapacity() { return (float) CapacityCalculator.calc(this); }
     private float calculateIntegration() { return (float) IntegrationCalculator.calc(this); }
-    /** deprecated - unused - always false */
+
+    /**
+     * @deprecated - unused - always false
+     */
+    @Deprecated
     void setIsFailing(boolean val) {}
-    
+
     /**
      *  Helper for calculators
      *  @since 0.9.2
