@@ -17,8 +17,10 @@ import java.util.TreeSet;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
+import net.i2p.crypto.SipHashInline;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
+import net.i2p.data.SessionKey;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.ClientManagerFacade;
@@ -51,8 +53,6 @@ public class ProfileOrganizer {
     private final Map<Hash, PeerProfile> _notFailingPeers;
     /** H(routerIdnetity), containing elements in _notFailingPeers */
     private final List<Hash> _notFailingPeersList;
-    /** TO BE REMOVED H(routerIdentity) to PeerProfile for all peers that ARE failing horribly (but that we haven't dropped reference to yet) */
-    private final Map<Hash, PeerProfile> _failingPeers;
     /** who are we? */
     private Hash _us;
     private final ProfilePersistenceHelper _persistenceHelper;
@@ -92,7 +92,7 @@ public class ProfileOrganizer {
     
     /** synchronized against this lock when updating the tier that peers are located in (and when fetching them from a peer) */
     private final ReentrantReadWriteLock _reorganizeLock = new ReentrantReadWriteLock(false);
-    
+
     public ProfileOrganizer(RouterContext context) {
         _context = context;
         _log = context.logManager().getLog(ProfileOrganizer.class);
@@ -102,7 +102,6 @@ public class ProfileOrganizer {
         _wellIntegratedPeers = new HashMap<Hash, PeerProfile>(128);
         _notFailingPeers = new HashMap<Hash, PeerProfile>(256);
         _notFailingPeersList = new ArrayList<Hash>(256);
-        _failingPeers = new HashMap<Hash, PeerProfile>(16);
         _strictCapacityOrder = new TreeSet<PeerProfile>(_comp);
         _persistenceHelper = new ProfilePersistenceHelper(_context);
         
@@ -297,7 +296,8 @@ public class ProfileOrganizer {
     @Deprecated
     public int countWellIntegratedPeers() { return count(_wellIntegratedPeers); }
     public int countNotFailingPeers() { return count(_notFailingPeers); }
-    public int countFailingPeers() { return count(_failingPeers); }
+    @Deprecated
+    public int countFailingPeers() { return 0; }
     
     public int countActivePeers() {
         int activePeers = 0;
@@ -305,12 +305,6 @@ public class ProfileOrganizer {
        
         getReadLock();
         try {
-            for (PeerProfile profile : _failingPeers.values()) {
-                if (profile.getLastSendSuccessful() >= hideBefore)
-                    activePeers++;
-                else if (profile.getLastHeardFrom() >= hideBefore)
-                    activePeers++;
-            }
             for (PeerProfile profile : _notFailingPeers.values()) {
                 if (profile.getLastSendSuccessful() >= hideBefore)
                     activePeers++;
@@ -334,7 +328,9 @@ public class ProfileOrganizer {
 
     /**
      *  Deprecated for now, always false
+     *  @deprecated unused
      */
+    @Deprecated
     public boolean isFailing(Hash peer) {
         // Always false so skip the lock
         //return isX(_failingPeers, peer);
@@ -346,7 +342,6 @@ public class ProfileOrganizer {
         if (!getWriteLock())
             return;
         try {
-            _failingPeers.clear();
             _fastPeers.clear();
             _highCapacityPeers.clear();
             _notFailingPeers.clear();
@@ -383,6 +378,8 @@ public class ProfileOrganizer {
     }
     
     /**
+     *  Only for console
+     *
      *  @return true if successful, false if not found
      */
     public boolean exportProfile(Hash profile, OutputStream out) throws IOException {
@@ -405,7 +402,7 @@ public class ProfileOrganizer {
      *
      */
     public void selectFastPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
-        selectFastPeers(howMany, exclude, matches, 0);
+        selectFastPeers(howMany, exclude, matches, 0, null);
     }
 
     /**
@@ -419,17 +416,19 @@ public class ProfileOrganizer {
      * @param matches set to store the return value in
      * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
      *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet in/out param, use for multiple calls, may be null only if mask is 0
+     * @since 0.9.53 added ipSet param
      *
      */
-    public void selectFastPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask) {
+    public void selectFastPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask, MaskedIPSet ipSet) {
         getReadLock();
         try {
-            locked_selectPeers(_fastPeers, howMany, exclude, matches, mask);
+            locked_selectPeers(_fastPeers, howMany, exclude, matches, mask, ipSet);
         } finally { releaseReadLock(); }
         if (matches.size() < howMany) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("selectFastPeers("+howMany+"), not enough fast (" + matches.size() + ") going on to highCap");
-            selectHighCapacityPeers(howMany, exclude, matches, mask);
+            selectHighCapacityPeers(howMany, exclude, matches, mask, ipSet);
         } else {
             if (_log.shouldDebug())
                 _log.debug("selectFastPeers("+howMany+"), found enough fast (" + matches.size() + ")");
@@ -480,8 +479,12 @@ public class ProfileOrganizer {
      *    6: return only from group 2
      *    7: return only from group 3
      *</pre>
+     * @param mask 0-4
+     * @param ipSet in/out param, use for multiple calls, may be null only if mask is 0
+     * @since 0.9.53 added mask and ipSet params
      */
-    public void selectFastPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, Hash randomKey, Slice subTierMode) {
+    public void selectFastPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, SessionKey randomKey,
+                                Slice subTierMode, int mask, MaskedIPSet ipSet) {
         getReadLock();
         try {
             if (subTierMode != Slice.SLICE_ALL) {
@@ -490,14 +493,14 @@ public class ProfileOrganizer {
                     subTierMode = Slice.SLICE_ALL;
             }
             if (subTierMode != Slice.SLICE_ALL)
-                locked_selectPeers(_fastPeers, howMany, exclude, matches, randomKey, subTierMode);
+                locked_selectPeers(_fastPeers, howMany, exclude, matches, randomKey, subTierMode, mask, ipSet);
             else
-                locked_selectPeers(_fastPeers, howMany, exclude, matches, 2);
+                locked_selectPeers(_fastPeers, howMany, exclude, matches, mask, ipSet);
         } finally { releaseReadLock(); }
         if (matches.size() < howMany) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("selectFastPeers("+howMany+"), not enough fast (" + matches.size() + ") going on to highCap");
-            selectHighCapacityPeers(howMany, exclude, matches, 2);
+            selectHighCapacityPeers(howMany, exclude, matches, mask, ipSet);
         } else {
             if (_log.shouldDebug())
                 _log.debug("selectFastPeers("+howMany+"), found enough fast (" + matches.size() + ")");
@@ -510,14 +513,16 @@ public class ProfileOrganizer {
      *
      */
     public void selectHighCapacityPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
-        selectHighCapacityPeers(howMany, exclude, matches, 0);
+        selectHighCapacityPeers(howMany, exclude, matches, 0, null);
     }
 
     /**
      * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
      *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet in/out param, use for multiple calls, may be null only if mask is 0
+     * @since 0.9.53 added ipSet param
      */
-    public void selectHighCapacityPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask) {
+    public void selectHighCapacityPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask, MaskedIPSet ipSet) {
         getReadLock();
         try {
             // we only use selectHighCapacityPeers when we are selecting for PURPOSE_TEST
@@ -529,12 +534,12 @@ public class ProfileOrganizer {
             else
                 exclude.addAll(_fastPeers.keySet());
              */
-            locked_selectPeers(_highCapacityPeers, howMany, exclude, matches, mask);
+            locked_selectPeers(_highCapacityPeers, howMany, exclude, matches, mask, ipSet);
         } finally { releaseReadLock(); }
         if (matches.size() < howMany) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("selectHighCap("+howMany+"), not enough highcap (" + matches.size() + ") going on to ANFP2");
-            selectActiveNotFailingPeers2(howMany, exclude, matches, mask);
+            selectActiveNotFailingPeers2(howMany, exclude, matches, mask, ipSet);
         } else {
             if (_log.shouldDebug())
                 _log.debug("selectHighCap("+howMany+"), found enough highCap (" + matches.size() + ")");
@@ -542,59 +547,26 @@ public class ProfileOrganizer {
         return;
     }
 
-    /**
-     * Return a set of Hashes for peers that are well integrated into the network.
-     *
-     * @deprecated unused
-     */
-    @Deprecated
-    public void selectWellIntegratedPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
-        selectWellIntegratedPeers(howMany, exclude, matches, 0);
-    }
-
-    /**
-     * Return a set of Hashes for peers that are well integrated into the network.
-     *
-     * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
-     *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
-     * @deprecated unused
-     */
-    @Deprecated
-    public void selectWellIntegratedPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask) {
-        getReadLock();
-        try {
-            locked_selectPeers(_wellIntegratedPeers, howMany, exclude, matches, mask);
-        } finally { releaseReadLock(); }
-        if (matches.size() < howMany) {
-            if (_log.shouldLog(Log.INFO))
-                _log.info("selectWellIntegrated("+howMany+"), not enough integrated (" + matches.size() + ") going on to notFailing");
-            selectNotFailingPeers(howMany, exclude, matches, mask);
-        } else {            
-            if (_log.shouldDebug())
-                _log.debug("selectWellIntegrated("+howMany+"), found enough well integrated (" + matches.size() + ")");
-        }
-        
-        return;
-    }
-
     /**
      * Return a set of Hashes for peers that are not failing, preferring ones that
      * we are already talking with
      *
      */
     public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
-        selectNotFailingPeers(howMany, exclude, matches, false, 0);
+        selectNotFailingPeers(howMany, exclude, matches, false, 0, null);
     }
 
     /**
      * @param mask ignored, should call locked_selectPeers, to be fixed
+     * @param ipSet ignored, should call locked_selectPeers, to be fixed
+     * @since 0.9.53 added ipSet param
      */
-    public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask) {
-        selectNotFailingPeers(howMany, exclude, matches, false, mask);
+    public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask, MaskedIPSet ipSet) {
+        selectNotFailingPeers(howMany, exclude, matches, false, mask, ipSet);
     }
 
     public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, boolean onlyNotFailing) {
-        selectNotFailingPeers(howMany, exclude, matches, onlyNotFailing, 0);
+        selectNotFailingPeers(howMany, exclude, matches, onlyNotFailing, 0, null);
     }
 
     /**
@@ -606,8 +578,11 @@ public class ProfileOrganizer {
      * @param matches set to store the matches in
      * @param onlyNotFailing if true, don't include any high capacity peers
      * @param mask ignored, should call locked_selectPeers, to be fixed
+     * @param ipSet ignored, should call locked_selectPeers, to be fixed
+     * @since 0.9.53 added ipSet param
      */
-    public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, boolean onlyNotFailing, int mask) {
+    public void selectNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, boolean onlyNotFailing,
+                                      int mask, MaskedIPSet ipSet) {
         if (matches.size() < howMany)
             selectAllNotFailingPeers(howMany, exclude, matches, onlyNotFailing, mask);
         return;
@@ -624,19 +599,37 @@ public class ProfileOrganizer {
      * Caution, this does NOT cascade further to non-connected peers, so it should only
      * be used when there is a good number of connected peers.
      *
-     * @param exclude non-null, WARNING - side effect, all not-connected peers are added
-     * No mask parameter, to be fixed
+     * @param exclude non-null, not-connected peers will NOT be added, as of 0.9.58
      */
     public void selectActiveNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
+        selectActiveNotFailingPeers(howMany, exclude, matches, 0, null);
+    }
+
+    /**
+     * Return a set of Hashes for peers that are both not failing and we're actively
+     * talking with.
+     *
+     * We use commSystem().isEstablished(), not profile.getIsActive(), as the
+     * NTCP idle time is now shorter than the 5 minute getIsActive() threshold,
+     * and we're using this to try and limit connections.
+     *
+     * Caution, this does NOT cascade further to non-connected peers, so it should only
+     * be used when there is a good number of connected peers.
+     *
+     * @param exclude non-null, not-connected peers will NOT be added, as of 0.9.58
+     * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
+     *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet may be null only if mask is 0
+     * @since 0.9.53
+     */
+    public void selectActiveNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask, MaskedIPSet ipSet) {
         if (matches.size() < howMany) {
-            Set<Hash> connected = _context.commSystem().getEstablished();
+            List<Hash> connected = _context.commSystem().getEstablished();
+            if (connected.isEmpty())
+                return;
             getReadLock();
             try {
-                for (Hash peer : _notFailingPeers.keySet()) {
-                    if (!connected.contains(peer))
-                        exclude.add(peer);
-                }
-                locked_selectPeers(_notFailingPeers, howMany, exclude, matches, 0);
+                locked_selectActive(connected, howMany, exclude, matches, mask, ipSet);
             } finally { releaseReadLock(); }
         }
     }
@@ -651,27 +644,26 @@ public class ProfileOrganizer {
      *
      * This DOES cascade further to non-connected peers.
      *
+     * @param exclude non-null, not-connected peers will NOT be added, as of 0.9.58
      * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
      *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet in/out param, use for multiple calls, may be null only if mask is 0
+     * @since 0.9.53 added ipSet param
      */
-    private void selectActiveNotFailingPeers2(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask) {
+    private void selectActiveNotFailingPeers2(int howMany, Set<Hash> exclude, Set<Hash> matches, int mask, MaskedIPSet ipSet) {
         if (matches.size() < howMany) {
-            Set<Hash> connected = _context.commSystem().getEstablished();
-            Map<Hash, PeerProfile> activePeers = new HashMap<Hash, PeerProfile>(connected.size());
-            getReadLock();
-            try {
-                for (Hash peer : connected) {
-                    PeerProfile prof = _notFailingPeers.get(peer);
-                    if (prof != null)
-                        activePeers.put(peer, prof);
-                }
-                locked_selectPeers(activePeers, howMany, exclude, matches, mask);
-            } finally { releaseReadLock(); }
+            List<Hash> connected = _context.commSystem().getEstablished();
+            if (!connected.isEmpty()) {
+                getReadLock();
+                try {
+                    locked_selectActive(connected, howMany, exclude, matches, mask, ipSet);
+                } finally { releaseReadLock(); }
+            }
         }
         if (matches.size() < howMany) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("selectANFP2("+howMany+"), not enough ANFP (" + matches.size() + ") going on to notFailing");
-            selectNotFailingPeers(howMany, exclude, matches, mask);
+            selectNotFailingPeers(howMany, exclude, matches, mask, ipSet);
         } else {
             if (_log.shouldDebug())
                 _log.debug("selectANFP2("+howMany+"), found enough ANFP (" + matches.size() + ")");
@@ -688,7 +680,6 @@ public class ProfileOrganizer {
 
     /**
      * @param mask ignored, should call locked_selectPeers, to be fixed
-     *
      */
     private void selectAllNotFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches, boolean onlyNotFailing, int mask) {
         if (matches.size() < howMany) {
@@ -735,78 +726,13 @@ public class ProfileOrganizer {
     /**
      * I'm not quite sure why you'd want this... (other than for failover from the better results)
      *
+     * @deprecated unused
      */
-    public void selectFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
-        getReadLock();
-        try {
-            locked_selectPeers(_failingPeers, howMany, exclude, matches);
-        } finally { releaseReadLock(); }
+    @Deprecated
+    private void selectFailingPeers(int howMany, Set<Hash> exclude, Set<Hash> matches) {
         return;        
     }                  
 
-    /**                
-     * Get the peers the transport layer thinks are unreachable,
-     * and peers requiring introducers.
-     *                 
-     */                
-    public List<Hash> selectPeersLocallyUnreachable() { 
-        List<Hash> n;
-        int count;
-        getReadLock();
-        try {
-            count = _notFailingPeers.size();
-            n = new ArrayList<Hash>(_notFailingPeers.keySet());
-        } finally { releaseReadLock(); }
-        List<Hash> l = new ArrayList<Hash>(count / 4);
-        for (Hash peer : n) {
-            if (_context.commSystem().wasUnreachable(peer)) {
-                l.add(peer);
-            } else {
-                // Blacklist all peers requiring SSU introducers, because either
-                //  a) it's slow; or
-                //  b) it doesn't work very often; or
-                //  c) in the event they are advertising NTCP, it probably won't work because
-                //     they probably don't have a TCP hole punched in their firewall either.
-                RouterInfo info = _context.netDb().lookupRouterInfoLocally(peer);
-                if (info != null) {
-                        RouterAddress ra = info.getTargetAddress("SSU");
-                        // peers with no SSU address at all are fine.
-                        // as long as they have NTCP
-                        if (ra == null) {
-                            if (info.getTargetAddresses("NTCP", "NTCP2").isEmpty())
-                                l.add(peer);
-                            continue;
-                        }
-                        // This is the quick way of doing UDPAddress.getIntroducerCount() > 0
-                        if (ra.getOption("ihost0") != null)
-                            l.add(peer);
-                }
-            }
-        }
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Unreachable: " + l);
-        return l;
-    }
-
-    /**
-     * Get the peers that have recently rejected us for bandwidth
-     * recent == last 20s
-     *
-     */
-    public List<Hash> selectPeersRecentlyRejecting() { 
-        getReadLock();
-        try {
-            long cutoff = _context.clock().now() - (20*1000);
-            int count = _notFailingPeers.size();
-            List<Hash> l = new ArrayList<Hash>(count / 128);
-            for (PeerProfile prof : _notFailingPeers.values()) {
-                if (prof.getTunnelHistory().getLastRejectedBandwidth() > cutoff)
-                    l.add(prof.getPeer());
-            }
-            return l;
-        } finally { releaseReadLock(); }
-    }
-
     /**
      * Find the hashes for all peers we are actively profiling
      *
@@ -814,8 +740,7 @@ public class ProfileOrganizer {
     public Set<Hash> selectAllPeers() {
         getReadLock();
         try {
-            Set<Hash> allPeers = new HashSet<Hash>(_failingPeers.size() + _notFailingPeers.size() + _highCapacityPeers.size() + _fastPeers.size());
-            allPeers.addAll(_failingPeers.keySet());
+            Set<Hash> allPeers = new HashSet<Hash>(_notFailingPeers.size() + _highCapacityPeers.size() + _fastPeers.size());
             allPeers.addAll(_notFailingPeers.keySet());
             allPeers.addAll(_highCapacityPeers.keySet());
             allPeers.addAll(_fastPeers.keySet());
@@ -823,11 +748,11 @@ public class ProfileOrganizer {
         } finally { releaseReadLock(); }
     }
     
-    private static final long MIN_EXPIRE_TIME = 2*60*60*1000;
+    private static final long MIN_EXPIRE_TIME = 60*60*1000;
     private static final long MAX_EXPIRE_TIME = 6*60*60*1000;
-    private static final long ADJUST_EXPIRE_TIME = 60*1000;
+    private static final long ADJUST_EXPIRE_TIME = 5*60*1000;
     private static final int ENOUGH_PROFILES = 600;
-    private long _currentExpireTime = MAX_EXPIRE_TIME;
+    private long _currentExpireTime = 2*60*60*1000;
 
     /**
      * Place peers into the correct tier, as well as expand/contract and even drop profiles
@@ -905,7 +830,6 @@ public class ProfileOrganizer {
             locked_calculateThresholds(allPeers);
             thresholdTime = System.currentTimeMillis()-thresholdStart;
 
-            _failingPeers.clear();
             _fastPeers.clear();
             _highCapacityPeers.clear();
             _notFailingPeers.clear();
@@ -918,7 +842,6 @@ public class ProfileOrganizer {
                 locked_placeProfile(profile);
             }
 
-            locked_unfailAsNecessary();
             locked_demoteHighCapAsNecessary();
             locked_promoteFastAsNecessary();
             locked_demoteFastAsNecessary();
@@ -969,15 +892,16 @@ public class ProfileOrganizer {
         if (numToPromote > 0) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("Need to explicitly promote " + numToPromote + " peers to the fast group");
+            long now = _context.clock().now();
             for (PeerProfile cur : _strictCapacityOrder) {
-                if ( (!_fastPeers.containsKey(cur.getPeer())) && (!cur.getIsFailing()) ) {
+                if (!_fastPeers.containsKey(cur.getPeer())) {
                     if (!isSelectable(cur.getPeer())) {
                         // skip peers we dont have in the netDb
                         // if (_log.shouldLog(Log.INFO))   
                         //     _log.info("skip unknown peer from fast promotion: " + cur.getPeer().toBase64());
                         continue;
                     }
-                    if (!cur.getIsActive()) {
+                    if (!cur.getIsActive(now)) {
                         // skip inactive
                         // if (_log.shouldLog(Log.INFO))
                         //     _log.info("skip inactive peer from fast promotion: " + cur.getPeer().toBase64());
@@ -1046,45 +970,8 @@ public class ProfileOrganizer {
                 _log.info("Demoted " + numToDemote + " peers from high cap, size now " + _highCapacityPeers.size());
         }
     }
-    
-    /** how many not failing/active peers must we have? */
-    private final static int MIN_NOT_FAILING_ACTIVE = 3;
 
-    /**
-     * I'm not sure how much I dislike the following - if there aren't enough
-     * active and not-failing peers, pick the most reliable active peers and
-     * override their 'failing' flag, resorting them into the not-failing buckets
-     *
-     */
-    private void locked_unfailAsNecessary() {
-        int notFailingActive = 0;
-        for (PeerProfile peer : _notFailingPeers.values()) {
-            if (peer.getIsActive())
-                notFailingActive++;
-            if (notFailingActive >= MIN_NOT_FAILING_ACTIVE) {
-                // we've got enough, no need to try further
-                return;
-            }
-        }
-        
-        // we dont have enough, lets unfail our best ones remaining
-        int needToUnfail = MIN_NOT_FAILING_ACTIVE - notFailingActive;
-        if (needToUnfail > 0) {
-            int unfailed = 0;
-            for (PeerProfile best : _strictCapacityOrder) {
-                if ( (best.getIsActive()) && (best.getIsFailing()) ) {
-                    if (_log.shouldLog(Log.WARN))
-                        _log.warn("All peers were failing, so we have overridden the failing flag for one of the most reliable active peers (" + best.getPeer().toBase64() + ")");
-                    best.setIsFailing(false);
-                    locked_placeProfile(best);
-                    unfailed++;
-                }
-                if (unfailed >= needToUnfail)
-                    break;
-            }
-        }
-    }
-    
+
     ////////
     // no more public stuff below
     ////////
@@ -1102,11 +989,12 @@ public class ProfileOrganizer {
         double totalCapacity = 0;
         double totalIntegration = 0;
         Set<PeerProfile> reordered = new TreeSet<PeerProfile>(_comp);
+        long now = _context.clock().now();
         for (PeerProfile profile : allPeers) {
             if (_us.equals(profile.getPeer())) continue;
             
             // only take into account active peers that aren't failing
-            if (profile.getIsFailing() || (!profile.getIsActive()))
+            if (!profile.getIsActive(now))
                 continue;
         
             // dont bother trying to make sense of things below the baseline
@@ -1273,9 +1161,6 @@ public class ProfileOrganizer {
     /** called after locking the reorganizeLock */
     private PeerProfile locked_getProfile(Hash peer) {
         PeerProfile cur = _notFailingPeers.get(peer);
-        if (cur != null) 
-            return cur;
-        cur = _failingPeers.get(peer);
         return cur;
     }
     
@@ -1285,19 +1170,21 @@ public class ProfileOrganizer {
      *
      */
     private void locked_selectPeers(Map<Hash, PeerProfile> peers, int howMany, Set<Hash> toExclude, Set<Hash> matches) {
-        locked_selectPeers(peers, howMany, toExclude, matches, 0);
+        locked_selectPeers(peers, howMany, toExclude, matches, 0, null);
     }
 
     /**
-      *
-      * As of 0.9.24, checks for a netdb family match as well, unless mask == 0.
-      *
+     *
+     * As of 0.9.24, checks for a netdb family match as well, unless mask == 0.
+     *
      * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
      *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet may be null only if mask is 0
+     * @since 0.9.53 added ipSet param
      */
-    private void locked_selectPeers(Map<Hash, PeerProfile> peers, int howMany, Set<Hash> toExclude, Set<Hash> matches, int mask) {
+    private void locked_selectPeers(Map<Hash, PeerProfile> peers, int howMany, Set<Hash> toExclude, Set<Hash> matches,
+                                    int mask, MaskedIPSet ipSet) {
         List<Hash> all = new ArrayList<Hash>(peers.keySet());
-        MaskedIPSet IPSet = new MaskedIPSet(16);
         // use RandomIterator to avoid shuffling the whole thing
         for (Iterator<Hash> iter = new RandomIterator<Hash>(all); (matches.size() < howMany) && iter.hasNext(); ) {
             Hash peer = iter.next();
@@ -1309,9 +1196,12 @@ public class ProfileOrganizer {
                 continue;
             boolean ok = isSelectable(peer);
             if (ok) {
-                ok = mask <= 0 || notRestricted(peer, IPSet, mask);
-                if ((!ok) && _log.shouldLog(Log.WARN))
+                ok = mask <= 0 || notRestricted(peer, ipSet, mask);
+                if ((!ok) && _log.shouldWarn())
                     _log.warn("IP restriction prevents " + peer + " from joining " + matches);
+            } else {
+                if (toExclude != null)
+                    toExclude.add(peer);
             }
             if (ok)
                 matches.add(peer);
@@ -1319,6 +1209,42 @@ public class ProfileOrganizer {
                 matches.remove(peer);
         }
     }
+
+    /**
+     *
+     * For efficiency. Rather than iterating through _notFailingPeers looking for connected peers,
+     * iterate through the connected peers and then check if failing.
+     *
+     * @param mask 0-4 Number of bytes to match to determine if peers in the same IP range should
+     *             not be in the same tunnel. 0 = disable check; 1 = /8; 2 = /16; 3 = /24; 4 = exact IP match
+     * @param ipSet may be null only if mask is 0
+     * @since 0.9.58
+     */
+    private void locked_selectActive(List<Hash> connected, int howMany, Set<Hash> toExclude, Set<Hash> matches,
+                                     int mask, MaskedIPSet ipSet) {
+        // use RandomIterator to avoid shuffling the whole thing
+        for (Iterator<Hash> iter = new RandomIterator<Hash>(connected); (matches.size() < howMany) && iter.hasNext(); ) {
+            Hash peer = iter.next();
+            if (toExclude != null && toExclude.contains(peer))
+                continue;
+            if (matches.contains(peer))
+                continue;
+            if (_us.equals(peer))
+                continue;
+            // we assume if connected, it's fine, don't look in _notFailingPeers
+            boolean ok = isSelectable(peer);
+            if (ok) {
+                ok = mask <= 0 || notRestricted(peer, ipSet, mask);
+                if ((!ok) && _log.shouldWarn())
+                    _log.warn("IP restriction prevents " + peer + " from joining " + matches);
+            } else {
+                if (toExclude != null)
+                    toExclude.add(peer);
+            }
+            if (ok)
+                matches.add(peer);
+        }
+    }
     
     /**
      * Does the peer's IP address NOT match the IP address of any peer already in the set,
@@ -1328,12 +1254,13 @@ public class ProfileOrganizer {
      *
      * @param mask is 1-4 (number of bytes to match)
      * @param IPMatches all IPs so far, modified by this routine
+     * @return true if ok, false if not
      */
-    private boolean notRestricted(Hash peer, MaskedIPSet IPSet, int mask) {
+    private boolean notRestricted(Hash peer, MaskedIPSet ipSet, int mask) {
         Set<String> peerIPs = new MaskedIPSet(_context, peer, mask);
-        if (IPSet.containsAny(peerIPs))
+        if (!ipSet.isEmpty() && ipSet.containsAny(peerIPs))
             return false;
-        IPSet.addAll(peerIPs);
+        ipSet.addAll(peerIPs);
         return true;
     }
 
@@ -1348,10 +1275,20 @@ public class ProfileOrganizer {
      *    6: return only from group 2
      *    7: return only from group 3
      *</pre>
+     * @param mask is 1-4 (number of bytes to match)
+     * @param IPMatches all IPs so far, modified by this routine
+     * @since 0.9.53 added mask/ipSet params
      */
     private void locked_selectPeers(Map<Hash, PeerProfile> peers, int howMany, Set<Hash> toExclude,
-                                    Set<Hash> matches, Hash randomKey, Slice subTierMode) {
+                                    Set<Hash> matches, SessionKey randomKey, Slice subTierMode,
+                                    int mask, MaskedIPSet ipSet) {
         List<Hash> all = new ArrayList<Hash>(peers.keySet());
+        byte[] rk = randomKey.getData();
+        // we use the first half of the random key here,
+        // the second half is used in TunnelPeerSelector.
+        long k0 = DataHelper.fromLong8(rk, 0);
+        long k1 = DataHelper.fromLong8(rk, 8);
+
         // use RandomIterator to avoid shuffling the whole thing
         for (Iterator<Hash> iter = new RandomIterator<Hash>(all); (matches.size() < howMany) && iter.hasNext(); ) {
             Hash peer = iter.next();
@@ -1361,10 +1298,18 @@ public class ProfileOrganizer {
                 continue;
             if (_us.equals(peer))
                 continue;
-            int subTier = getSubTier(peer, randomKey);
+            int subTier = getSubTier(peer, k0, k1);
             if ((subTier & subTierMode.mask) != subTierMode.val)
                 continue;
             boolean ok = isSelectable(peer);
+            if (ok) {
+                ok = mask <= 0 || notRestricted(peer, ipSet, mask);
+                if ((!ok) && _log.shouldWarn())
+                    _log.warn("IP restriction prevents " + peer + " from joining " + matches);
+            } else {
+                if (toExclude != null)
+                    toExclude.add(peer);
+            }
             if (ok)
                 matches.add(peer);
             else
@@ -1375,15 +1320,13 @@ public class ProfileOrganizer {
     /**
      *  Implement a random, deterministic split into 4 groups that cannot be predicted by
      *  others.
+     *
+     *  @param k0 random key part 1
+     *  @param k1 random key part 2
      *  @return 0-3
      */
-    private int getSubTier(Hash peer, Hash randomKey) {
-        // input is first 64 bytes; output is last 32 bytes
-        byte[] data = new byte[96];
-        System.arraycopy(peer.getData(), 0, data, 0, 32);
-        System.arraycopy(randomKey.getData(), 0, data, 32, 32);
-        _context.sha().calculateHash(data, 0, 64, data, 64);
-        return data[64] & 0x03;
+    private int getSubTier(Hash peer, long k0, long k1) {
+        return ((int) SipHashInline.hash24(k0, k1, peer.getData())) & 0x03;
     }
 
     public boolean isSelectable(Hash peer) {
@@ -1397,7 +1340,7 @@ public class ProfileOrganizer {
             return false; // never select a banlisted peer
         }
             
-        RouterInfo info = _context.netDb().lookupRouterInfoLocally(peer);
+        RouterInfo info = (RouterInfo) _context.netDb().lookupLocallyWithoutValidation(peer);
         if (null != info) {
             if (info.isHidden()) {
                if (_log.shouldLog(Log.WARN))
@@ -1429,16 +1372,7 @@ public class ProfileOrganizer {
      */
     private void locked_placeProfile(PeerProfile profile) {
         Hash peer = profile.getPeer();
-        if (profile.getIsFailing()) {
-            if (!shouldDrop(profile))
-                _failingPeers.put(peer, profile);
-            _fastPeers.remove(peer);
-            _highCapacityPeers.remove(peer);
-            _wellIntegratedPeers.remove(peer);
-            _notFailingPeers.remove(peer);
-            _notFailingPeersList.remove(peer);
-        } else {
-            _failingPeers.remove(peer);
+
             _fastPeers.remove(peer);
             _highCapacityPeers.remove(peer);
             _wellIntegratedPeers.remove(peer);
@@ -1478,7 +1412,7 @@ public class ProfileOrganizer {
                 if (_log.shouldLog(Log.DEBUG))
                     _log.debug("Integrated: \t" + peer);
             }
-        }
+
     }
     
     /**
@@ -1554,7 +1488,7 @@ public class ProfileOrganizer {
         organizer.setUs(Hash.FAKE_HASH);
         ProfilePersistenceHelper helper = new ProfilePersistenceHelper(ctx);
         for (int i = 0; i < args.length; i++) {
-            PeerProfile profile = helper.readProfile(new java.io.File(args[i]));
+            PeerProfile profile = helper.readProfile(new java.io.File(args[i]), 0);
             if (profile == null) {
                 System.err.println("Could not load profile " + args[i]);
                 continue;
@@ -1564,9 +1498,10 @@ public class ProfileOrganizer {
         organizer.reorganize();
         DecimalFormat fmt = new DecimalFormat("0000.0");
         
+        long now = ctx.clock().now();
         for (Hash peer : organizer.selectAllPeers()) {
             PeerProfile profile = organizer.getProfile(peer);
-            if (!profile.getIsActive()) {
+            if (!profile.getIsActive(now)) {
                 System.out.println("Peer " + peer.toBase64().substring(0,4) 
                            + " [" + (organizer.isFast(peer) ? "IF+R" : 
                                      organizer.isHighCapacity(peer) ? "IR  " :
@@ -1574,8 +1509,7 @@ public class ProfileOrganizer {
                            + " Speed:\t" + fmt.format(profile.getSpeedValue())
                            + " Capacity:\t" + fmt.format(profile.getCapacityValue())
                            + " Integration:\t" + fmt.format(profile.getIntegrationValue())
-                           + " Active?\t" + profile.getIsActive() 
-                           + " Failing?\t" + profile.getIsFailing());
+                           + " Active?\t" + profile.getIsActive(now) );
             } else {
                 System.out.println("Peer " + peer.toBase64().substring(0,4) 
                            + " [" + (organizer.isFast(peer) ? "F+R " : 
@@ -1584,8 +1518,7 @@ public class ProfileOrganizer {
                            + " Speed:\t" + fmt.format(profile.getSpeedValue())
                            + " Capacity:\t" + fmt.format(profile.getCapacityValue())
                            + " Integration:\t" + fmt.format(profile.getIntegrationValue())
-                           + " Active?\t" + profile.getIsActive() 
-                           + " Failing?\t" + profile.getIsFailing());
+                           + " Active?\t" + profile.getIsActive(now));
             }
         }
         
