@@ -9,11 +9,10 @@ import java.io.InputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Date;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Properties;
-import java.util.Set;
 import java.util.zip.GZIPInputStream;
 import java.util.zip.GZIPOutputStream;
 
@@ -25,6 +24,7 @@ import net.i2p.util.FileUtil;
 import net.i2p.util.Log;
 import net.i2p.util.SecureDirectory;
 import net.i2p.util.SecureFileOutputStream;
+import net.i2p.util.SystemVersion;
 
 /**
  *  Write profiles to disk at shutdown,
@@ -47,14 +47,8 @@ class ProfilePersistenceHelper {
     private static final int MIN_NAME_LENGTH = PREFIX.length() + 44 + OLD_SUFFIX.length();
     private static final String DIR_PREFIX = "p";
     private static final String B64 = Base64.ALPHABET_I2P;
-    
-    /**
-     * If we haven't been able to get a message through to the peer in this much time,
-     * drop the profile.  They may reappear, but if they do, their config may
-     * have changed (etc).
-     *
-     */
-    private static final long EXPIRE_AGE = 15*24*60*60*1000;
+    // Max to read in at startup
+    private static final int LIMIT_PROFILES = SystemVersion.isSlow() ? 1000 : 4000;
     
     private final File _profileDir;
     private Hash _us;
@@ -77,25 +71,21 @@ class ProfilePersistenceHelper {
     
     /**
      * write out the data from the profile to the file
+     * @return success
      */
-    public void writeProfile(PeerProfile profile) {
-        if (isExpired(profile.getLastSendSuccessful()))
-            return;
-        
+    public boolean writeProfile(PeerProfile profile) {
         File f = pickFile(profile);
-        long before = _context.clock().now();
         OutputStream fos = null;
         try {
             fos = new BufferedOutputStream(new GZIPOutputStream(new SecureFileOutputStream(f)));
             writeProfile(profile, fos, false);
         } catch (IOException ioe) {
             _log.error("Error writing profile to " + f);
+            return false;
         } finally {
             if (fos != null) try { fos.close(); } catch (IOException ioe) {}
         }
-        long delay = _context.clock().now() - before;
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Writing the profile to " + f.getName() + " took " + delay + "ms");
+        return true;
     }
 
     /**
@@ -111,6 +101,7 @@ class ProfilePersistenceHelper {
      * @param addComments add comment lines to the output
      * @since 0.9.41
      */
+    @SuppressWarnings("deprecation")
     public void writeProfile(PeerProfile profile, OutputStream out, boolean addComments) throws IOException {
         String groups = null;
         if (_context.profileOrganizer().isFailing(profile.getPeer())) {
@@ -150,7 +141,8 @@ class ProfilePersistenceHelper {
         addDate(buf, addComments, "lastHeardFrom", profile.getLastHeardFrom(), "When did we last get a message from the peer?");
         addDate(buf, addComments, "lastSentToSuccessfully", profile.getLastSendSuccessful(), "When did we last send the peer a message successfully?");
         addDate(buf, addComments, "lastFailedSend", profile.getLastSendFailed(), "When did we last fail to send a message to the peer?");
-        add(buf, addComments, "tunnelTestTimeAverage", profile.getTunnelTestTimeAverage(), "Moving average as to how fast the peer replies");
+        if (PeerProfile.ENABLE_TUNNEL_TEST_RESPONSE_TIME)
+            add(buf, addComments, "tunnelTestTimeAverage", profile.getTunnelTestTimeAverage(), "Moving average as to how fast the peer replies");
         add(buf, addComments, "tunnelPeakThroughput", profile.getPeakThroughputKBps(), "KBytes/sec");
         add(buf, addComments, "tunnelPeakTunnelThroughput", profile.getPeakTunnelThroughputKBps(), "KBytes/sec");
         add(buf, addComments, "tunnelPeakTunnel1mThroughput", profile.getPeakTunnel1mThroughputKBps(), "KBytes/sec");
@@ -165,7 +157,8 @@ class ProfilePersistenceHelper {
             //profile.getReceiveSize().store(out, "receiveSize");
             //profile.getSendSuccessSize().store(out, "sendSuccessSize");
             profile.getTunnelCreateResponseTime().store(out, "tunnelCreateResponseTime", addComments);
-            profile.getTunnelTestResponseTime().store(out, "tunnelTestResponseTime", addComments);
+            if (PeerProfile.ENABLE_TUNNEL_TEST_RESPONSE_TIME)
+                profile.getTunnelTestResponseTime().store(out, "tunnelTestResponseTime", addComments);
         }
 
         if (profile.getIsExpandedDB()) {
@@ -203,18 +196,29 @@ class ProfilePersistenceHelper {
             buf.append(NL);
     }
     
-    public Set<PeerProfile> readProfiles() {
-        long start = _context.clock().now();
+    public List<PeerProfile> readProfiles() {
+        long start = System.currentTimeMillis();
+        long down = _context.router().getEstimatedDowntime();
+        long cutoff = down < 15*24*60*60*1000L ? start - down - 24*60*60*1000 : start;
         List<File> files = selectFiles();
-        Set<PeerProfile> profiles = new HashSet<PeerProfile>(files.size());
+        if (files.size() > LIMIT_PROFILES)
+            Collections.shuffle(files, _context.random());
+        List<PeerProfile> profiles = new ArrayList<PeerProfile>(Math.min(LIMIT_PROFILES, files.size()));
+        int count = 0;
         for (File f :  files) {
-            PeerProfile profile = readProfile(f);
-            if (profile != null)
+            if (count >= LIMIT_PROFILES) {
+                f.delete();
+                continue;
+            }
+            PeerProfile profile = readProfile(f, cutoff);
+            if (profile != null) {
                 profiles.add(profile);
+                count++;
+            }
         }
-        long duration = _context.clock().now() - start;
+        long duration = System.currentTimeMillis() - start;
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Loading " + profiles.size() + " took " + duration + "ms");
+            _log.debug("Loading " + count + " profiles took " + duration + "ms");
         return profiles;
     }
     
@@ -260,9 +264,10 @@ class ProfilePersistenceHelper {
     
     /**
      *  Delete profile files with timestamps older than 'age' ago
+     *  @return number deleted
      *  @since 0.9.28
      */
-    public void deleteOldProfiles(long age) {
+    public int deleteOldProfiles(long age) {
         long cutoff = System.currentTimeMillis() - age;
         List<File> files = selectFiles();
         int i = 0;
@@ -274,16 +279,20 @@ class ProfilePersistenceHelper {
                 f.delete();
             }
         }
-        if (_log.shouldWarn())
-            _log.warn("Deleted " + i + " old profiles");
+        if (_log.shouldInfo())
+            _log.info("Deleted " + i + " old profiles");
+        return i;
     }
 
-    private boolean isExpired(long lastSentToSuccessfully) {
-        long timeSince = _context.clock().now() - lastSentToSuccessfully;
-        return (timeSince > EXPIRE_AGE);
-    }
-    
-    public PeerProfile readProfile(File file) {
+    /**
+     *  @param cutoff delete and return null if older than this (absolute time)
+     */
+    @SuppressWarnings("deprecation")
+    public PeerProfile readProfile(File file, long cutoff) {
+        if (file.lastModified() < cutoff) {
+            file.delete();
+            return null;
+        }
         Hash peer = getHash(file.getName());
         try {
             if (peer == null) {
@@ -296,7 +305,7 @@ class ProfilePersistenceHelper {
             loadProps(props, file);
             
             long lastSentToSuccessfully = getLong(props, "lastSentToSuccessfully");
-            if (isExpired(lastSentToSuccessfully)) {
+            if (lastSentToSuccessfully <= cutoff) {
                 if (_log.shouldLog(Log.INFO))
                     _log.info("Dropping old profile " + file.getName() + 
                               ", since we haven't heard from them in a long time");
@@ -321,7 +330,10 @@ class ProfilePersistenceHelper {
             profile.setLastSendSuccessful(getLong(props, "lastSentToSuccessfully"));
             profile.setLastSendFailed(getLong(props, "lastFailedSend"));
             profile.setLastHeardFrom(getLong(props, "lastHeardFrom"));
-            profile.setTunnelTestTimeAverage(getFloat(props, "tunnelTestTimeAverage"));
+
+            if (PeerProfile.ENABLE_TUNNEL_TEST_RESPONSE_TIME)
+                profile.setTunnelTestTimeAverage(getFloat(props, "tunnelTestTimeAverage"));
+
             profile.setPeakThroughputKBps(getFloat(props, "tunnelPeakThroughput"));
             profile.setPeakTunnelThroughputKBps(getFloat(props, "tunnelPeakTunnelThroughput"));
             profile.setPeakTunnel1mThroughputKBps(getFloat(props, "tunnelPeakTunnel1mThroughput"));
@@ -344,14 +356,16 @@ class ProfilePersistenceHelper {
             //profile.getReceiveSize().load(props, "receiveSize", true);
             //profile.getSendSuccessSize().load(props, "sendSuccessSize", true);
             profile.getTunnelCreateResponseTime().load(props, "tunnelCreateResponseTime", true);
-            profile.getTunnelTestResponseTime().load(props, "tunnelTestResponseTime", true);
+
+            if (PeerProfile.ENABLE_TUNNEL_TEST_RESPONSE_TIME)
+                profile.getTunnelTestResponseTime().load(props, "tunnelTestResponseTime", true);
             
             if (_log.shouldLog(Log.DEBUG))
                 _log.debug("Loaded the profile for " + peer.toBase64() + " from " + file.getName());
             
             fixupFirstHeardAbout(profile);
             return profile;
-        } catch (IOException e) {
+        } catch (Exception e) {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Error loading properties from " + file.getAbsolutePath(), e);
             file.delete();
