@@ -10,6 +10,7 @@ package net.i2p.router.startup;
 
 import net.i2p.router.JobImpl;
 import net.i2p.router.RouterContext;
+import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseSegmentor;
 
 /** start up the network database */
 class BootNetworkDbJob extends JobImpl {
@@ -21,6 +22,6 @@ class BootNetworkDbJob extends JobImpl {
     public String getName() { return "Boot Network Database"; }
     
     public void runJob() {
-        getContext().netDb().startup();
+        getContext().netDbSegmentor().startup();
     }
 }
