@@ -80,23 +80,40 @@ public class WorkingDir {
         } else {
             String home = System.getProperty("user.home");
             if (isWindows) {
-                String appdata = System.getenv("LOCALAPPDATA");
-                if (appdata != null)
-                    home = appdata;
+                String localappdata = System.getenv("LOCALAPPDATA");
+                if (localappdata != null) {
+                    home = localappdata;
+                }
                 // Don't mess with existing Roaming Application Data installs,
                 // in case somebody is using roaming appdata for a reason
                 // already. In new installs, use local appdata by default. -idk
-                appdata = System.getenv("APPDATA");
+                String appdata = System.getenv("APPDATA");
                 if (appdata != null) {
                     File checkOld = new File(appdata, WORKING_DIR_DEFAULT_WINDOWS);
-                    if (checkOld.exists() && checkOld.isDirectory())
-                        home = appdata;
+                    if (checkOld.exists() && checkOld.isDirectory()){
+                        File routerConfig = new File(checkOld.getAbsolutePath(), "router.config");
+                        // The Firefox profile installer was mistakenly using the Roaming application data
+                        // which is synced between devices on some Windows machines using MS cloud services,
+                        // instead of the local application data which is used by default.
+                        // It would create the router.config file in an empty directory, which the router would
+                        // then attempt to use, resulting in a router with no client applications. Checking
+                        // for clients.config.d determines if the directory is "Real" or not.
+                        File clientAppsConfig = new File(checkOld.getAbsolutePath(), "clients.config.d");
+                        if (routerConfig.exists() && clientAppsConfig.exists() && clientAppsConfig.isDirectory()) {
+                            home = appdata;
+                        } else {
+                            clientAppsConfig = new File(checkOld.getAbsolutePath(), "clients.config");
+                            if (routerConfig.exists() && clientAppsConfig.exists())
+                                home = appdata;
+                        }
+                        System.err.println("System is Windows: " + home);
+                    }
                 }
                 dirf = new SecureDirectory(home, WORKING_DIR_DEFAULT_WINDOWS);
             } else if (SystemVersion.isMac()) {
                 String appdata = "/Library/Application Support/";
-		File old = new File(home,WORKING_DIR_DEFAULT);
-		if (old.exists() && old.isDirectory())
+                File old = new File(home,WORKING_DIR_DEFAULT);
+                if (old.exists() && old.isDirectory())
                     dirf = new SecureDirectory(home, WORKING_DIR_DEFAULT);
                 else {
                     home = home+appdata;
@@ -350,6 +367,8 @@ public class WorkingDir {
      */
     private static boolean migrateClientsConfig(File olddir, File todir) {
         File oldFile = new File(olddir, "clients.config");
+        if (!oldFile.exists())
+            return true;
         File newFile = new File(todir, "clients.config");
         FileInputStream in = null;
         PrintWriter out = null;
@@ -373,14 +392,12 @@ public class WorkingDir {
                 }
                 out.println(s);
             }
-            System.err.println("Copied " + oldFile + " with modifications");
+            System.err.println("Copied file " + oldFile + " with modifications");
             if (out.checkError())
                 throw new IOException("Failed write to " + newFile);
             return true;
         } catch (IOException ioe) {
-            if (in != null) {
-                System.err.println("FAILED copy " + oldFile + ": " + ioe);
-            }
+            System.err.println("FAILED copy " + oldFile + ": " + ioe);
             return false;
         } finally {
             if (in != null) try { in.close(); } catch (IOException ioe) {}
@@ -395,6 +412,8 @@ public class WorkingDir {
      */
     static boolean migrateJettyXml(File olddir, File todir, String filename, String oldString, String newString) {
         File oldFile = new File(olddir, filename);
+        if (!oldFile.exists())
+            return true;
         File newFile = new File(todir, filename);
         FileInputStream in = null;
         PrintWriter out = null;
@@ -415,9 +434,7 @@ public class WorkingDir {
             System.err.println("Copied " + oldFile + " with modifications");
             return true;
         } catch (IOException ioe) {
-            if (in != null) {
-                System.err.println("FAILED copy " + oldFile + ": " + ioe);
-            }
+            System.err.println("FAILED copy " + oldFile + ": " + ioe);
             return false;
         } finally {
             if (in != null) try { in.close(); } catch (IOException ioe) {}
@@ -440,7 +457,7 @@ public class WorkingDir {
                 System.err.println("FAILED copy " + src.getPath());
                 return false;
             }
-            System.err.println("Created " + targetDir.getPath());
+            System.err.println("Created Directory " + targetDir.getPath());
         }
         // SecureDirectory is a File so this works for non-directories too
         File targetFile = new SecureDirectory(targetDir, src.getName());
@@ -457,7 +474,7 @@ public class WorkingDir {
                 System.err.println("FAILED copy " + src.getPath());
                 return false;
             }
-            System.err.println("Created " + targetFile.getPath());
+            System.err.println("Created File " + targetFile.getPath());
         }
         boolean rv = true;
         for (int i = 0; i < children.length; i++) {
@@ -481,7 +498,7 @@ public class WorkingDir {
             in = new FileInputStream(src);
             out = new SecureFileOutputStream(dst);
             DataHelper.copy(in, out);
-            System.err.println("Copied " + src.getPath());
+            System.err.println("Copied File " + src.getPath());
         } catch (IOException ioe) {
             System.err.println("FAILED copy " + src.getPath() + ": " + ioe);
             rv = false;
