@@ -9,6 +9,7 @@ import java.io.BufferedWriter;
 import java.io.IOException;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.util.Arrays;
@@ -92,7 +93,7 @@ public class GeoIP {
     private static final String DEBIAN_GEOIPV6_FILE = "/usr/share/GeoIP/GeoIPv6.dat";
     private static final boolean DISABLE_DEBIAN = false;
     private static final boolean ENABLE_DEBIAN = !DISABLE_DEBIAN && !(SystemVersion.isWindows() || SystemVersion.isAndroid());
-    private static final String PROP_BLOCK_MY_COUNTRY = "i2np.blockMyCountry";
+    public static final String PROP_BLOCK_MY_COUNTRY = "i2np.blockMyCountry";
     /** maxmind API */
     private static final String UNKNOWN_COUNTRY_CODE = "--";
     /** db-ip.com https://db-ip.com/faq.php */
@@ -520,20 +521,15 @@ public class GeoIP {
     *
     */
     private void readCountryFile() {
-        String geoDir = _context.getProperty(PROP_GEOIP_DIR, GEOIP_DIR_DEFAULT);
-        File geoFile = new File(geoDir);
-        if (!geoFile.isAbsolute())
-            geoFile = new File(_context.getBaseDir(), geoDir);
-        geoFile = new File(geoFile, COUNTRY_FILE_DEFAULT);
-        if (!geoFile.exists()) {
+        InputStream is = GeoIP.class.getResourceAsStream("/net/i2p/util/resources/" + COUNTRY_FILE_DEFAULT);
+        if (is == null) {
             if (_log.shouldLog(Log.WARN))
-                _log.warn("Country file not found: " + geoFile.getAbsolutePath());
+                _log.warn("Country file not found");
             return;
         }
         BufferedReader br = null;
         try {
-            br = new BufferedReader(new InputStreamReader(
-                    new FileInputStream(geoFile), "UTF-8"));
+            br = new BufferedReader(new InputStreamReader(is, "UTF-8"));
             String line = null;
             while ( (line = br.readLine()) != null) {
                 try {
@@ -551,6 +547,7 @@ public class GeoIP {
             if (_log.shouldLog(Log.ERROR))
                 _log.error("Error reading the Country File", ioe);
         } finally {
+            try { is.close(); } catch (IOException ioe) {}
             if (br != null) try { br.close(); } catch (IOException ioe) {}
         }
     }
@@ -773,7 +770,7 @@ public class GeoIP {
      * @param ip IPv4 or IPv6
      */
     public void add(String ip) {
-        byte[] pib = Addresses.getIP(ip);
+        byte[] pib = Addresses.getIPOnly(ip);
         if (pib == null) return;
         add(pib);
     }
@@ -785,6 +782,14 @@ public class GeoIP {
      * @param ip IPv4 or IPv6
      */
     public void add(byte ip[]) {
+        // skip he.net tunnel 2001:470:: so we will get correct geoip from IPv4
+        // ditto route48
+        if (ip.length == 16 &&
+            ((ip[0] == 0x20 && ip[1] == 0x01 &&
+              ip[2] == 0x04 && ip[3] == 0x70) ||
+             (ip[0] == 0x2a && ip[1] == 0x06 &&
+              ip[2] == (byte) 0xa0 && ip[3] == 0x04)))
+            return;
         add(toLong(ip));
     }
 
@@ -807,7 +812,7 @@ public class GeoIP {
      * @return lower-case code, generally two letters, or null.
      */
     public String get(String ip) {
-        byte[] pib = Addresses.getIP(ip);
+        byte[] pib = Addresses.getIPOnly(ip);
         if (pib == null) return null;
         return get(pib);
     }
@@ -818,6 +823,11 @@ public class GeoIP {
      * @return lower-case code, generally two letters, or null.
      */
     String get(byte ip[]) {
+        // skip he.net tunnel 2001:470:: so we will get correct geoip from IPv4
+        if (ip.length == 16 &&
+            ip[0] == 0x20 && ip[1] == 0x01 &&
+            ip[2] == 0x04 && ip[3] == 0x70)
+            return null;
         return get(toLong(ip));
     }
 
@@ -880,6 +890,16 @@ public class GeoIP {
         return _codeToName.get(code);
     }
 
+    /**
+     * Get the country code map
+     *
+     * @return Map of two-letter lower case code to untranslated country name, unmodifiable
+     * @since 0.9.53
+     */
+    public Map<String, String> getCountries() {
+        return Collections.unmodifiableMap(_codeToName);
+    }
+
     public static void main(String args[]) {
         if (args.length <= 0) {
             System.out.println("Usage: GeoIP ip...");
