@@ -30,6 +30,7 @@ import java.util.TreeMap;
 import java.util.concurrent.ConcurrentHashMap;
 
 import net.i2p.crypto.SigType;
+import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterIdentity;
@@ -73,6 +74,7 @@ public class TransportManager implements TransportEventListener {
     private final Map<String, Transport> _pluggableTransports;
     private final RouterContext _context;
     private final UPnPManager _upnpManager;
+    private final SimpleTimer2.TimedEvent _upnpRefresher;
     private final DHSessionKeyBuilder.PrecalcRunner _dhThread;
     private final X25519KeyFactory _xdhThread;
     private final boolean _enableUDP;
@@ -81,26 +83,36 @@ public class TransportManager implements TransportEventListener {
 
     /** default true */
     public final static String PROP_ENABLE_UDP = "i2np.udp.enable";
+    /**
+     * Default true for now
+     * @since 0.9.57
+     */
+    public final static String PROP_ENABLE_SSU1 = "i2np.ssu1.enable";
+    /**
+     * Default true as of 0.9.56
+     * @since 0.9.54
+     */
+    public final static String PROP_ENABLE_SSU2 = "i2np.ssu2.enable";
     /** default true */
     public final static String PROP_ENABLE_NTCP = "i2np.ntcp.enable";
     /** default true */
     public final static String PROP_ENABLE_UPNP = "i2np.upnp.enable";
+    public final static String PROP_ENABLE_UPNP_IPV6 = "i2np.upnp.ipv6.enable";
+    public static final boolean DEFAULT_ENABLE_UPNP_IPV6 = true;
     private static final String PROP_JAVA_PROXY1 = "socksProxyHost";
     private static final String PROP_JAVA_PROXY2 = "java.net.useSystemProxies";
     private static final String PROP_JAVA_PROXY3 = "http.proxyHost";
     private static final String PROP_JAVA_PROXY4 = "https.proxyHost";
 
-    /** default true */
-    private static final String PROP_NTCP1_ENABLE = "i2np.ntcp1.enable";
-    private static final boolean DEFAULT_NTCP1_ENABLE = false;
-    private static final String PROP_NTCP2_ENABLE = "i2np.ntcp2.enable";
-    private static final boolean DEFAULT_NTCP2_ENABLE = true;
-
     private static final String PROP_ADVANCED = "routerconsole.advanced";
     
     /** not forever, since they may update */
     private static final long SIGTYPE_BANLIST_DURATION = 36*60*60*1000L;
 
+    private static final long UPNP_REFRESH_TIME = UPnP.LEASE_TIME_SECONDS * 1000L / 3;
+
+    private final long _msgIDBloomXor;
+
     public TransportManager(RouterContext context) {
         _context = context;
         _log = _context.logManager().getLog(TransportManager.class);
@@ -117,14 +129,14 @@ public class TransportManager implements TransportEventListener {
         boolean isProxied = isProxied();
         boolean enableUPnP = !isProxied && _context.getBooleanPropertyDefaultTrue(PROP_ENABLE_UPNP);
         _upnpManager = enableUPnP ? new UPnPManager(context, this) : null;
+        _upnpRefresher = enableUPnP ? new UPnPRefresher() : null;
         _enableUDP = _context.getBooleanPropertyDefaultTrue(PROP_ENABLE_UDP);
-        _enableNTCP1 = isNTCPEnabled(context) &&
-                       context.getProperty(PROP_NTCP1_ENABLE, DEFAULT_NTCP1_ENABLE);
-        boolean enableNTCP2 = isNTCPEnabled(context) &&
-                              context.getProperty(PROP_NTCP2_ENABLE, DEFAULT_NTCP2_ENABLE);
+        _enableNTCP1 = false;
+        boolean enableNTCP2 = isNTCPEnabled(context);
         _dhThread = (_enableUDP || enableNTCP2) ? new DHSessionKeyBuilder.PrecalcRunner(context) : null;
         // always created, even if NTCP2 is not enabled, because ratchet needs it
         _xdhThread = new X25519KeyFactory(context);
+        _msgIDBloomXor = _context.random().nextLong(I2NPMessage.MAX_ID_VALUE);
     }
 
     /**
@@ -144,7 +156,7 @@ public class TransportManager implements TransportEventListener {
             System.out.println(msg);
             _log.logAlways(Log.WARN, msg);
             rv = true;
-        } else if (!SystemVersion.isMac() && Boolean.valueOf(System.getProperty(PROP_JAVA_PROXY2))) {
+        } else if (!SystemVersion.isMac() && Boolean.parseBoolean(System.getProperty(PROP_JAVA_PROXY2))) {
             try {
                 // Use ProxySelector to see if we would be proxied
                 // using a dummy address.
@@ -256,15 +268,16 @@ public class TransportManager implements TransportEventListener {
     private void configTransports() {
         Transport udp = null;
         if (_enableUDP) {
-            udp = new UDPTransport(_context, _dhThread);
+            boolean enableSSU1 = _context.getBooleanProperty(PROP_ENABLE_SSU1);
+            boolean enableSSU2 = _context.getBooleanPropertyDefaultTrue(PROP_ENABLE_SSU2);
+            DHSessionKeyBuilder.PrecalcRunner dh = enableSSU1 ? _dhThread : null;
+            X25519KeyFactory xdh = enableSSU2 ? _xdhThread : null;
+            udp = new UDPTransport(_context, dh, xdh);
             addTransport(udp);
             initializeAddress(udp);
         }
         if (isNTCPEnabled(_context)) {
-            DHSessionKeyBuilder.PrecalcRunner dh = _enableNTCP1 ? _dhThread : null;
-            boolean enableNTCP2 = _context.getProperty(PROP_NTCP2_ENABLE, DEFAULT_NTCP2_ENABLE);
-            X25519KeyFactory xdh = enableNTCP2 ? _xdhThread : null;
-            Transport ntcp = new NTCPTransport(_context, dh, xdh);
+            Transport ntcp = new NTCPTransport(_context, null, _xdhThread);
             addTransport(ntcp);
             initializeAddress(ntcp);
             if (udp != null) {
@@ -285,9 +298,7 @@ public class TransportManager implements TransportEventListener {
     }
     
     public static boolean isNTCPEnabled(RouterContext ctx) {
-        return ctx.getBooleanPropertyDefaultTrue(PROP_ENABLE_NTCP) &&
-               (ctx.getProperty(PROP_NTCP1_ENABLE, DEFAULT_NTCP1_ENABLE) ||
-                ctx.getProperty(PROP_NTCP2_ENABLE, DEFAULT_NTCP2_ENABLE));
+        return ctx.getBooleanPropertyDefaultTrue(PROP_ENABLE_NTCP);
     }
     
     /**
@@ -319,10 +330,10 @@ public class TransportManager implements TransportEventListener {
         Set<String> ipset = Addresses.getAddresses(_context.getBooleanProperty("i2np.allowLocal"), false, true);
         String lastv4 = _context.getProperty(UDPTransport.PROP_IP);
         String lastv6 = _context.getProperty(UDPTransport.PROP_IPV6);
-        boolean preferTemp = _context.getBooleanProperty(UDPTransport.PROP_LAPTOP_MODE);
+        boolean preferTemp = Boolean.parseBoolean(Addresses.useIPv6TempAddresses());
         //
         // Avoid IPv6 temporary addresses if we have a non-temporary one,
-        // unless laptop mode
+        // unless the kernel prefers them
         //
         boolean hasPreferredV6Address = false;
         List<InetAddress> addresses = new ArrayList<InetAddress>(4);
@@ -450,8 +461,10 @@ public class TransportManager implements TransportEventListener {
         // Always start on Android, as we may have a cellular IPv4 address but
         // are routing all traffic through WiFi.
         // Also, conditions may change rapidly.
-        if (_upnpManager != null && (SystemVersion.isAndroid() || Addresses.getAnyAddress() == null))
+        if (_upnpManager != null && (SystemVersion.isAndroid() || Addresses.getAnyAddress() == null)) {
             _upnpManager.start();
+            _upnpRefresher.schedule(UPNP_REFRESH_TIME);
+        }
         configTransports();
         _log.debug("Starting up the transport manager");
         // Let's do this in a predictable order to make testing easier
@@ -488,8 +501,10 @@ public class TransportManager implements TransportEventListener {
      *  Can be restarted.
      */
     synchronized void stopListening() {
-        if (_upnpManager != null)
+        if (_upnpManager != null) {
+            _upnpRefresher.cancel();
             _upnpManager.stop();
+        }
         for (Transport t : _transports.values()) {
             t.stopListening();
         }
@@ -655,20 +670,24 @@ public class TransportManager implements TransportEventListener {
     }    
     
     /**
-     *  @return a new set, may be modified
+     *  @return a new list, may be modified
      *  @since 0.9.34
      */
-    public Set<Hash> getEstablished() {
-        // for efficiency. NTCP is modifiable, SSU isn't
+    public List<Hash> getEstablished() {
+        // for efficiency
         Transport t = _transports.get("NTCP");
-        Set<Hash> rv;
+        List<Hash> rv = null;
         if (t != null)
             rv = t.getEstablished();
-        else
-            rv = new HashSet<Hash>(256);
         t = _transports.get("SSU");
-        if (t != null)
-            rv.addAll(t.getEstablished());
+        if (t != null) {
+            if (rv != null)
+                rv.addAll(t.getEstablished());
+            else
+                rv = t.getEstablished();
+        } else if (rv == null) {
+            rv = new ArrayList<Hash>(0);
+        }
         return rv;
     }
     
@@ -743,25 +762,43 @@ public class TransportManager implements TransportEventListener {
     static class Port {
         public final String style;
         public final int port;
+        public final boolean isIPv6;
+        public final String ip;
 
+        /**
+         *  IPv4 only
+         */
         public Port(String style, int port) {
             this.style = style;
             this.port = port;
+            isIPv6 = false;
+            ip = null;
+        }
+
+        /**
+         *  IPv6 only
+         *  @since 0.9.50
+         */
+        public Port(String style, String host, int port) {
+            this.style = style;
+            this.port = port;
+            isIPv6 = true;
+            ip = host;
         }
 
         @Override
         public int hashCode() {
-            return style.hashCode() ^ port;
+            return style.hashCode() ^ port ^ DataHelper.hashCode(ip);
         }
 
         @Override
         public boolean equals(Object o) {
-            if (o == null)
-                return false;
+            if (o == this)
+                return true;
             if (! (o instanceof Port))
                 return false;
             Port p = (Port) o;
-            return port == p.port && style.equals(p.style);
+            return port == p.port && style.equals(p.style) && DataHelper.eq(ip, p.ip);
         }
     }
 
@@ -771,6 +808,8 @@ public class TransportManager implements TransportEventListener {
      */
     private Set<Port> getPorts() {
         Set<Port> rv = new HashSet<Port>(4);
+        if (_context.router().isHidden())
+            return rv;
         for (Transport t : _transports.values()) {
             int port = t.getRequestedPort();
             // Use UDP port for NTCP too - see comment in NTCPTransport.getRequestedPort() for why this is here
@@ -780,8 +819,31 @@ public class TransportManager implements TransportEventListener {
                 if (udp != null)
                     port = udp.getRequestedPort();
             }
-            if (port > 0)
-                rv.add(new Port(t.getStyle(), port));
+            if (port > 0) {
+                TransportUtil.IPv6Config config = t.getIPv6Config();
+                // ipv4
+                if (config != TransportUtil.IPv6Config.IPV6_ONLY &&
+                    !t.isIPv4Firewalled()) {
+                    rv.add(new Port(t.getStyle(), port));
+                }
+                // ipv6
+                if (_context.getProperty(PROP_ENABLE_UPNP_IPV6, DEFAULT_ENABLE_UPNP_IPV6) &&
+                    config != TransportUtil.IPv6Config.IPV6_DISABLED &&
+                    !t.isIPv6Firewalled()) {
+                    RouterAddress ra = t.getCurrentAddress(true);
+                    if (ra == null || ra.getHost() == null) {
+                        if (t.getStyle().equals(UDPTransport.STYLE)) {
+                            UDPTransport udp = (UDPTransport) t;
+                            ra = udp.getCurrentExternalAddress(true);
+                        }
+                    }
+                    if (ra != null) {
+                        String host = ra.getHost();
+                        if (host != null)
+                            rv.add(new Port(t.getStyle(), host, port));
+                    }
+                }
+            }
         }
         return rv;
     }
@@ -801,7 +863,7 @@ public class TransportManager implements TransportEventListener {
             throw new IllegalArgumentException("Bids for a message bound to ourselves?");
 
         List<TransportBid> rv = new ArrayList<TransportBid>(_transports.size());
-        Set<String> failedTransports = msg.getFailedTransports();
+        List<String> failedTransports = msg.getFailedTransports();
         for (Transport t : _transports.values()) {
             if (failedTransports.contains(t.getStyle())) {
                 if (_log.shouldLog(Log.DEBUG))
@@ -827,7 +889,7 @@ public class TransportManager implements TransportEventListener {
     TransportBid getNextBid(OutNetMessage msg) {
         int unreachableTransports = 0;
         Hash peer = msg.getTarget().getIdentity().calculateHash();
-        Set<String> failedTransports = msg.getFailedTransports();
+        List<String> failedTransports = msg.getFailedTransports();
         TransportBid rv = null;
         for (Transport t : _transports.values()) {
             if (t.isUnreachable(peer)) {
@@ -908,7 +970,7 @@ public class TransportManager implements TransportEventListener {
         if (_log.shouldLog(Log.DEBUG))
             _log.debug("I2NPMessage received: " + message.getClass().getSimpleName() /*, new Exception("Where did I come from again?") */ );
         try {
-            _context.inNetMessagePool().add(message, fromRouter, fromRouterHash);
+            _context.inNetMessagePool().add(message, fromRouter, fromRouterHash, _msgIDBloomXor);
             //if (_log.shouldLog(Log.DEBUG))
             //    _log.debug("Added to in pool");
         } catch (IllegalArgumentException iae) {
@@ -931,7 +993,10 @@ public class TransportManager implements TransportEventListener {
                         _upnpUpdateQueued = true;
                         _context.simpleTimer2().addEvent(new UpdatePorts(), 3250);
                     } else {
-                        _upnpManager.update(getPorts());
+                        // throw onto timer to avoid deadlock
+                        //_upnpManager.update(getPorts());
+                        _upnpUpdateQueued = true;
+                        _context.simpleTimer2().addEvent(new UpdatePorts(), 0);
                     }
                 }
             }
@@ -945,13 +1010,31 @@ public class TransportManager implements TransportEventListener {
      */
     private class UpdatePorts implements SimpleTimer.TimedEvent {
         public void timeReached() {
+            Set<Port> ports = getPorts();
             synchronized (_upnpManager) {
                 _upnpUpdateQueued = false;
-                _upnpManager.update(getPorts());
+                _upnpManager.update(ports);
             }
         }
     }
 
+    /**
+     * Periodic refresh of UPnP ports.
+     * This is required because UPnP leases expire.
+     * UPnPManager.Rescanner finds new devices but does not refresh the ports.
+     * Caller must schedule.
+     *
+     * @since 0.9.50
+     */
+    private class UPnPRefresher extends SimpleTimer2.TimedEvent {
+        public UPnPRefresher() { super(_context.simpleTimer2()); }
+
+        public void timeReached() {
+            transportAddressChanged();
+            reschedule(UPNP_REFRESH_TIME);
+        }
+    }
+
     List<String> getMostRecentErrorMessages() { 
         List<String> rv = new ArrayList<String>(16);
         for (Transport t : _transports.values()) {
