@@ -24,12 +24,14 @@ import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.LinkedBlockingQueue;
 
 import net.i2p.I2PAppContext;
-import net.i2p.data.ByteArray;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterIdentity;
 import net.i2p.router.CommSystemFacade.Status;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.FIFOBandwidthLimiter;
+import net.i2p.stat.Rate;
+import net.i2p.stat.RateAverages;
+import net.i2p.stat.RateStat;
 import net.i2p.util.TryCache;
 import net.i2p.util.Addresses;
 import net.i2p.util.ConcurrentHashSet;
@@ -54,7 +56,7 @@ class EventPumper implements Runnable {
     private final Queue<ServerSocketChannel> _wantsRegister = new ConcurrentLinkedQueue<ServerSocketChannel>();
     private final Queue<NTCPConnection> _wantsConRegister = new ConcurrentLinkedQueue<NTCPConnection>();
     private final NTCPTransport _transport;
-    private final ObjectCounter<ByteArray> _blockedIPs;
+    private final ObjectCounter<String> _blockedIPs;
     private long _expireIdleWriteTime;
     private static final boolean _useDirect = false;
     private final boolean _nodelay;
@@ -87,12 +89,13 @@ class EventPumper implements Runnable {
     private static final long FAILSAFE_ITERATION_FREQ = 2*1000l;
     private static final int FAILSAFE_LOOP_COUNT = 512;
     private static final long SELECTOR_LOOP_DELAY = 200;
-    private static final long BLOCKED_IP_FREQ = 3*60*1000;
+    private static final long BLOCKED_IP_FREQ = 12*60*1000;
 
     /** tunnel test now disabled, but this should be long enough to allow an active tunnel to get started */
     private static final long MIN_EXPIRE_IDLE_TIME = 120*1000l;
     private static final long MAX_EXPIRE_IDLE_TIME = 11*60*1000l;
     private static final long MAY_DISCON_TIMEOUT = 10*1000;
+    private static final long RI_STORE_INTERVAL = 29*60*1000;
 
     /**
      *  Do we use direct buffers for reading? Default false.
@@ -123,13 +126,14 @@ class EventPumper implements Runnable {
         _log = ctx.logManager().getLog(getClass());
         _transport = transport;
         _expireIdleWriteTime = MAX_EXPIRE_IDLE_TIME;
-        _blockedIPs = new ObjectCounter<ByteArray>();
+        _blockedIPs = new ObjectCounter<String>();
         _context.statManager().createRateStat("ntcp.pumperKeySetSize", "", "ntcp", new long[] {10*60*1000} );
         //_context.statManager().createRateStat("ntcp.pumperKeysPerLoop", "", "ntcp", new long[] {10*60*1000} );
         _context.statManager().createRateStat("ntcp.pumperLoopsPerSecond", "", "ntcp", new long[] {10*60*1000} );
         _context.statManager().createRateStat("ntcp.zeroRead", "", "ntcp", new long[] {10*60*1000} );
         _context.statManager().createRateStat("ntcp.zeroReadDrop", "", "ntcp", new long[] {10*60*1000} );
         _context.statManager().createRateStat("ntcp.dropInboundNoMessage", "", "ntcp", new long[] {10*60*1000} );
+        _context.statManager().createRequiredRateStat("ntcp.inboundConn", "Inbound NTCP Connection", "ntcp", new long[] { 60*1000L } );
         _nodelay = ctx.getBooleanPropertyDefaultTrue(PROP_NODELAY);
     }
     
@@ -280,15 +284,17 @@ class EventPumper implements Runnable {
                                     continue;
                                 }
 
-                                if ( (!con.isWriteBufEmpty()) &&
-                                     ((key.interestOps() & SelectionKey.OP_WRITE) == 0) ) {
-                                    // the data queued to be sent has already passed through
-                                    // the bw limiter and really just wants to get shoved
-                                    // out the door asap.
-                                    if (_log.shouldLog(Log.INFO))
-                                        _log.info("Failsafe write for " + con);
-                                    key.interestOps(SelectionKey.OP_WRITE | key.interestOps());
-                                    failsafeWrites++;
+                                synchronized(con.getWriteLock()) {
+                                    if ( (!con.isWriteBufEmpty()) &&
+                                         ((key.interestOps() & SelectionKey.OP_WRITE) == 0) ) {
+                                        // the data queued to be sent has already passed through
+                                        // the bw limiter and really just wants to get shoved
+                                        // out the door asap.
+                                        if (_log.shouldLog(Log.INFO))
+                                            _log.info("Failsafe write for " + con);
+                                        setInterest(key, SelectionKey.OP_WRITE);
+                                        failsafeWrites++;
+                                    }
                                 }
                                 
                                 final long expire;
@@ -310,6 +316,18 @@ class EventPumper implements Runnable {
                                     if (_log.shouldInfo())
                                         _log.info("Failsafe or expire close for " + con);
                                     failsafeCloses++;
+                                } else {
+                                    // periodically send our RI
+                                    long estab = con.getEstablishedOn();
+                                    if (estab > 0) {
+                                        long uptime = now - estab;
+                                        if (uptime >= RI_STORE_INTERVAL) {
+                                            long mod = uptime % RI_STORE_INTERVAL;
+                                            if (mod < FAILSAFE_ITERATION_FREQ) {
+                                                con.sendOurRouterInfo(false);
+                                            }
+                                        }
+                                    }
                                 }
                             } catch (CancelledKeyException cke) {
                                 // cancelled while updating the interest ops.  ah well
@@ -406,7 +424,7 @@ class EventPumper implements Runnable {
                     processAccept(key);
                 }
                 if (connect) {
-                    key.interestOps(key.interestOps() & ~SelectionKey.OP_CONNECT);
+                    clearInterest(key, SelectionKey.OP_CONNECT);
                     processConnect(key);
                 }
                 if (read) {
@@ -426,35 +444,6 @@ class EventPumper implements Runnable {
         }
     }
 
-    /**
-     *  Called by the connection when it has data ready to write.
-     *  If we have bandwidth, calls con.Write() which calls wantsWrite(con).
-     *  If no bandwidth, calls con.queuedWrite().
-     */
-    public void wantsWrite(NTCPConnection con, byte data[]) {
-        wantsWrite(con, data, 0, data.length);
-    }
-
-    /**
-     *  Called by the connection when it has data ready to write.
-     *  If we have bandwidth, calls con.Write() which calls wantsWrite(con).
-     *  If no bandwidth, calls con.queuedWrite().
-     *
-     *  @since 0.9.35 off/len version
-     */
-    public void wantsWrite(NTCPConnection con, byte data[], int off, int len) {
-        ByteBuffer buf = ByteBuffer.wrap(data, off, len);
-        FIFOBandwidthLimiter.Request req = _context.bandwidthLimiter().requestOutbound(len, 0, "NTCP write");//con, buf);
-        if (req.getPendingRequested() > 0) {
-            if (_log.shouldLog(Log.INFO))
-                _log.info("queued write on " + con + " for " + len);
-            _context.statManager().addRateData("ntcp.wantsQueuedWrite", 1);
-            con.queuedWrite(buf, req);
-        } else {
-            con.write(buf);
-        }
-    }
-
     /**
      *  Called by the connection when it has data ready to write (after bw allocation).
      *  Only wakeup if new.
@@ -506,31 +495,39 @@ class EventPumper implements Runnable {
                 return;
             chan.configureBlocking(false);
 
-            if (!_transport.allowConnection()) {
-                if (_log.shouldLog(Log.WARN))
-                    _log.warn("Receive session request but at connection limit: " + chan.socket().getInetAddress());
-                try { chan.close(); } catch (IOException ioe) { }
-                return;
-            }
-
             byte[] ip = chan.socket().getInetAddress().getAddress();
+            String ba = Addresses.toString(ip);
             if (_context.blocklist().isBlocklisted(ip)) {
                 if (_log.shouldLog(Log.WARN))
-                    _log.warn("Receive session request from blocklisted IP: " + chan.socket().getInetAddress());
+                    _log.warn("Receive session request from blocklisted IP: " + ba);
                 try { chan.close(); } catch (IOException ioe) { }
                 return;
             }
+            if (!_context.commSystem().isExemptIncoming(Addresses.toCanonicalString(ba))) {
+                if (!_transport.allowConnection()) {
+                    if (_log.shouldLog(Log.WARN))
+                        _log.warn("Receive session request but at connection limit: " + ba);
+                    try { chan.close(); } catch (IOException ioe) { }
+                    return;
+                }
+
+                int count = _blockedIPs.count(ba);
+                if (count > 0) {
+                    count = _blockedIPs.increment(ba);
+                    if (_log.shouldLog(Log.WARN))
+                       _log.warn("Blocking accept of IP with count " + count + ": " + ba);
+                    _context.statManager().addRateData("ntcp.dropInboundNoMessage", count);
+                    try { chan.close(); } catch (IOException ioe) { }
+                    return;
+                }
+
+                if (!shouldAllowInboundEstablishment()) {
+                    try { chan.close(); } catch (IOException ioe) { }
+                    return;
+                }
+            }
 
-            ByteArray ba = new ByteArray(ip);
-            int count = _blockedIPs.count(ba);
-            if (count > 0) {
-                count = _blockedIPs.increment(ba);
-                if (_log.shouldLog(Log.WARN))
-                   _log.warn("Blocking accept of IP with count " + count + ": " + Addresses.toString(ip));
-                _context.statManager().addRateData("ntcp.dropInboundNoMessage", count);
-                try { chan.close(); } catch (IOException ioe) { }
-                return;
-            }
+            _context.statManager().addRateData("ntcp.inboundConn", 1);
 
             if (shouldSetKeepAlive(chan))
                 chan.socket().setKeepAlive(true);
@@ -545,6 +542,60 @@ class EventPumper implements Runnable {
             _log.error("Error accepting", ioe);
         }
     }
+
+    /**
+     * Should we allow another inbound establishment?
+     * Used to throttle outbound hole punches.
+     * @since 0.9.2
+     */
+    private boolean shouldAllowInboundEstablishment() {
+        RateStat rs = _context.statManager().getRate("ntcp.inboundConn");
+        if (rs == null)
+            return true;
+        Rate r = rs.getRate(60*1000);
+        if (r == null)
+            return true;
+        int last;
+        long periodStart;
+        RateAverages ra = RateAverages.getTemp();
+        synchronized(r) {
+            last = (int) r.getLastEventCount();
+            periodStart = r.getLastCoalesceDate();
+            r.computeAverages(ra, true);
+        }
+        // compare incoming conns per ms, min of 1 per second or 60/minute
+        if (last < 15)
+            last = 15;
+        int total = (int) ra.getTotalEventCount();
+        int current = total - last;
+        if (current <= 0)
+            return true;
+        // getLastEventCount() is normalized to the rate, so we use the canonical period
+        int lastPeriod = 60*1000;
+        double avg = ra.getAverage();
+        int currentTime = (int) (_context.clock().now() - periodStart);
+        if (currentTime <= 5*1000)
+            return true;
+        // compare incoming conns per ms
+        // both of these are scaled by actual period in coalesce
+        float lastRate = last / (float) lastPeriod;
+        float currentRate = (float) (current / (double) currentTime);
+        float factor = _transport.haveCapacity(95) ? 1.05f : 0.95f;
+        float minThresh = factor * lastRate;
+        if (currentRate > minThresh) {
+            // chance in 128
+            // max out at about 25% over the last rate
+            int probAccept = Math.max(1, ((int) (4 * 128 * currentRate / minThresh)) - 512);
+            if (probAccept >= 128 || _context.random().nextInt(128) < probAccept) {
+                if (_log.shouldWarn())
+                    _log.warn("Probabalistic drop incoming (p=" + probAccept  +
+                              "/128) last rate " + last + "/min current rate " +
+                              (int) (currentRate * 60*1000));
+                return false;
+            }
+        }
+        return true; 
+    }
     
     private void processConnect(SelectionKey key) {
         final NTCPConnection con = (NTCPConnection)key.attachment();
@@ -619,10 +670,10 @@ class EventPumper implements Runnable {
                         int count;
                         if (addr != null) {
                             byte[] ip = addr.getAddress();
-                            ByteArray ba = new ByteArray(ip);
+                            String ba = Addresses.toString(ip);
                             count = _blockedIPs.increment(ba);
                             if (_log.shouldLog(Log.WARN))
-                                _log.warn("EOF on inbound before receiving any, blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con);
+                                _log.warn("EOF on inbound before receiving any, blocking IP " + ba + " with count " + count + ": " + con);
                         } else {
                             count = 1;
                             if (_log.shouldLog(Log.WARN))
@@ -666,7 +717,7 @@ class EventPumper implements Runnable {
                 FIFOBandwidthLimiter.Request req = _context.bandwidthLimiter().requestInbound(read, "NTCP read"); //con, buf);
                 if (req.getPendingRequested() > 0) {
                     // rare since we generally don't throttle inbound
-                    key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
+                    clearInterest(key, SelectionKey.OP_READ);
                     _context.statManager().addRateData("ntcp.queuedRecv", read);
                     con.queuedRecv(buf, req);
                     break;
@@ -695,14 +746,13 @@ class EventPumper implements Runnable {
             if (buf != null)
                 releaseBuf(buf);
             if (con.isInbound() && con.getMessagesReceived() <= 0) {
-                InetAddress addr = chan.socket().getInetAddress();
+                byte[] ip = con.getRemoteIP();
                 int count;
-                if (addr != null) {
-                    byte[] ip = addr.getAddress();
-                    ByteArray ba = new ByteArray(ip);
+                if (ip != null) {
+                    String ba = Addresses.toString(ip);
                     count = _blockedIPs.increment(ba);
                     if (_log.shouldLog(Log.WARN))
-                        _log.warn("Blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con, ioe);
+                        _log.warn("Blocking IP " + ba + " with count " + count + ": " + con, ioe);
                 } else {
                     count = 1;
                     if (_log.shouldLog(Log.WARN))
@@ -729,7 +779,7 @@ class EventPumper implements Runnable {
             if (buf != null)
                 releaseBuf(buf);
             // ???
-            key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
+            clearInterest(key, SelectionKey.OP_READ);
             if (_log.shouldLog(Log.WARN))
                 _log.warn("error reading on " + con, nyce);
         }
@@ -742,50 +792,75 @@ class EventPumper implements Runnable {
      */
     private void processWrite(SelectionKey key) {
         final NTCPConnection con = (NTCPConnection)key.attachment();
+        processWrite(con, key);
+    }
+
+    /**
+     *  Asynchronous write all buffers to the channel.
+     *  This method will disable the interest if no more writes remain.
+     *  If this returns false, caller MUST call wantsWrite(con)
+     *
+     *  @param key non-null
+     *  @return true if all buffers were completely written, false if buffers remain
+     *  @since 0.9.53
+     */
+    public boolean processWrite(final NTCPConnection con, final SelectionKey key) {
+        boolean rv = false;
         final SocketChannel chan = con.getChannel();
         try {
-            while (true) {
-                ByteBuffer buf = con.getNextWriteBuf();
-                if (buf != null) {
-                    if (buf.remaining() <= 0) {
-                        con.removeWriteBuf(buf);
-                        continue;                    
-                    }
-                    int written = chan.write(buf);
-                    //totalWritten += written;
-                    if (written == 0) {
-                        if ( (buf.remaining() > 0) || (!con.isWriteBufEmpty()) ) {
+            synchronized(con.getWriteLock()) {
+                while (true) {
+                    ByteBuffer buf = con.getNextWriteBuf();
+                    if (buf != null) {
+                        if (buf.remaining() <= 0) {
+                            con.removeWriteBuf(buf);
+                            continue;
+                        }
+                        int written = chan.write(buf);
+                        //totalWritten += written;
+                        if (written == 0) {
+                            if ( (buf.remaining() > 0) || (!con.isWriteBufEmpty()) ) {
+                                // stay interested
+                                //key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
+                            } else {
+                                rv = true;
+                            }
+                            break;
+                        } else if (buf.remaining() > 0) {
                             // stay interested
                             //key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
+                            break;
                         } else {
-                            key.interestOps(key.interestOps() & ~SelectionKey.OP_WRITE);
+                            con.removeWriteBuf(buf);
+                            //if (buffer time is too much, add OP_WRITE to the interest ops and break?)
+                            // LOOP
+                        }
+                    } else {
+                        // Nothing more to write
+                        if (key.isValid()) {
+                            rv = true;
                         }
                         break;
-                    } else if (buf.remaining() > 0) {
-                        // stay interested
-                        //key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
-                        continue;
-                    } else {
-                        con.removeWriteBuf(buf);
-                        //if (buffer time is too much, add OP_WRITe to the interest ops and break?)
-                        // LOOP
                     }
-                } else {
-                    // Nothing more to write
-		    if (key.isValid())
-                    	key.interestOps(key.interestOps() & ~SelectionKey.OP_WRITE);
-                    break;
                 }
+                if (rv)
+                    clearInterest(key, SelectionKey.OP_WRITE);
+                else
+                    setInterest(key, SelectionKey.OP_WRITE);
             }
+        // catch and close outside the write lock to avoid deadlocks in NTCPCon.locked_close()
         } catch (CancelledKeyException cke) {
             if (_log.shouldLog(Log.WARN)) _log.warn("error writing on " + con, cke);
             _context.statManager().addRateData("ntcp.writeError", 1);
             con.close();
+            rv = true;
         } catch (IOException ioe) {
             if (_log.shouldLog(Log.WARN)) _log.warn("error writing on " + con, ioe);
             _context.statManager().addRateData("ntcp.writeError", 1);
             con.close();
+            rv = true;
         }
+        return rv;
     }
     
     /**
@@ -798,7 +873,7 @@ class EventPumper implements Runnable {
         while ((con = _wantsRead.poll()) != null) {
             SelectionKey key = con.getKey();
             try {
-                key.interestOps(key.interestOps() | SelectionKey.OP_READ);
+                setInterest(key, SelectionKey.OP_READ);
             } catch (CancelledKeyException cke) {
                 // ignore, we remove/etc elsewhere
                 if (_log.shouldLog(Log.WARN))
@@ -833,7 +908,7 @@ class EventPumper implements Runnable {
                 if (key == null)
                     continue;
                 try {
-                    key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
+                    setInterest(key, SelectionKey.OP_WRITE);
                 } catch (CancelledKeyException cke) {
                    if (_log.shouldLog(Log.WARN))
                        _log.warn("RDE CKE 2", cke);
@@ -865,7 +940,7 @@ class EventPumper implements Runnable {
                 con.setKey(key);
                 RouterAddress naddr = con.getRemoteAddress();
                 try {
-                    // no DNS lookups, do not use host names
+                    // no DNS lookups, do not use hostnames
                     int port = naddr.getPort();
                     byte[] ip = naddr.getIP();
                     if (port <= 0 || ip == null)
@@ -874,7 +949,7 @@ class EventPumper implements Runnable {
                     boolean connected = schan.connect(saddr);
                     if (connected) {
                         // Never happens, we use nonblocking
-                        key.interestOps(SelectionKey.OP_READ);
+                        setInterest(key, SelectionKey.OP_READ);
                         processConnect(key);
                     }
                 } catch (IOException ioe) {
@@ -903,6 +978,18 @@ class EventPumper implements Runnable {
         }
     }
 
+    /**
+     *  Temp. block inbound from this IP
+     *
+     *  @since 0.9.58
+     */
+    public void blockIP(byte[] ip) {
+        if (ip == null)
+            return;
+        String ba = Addresses.toString(ip);
+        _blockedIPs.increment(ba);
+    }
+
     private long _lastExpired;
 
     private void expireTimedOut() {
@@ -910,4 +997,32 @@ class EventPumper implements Runnable {
     }
 
     public long getIdleTimeout() { return _expireIdleWriteTime; }
+
+    /**
+     *  Warning - caller should catch unchecked CancelledKeyException
+     *
+     *  @throws CancelledKeyException which is unchecked
+     *  @since 0.9.53
+     */
+    public static void setInterest(SelectionKey key, int op) throws CancelledKeyException {
+        synchronized(key) {
+            int old = key.interestOps();
+            if ((old & op) == 0)
+                key.interestOps(old | op);
+        }
+    }
+
+    /**
+     *  Warning - caller should catch unchecked CancelledKeyException
+     *
+     *  @throws CancelledKeyException which is unchecked
+     *  @since 0.9.53
+     */
+    public static void clearInterest(SelectionKey key, int op) throws CancelledKeyException {
+        synchronized(key) {
+            int old = key.interestOps();
+            if ((old & op) != 0)
+                key.interestOps(old & ~op);
+        }
+    }
 }
