@@ -33,6 +33,7 @@ import net.i2p.router.RouterContext;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
 import static net.i2p.router.transport.ntcp.OutboundNTCP2State.*;
+import net.i2p.util.Addresses;
 import net.i2p.util.ByteArrayStream;
 import net.i2p.util.ByteCache;
 import net.i2p.util.Log;
@@ -40,7 +41,7 @@ import net.i2p.util.SimpleByteCache;
 
 /**
  *
- *  NTCP 1 or 2. We are Bob.
+ *  NTCP 2. We are Bob.
  *
  *  @since 0.9.35 pulled out of EstablishState
  */
@@ -74,7 +75,6 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
     private static final int MAX_DATA_READ_BUFS = 32;
     private static final ByteCache _dataReadBufs = ByteCache.getInstance(MAX_DATA_READ_BUFS, BUFFER_SIZE);
 
-    private static final int NTCP1_MSG1_SIZE = XY_SIZE + HXY_SIZE;
     // 287 - 64 = 223
     private static final int PADDING1_MAX = TOTAL1_MAX - MSG1_SIZE;
     private static final int PADDING1_FAIL_MAX = 128;
@@ -89,7 +89,6 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
     private static final Set<State> STATES_NTCP2 =
         EnumSet.of(State.IB_NTCP2_INIT, State.IB_NTCP2_GOT_X, State.IB_NTCP2_GOT_PADDING,
                    State.IB_NTCP2_SENT_Y, State.IB_NTCP2_GOT_RI, State.IB_NTCP2_READ_RANDOM);
-    private static final Set<State> STATES_MSG3 = EnumSet.of(State.IB_SENT_Y, State.IB_GOT_RI_SIZE, State.IB_GOT_RI);
 
     
     public InboundEstablishState(RouterContext ctx, NTCPTransport transport, NTCPConnection con) {
@@ -123,17 +122,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
      *  @since 0.9.35
      */
     public int getVersion() {
-        if (!_transport.isNTCP2Enabled())
-            return 1;
-        if (!_transport.isNTCP1Enabled())
             return 2;
-        synchronized (_stateLock) {
-            if (_state == State.IB_INIT)
-                return 0;
-            if (STATES_NTCP2.contains(_state))
-                return 2;
-            return 1;
-        } 
     } 
 
     /**
@@ -153,7 +142,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         }
         if (_state == State.IB_INIT && src.hasRemaining()) {
             int remaining = src.remaining();
-            if (_transport.isNTCP2Enabled()) {
+
                 if (remaining + _received < MSG1_SIZE) {
                     // Less than 64 total received, so we defer the NTCP 1 or 2 decision.
                     // Buffer in _X.
@@ -164,8 +153,8 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                         _log.warn("Short buffer got " + remaining + " total now " + _received + " on " + this);
                     return;
                 }
-                if (remaining + _received < NTCP1_MSG1_SIZE ||
-                    !_transport.isNTCP1Enabled()) {
+                //if (remaining + _received < NTCP1_MSG1_SIZE ||
+                //    !_transport.isNTCP1Enabled()) {
                     // Less than 288 total received, assume NTCP2
                     // TODO can't change our mind later if we get more than 287
                     _con.setVersion(2);
@@ -173,343 +162,8 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                     receiveInboundNTCP2(src);
                     // releaseBufs() will return the unused DH
                     return;
-                }
-            }
-            int toGet = Math.min(remaining, XY_SIZE - _received);
-            src.get(_X, _received, toGet);
-            _received += toGet;
-            if (_received < XY_SIZE)
-                return;
-            changeState(State.IB_GOT_X);
-            _received = 0;
-        }
+                //}
 
-        if (_state == State.IB_GOT_X && src.hasRemaining()) {
-            int toGet = Math.min(src.remaining(), HXY_SIZE - _received);
-            src.get(_hX_xor_bobIdentHash, _received, toGet);
-            _received += toGet;
-            if (_received < HXY_SIZE)
-                return;
-            changeState(State.IB_GOT_HX);
-            _received = 0;
-        }
-
-        if (_state == State.IB_GOT_HX) {
-
-                if (_log.shouldLog(Log.DEBUG))
-                    _log.debug(prefix()+"Enough data for a DH received");
-
-                // first verify that Alice knows who she is trying to talk with and that the X
-                // isn't corrupt
-                byte[] realXor = SimpleByteCache.acquire(HXY_SIZE);
-                _context.sha().calculateHash(_X, 0, XY_SIZE, realXor, 0);
-                xor32(_context.routerHash().getData(), realXor);
-                if (!DataHelper.eq(realXor, _hX_xor_bobIdentHash)) {
-                    SimpleByteCache.release(realXor);
-                    _context.statManager().addRateData("ntcp.invalidHXxorBIH", 1);
-                    fail("Invalid hX_xor");
-                    return;
-                }
-                SimpleByteCache.release(realXor);
-                if (!_transport.isHXHIValid(_hX_xor_bobIdentHash)) {
-                    // blocklist source? but spoofed IPs could DoS us
-                    _context.statManager().addRateData("ntcp.replayHXxorBIH", 1);
-                    fail("Replay hX_xor");
-                    return;
-                }
-
-                try {
-                    // ok, they're actually trying to talk to us, and we got their (unauthenticated) X
-                    _dh.setPeerPublicValue(_X);
-                    _dh.getSessionKey(); // force the calc
-                    System.arraycopy(_hX_xor_bobIdentHash, AES_SIZE, _prevEncrypted, 0, AES_SIZE);
-                    if (_log.shouldLog(Log.DEBUG))
-                        _log.debug(prefix()+"DH session key calculated (" + _dh.getSessionKey().toBase64() + ")");
-
-                    // now prepare our response: Y+E(H(X+Y)+tsB+padding, sk, Y[239:255])
-                    byte xy[] = new byte[XY_SIZE + XY_SIZE];
-                    System.arraycopy(_X, 0, xy, 0, XY_SIZE);
-                    System.arraycopy(_Y, 0, xy, XY_SIZE, XY_SIZE);
-                    byte[] hxy = SimpleByteCache.acquire(HXY_SIZE);
-                    _context.sha().calculateHash(xy, 0, XY_SIZE + XY_SIZE, hxy, 0);
-                    // our (Bob's) timestamp in seconds
-                    _tsB = (_context.clock().now() + 500) / 1000l;
-                    byte toEncrypt[] = new byte[HXY_TSB_PAD_SIZE];  // 48
-                    System.arraycopy(hxy, 0, toEncrypt, 0, HXY_SIZE);
-                    byte tsB[] = DataHelper.toLong(4, _tsB);
-                    System.arraycopy(tsB, 0, toEncrypt, HXY_SIZE, tsB.length);
-                    _context.random().nextBytes(toEncrypt, HXY_SIZE + 4, 12);
-                    if (_log.shouldLog(Log.DEBUG)) {
-                        _log.debug(prefix()+"h(x+y)="+Base64.encode(hxy));
-                        _log.debug(prefix() + "tsb = " + _tsB);
-                        _log.debug(prefix()+"unencrypted H(X+Y)+tsB+padding: " + Base64.encode(toEncrypt));
-                        _log.debug(prefix()+"encryption iv= " + Base64.encode(_Y, XY_SIZE-AES_SIZE, AES_SIZE));
-                        _log.debug(prefix()+"encryption key= " + _dh.getSessionKey().toBase64());
-                    }
-                    SimpleByteCache.release(hxy);
-                    _context.aes().encrypt(toEncrypt, 0, _e_hXY_tsB, 0, _dh.getSessionKey(),
-                                           _Y, XY_SIZE-AES_SIZE, HXY_TSB_PAD_SIZE);
-                    if (_log.shouldLog(Log.DEBUG))
-                        _log.debug(prefix()+"encrypted H(X+Y)+tsB+padding: " + Base64.encode(_e_hXY_tsB));
-                    byte write[] = new byte[XY_SIZE + HXY_TSB_PAD_SIZE];
-                    System.arraycopy(_Y, 0, write, 0, XY_SIZE);
-                    System.arraycopy(_e_hXY_tsB, 0, write, XY_SIZE, HXY_TSB_PAD_SIZE);
-
-                    // ok, now that is prepared, we want to actually send it, so make sure we are up for writing
-                    changeState(State.IB_SENT_Y);
-                    _transport.getPumper().wantsWrite(_con, write);
-                    if (!src.hasRemaining()) return;
-                } catch (DHSessionKeyBuilder.InvalidPublicParameterException e) {
-                    _context.statManager().addRateData("ntcp.invalidDH", 1);
-                    fail("Invalid X", e);
-                    return;
-                } catch (IllegalStateException ise) {
-                    // setPeerPublicValue()
-                    fail("reused keys?", ise);
-                    return;
-                }
-
-        }
-
-        // ok, we are onto the encrypted area, i.e. Message #3
-        while (STATES_MSG3.contains(_state) && src.hasRemaining()) {
-
-                // Collect a 16-byte block
-                if (_received < AES_SIZE && src.hasRemaining()) {
-                    int toGet = Math.min(src.remaining(), AES_SIZE - _received);
-                    src.get(_curEncrypted, _received, toGet);
-                    _received += toGet;
-                    if (_received < AES_SIZE) {
-                        // no more bytes available in the buffer, and only a partial
-                        // block was read, so we can't decrypt it.
-                        if (_log.shouldLog(Log.DEBUG))
-                            _log.debug(prefix() + "end of available data with only a partial block read (" +
-                                       + _received + ")");
-                        return;
-                    }
-                }
-                // Decrypt the 16-byte block
-                if (_received >= AES_SIZE) {
-                    _context.aes().decrypt(_curEncrypted, 0, _curDecrypted, 0, _dh.getSessionKey(),
-                                           _prevEncrypted, 0, AES_SIZE);
-
-                    byte swap[] = _prevEncrypted;
-                    _prevEncrypted = _curEncrypted;
-                    _curEncrypted = swap;
-                    _received = 0;
-
-                    if (_state == State.IB_SENT_Y) { // we are on the first decrypted block
-                        int sz = (int)DataHelper.fromLong(_curDecrypted, 0, 2);
-                        if (sz < MIN_RI_SIZE || sz > MAX_RI_SIZE) {
-                            _context.statManager().addRateData("ntcp.invalidInboundSize", sz);
-                            fail("size is invalid", new Exception("size is " + sz));
-                            return;
-                        }
-                        if (_log.shouldLog(Log.DEBUG))
-                            _log.debug(prefix() + "got the RI size: " + sz);
-                        _aliceIdentSize  = sz;
-                        changeState(State.IB_GOT_RI_SIZE);
-
-                        // We must defer the calculations for total size of the message until
-                        //  we get the full alice ident so
-                        // we can determine how long the signature is.
-                        // See below
-
-                    }
-                    try {
-                        _sz_aliceIdent_tsA_padding_aliceSig.write(_curDecrypted);
-                    } catch (IOException ioe) {
-                        if (_log.shouldLog(Log.ERROR)) _log.error(prefix()+"Error writing to the baos?", ioe);
-                    }
-
-                    if (_state == State.IB_GOT_RI_SIZE &&
-                        _sz_aliceIdent_tsA_padding_aliceSig.size() >= 2 + _aliceIdentSize) {
-                        // we have enough to get Alice's RI and determine the sig+padding length
-                        readAliceRouterIdentity();
-                        if (_log.shouldLog(Log.DEBUG))
-                            _log.debug(prefix() + "got the RI");
-                        if (_aliceIdent == null) {
-                            // readAliceRouterIdentity already called fail
-                            return;
-                        }
-                        SigType type = _aliceIdent.getSigningPublicKey().getType();
-                        if (type == null) {
-                            fail("Unsupported sig type");
-                            return;
-                        }
-                        changeState(State.IB_GOT_RI);
-                        // handle variable signature size
-                        _sz_aliceIdent_tsA_padding_aliceSigSize = 2 + _aliceIdentSize + 4 + type.getSigLen();
-                        int rem = (_sz_aliceIdent_tsA_padding_aliceSigSize % AES_SIZE);
-                        int padding = 0;
-                        if (rem > 0)
-                            padding = AES_SIZE-rem;
-                        _sz_aliceIdent_tsA_padding_aliceSigSize += padding;
-                        if (_log.shouldLog(Log.DEBUG))
-                            _log.debug(prefix() + "alice ident size decrypted as " + _aliceIdentSize +
-                                       ", making the padding at " + padding + " and total size at " +
-                                       _sz_aliceIdent_tsA_padding_aliceSigSize);
-                    }
-
-                    if (_state == State.IB_GOT_RI &&
-                        _sz_aliceIdent_tsA_padding_aliceSig.size() >= _sz_aliceIdent_tsA_padding_aliceSigSize) {
-                        // we have the remainder of Message #3, i.e. the padding+signature
-                        // Time to verify.
-
-                            if (_log.shouldLog(Log.DEBUG))
-                                _log.debug(prefix() + "got the sig");
-                            verifyInbound(src);
-                            if (_log.shouldLog(Log.DEBUG))
-                                _log.debug(prefix()+"verifying size (sz=" + _sz_aliceIdent_tsA_padding_aliceSig.size()
-                                           + " expected=" + _sz_aliceIdent_tsA_padding_aliceSigSize
-                                           + ' ' + _state
-                                           + ')');
-                            return;
-                    }
-                }
-        }
-
-        // check for remaining data
-        if (STATES_DONE.contains(_state) && src.hasRemaining()) {
-            if (_log.shouldWarn())
-                _log.warn("Received unexpected " + src.remaining() + " on " + this, new Exception());
-        }
-
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug(prefix()+"done with the data, not yet complete or corrupt");
-    }
-
-    /**
-     * We are Bob. We have received enough of message #3 from Alice
-     * to get Alice's RouterIdentity.
-     *
-     * _aliceIdentSize must be set.
-     * _sz_aliceIdent_tsA_padding_aliceSig must contain at least 2 + _aliceIdentSize bytes.
-     *
-     * Sets _aliceIdent so that we
-     * may determine the signature and padding sizes.
-     *
-     * After all of message #3 is received including the signature and
-     * padding, verifyIdentity() must be called.
-     *
-     *  State must be IB_GOT_RI_SIZE.
-     *  Caller must synch.
-     *
-     * @since 0.9.16 pulled out of verifyInbound()
-     */
-    private void readAliceRouterIdentity() {
-        byte b[] = _sz_aliceIdent_tsA_padding_aliceSig.toByteArray();
-
-        try {
-            int sz = _aliceIdentSize;
-            if (sz < MIN_RI_SIZE || sz > MAX_RI_SIZE ||
-                sz > b.length-2) {
-                _context.statManager().addRateData("ntcp.invalidInboundSize", sz);
-                fail("size is invalid", new Exception("size is " + sz));
-                return;
-            }
-            RouterIdentity alice = new RouterIdentity();
-            ByteArrayInputStream bais = new ByteArrayInputStream(b, 2, sz);
-            alice.readBytes(bais);
-            _aliceIdent = alice;
-        } catch (IOException ioe) {
-            _context.statManager().addRateData("ntcp.invalidInboundIOE", 1);
-            fail("Error verifying peer", ioe);
-        } catch (DataFormatException dfe) {
-            _context.statManager().addRateData("ntcp.invalidInboundDFE", 1);
-            fail("Error verifying peer", dfe);
-        }
-    }
-
-    /**
-     * We are Bob. Verify message #3 from Alice, then send message #4 to Alice.
-     * NTCP 1 only.
-     *
-     * _aliceIdentSize and _aliceIdent must be set.
-     * _sz_aliceIdent_tsA_padding_aliceSig must contain at least
-     *  (2 + _aliceIdentSize + 4 + padding + sig) bytes.
-     *
-     * Sets _aliceIdent so that we
-     *
-     * readAliceRouterIdentity() must have been called previously
-     *
-     * Make sure the signatures are correct, and if they are, update the
-     * NIOConnection with the session key / peer ident / clock skew / iv.
-     * The NIOConnection itself is responsible for registering with the
-     * transport
-     *
-     *  State must be IB_GOT_RI.
-     *  This will always change the state to VERIFIED or CORRUPT.
-     *  Caller must synch.
-     *
-     *  @param buf possibly containing "extra" data for data phase
-     */
-    private void verifyInbound(ByteBuffer buf) {
-        byte b[] = _sz_aliceIdent_tsA_padding_aliceSig.toByteArray();
-        try {
-            int sz = _aliceIdentSize;
-            // her timestamp from message #3
-            long tsA = DataHelper.fromLong(b, 2+sz, 4);
-            // _tsB is when we sent message #2
-            // Adjust backward by RTT/2
-            long now = _context.clock().now();
-            // rtt from sending #2 to receiving #3
-            long rtt = now - _con.getCreated();
-            _peerSkew = (now - (tsA * 1000) - (rtt / 2) + 500) / 1000; 
-
-            ByteArrayStream baos = new ByteArrayStream(256 + 256 + 32 + 4 + 4);
-            baos.write(_X);
-            baos.write(_Y);
-            baos.write(_context.routerHash().getData());
-            baos.write(DataHelper.toLong(4, tsA));
-            baos.write(DataHelper.toLong(4, _tsB));
-            //baos.write(b, 2+sz+4, b.length-2-sz-4-Signature.SIGNATURE_BYTES);
-
-            byte toVerify[] = baos.toByteArray();
-
-            // handle variable signature size
-            SigType type = _aliceIdent.getSigningPublicKey().getType();
-            if (type == null) {
-                fail("unsupported sig type");
-                return;
-            }
-            byte s[] = new byte[type.getSigLen()];
-            System.arraycopy(b, b.length-s.length, s, 0, s.length);
-            Signature sig = new Signature(type, s);
-            boolean ok = _context.dsa().verifySignature(sig, toVerify, _aliceIdent.getSigningPublicKey());
-            Hash aliceHash = _aliceIdent.calculateHash();
-            if (ok) {
-                ok = verifyInbound(aliceHash);
-            }
-            if (ok) {
-                _con.setRemotePeer(_aliceIdent);
-                sendInboundConfirm(aliceHash, tsA);
-                if (_log.shouldLog(Log.DEBUG))
-                    _log.debug(prefix()+"e_bobSig is " + _e_bobSig.length + " bytes long");
-                byte iv[] = _curEncrypted;  // reuse buf
-                System.arraycopy(_e_bobSig, _e_bobSig.length-AES_SIZE, iv, 0, AES_SIZE);
-                // this does not copy the IV, do not release to cache
-                // We are Bob, she is Alice, clock skew is Alice-Bob
-                // skew in seconds
-                _con.finishInboundEstablishment(_dh.getSessionKey(), _peerSkew, iv, _prevEncrypted);
-                changeState(State.VERIFIED);
-                if (buf.hasRemaining()) {
-                    // process "extra" data
-                    // This is unlikely for inbound, as we must reply with message 4
-                    if (_log.shouldWarn())
-                        _log.warn("extra data " + buf.remaining() + " on " + this);
-                     _con.recvEncryptedI2NP(buf);
-                }
-                releaseBufs(true);
-                if (_log.shouldLog(Log.INFO))
-                    _log.info(prefix()+"Verified remote peer as " + aliceHash);
-            } else {
-                _context.statManager().addRateData("ntcp.invalidInboundSignature", 1);
-                // verifyInbound(aliceHash) called fail()
-            }
-        } catch (IOException ioe) {
-            _context.statManager().addRateData("ntcp.invalidInboundIOE", 1);
-            fail("Error verifying peer", ioe);
         }
     }
 
@@ -525,11 +179,10 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
      */
     private boolean verifyInbound(Hash aliceHash) {
         // get inet-addr
-        InetAddress addr = this._con.getChannel().socket().getInetAddress();
-        byte[] ip = (addr == null) ? null : addr.getAddress();
+        byte[] ip = _con.getRemoteIP();
         if (_context.banlist().isBanlistedForever(aliceHash)) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn("Dropping inbound connection from permanently banlisted peer: " + aliceHash);
+            if (_log.shouldWarn())
+                _log.warn("Dropping inbound connection from permanently banlisted peer at " + Addresses.toString(ip) + " : " + aliceHash);
             // So next time we will not accept the con from this IP,
             // rather than doing the whole handshake
             if(ip != null)
@@ -546,26 +199,29 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         if (_log.shouldLog(Log.DEBUG))
             _log.debug(prefix() + "verification successful for " + _con);
 
+        // Adjust skew calculation now that we know RTT
+        // rtt from receiving #1 to receiving #3
+        long rtt = _context.clock().now() - _con.getCreated();
+        _peerSkew -= ((rtt / 2) + 500) / 1000;
         long diff = 1000*Math.abs(_peerSkew);
-        if (!_context.clock().getUpdatedSuccessfully()) {
+        boolean skewOK = diff < Router.CLOCK_FUDGE_FACTOR;
+        if (skewOK && !_context.clock().getUpdatedSuccessfully()) {
             // Adjust the clock one time in desperation
             // This isn't very likely, outbound will do it first
+            // Don't adjust to large skews inbound.
             // We are Bob, she is Alice, adjust to match Alice
             _context.clock().setOffset(1000 * (0 - _peerSkew), true);
             _peerSkew = 0;
             if (diff != 0)
-                _log.logAlways(Log.WARN, "NTP failure, NTCP adjusting clock by " + DataHelper.formatDuration(diff));
-        } else if (diff >= Router.CLOCK_FUDGE_FACTOR) {
-            _context.statManager().addRateData("ntcp.invalidInboundSkew", diff);
-            _transport.markReachable(aliceHash, true);
+                _log.logAlways(Log.WARN, "NTP failure, NTCP adjusted clock by " + DataHelper.formatDuration(diff) +
+                                         " source router: " + aliceHash.toBase64());
+        } else if (!skewOK) {
             // Only banlist if we know what time it is
             _context.banlist().banlistRouter(DataHelper.formatDuration(diff),
                                              aliceHash,
                                                _x("Excessive clock skew: {0}"));
             _transport.setLastBadSkew(_peerSkew);
-            if (getVersion() < 2)
-                fail("Clocks too skewed (" + diff + " ms)", null, true);
-            else if (_log.shouldWarn())
+            if (_log.shouldWarn())
                 _log.warn("Clocks too skewed (" + diff + " ms)");
             _msg3p2FailReason = NTCPConnection.REASON_SKEW;
             return false;
@@ -577,11 +233,9 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
 
     /**
      *  Validate network ID, NTCP 2 only.
-     *  Call after receiving Alice's RouterInfo,
-     *  but before storing it in the netdb.
+     *  Call if storing in netdb fails.
      *
-     *  Side effects: When returning false, sets _msg3p2FailReason,
-     *  banlists permanently and blocklists
+     *  Side effects: When returning false, sets _msg3p2FailReason
      *
      *  @return success
      *  @since 0.9.38
@@ -595,57 +249,16 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                 _log.warn("Not in our network: " + alice, new Exception());
             // So next time we will not accept the con from this IP,
             // rather than doing the whole handshake
-            InetAddress addr = _con.getChannel().socket().getInetAddress();
-            if (addr != null) {
-                byte[] ip = addr.getAddress();
+            byte[] ip = _con.getRemoteIP();
+            if (ip != null) {
                 _context.blocklist().add(ip);
             }
-            _context.banlist().banlistRouterForever(aliceHash, "Not in our network: " + aliceID);
             _transport.markUnreachable(aliceHash);
             _msg3p2FailReason = NTCPConnection.REASON_BANNED;
         }
         return rv;
     }
 
-    /**
-     *  We are Bob. Send message #4 to Alice.
-     *
-     *  State must be VERIFIED.
-     *  Caller must synch.
-     *
-     *  @param h Alice's Hash
-     */
-    private void sendInboundConfirm(Hash h, long tsA) {
-        // send Alice E(S(X+Y+Alice.identHash+tsA+tsB), sk, prev)
-        byte toSign[] = new byte[XY_SIZE + XY_SIZE + 32+4+4];
-        int off = 0;
-        System.arraycopy(_X, 0, toSign, off, XY_SIZE); off += XY_SIZE;
-        System.arraycopy(_Y, 0, toSign, off, XY_SIZE); off += XY_SIZE;
-        System.arraycopy(h.getData(), 0, toSign, off, 32); off += 32;
-        DataHelper.toLong(toSign, off, 4, tsA); off += 4;
-        DataHelper.toLong(toSign, off, 4, _tsB); off += 4;
-
-        // handle variable signature size
-        Signature sig = _context.dsa().sign(toSign, _context.keyManager().getSigningPrivateKey());
-        int siglen = sig.length();
-        int rem = siglen % AES_SIZE;
-        int padding;
-        if (rem > 0)
-            padding = AES_SIZE - rem;
-        else
-            padding = 0;
-        byte preSig[] = new byte[siglen + padding];
-        System.arraycopy(sig.getData(), 0, preSig, 0, siglen);
-        if (padding > 0)
-            _context.random().nextBytes(preSig, siglen, padding);
-        _e_bobSig = new byte[preSig.length];
-        _context.aes().encrypt(preSig, 0, _e_bobSig, 0, _dh.getSessionKey(), _e_hXY_tsB, HXY_TSB_PAD_SIZE - AES_SIZE, _e_bobSig.length);
-
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug(prefix() + "Sending encrypted inbound confirmation");
-        _transport.getPumper().wantsWrite(_con, _e_bobSig);
-    }
-
     //// NTCP2 below here
 
     /**
@@ -737,11 +350,10 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             // network ID cross-check, proposal 147, as of 0.9.42
             v = options[0] & 0xff;
             if (v != 0 && v != _context.router().getNetworkID()) {
-                InetAddress addr = _con.getChannel().socket().getInetAddress();
-                if (addr != null) {
+                byte[] ip = _con.getRemoteIP();
+                if (ip != null) {
                     if (_log.shouldLog(Log.WARN))
-                        _log.warn("Dropping inbound connection from wrong network: " + addr);
-                    byte[] ip = addr.getAddress();
+                        _log.warn("Dropping inbound connection from wrong network: " + Addresses.toString(ip));
                     // So next time we will not accept the con from this IP
                     _context.blocklist().add(ip);
                 }
@@ -752,23 +364,21 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             _msg3p2len = (int) DataHelper.fromLong(options, 4, 2);
             long tsA = DataHelper.fromLong(options, 8, 4);
             long now = _context.clock().now();
-            // In NTCP1, timestamp comes in msg 3 so we know the RTT.
-            // In NTCP2, it comes in msg 1, so just guess.
-            // We could defer this to msg 3 to calculate the RTT?
-            long rtt = 250;
-            _peerSkew = (now - (tsA * 1000) - (rtt / 2) + 500) / 1000; 
-            if ((_peerSkew > MAX_SKEW || _peerSkew < 0 - MAX_SKEW) &&
-                !_context.clock().getUpdatedSuccessfully()) {
-                // If not updated successfully, allow it.
-                // This isn't very likely, outbound will do it first
-                // See verifyInbound() above.
-                fail("Clock Skew: " + _peerSkew, null, true);
-                return;
-            }
-            // If NTCP1 disabled, we allow longer padding
-            if (_padlen1 > PADDING1_MAX && _transport.isNTCP1Enabled()) {
-                fail("bad msg 1 padlen: " + _padlen1);
-                return;
+            // Will be adjusted for RTT in verifyInbound()
+            _peerSkew = (now - (tsA * 1000) + 500) / 1000; 
+            if (_peerSkew > MAX_SKEW || _peerSkew < 0 - MAX_SKEW) {
+                long diff = 1000*Math.abs(_peerSkew);
+                _context.statManager().addRateData("ntcp.invalidInboundSkew", diff);
+                _transport.setLastBadSkew(_peerSkew);
+                if (_log.shouldWarn())
+                    _log.warn("Clock Skew: " + _peerSkew + " on " + this);
+                // Do them a favor, keep going with msg 2 so they get our timestamp.
+                // They should disconnect after that.
+                // If they do send a msg 3, verifyInbound() will catch it and
+                // will send a destroy with code 7 and
+                // ban the peer for a while.
+                //fail("Clock Skew: " + _peerSkew, null, true);
+                //return;
             }
             if (_msg3p2len < MSG3P2_MIN || _msg3p2len > MSG3P2_MAX) {
                 fail("bad msg3p2 len: " + _msg3p2len);
@@ -867,10 +477,17 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             } catch (DataFormatException dfe) {
                 if (_log.shouldWarn())
                     _log.warn("Bad msg 3 payload", dfe);
-                // probably RI problems
+                // probably RI signature failure
                 // setDataPhase() will send termination
-                if (_msg3p2FailReason < 0)
+                if (_msg3p2FailReason < 0) {
                     _msg3p2FailReason = NTCPConnection.REASON_SIGFAIL;
+                    // buggy/forked and very persistent i2pd
+                    // So next time we will not accept the con from this IP,
+                    // rather than doing the whole handshake
+                    byte[] ip = _con.getRemoteIP();
+                    if (ip != null)
+                        _context.blocklist().add(ip);
+                }
                 _context.statManager().addRateData("ntcp.invalidInboundSignature", 1);
             } catch (I2NPMessageException ime) {
                 // shouldn't happen, no I2NP msgs in msg3p2
@@ -897,14 +514,15 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
      */
     private synchronized void prepareOutbound2() {
         // create msg 2 payload
-        byte[] options2 = new byte[OPTIONS2_SIZE];
         int padlen2 = _context.random().nextInt(PADDING2_MAX);
-        DataHelper.toLong(options2, 2, 2, padlen2);
-        long now = _context.clock().now() / 1000;
-        DataHelper.toLong(options2, 8, 4, now);
         byte[] tmp = new byte[MSG2_SIZE + padlen2];
+        // write options directly to tmp with 32 byte offset
+        DataHelper.toLong(tmp, KEY_SIZE + 2, 2, padlen2);
+        long now = (_context.clock().now() + 500) / 1000;
+        DataHelper.toLong(tmp, KEY_SIZE + 8, 4, now);
         try {
-            _handshakeState.writeMessage(tmp, 0, options2, 0, OPTIONS2_SIZE);
+            // encrypt in-place
+            _handshakeState.writeMessage(tmp, 0, tmp, KEY_SIZE, OPTIONS2_SIZE);
         } catch (GeneralSecurityException gse) {
             // buffer length error
             if (!_log.shouldWarn())
@@ -931,7 +549,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
 
         changeState(State.IB_NTCP2_SENT_Y);
         // send it all at once
-        _transport.getPumper().wantsWrite(_con, tmp);
+        _con.wantsWrite(tmp);
     }
 
     /**
@@ -1011,15 +629,32 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
             throw new DataFormatException("no NTCP in RI: " + ri);
         }
         String s = null;
+        String mismatchMessage = null;
+        byte[] realIP = _con.getRemoteIP();
         for (RouterAddress addr : addrs) {
             String v = addr.getOption("v");
             if (v == null ||
                 (!v.equals(NTCPTransport.NTCP2_VERSION) && !v.startsWith(NTCPTransport.NTCP2_VERSION_ALT))) {
                  continue;
             }
-            s = addr.getOption("s");
-            if (s != null)
-                break;
+            if (s == null)
+                s = addr.getOption("s");
+            if (realIP != null) {
+                byte[] infoIP = addr.getIP();
+                if (infoIP != null && infoIP.length == realIP.length) {
+                    if (infoIP.length == 16) {
+                        if ((((int) infoIP[0]) & 0xfe) == 0x02)
+                            continue; // ygg
+                        if (DataHelper.eq(realIP, 0, infoIP, 0, 8))
+                            continue;
+                    } else {
+                        if (DataHelper.eq(realIP, infoIP))
+                            continue;
+                    }
+                    // We will ban and throw below after checking s
+                    mismatchMessage = "IP mismatch actual IP " + Addresses.toString(realIP) + " in RI: ";
+                }
+            }
         }
         if (s == null) {
             _msg3p2FailReason = NTCPConnection.REASON_S_MISMATCH;
@@ -1039,13 +674,21 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         }
         _aliceIdent = ri.getIdentity();
         Hash h = _aliceIdent.calculateHash();
-        // this sets the reason
+        // this sets the reasonH
         boolean ok = verifyInbound(h);
         if (!ok)
             throw new DataFormatException("NTCP2 verifyInbound() fail");
-        ok = verifyInboundNetworkID(ri);
-        if (!ok)
-            throw new DataFormatException("NTCP2 network ID mismatch");
+
+        // s is verified, we may now ban the hash
+        if (mismatchMessage != null) {
+            _context.banlist().banlistRouter(h, "IP mismatch", null,
+                                             //_context.banlist().BANLIST_CODE_FOREVER,
+                                             null,
+                                             _context.clock().now() + 2*60*60*1000);
+            _msg3p2FailReason = NTCPConnection.REASON_BANNED;
+            throw new DataFormatException(mismatchMessage + ri);
+        }
+
         try {
             RouterInfo old = _context.netDb().store(h, ri);
             if (flood && !ri.equals(old)) {
@@ -1059,9 +702,14 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
                 }
             }
         } catch (IllegalArgumentException iae) {
-            // hash collision?
-            // expired RI?
-            _msg3p2FailReason = NTCPConnection.REASON_MSG3;
+            // sets _msg3p2FailReason
+            ok = verifyInboundNetworkID(ri);
+            if (!ok)
+                throw new DataFormatException("NTCP2 network ID mismatch");
+            // generally expired/future RI
+            // don't change reason if already set as clock skew
+            if (_msg3p2FailReason <= 0)
+                _msg3p2FailReason = NTCPConnection.REASON_MSG3;
             throw new DataFormatException("RI store fail: " + ri, iae);
         }
         _con.setRemotePeer(_aliceIdent);
