@@ -39,6 +39,7 @@ import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.data.i2np.DatabaseStoreMessage;
 import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.router.Banlist;
 import net.i2p.router.CommSystemFacade.Status;
 import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
@@ -51,6 +52,7 @@ import net.i2p.router.transport.TransportUtil;
 import static net.i2p.router.transport.TransportUtil.IPv6Config.*;
 import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
 import net.i2p.router.transport.crypto.X25519KeyFactory;
+import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.router.util.DecayingHashSet;
 import net.i2p.router.util.DecayingBloomFilter;
 import net.i2p.router.util.EventLog;
@@ -112,7 +114,6 @@ public class NTCPTransport extends TransportImpl {
     public static final String PROP_BIND_INTERFACE = "i2np.ntcp.bindInterface";
 
     private final NTCPSendFinisher _finisher;
-    private final DHSessionKeyBuilder.Factory _dhFactory;
     private final X25519KeyFactory _xdhFactory;
     private long _lastBadSkew;
     private static final long[] RATES = { 10*60*1000 };
@@ -138,8 +139,6 @@ public class NTCPTransport extends TransportImpl {
     private static final int NTCP2_KEY_LEN = OutboundNTCP2State.KEY_SIZE;
     private static final long MIN_DOWNTIME_TO_REKEY = 30*24*60*60*1000L;
     private static final long MIN_DOWNTIME_TO_REKEY_HIDDEN = 24*60*60*1000L;
-    private final boolean _enableNTCP1;
-    private final boolean _enableNTCP2;
     private final byte[] _ntcp2StaticPubkey;
     private final byte[] _ntcp2StaticPrivkey;
     private final byte[] _ntcp2StaticIV;
@@ -147,12 +146,11 @@ public class NTCPTransport extends TransportImpl {
     private final String _b64Ntcp2StaticIV;
 
     /**
-     *  @param dh null to disable NTCP1
+     *  @param dh unused, ignored
      *  @param xdh null to disable NTCP2
      */
     public NTCPTransport(RouterContext ctx, DHSessionKeyBuilder.Factory dh, X25519KeyFactory xdh) {
         super(ctx);
-        _dhFactory = dh;
         _xdhFactory = xdh;
         _log = ctx.logManager().getLog(getClass());
 
@@ -242,11 +240,9 @@ public class NTCPTransport extends TransportImpl {
         _transientFail = new SharedBid(TransportBid.TRANSIENT_FAIL);
 
         setupPort();
-        _enableNTCP1 = dh != null;
-        _enableNTCP2 = xdh != null;
-        if (!_enableNTCP1 && !_enableNTCP2)
+        if (xdh == null)
             throw new IllegalArgumentException();
-        if (_enableNTCP2) {
+
             boolean shouldSave = false;
             byte[] priv = null;
             byte[] iv = null;
@@ -254,7 +250,7 @@ public class NTCPTransport extends TransportImpl {
             String s = null;
             // try to determine if we've been down for 30 days or more
             long minDowntime = _context.router().isHidden() ? MIN_DOWNTIME_TO_REKEY_HIDDEN : MIN_DOWNTIME_TO_REKEY;
-            boolean shouldRekey = _context.getEstimatedDowntime() >= minDowntime;
+            boolean shouldRekey = !allowLocal() && _context.getEstimatedDowntime() >= minDowntime;
             if (!shouldRekey) {
                 s = ctx.getProperty(PROP_NTCP2_SP);
                 if (s != null) {
@@ -295,35 +291,42 @@ public class NTCPTransport extends TransportImpl {
             _ntcp2StaticIV = iv;
             _b64Ntcp2StaticPubkey = Base64.encode(_ntcp2StaticPubkey);
             _b64Ntcp2StaticIV = b64IV;
-        } else {
-            _ntcp2StaticPubkey = null;
-            _ntcp2StaticPrivkey = null;
-            _ntcp2StaticIV = null;
-            _b64Ntcp2StaticPubkey = null;
-            _b64Ntcp2StaticIV = null;
-        }
+
     }
     
     /**
      *  Pick a port if not previously configured.
      *  Only if UDP is disabled.
      *
+     *  @return the port or -1
      *  @since 0.9.39
      */
-    private void setupPort() {
+    private int setupPort() {
         if (_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP))
-            return;
+            return -1;
         int port = getRequestedPort();
         if (port > 0 && !TransportUtil.isValidPort(port)) {
             TransportUtil.logInvalidPort(_log, STYLE, port);
+            port = -1;
+        }
+        if (port <= 0) {
+            // If we previously had a UDP port, use it
+            port = _context.getProperty(UDPTransport.PROP_INTERNAL_PORT, -1);
+            if (port <= 0)
+                port = _context.getProperty(UDPTransport.PROP_EXTERNAL_PORT, -1);
+            if (port > 0 && !TransportUtil.isValidPort(port)) {
+                TransportUtil.logInvalidPort(_log, STYLE, port);
+                port = -1;
+            }
+            if (port > 0)
+                _context.router().saveConfig(PROP_I2NP_NTCP_PORT, Integer.toString(port));
         }
         if (port <= 0) {
             port = TransportUtil.selectRandomPort(_context, STYLE);
-            Map<String, String> changes = new HashMap<String, String>(2);
-            changes.put(PROP_I2NP_NTCP_PORT, Integer.toString(port));
-            _context.router().saveConfig(changes, null);
+            _context.router().saveConfig(PROP_I2NP_NTCP_PORT, Integer.toString(port));
             _log.logAlways(Log.INFO, "NTCP selected random port " + port);
         }
+        return port;
     }
 
     /**
@@ -334,16 +337,33 @@ public class NTCPTransport extends TransportImpl {
         _context.statManager().addRateData("ntcp.inboundEstablished", 1);
         Hash peer = con.getRemotePeer().calculateHash();
         markReachable(peer, true);
-        //_context.banlist().unbanlistRouter(con.getRemotePeer().calculateHash());
         NTCPConnection old;
         synchronized (_conLock) {
             old = _conByIdent.put(peer, con);
         }
         if (con.isIPv6()) {
+            long last = _lastInboundIPv6;
+            Status oldStatus;
+            if (last <= 0)
+                oldStatus = getReachabilityStatus();
+            else
+                oldStatus = null;
             _lastInboundIPv6 = con.getCreated();
+            // Get that "R" cap in the netdb ASAP, esp. when SSU disabled
+            if (last <= 0)
+                addressChanged(oldStatus);
             _context.statManager().addRateData("ntcp.inboundIPv6Conn", 1);
         } else {
+            long last = _lastInboundIPv4;
+            Status oldStatus;
+            if (last <= 0)
+                oldStatus = getReachabilityStatus();
+            else
+                oldStatus = null;
             _lastInboundIPv4 = con.getCreated();
+            // Get that "R" cap in the netdb ASAP, esp. when SSU disabled
+            if (last <= 0)
+                addressChanged(oldStatus);
             _context.statManager().addRateData("ntcp.inboundIPv4Conn", 1);
         }
         return old;
@@ -416,7 +436,8 @@ public class NTCPTransport extends TransportImpl {
                     // Queue the message, and our RI
                     // doesn't do anything yet, just enqueues it
                     con.send(msg);
-                    con.enqueueInfoMessage();
+                    // does nothing for outbound NTCP2
+                    //con.enqueueInfoMessage();
                 } else if (shouldFlood || newVersion == 1) {
                     // Queue the message, which is a DSM of our RI
                     con.send(msg);
@@ -501,9 +522,8 @@ public class NTCPTransport extends TransportImpl {
             return null;
         // passed in dataSize assumes 16 byte header, if NTCP2 then
         // we have a 9-byte header so there's 7 to spare
-        if (dataSize > NTCPConnection.NTCP2_MAX_MSG_SIZE + 7 ||
-            (!_enableNTCP2 && dataSize > NTCPConnection.NTCP1_MAX_MSG_SIZE)) {
-            // Too big for NTCP2, or NTCP2 disabled and too big for NTCP1
+        if (dataSize > NTCPConnection.NTCP2_MAX_MSG_SIZE + 7) {
+            // Too big for NTCP2
             // Let SSU deal with it
             _context.statManager().addRateData("ntcp.noBidTooLargeI2NP", dataSize);
             return null;
@@ -521,29 +541,19 @@ public class NTCPTransport extends TransportImpl {
 
         boolean established = isEstablished(peer);
         if (established) { // should we check the queue size?  nah, if its valid, use it
-            if (dataSize > NTCPConnection.NTCP1_MAX_MSG_SIZE) {
-                // Must be version 2 to send a big message
-                NTCPConnection con = _conByIdent.get(peer);
-                if (con == null || con.getVersion() < NTCP2_INT_VERSION) {
-                    _context.statManager().addRateData("ntcp.noBidTooLargeI2NP", dataSize);
-                    return null;
-                }
-            }
             return _fastBid;
         }
-        if (toAddress.getNetworkId() != _networkID) {
-            _context.banlist().banlistRouterForever(peer, "Not in our network: " + toAddress.getNetworkId());
+        int nid = toAddress.getNetworkId();
+        if (nid != _networkID) {
+            if (nid == -1)
+                _context.banlist().banlistRouter(peer, "No network specified", null, null, _context.clock().now() + Banlist.BANLIST_DURATION_NO_NETWORK);
+            else
+                _context.banlist().banlistRouterForever(peer, "Not in our network: " + nid);
             if (_log.shouldWarn())
                 _log.warn("Not in our network: " + toAddress, new Exception());
             markUnreachable(peer);
             return null;    
         }
-        if (dataSize > NTCPConnection.NTCP1_MAX_MSG_SIZE) {
-            // Not established, too big for NTCP 1, let SSU deal with it
-            // TODO look at his addresses to see if NTCP2 supported?
-            _context.statManager().addRateData("ntcp.noBidTooLargeI2NP", dataSize);
-            return null;
-        }
 
         RouterAddress addr = getTargetAddress(toAddress);
         if (addr == null) {
@@ -713,6 +723,25 @@ public class NTCPTransport extends TransportImpl {
     public int countPeers() {
             return _conByIdent.size();
     }
+
+    /**
+     * @return 8 bytes:
+     *         version 1 4 bytes all zeros
+     *         version 2 ipv4 in/out, ipv6 in/out
+     * @since 0.9.57
+     */
+    public int[] getPeerCounts() {
+        int[] rv = new int[8];
+        for (NTCPConnection con : _conByIdent.values()) {
+            int idx = 4;
+            if (con.isIPv6())
+                idx += 2;
+            if (!con.isInbound())
+                idx++;
+            rv[idx]++;
+        }
+        return rv;
+    }
     
     /** 
      * For /peers UI only. Not a public API, not for external use.
@@ -730,12 +759,12 @@ public class NTCPTransport extends TransportImpl {
      * @return a copy, modifiable
      * @since 0.9.34
      */
-    public Set<Hash> getEstablished() {
-        Set<Hash> rv = new HashSet<Hash>(_conByIdent.keySet());
+    public List<Hash> getEstablished() {
+        List<Hash> rv = new ArrayList<Hash>(_conByIdent.size());
         for (Map.Entry<Hash, NTCPConnection> e : _conByIdent.entrySet()) {
             NTCPConnection con = e.getValue();
-            if (!con.isEstablished() || con.isClosed())
-                rv.remove(e.getKey());
+            if (con.isEstablished() && !con.isClosed())
+                rv.add(e.getKey());
         }
         return rv;
     }
@@ -828,7 +857,7 @@ public class NTCPTransport extends TransportImpl {
     private static final int MIN_CONCURRENT_READERS = 2;  // unless < 32MB
     private static final int MIN_CONCURRENT_WRITERS = 2;  // unless < 32MB
     private static final int MAX_CONCURRENT_READERS = 4;
-    private static final int MAX_CONCURRENT_WRITERS = 4;
+    private static final int MAX_CONCURRENT_WRITERS = 3;
 
     /**
      *  Called by TransportManager.
@@ -845,10 +874,13 @@ public class NTCPTransport extends TransportImpl {
 
         startIt();
         RouterAddress addr = configureLocalAddress();
+        boolean ssuDisabled = !_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP);
         int port;
         if (addr != null)
             // probably not set
             port = addr.getPort();
+        else if (ssuDisabled)
+            port = setupPort();
         else
             // received by externalAddressReceived() from TransportManager
             port = _ssuPort;
@@ -864,30 +896,54 @@ public class NTCPTransport extends TransportImpl {
         } else if (port > 0 && !isFixedOrForceFirewalled) {
             // all detected interfaces
             Collection<InetAddress> addrs = getSavedLocalAddresses();
-            if (!addrs.isEmpty()) {
+            if (!addrs.isEmpty() && !_context.router().isHidden()) {
+                int count = 0;
+                boolean skipv4 = false;
+                boolean skipv6 = false;
                 for (InetAddress ia : addrs) {
+                    boolean ipv6 = ia instanceof Inet6Address;
+                    if ((ipv6 && (isIPv6Firewalled() || (_context.getBooleanProperty(PROP_IPV6_FIREWALLED) && !ssuDisabled))) ||
+                        (!ipv6 && isIPv4Firewalled())) {
+                        if (ipv6)
+                            skipv6 = true;
+                        else
+                            skipv4 = true;
+                        continue;
+                    }
                     OrderedProperties props = new OrderedProperties();
                     props.setProperty(RouterAddress.PROP_HOST, ia.getHostAddress());
                     props.setProperty(RouterAddress.PROP_PORT, Integer.toString(port));
                     addNTCP2Options(props);
-                    boolean ipv6 = ia instanceof Inet6Address;
-                    if (!ipv6 || !_context.getBooleanProperty(PROP_IPV6_FIREWALLED)) {
-                        int cost = getDefaultCost(ipv6);
-                        myAddress = new RouterAddress(getPublishStyle(), props, cost);
-                        replaceAddress(myAddress);
-                    }
+                    int cost = getDefaultCost(ipv6);
+                    myAddress = new RouterAddress(getPublishStyle(), props, cost);
+                    replaceAddress(myAddress);
+                    count++;
                 }
-            } else if (_enableNTCP2) {
+                if (count <= 0) {
+                    setOutboundNTCP2Address();
+                } else if (skipv6) {
+                    setOutboundNTCP2Address(true);
+                } else if (skipv4) {
+                    setOutboundNTCP2Address(false);
+                }
+            } else {
                 setOutboundNTCP2Address();
             }
-        } else if (_enableNTCP2) {
+        } else {
             setOutboundNTCP2Address();
         }
+        if (ssuDisabled) {
+            // Since we don't have peer testing, start out reachable,
+            // so peers will attempt to connect to us
+            long now = _context.clock().now();
+            _lastInboundIPv4 = now;
+            _lastInboundIPv6 = now;
+        }
         // TransportManager.startListening() calls router.rebuildRouterInfo()
     }
 
     /**
-     *  Outbound only, NTCP2 with "s" and "v" only
+     *  Outbound only, both IPv4 and IPv6, NTCP2 with "s" and "v" only
      *  @since 0.9.36
      */
     private void setOutboundNTCP2Address() {
@@ -897,6 +953,32 @@ public class NTCPTransport extends TransportImpl {
         replaceAddress(myAddress);
     }
 
+    /**
+     *  Outbound only, either IPv4 or IPv6, NTCP2 with "s" and "v" only.
+     *  @since 0.9.50
+     */
+    private void setOutboundNTCP2Address(boolean ipv6) {
+        // following is like addNTCP2Options() but adds 4 or 6 only,
+        // and returns if not appropriately configured
+        String caps;
+        TransportUtil.IPv6Config config = getIPv6Config();
+        if (ipv6) {
+            if (config == IPV6_DISABLED)
+                return;
+            caps = CAP_IPV6;
+        } else {
+            if (config == IPV6_ONLY)
+                return;
+            caps = CAP_IPV4;
+        }
+        OrderedProperties props = new OrderedProperties();
+        props.setProperty("caps", caps);
+        props.setProperty("s", _b64Ntcp2StaticPubkey);
+        props.setProperty("v", NTCP2_VERSION);
+        RouterAddress myAddress = new RouterAddress(STYLE2, props, NTCP2_OUTBOUND_COST);
+        replaceAddress(myAddress);
+    }
+
     /**
      *  Only called by externalAddressReceived().
      *  Calls replaceAddress() or removeAddress().
@@ -1016,6 +1098,15 @@ public class NTCPTransport extends TransportImpl {
                             _log.warn("Already listening on " + addr);
                         return null;
                     }
+                    // if UDP only changed external port and not internal port,
+                    // do not rebind internally and restart, just change the address
+                    int eport = _context.getProperty(UDPTransport.PROP_EXTERNAL_PORT, 0);
+                    int iport = _context.getProperty(UDPTransport.PROP_INTERNAL_PORT, 0);
+                    if (port == eport && iport > 0 && eport != iport) {
+                        if (_log.shouldWarn())
+                            _log.warn("External port changed to " + eport + ", keep listening on internal port " + iport);
+                        return null;
+                    }
                     // FIXME support multiple binds
                     // FIXME just close and unregister
                     stopWaitAndRestart();
@@ -1088,26 +1179,26 @@ public class NTCPTransport extends TransportImpl {
     net.i2p.router.transport.ntcp.Writer getWriter() { return _writer; }
 
     /**
-     * @return always "NTCP" even if NTCP1 is disabled
+     * @return always "NTCP"
      */
     public String getStyle() { return STYLE; }
 
     /**
-     * An alternate supported style, or null.
-     * @return "NTCP2" or null
+     * An alternate supported style
+     * @return "NTCP2" always
      * @since 0.9.35
      */
     @Override
     public String getAltStyle() {
-        return _enableNTCP2 ? STYLE2 : null;
+        return STYLE2;
     }
 
     /**
-     * @return "NTCP" if NTCP1 is enabled, else "NTCP2"
+     * @return "NTCP2" always
      * @since 0.9.39
      */
     private String getPublishStyle() {
-        return _enableNTCP1 ? STYLE : STYLE2;
+        return STYLE2;
     }
 
     /**
@@ -1115,14 +1206,6 @@ public class NTCPTransport extends TransportImpl {
      */
     EventPumper getPumper() { return _pumper; }
 
-    /**
-     *  @return null if not configured for NTCP1
-     *  @since 0.9
-     */
-    DHSessionKeyBuilder getDHBuilder() {
-        return _dhFactory != null ? _dhFactory.getBuilder() : null;
-    }
-
     /**
      *  @return null if not configured for NTCP2
      *  @since 0.9.36
@@ -1131,18 +1214,6 @@ public class NTCPTransport extends TransportImpl {
         return _xdhFactory;
     }
 
-    /**
-     * Return an unused DH key builder
-     * to be put back onto the queue for reuse.
-     *
-     * @param builder must not have a peerPublicValue set
-     * @since 0.9.16
-     */
-    void returnUnused(DHSessionKeyBuilder builder) {
-        if (_dhFactory != null)
-            _dhFactory.returnUnused(builder);
-    }
-
     /**
      * how long from initial connection attempt (accept() or connect()) until
      * the con must be established to avoid premature close()ing
@@ -1244,29 +1315,25 @@ public class NTCPTransport extends TransportImpl {
      * @since 0.9.35
      */
     private void addNTCP2Options(Properties props) {
-        if (!_enableNTCP2)
-            return;
         // only set i if we are not firewalled
-        if (props.containsKey("host"))
+        if (props.containsKey("host")) {
             props.setProperty("i", _b64Ntcp2StaticIV);
+            props.remove("caps");
+        } else {
+            String caps;
+            TransportUtil.IPv6Config config = getIPv6Config();
+            if (config == IPV6_ONLY)
+                caps = CAP_IPV6;
+            else if (config != IPV6_DISABLED && _haveIPv6Address)
+                caps = CAP_IPV4_IPV6;
+            else
+                caps = CAP_IPV4;
+            props.setProperty("caps", caps);
+        }
         props.setProperty("s", _b64Ntcp2StaticPubkey);
         props.setProperty("v", NTCP2_VERSION);
     }
 
-    /**
-     * Is NTCP1 enabled?
-     *
-     * @since 0.9.39
-     */
-    boolean isNTCP1Enabled() { return _enableNTCP1; }
-
-    /**
-     * Is NTCP2 enabled?
-     *
-     * @since 0.9.35
-     */
-    boolean isNTCP2Enabled() { return _enableNTCP2; }
-
     /**
      * The static priv key
      *
@@ -1305,12 +1372,8 @@ public class NTCPTransport extends TransportImpl {
         int rv;
         String style = addr.getTransportStyle();
         if (style.equals(STYLE)) {
-            if (!_enableNTCP2)
-                return 1;
             rv = 1;
         } else if (style.equals(STYLE2)) {
-            if (!_enableNTCP2)
-                return 0;
             rv = NTCP2_INT_VERSION;
         } else {
             return 0;
@@ -1323,7 +1386,7 @@ public class NTCPTransport extends TransportImpl {
             addr.getOption("s") == null ||
             (!v.equals(NTCP2_VERSION) && !v.startsWith(NTCP2_VERSION_ALT))) {
             // his address is NTCP1 or is outbound NTCP2 only
-            return (rv == 1 && _enableNTCP1) ? 1 : 0;
+            return 0;
         }
         // his address is NTCP2
         // do not validate the s/i b64, we will just catch it later
@@ -1446,7 +1509,7 @@ public class NTCPTransport extends TransportImpl {
             // must be set before isValid() call
             _haveIPv6Address = true;
         }
-        if (ip != null && !isValid(ip)) {
+        if (ip != null && !isValid(ip) && !allowLocal()) {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Invalid address: " + Addresses.toString(ip, port) + " from: " + source);
             return;
@@ -1518,9 +1581,10 @@ public class NTCPTransport extends TransportImpl {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Old status: " + old + " New status: " + status +
                           " from: ", new Exception("traceback"));
-            if (old != Status.UNKNOWN)
+            if (old != Status.UNKNOWN && _context.router().getUptime() > 5*60*1000L) {
                 _context.router().eventLog().addEvent(EventLog.REACHABILITY,
                    "from " + _t(old.toStatusString()) + " to " +  _t(status.toStatusString()));
+            }
         }
         _context.router().rebuildRouterInfo();
     }
@@ -1535,10 +1599,11 @@ public class NTCPTransport extends TransportImpl {
      */
     private synchronized boolean externalAddressReceived(byte[] ip, boolean isIPv6, int port) {
         // FIXME just take first address for now
-        // FIXME if SSU set to hostname, NTCP will be set to IP
+        // Warning, this returns null when isIPv6 == true and it's an empty "46" address
+        // See below
         RouterAddress oldAddr = getCurrentAddress(isIPv6);
         if (_log.shouldLog(Log.INFO))
-            _log.info("Changing NTCP Address? was " + oldAddr);
+            _log.info("Changing NTCP IPv" + (isIPv6 ? '6' : '4') + " Address? was " + oldAddr);
 
         OrderedProperties newProps = new OrderedProperties();
         int cost;
@@ -1561,6 +1626,8 @@ public class NTCPTransport extends TransportImpl {
         String cport = _context.getProperty(PROP_I2NP_NTCP_PORT);
         if (cport != null && cport.length() > 0) {
             nport = cport;
+            if (port > 0 && !nport.equals(Integer.toString(port)))
+                _log.logAlways(Log.WARN, "UDP detected external port is " + port + " but TCP configured port is " + nport);
         } else if (_context.getBooleanPropertyDefaultTrue(PROP_I2NP_NTCP_AUTO_PORT)) {
             // 0.9.6 change
             // This wasn't quite right, as udpAddr is the EXTERNAL port and we really
@@ -1636,6 +1703,27 @@ public class NTCPTransport extends TransportImpl {
                 newAddr.setCost(DEFAULT_COST);
             changed = true;
         } else if (ohost == null || ohost.length() <= 0) {
+            // SSU2 told us to remove our IPv6 address
+            // getCurrentAddress(true) returns null for a "46" address
+            // Get v4 address and see if it has a "6" in it,
+            // if not, put in a "6" address
+            if (isIPv6 && _haveIPv6Address && oldAddr == null && ip == null && port <= 0) {
+                RouterAddress v4Addr = getCurrentAddress(false);
+                if (v4Addr != null) {
+                    String caps = v4Addr.getOption("caps");
+                    if (caps != null && caps.contains(CAP_IPV6)) {
+                        if (_log.shouldInfo())
+                            _log.info("No old host, no new host, no change to NTCP Address");
+                        return false;
+                    }
+                }
+                if (_log.shouldInfo())
+                    _log.info("IPv6 now firewalled, adding 6 address");
+                setOutboundNTCP2Address(true);
+                return true;
+            }
+            if (_log.shouldInfo())
+                _log.info("No old host, no new host, no change to NTCP Address");
             return false;
         } else if (Boolean.parseBoolean(enabled) && !ssuOK) {
             // UDP transitioned to not-OK, turn off NTCP address
@@ -1644,13 +1732,9 @@ public class NTCPTransport extends TransportImpl {
             // we are still firewalled (SW firewall, bad UPnP indication, etc.)
             if (_log.shouldLog(Log.INFO))
                 _log.info("old host: " + ohost + " config: " + name + " new: null");
-            if (_enableNTCP2) {
-                // addNTCP2Options() called below
-                newProps.clear();
-                newAddr = new RouterAddress(STYLE2, newProps, NTCP2_OUTBOUND_COST);
-            } else {
-                newAddr = null;
-            }
+            // addNTCP2Options() called below
+            newProps.clear();
+            newAddr = new RouterAddress(STYLE2, newProps, NTCP2_OUTBOUND_COST);
             changed = true;
         }
 
@@ -1681,8 +1765,10 @@ public class NTCPTransport extends TransportImpl {
         } else {
             // IPv6
             // We have an IPv4 address, IPv6 transitioned to firewalled,
-            // so just remove the v6 address
-            newAddr = null;
+            if (_log.shouldInfo())
+                _log.info("IPv6 now firewalled, adding 6 address");
+            setOutboundNTCP2Address(true);
+            return true;
         }
 
         // stopListening stops the pumper, readers, and writers, so required even if
@@ -1699,7 +1785,12 @@ public class NTCPTransport extends TransportImpl {
         //while (isAlive()) {
         //    try { Thread.sleep(5*1000); } catch (InterruptedException ie) {}
         //}
-        restartListening(newAddr, isIPv6);
+
+        // do not restart on transition to firewalled
+        if (ip != null || port > 0)
+            restartListening(newAddr, isIPv6);
+        else
+            replaceAddress(newAddr);
         if (_log.shouldLog(Log.WARN))
             _log.warn("Updating NTCP Address (ipv6? " + isIPv6 + ") with " + newAddr);
         return true;     	
@@ -1721,7 +1812,7 @@ public class NTCPTransport extends TransportImpl {
             if (success)
                 _log.warn("UPnP has opened the NTCP port: " + port + " via " + Addresses.toString(ip, externalPort));
             else
-                _log.warn("UPnP has failed to open the NTCP port: " + port + " reason: " + reason);
+                _log.warn("UPnP has failed to open the NTCP port: " + Addresses.toString(ip, externalPort) + " reason: " + reason);
         }
         // if SSU is disabled, externalAddressReceived() will update our address and call rebuildRouterInfo().
         // getReachabilityStatus() should report correctly after address is updated.
@@ -1765,6 +1856,10 @@ public class NTCPTransport extends TransportImpl {
      * Previously returned short, now enum as of 0.9.20
      */
     public Status getReachabilityStatus() { 
+        boolean fwV4 = isIPv4Firewalled();
+        boolean fwV6 = isIPv6Firewalled();
+        if (fwV4 && fwV6)
+            return Status.REJECT_UNSOLICITED;
         if (!isAlive())
             return Status.UNKNOWN;
         TransportUtil.IPv6Config config = getIPv6Config();
@@ -1779,8 +1874,8 @@ public class NTCPTransport extends TransportImpl {
             v4Disabled = false;
             v6Disabled = false;
         }
-        boolean hasV4 = getCurrentAddress(false) != null;
-        boolean hasV6 = getCurrentAddress(true) != null;
+        boolean hasV4 = !fwV4 && getCurrentAddress(false) != null;
+        boolean hasV6 = !fwV6 && getCurrentAddress(true) != null;
         boolean showFirewalled = !_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP) &&
                                  _context.router().getUptime() > 10*60*1000;
         if (!hasV4 && !hasV6) {
@@ -1796,12 +1891,16 @@ public class NTCPTransport extends TransportImpl {
                 return Status.OK;
             if (!_haveIPv6Address)
                 return Status.OK;
+            if (fwV6)
+                return Status.IPV4_OK_IPV6_FIREWALLED;
             if (!hasV6)
                 return Status.IPV4_OK_IPV6_UNKNOWN;
         }
         if (v6OK) {
             if (v4Disabled)
                 return Status.IPV4_DISABLED_IPV6_OK;
+            if (fwV4)
+                return Status.IPV4_FIREWALLED_IPV6_OK;
             if (!hasV4)
                 return showFirewalled ? Status.IPV4_FIREWALLED_IPV6_OK : Status.IPV4_UNKNOWN_IPV6_OK;
         }
