@@ -76,7 +76,6 @@ class OutboundNTCP2State implements EstablishState {
     private final HandshakeState _handshakeState;
     private final RouterInfo _aliceRI;
     private final int _aliceRISize;
-    private int _padlen1;
     private int _padlen2;
     private final int _padlen3;
     private final SessionKey _bobHash;
@@ -190,19 +189,22 @@ class OutboundNTCP2State implements EstablishState {
         }
         if (_log.shouldLog(Log.DEBUG))
             _log.debug(this + "send X");
-        byte options[] = new byte[OPTIONS1_SIZE];
+        // write options directly to tmp, offset 32, will encrypt in-place
         // network ID cross-check, proposal 147
-        options[0] = (byte) (_context.router().getNetworkID());
-        options[1] = NTCPTransport.NTCP2_INT_VERSION;
+        _tmp[KEY_SIZE] = (byte) (_context.router().getNetworkID());
+        _tmp[KEY_SIZE + 1] = NTCPTransport.NTCP2_INT_VERSION;
         int padlen1 = _context.random().nextInt(PADDING1_MAX);
-        DataHelper.toLong(options, 2, 2, padlen1);
+        DataHelper.toLong(_tmp, KEY_SIZE + 2, 2, padlen1);
         int msg3p2len = NTCP2Payload.BLOCK_HEADER_SIZE + 1 + _aliceRISize +
                         NTCP2Payload.BLOCK_HEADER_SIZE + OPTIONS3_SIZE +
                         NTCP2Payload.BLOCK_HEADER_SIZE + _padlen3 +
                         MAC_SIZE;
-        DataHelper.toLong(options, 4, 2, msg3p2len);
+        DataHelper.toLong(_tmp, KEY_SIZE + 4, 2, msg3p2len);
+        _tmp[KEY_SIZE + 6] = 0;
+        _tmp[KEY_SIZE + 7] = 0;
         long now = (_context.clock().now() + 500) / 1000;
-        DataHelper.toLong(options, 8, 4, now);
+        DataHelper.toLong(_tmp, KEY_SIZE + 8, 4, now);
+        Arrays.fill(_tmp, KEY_SIZE + 12, KEY_SIZE + 16, (byte) 0);
 
         // set keys
         String s = _con.getRemoteAddress().getOption("s");
@@ -224,7 +226,8 @@ class OutboundNTCP2State implements EstablishState {
             _handshakeState.start();
             if (_log.shouldDebug())
                 _log.debug("After start: " + _handshakeState.toString());
-            _handshakeState.writeMessage(_tmp, 0, options, 0, OPTIONS1_SIZE);
+            // encrypt in-place
+            _handshakeState.writeMessage(_tmp, 0, _tmp, KEY_SIZE, OPTIONS1_SIZE);
         } catch (GeneralSecurityException gse) {
             // buffer length error
             if (!_log.shouldWarn())
@@ -254,7 +257,7 @@ class OutboundNTCP2State implements EstablishState {
 
         changeState(State.OB_SENT_X);
         // send it all at once
-        _transport.getPumper().wantsWrite(_con, _tmp, 0, MSG1_SIZE + padlen1);
+        _con.wantsWrite(_tmp, 0, MSG1_SIZE + padlen1);
     }
 
     /**
@@ -300,8 +303,24 @@ class OutboundNTCP2State implements EstablishState {
             long rtt = now - _con.getCreated();
             _peerSkew = (now - (tsB * 1000) - (rtt / 2) + 500) / 1000; 
             if (_peerSkew > MAX_SKEW || _peerSkew < 0 - MAX_SKEW) {
-                fail("Clock Skew: " + _peerSkew, null, true);
-                return;
+                long diff = 1000 * Math.abs(_peerSkew);
+                if (_context.clock().getUpdatedSuccessfully()) {
+                    // usual case
+                    _context.statManager().addRateData("ntcp.invalidOutboundSkew", diff);
+                    fail("Clock Skew: " + _peerSkew, null, true);
+                    // Only banlist if we know what time it is
+                    _context.banlist().banlistRouter(DataHelper.formatDuration(diff),
+                                                     _con.getRemotePeer().calculateHash(),
+                                                     "Excessive clock skew: {0}");  // _x in IES
+                    _transport.setLastBadSkew(_peerSkew);
+                    return;
+                }
+                // Adjust the clock one time in desperation
+                // We are Alice, he is Bob, adjust to match Bob
+                _context.clock().setOffset(1000 * (0 - _peerSkew), true);
+                _peerSkew = 0;
+                _log.logAlways(Log.WARN, "NTP failure, NTCP adjusted clock by " + DataHelper.formatDuration(diff) +
+                                         " source router: " + _con.getRemotePeer().calculateHash().toBase64());
             }
             changeState(State.OB_GOT_HXY);
             _received = 0;
@@ -392,7 +411,7 @@ class OutboundNTCP2State implements EstablishState {
         // send it all at once
         if (_log.shouldDebug())
             _log.debug("Sending msg3, part 1 is:\n" + net.i2p.util.HexDump.dump(tmp, 0, MSG3P1_SIZE));
-        _transport.getPumper().wantsWrite(_con, tmp);
+        _con.wantsWrite(tmp);
         if (_log.shouldDebug())
             _log.debug("After msg 3: " + _handshakeState.toString());
         setDataPhase();
