@@ -1,6 +1,7 @@
 package net.i2p.router.transport.udp;
 
 import java.io.ByteArrayInputStream;
+import java.net.InetSocketAddress;
 import java.io.IOException;
 import java.util.Queue;
 import java.util.concurrent.LinkedBlockingQueue;
@@ -10,6 +11,7 @@ import net.i2p.data.Base64;
 import net.i2p.data.ByteArray;
 import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
+import net.i2p.data.Hash;
 import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.SessionKey;
 import net.i2p.data.Signature;
@@ -27,18 +29,18 @@ import net.i2p.util.Log;
  * TODO do all these methods need to be synchronized?
  */
 class InboundEstablishState {
-    private final RouterContext _context;
-    private final Log _log;
+    protected final RouterContext _context;
+    protected final Log _log;
     // SessionRequest message
     private byte _receivedX[];
-    private byte _bobIP[];
-    private final int _bobPort;
+    protected byte _bobIP[];
+    protected final int _bobPort;
     private final DHSessionKeyBuilder _keyBuilder;
     // SessionCreated message
     private byte _sentY[];
-    private final byte _aliceIP[];
-    private final int _alicePort;
-    private long _sentRelayTag;
+    protected final byte _aliceIP[];
+    protected final int _alicePort;
+    protected long _sentRelayTag;
     private long _sentSignedOnTime;
     private SessionKey _sessionKey;
     private SessionKey _macKey;
@@ -49,23 +51,23 @@ class InboundEstablishState {
     private byte _receivedSignature[];
     private boolean _verificationAttempted;
     // sig not verified
-    private RouterIdentity _receivedUnconfirmedIdentity;
+    protected RouterIdentity _receivedUnconfirmedIdentity;
     // identical to uncomfirmed, but sig now verified
-    private RouterIdentity _receivedConfirmedIdentity;
+    protected RouterIdentity _receivedConfirmedIdentity;
     // general status 
-    private final long _establishBegin;
+    protected final long _establishBegin;
     //private long _lastReceive;
-    private long _lastSend;
-    private long _nextSend;
-    private final RemoteHostId _remoteHostId;
-    private InboundState _currentState;
+    protected long _lastSend;
+    protected long _nextSend;
+    protected final RemoteHostId _remoteHostId;
+    protected InboundState _currentState;
     private final Queue<OutNetMessage> _queuedMessages;
     // count for backoff
-    private int _createdSentCount;
-    // default true
-    private boolean _introductionRequested = true;
+    protected int _createdSentCount;
+    // default true for SSU 1, false for SSU 2
+    protected boolean _introductionRequested;
 
-    private int _rtt;
+    protected int _rtt;
     
     public enum InboundState {
         /** nothin known yet */
@@ -82,17 +84,36 @@ class InboundEstablishState {
         /** we are explicitly failing it */
         IB_STATE_FAILED,
         /** Successful completion, PeerState created and added to transport */
-        IB_STATE_COMPLETE
+        IB_STATE_COMPLETE,
+
+        /**
+         * SSU2: We have received a token request
+         * @since 0.9.54
+         */
+        IB_STATE_TOKEN_REQUEST_RECEIVED,
+        /**
+         * SSU2: We have received a request but the token is bad
+         * @since 0.9.54
+         */
+        IB_STATE_REQUEST_BAD_TOKEN_RECEIVED,
+        /**
+         * SSU2: We have sent a retry
+         * @since 0.9.54
+         */
+        IB_STATE_RETRY_SENT,
     }
     
     /** basic delay before backoff
-     *  Transmissions at 0, 3, 9 sec
-     *  Previously: 1500 (0, 1.5, 4.5, 10.5)
+     *  Transmissions at 0, 1, 3, 7 sec
+     *  This should be a little shorter than for outbound.
      */
-    private static final long RETRANSMIT_DELAY = 3000;
+    protected static final long RETRANSMIT_DELAY = 1000;
 
-    /** max delay including backoff */
-    private static final long MAX_DELAY = 15*1000;
+    /**
+     *  max delay including backoff
+     *  This should be a little shorter than for outbound.
+     */
+    protected static final long MAX_DELAY = EstablishmentManager.MAX_IB_ESTABLISH_TIME;
 
     /**
      *  @param localPort Must be our external port, otherwise the signature of the
@@ -110,9 +131,33 @@ class InboundEstablishState {
         _establishBegin = ctx.clock().now();
         _keyBuilder = dh;
         _queuedMessages = new LinkedBlockingQueue<OutNetMessage>();
+        _introductionRequested = true;
         receiveSessionRequest(req);
     }
+
+    /**
+     *  For SSU2
+     *
+     *  @since 0.9.54
+     */
+    protected InboundEstablishState(RouterContext ctx, InetSocketAddress addr) {
+        _context = ctx;
+        _log = ctx.logManager().getLog(getClass());
+        _aliceIP = addr.getAddress().getAddress();
+        _alicePort = addr.getPort();
+        _remoteHostId = new RemoteHostId(_aliceIP, _alicePort);
+        _bobPort = 0;
+        _currentState = InboundState.IB_STATE_UNKNOWN;
+        _establishBegin = ctx.clock().now();
+        _keyBuilder = null;
+        _queuedMessages = new LinkedBlockingQueue<OutNetMessage>();
+    }
     
+    /**
+     * @since 0.9.54
+     */
+    public int getVersion() { return 1; }
+
     public synchronized InboundState getState() { return _currentState; }
 
     /** @return if previously complete */
@@ -288,8 +333,17 @@ class InboundEstablishState {
             _currentState = InboundState.IB_STATE_CREATED_SENT;
     }
 
-    /** how long have we been trying to establish this session? */
-    public long getLifetime() { return _context.clock().now() - _establishBegin; }
+    /**
+     * how long have we been trying to establish this session?
+     */
+    public long getLifetime() { return getLifetime(_context.clock().now()); }
+
+    /**
+     * how long have we been trying to establish this session?
+     * @since 0.9.57
+     */
+    public long getLifetime(long now) { return now - _establishBegin; }
+
     public long getEstablishBeginTime() { return _establishBegin; }
 
     /**
@@ -344,6 +398,15 @@ class InboundEstablishState {
                                                             "Unsupported signature type");
                     fail();
                 }
+                Hash h = _receivedUnconfirmedIdentity.calculateHash();
+                if (_context.banlist().isBanlistedForever(h)) {
+                    // validate sig to prevent spoofing
+                    if (getConfirmedIdentity() != null)
+                       _context.blocklist().add(_aliceIP);
+                    if (_log.shouldLog(Log.WARN))
+                        _log.warn("Router is banned: " + h.toBase64() + " on " + this);
+                    fail();
+                }
             } else {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn("Bad ident from: " + toString());
@@ -371,7 +434,7 @@ class InboundEstablishState {
      *  Have we fully received the SessionConfirmed messages from Alice?
      *  Caller must synch on this.
      */
-    private boolean confirmedFullyReceived() {
+    protected boolean confirmedFullyReceived() {
         if (_receivedIdentity != null) {
             for (int i = 0; i < _receivedIdentity.length; i++) {
                 if (_receivedIdentity[i] == null)
@@ -488,14 +551,16 @@ class InboundEstablishState {
                 _receivedConfirmedIdentity = _receivedUnconfirmedIdentity;
             } else {
                 if (_log.shouldLog(Log.WARN))
-                    _log.warn("Signature failed from " + _receivedUnconfirmedIdentity);
+                    _log.warn("Signature failed from " + _receivedUnconfirmedIdentity + "\non: " + this);
+                // we can't ban the hash because it could be spoofed, but we can block the IP
+                _context.blocklist().add(_aliceIP);
             }
     }
     
     /**
      *  Call from synchronized method only
      */
-    private void packetReceived() {
+    protected void packetReceived() {
         _nextSend = _context.clock().now();
     }
     
@@ -509,7 +574,9 @@ class InboundEstablishState {
         //if (_sentY != null)
         //    buf.append(" SentY: ").append(Base64.encode(_sentY, 0, 4));
         //buf.append(" Bob: ").append(Addresses.toString(_bobIP, _bobPort));
-        buf.append(" RelayTag: ").append(_sentRelayTag);
+        buf.append(" lifetime: ").append(DataHelper.formatDuration(getLifetime()));
+        if (_sentRelayTag > 0)
+            buf.append(" RelayTag: ").append(_sentRelayTag);
         //buf.append(" SignedOn: ").append(_sentSignedOnTime);
         buf.append(' ').append(_currentState);
         return buf.toString();
