@@ -45,6 +45,9 @@ class InboundMessageState implements CDQEntry {
     private static final int MAX_FRAGMENT_SIZE = UDPPacket.MAX_PACKET_SIZE;
     private static final ByteCache _fragmentCache = ByteCache.getInstance(64, MAX_FRAGMENT_SIZE);
     
+    /**
+     * Only for Poison right now.
+     */
     public InboundMessageState(RouterContext ctx, long messageId, Hash from) {
         _context = ctx;
         _log = ctx.logManager().getLog(InboundMessageState.class);
@@ -62,6 +65,9 @@ class InboundMessageState implements CDQEntry {
      * This is more efficient if the fragment is the last (and probably only) fragment.
      * The main savings is not allocating ByteArray[64].
      *
+     * SSU 1 only.
+     *
+     * @param dataFragment the fragment index in the DataReader, NOT the fragment number
      * @throws DataFormatException if the fragment was corrupt
      * @since 0.9.9
      */
@@ -86,11 +92,47 @@ class InboundMessageState implements CDQEntry {
         if (!receiveFragment(data, dataFragment))
             throw new DataFormatException("corrupt");
     }
-    
+
+    /**
+     * Create a new IMS and read in the data from the fragment.
+     * Do NOT call receiveFragment for the same fragment afterwards.
+     * This is more efficient if the fragment is the last (and probably only) fragment.
+     * The main savings is not allocating ByteArray[64].
+     *
+     * SSU 2 only.
+     *
+     * @param fragmentNum the fragment number
+     * @throws DataFormatException if the fragment was corrupt
+     * @since 0.9.54
+     */
+    public InboundMessageState(RouterContext ctx, long messageId, Hash from,
+                               byte[] data, int off, int len, int fragmentNum, boolean isLast)
+                               throws DataFormatException {
+        _context = ctx;
+        _log = ctx.logManager().getLog(InboundMessageState.class);
+        _messageId = messageId;
+        _from = from;
+        if (isLast) {
+            if (fragmentNum > MAX_FRAGMENTS)
+                throw new DataFormatException("corrupt - too many fragments: " + fragmentNum);
+            _fragments = new ByteArray[fragmentNum + 1];
+        } else {
+            _fragments = new ByteArray[MAX_FRAGMENTS];
+        }
+        _lastFragment = -1;
+        _completeSize = -1;
+        _receiveBegin = ctx.clock().now();
+        if (!receiveFragment(data, off, len, fragmentNum, isLast))
+            throw new DataFormatException("corrupt");
+    }
+
     /**
      * Read in the data from the fragment.
      * Caller should synchronize.
      *
+     * SSU 1 only.
+     *
+     * @param dataFragment the fragment index in the DataReader, NOT the fragment number
      * @return true if the data was ok, false if it was corrupt
      */
     public boolean receiveFragment(UDPPacketReader.DataReader data, int dataFragment) throws DataFormatException {
@@ -123,16 +165,16 @@ class InboundMessageState implements CDQEntry {
                 if (isLast) {
                     // don't allow _lastFragment to be set twice
                     if (_lastFragment >= 0) {
-                        if (_log.shouldLog(Log.ERROR))
-                            _log.error("Multiple last fragments for message " + _messageId + " from " + _from);
+                        if (_log.shouldWarn())
+                            _log.warn("Multiple last fragments for message " + _messageId + " from " + _from);
                         return false;
                     }
                     // TODO - check for non-last fragments after this one?
                     _lastFragment = fragmentNum;
                 } else if (_lastFragment >= 0 && fragmentNum >= _lastFragment) {
                     // don't allow non-last after last
-                    if (_log.shouldLog(Log.ERROR))
-                        _log.error("Non-last fragment " + fragmentNum + " when last is " + _lastFragment + " for message " + _messageId + " from " + _from);
+                    if (_log.shouldWarn())
+                        _log.warn("Non-last fragment " + fragmentNum + " when last is " + _lastFragment + " for message " + _messageId + " from " + _from);
                     return false;
                 }
                 if (_log.shouldLog(Log.DEBUG))
@@ -153,7 +195,73 @@ class InboundMessageState implements CDQEntry {
         }
         return true;
     }
-    
+
+    /**
+     * Read in the data from the fragment.
+     * Caller should synchronize.
+     *
+     * SSU 2 only.
+     *
+     * @param fragmentNum the fragment number
+     * @return true if the data was ok, false if it was corrupt
+     * @since 0.9.54
+     */
+    public boolean receiveFragment(byte[] data, int off, int len, int fragmentNum, boolean isLast) throws DataFormatException {
+        if (fragmentNum >= _fragments.length) {
+            if (_log.shouldLog(Log.WARN))
+                _log.warn("Invalid fragment " + fragmentNum + '/' + _fragments.length);
+            return false;
+        }
+        if (_fragments[fragmentNum] == null) {
+            // new fragment, read it
+            ByteArray message = _fragmentCache.acquire();
+            System.arraycopy(data, off, message.getData(), 0, len);
+            message.setValid(len);
+            _fragments[fragmentNum] = message;
+            if (isLast) {
+                // don't allow _lastFragment to be set twice
+                if (_lastFragment >= 0) {
+                    if (_log.shouldWarn())
+                        _log.warn("Multiple last fragments for message " + _messageId + " from " + _from);
+                    return false;
+                }
+                // TODO - check for non-last fragments after this one?
+                _lastFragment = fragmentNum;
+            } else if (_lastFragment >= 0 && fragmentNum >= _lastFragment) {
+                // don't allow non-last after last
+                if (_log.shouldWarn())
+                    _log.warn("Non-last fragment " + fragmentNum + " when last is " + _lastFragment + " for message " + _messageId + " from " + _from);
+                return false;
+            }
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("New fragment " + fragmentNum + " for message " + _messageId
+                           + ", size=" + len
+                           + ", isLast=" + isLast
+                      /*   + ", data=" + Base64.encode(message.getData(), 0, size)   */  );
+        } else {
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("Received fragment " + fragmentNum + " for message " + _messageId
+                           + " again, old size=" + _fragments[fragmentNum].getValid()
+                           + " and new size=" + len);
+        }
+        return true;
+    }
+
+    /**
+     * Do we have this fragment?
+     *
+     * SSU 2 only.
+     *
+     * @param fragmentNum the fragment number
+     * @return true if we have the fragment
+     * @since 0.9.54
+     */
+    public boolean hasFragment(int fragmentNum) {
+        if (fragmentNum >= _fragments.length)
+            return false;
+        return _fragments[fragmentNum] != null;
+    }
+
     /**
      *  May not be valid after released.
      *  Probably doesn't need to be synced by caller, given the order of
@@ -226,7 +334,11 @@ class InboundMessageState implements CDQEntry {
         return _completeSize;
     }
 
-    /** FIXME synch here or PeerState.fetchPartialACKs() */
+    /**
+     *  Only call this if not complete.
+     *  TODO remove this, have InboundMessageState implement ACKBitfield.
+     *  FIXME synch here or PeerState.fetchPartialACKs()
+     */
     public ACKBitfield createACKBitfield() {
         int last = _lastFragment;
         int sz = (last >= 0) ? last + 1 : _fragments.length;
@@ -236,9 +348,12 @@ class InboundMessageState implements CDQEntry {
     /**
      *  A true partial bitfield that is probably not complete.
      *  fragmentCount() will return 64 if unknown.
+     *
+     *  TODO remove this, have InboundMessageState implement ACKBitfield.
      */
     private static final class PartialBitfield implements ACKBitfield {
         private final long _bitfieldMessageId;
+        private final int _fragmentCount;
         private final int _ackCount;
         private final int _highestReceived;
         // bitfield, 1 for acked
@@ -263,6 +378,7 @@ class InboundMessageState implements CDQEntry {
                 }
             }
             _fragmentAcks = acks;
+            _fragmentCount = size;
             _ackCount = ackCount;
             _highestReceived = highestReceived;
         }
@@ -274,7 +390,12 @@ class InboundMessageState implements CDQEntry {
             return 1L << fragment;
         }
 
-        public int fragmentCount() { return _highestReceived + 1; }
+        /**
+         *  Don't use for serialization since it may be unknown;
+         *  use highestReceived() instead.
+         *  @return 64 if unknown
+         */
+        public int fragmentCount() { return _fragmentCount; }
 
         public int ackCount() { return _ackCount; }
 
@@ -288,7 +409,10 @@ class InboundMessageState implements CDQEntry {
             return (_fragmentAcks & mask(fragmentNum)) != 0;
         }
 
-        public boolean receivedComplete() { return _ackCount == _highestReceived + 1; }
+        /**
+         *  @return should always be false
+         */
+        public boolean receivedComplete() { return _ackCount == _fragmentCount; }
         
         @Override
         public String toString() { 
@@ -347,8 +471,9 @@ class InboundMessageState implements CDQEntry {
         } else {
             for (int i = 0; i <= _lastFragment; i++) {
                 buf.append(" fragment ").append(i);
-                if (_fragments[i] != null)
-                    buf.append(": known at size ").append(_fragments[i].getValid());
+                ByteArray ba = _fragments[i];
+                if (ba != null)
+                    buf.append(": known at size ").append(ba.getValid());
                 else
                     buf.append(": unknown");
             }
