@@ -25,19 +25,19 @@ import net.i2p.util.Log;
  *
  */
 class OutboundEstablishState {
-    private final RouterContext _context;
-    private final Log _log;
+    protected final RouterContext _context;
+    protected final Log _log;
     // SessionRequest message
     private byte _sentX[];
-    private byte _bobIP[];
-    private int _bobPort;
+    protected byte _bobIP[];
+    protected int _bobPort;
     private final DHSessionKeyBuilder.Factory _keyFactory;
     private DHSessionKeyBuilder _keyBuilder;
     // SessionCreated message
     private byte _receivedY[];
-    private byte _aliceIP[];
-    private int _alicePort;
-    private long _receivedRelayTag;
+    protected byte _aliceIP[];
+    protected int _alicePort;
+    protected long _receivedRelayTag;
     private long _receivedSignedOnTime;
     private SessionKey _sessionKey;
     private SessionKey _macKey;
@@ -49,18 +49,18 @@ class OutboundEstablishState {
     private long _sentSignedOnTime;
     private Signature _sentSignature;
     // general status 
-    private final long _establishBegin;
+    protected final long _establishBegin;
     //private long _lastReceive;
-    private long _lastSend;
-    private long _nextSend;
-    private RemoteHostId _remoteHostId;
+    protected long _lastSend;
+    protected long _nextSend;
+    protected RemoteHostId _remoteHostId;
     private final RemoteHostId _claimedAddress;
-    private final RouterIdentity _remotePeer;
+    protected final RouterIdentity _remotePeer;
     private final boolean _allowExtendedOptions;
     private final boolean _needIntroduction;
     private final SessionKey _introKey;
     private final Queue<OutNetMessage> _queuedMessages;
-    private OutboundState _currentState;
+    protected OutboundState _currentState;
     private long _introductionNonce;
     private boolean _isFirstMessageOurDSM;
     // intro
@@ -68,13 +68,13 @@ class OutboundEstablishState {
     private boolean _complete;
     // counts for backoff
     private int _confirmedSentCount;
-    private int _requestSentCount;
+    protected int _requestSentCount;
     private int _introSentCount;
     // Times for timeout
     private long _confirmedSentTime;
-    private long _requestSentTime;
+    protected long _requestSentTime;
     private long _introSentTime;
-    private int _rtt;
+    protected int _rtt;
     
     public enum OutboundState {
         /** nothin sent yet */
@@ -92,16 +92,40 @@ class OutboundEstablishState {
         /** RelayResponse received */
         OB_STATE_INTRODUCED,
         /** SessionConfirmed failed validation */
-        OB_STATE_VALIDATION_FAILED
+        OB_STATE_VALIDATION_FAILED,
+
+        /**
+         * SSU2: We don't have a token
+         * @since 0.9.54
+         */
+        OB_STATE_NEEDS_TOKEN,
+        /**
+         * SSU2: We have sent a token request
+         * @since 0.9.54
+         */
+        OB_STATE_TOKEN_REQUEST_SENT,
+        /**
+         * SSU2: We have received a retry
+         * @since 0.9.54
+         */
+        OB_STATE_RETRY_RECEIVED,
+        /**
+         * SSU2: We have sent a session request after receiving a retry
+         * @since 0.9.54
+         */
+        OB_STATE_REQUEST_SENT_NEW_TOKEN
     }
     
     /** basic delay before backoff
-     *  Transmissions at 0, 3, 9 sec
-     *  Previously: 1500 (0, 1.5, 4.5, 10.5)
+     *  Transmissions at 0, 1.25, 3.75, 8.75 sec
+     *  This should be a little longer than for inbound.
      */
-    private static final long RETRANSMIT_DELAY = 3000;
+    protected static final long RETRANSMIT_DELAY = 1250;
 
-    /** max delay including backoff */
+    /**
+     *  max delay including backoff
+     *  This should be a little longer than for inbound.
+     */
     private static final long MAX_DELAY = 15*1000;
 
     private static final long WAIT_FOR_HOLE_PUNCH_DELAY = 500;
@@ -151,6 +175,50 @@ class OutboundEstablishState {
         }
     }
     
+    /**
+     *  For SSU2
+     *
+     *  @since 0.9.54
+     */
+    protected OutboundEstablishState(RouterContext ctx, RemoteHostId claimedAddress,
+                                  RemoteHostId remoteHostId,
+                                  RouterIdentity remotePeer,
+                                  boolean needIntroduction,
+                                  SessionKey introKey, UDPAddress addr) {
+        _context = ctx;
+        _log = ctx.logManager().getLog(getClass());
+        if (claimedAddress != null) {
+            _bobIP = claimedAddress.getIP();
+            _bobPort = claimedAddress.getPort();
+        } else {
+            //_bobIP = null;
+            _bobPort = -1;
+        }
+        _claimedAddress = claimedAddress;
+        _remoteHostId = remoteHostId;
+        _allowExtendedOptions = false;
+        _needIntroduction = needIntroduction;
+        _remotePeer = remotePeer;
+        _introKey = introKey;
+        _queuedMessages = new LinkedBlockingQueue<OutNetMessage>();
+        _establishBegin = ctx.clock().now();
+        _remoteAddress = addr;
+        _introductionNonce = -1;
+        _keyFactory = null;
+        if (addr.getIntroducerCount() > 0) {
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("new outbound establish to " + remotePeer.calculateHash() + ", with address: " + addr);
+            _currentState = OutboundState.OB_STATE_PENDING_INTRO;
+        } else {
+            _currentState = OutboundState.OB_STATE_UNKNOWN;
+        }
+    }
+    
+    /**
+     * @since 0.9.54
+     */
+    public int getVersion() { return 1; }
+    
     public synchronized OutboundState getState() { return _currentState; }
 
     /** @return if previously complete */
@@ -192,6 +260,9 @@ class OutboundEstablishState {
             if (m.getType() == DatabaseStoreMessage.MESSAGE_TYPE) {
                DatabaseStoreMessage dsm = (DatabaseStoreMessage) m;
                if (dsm.getKey().equals(_context.routerHash())) {
+                   // version 2 sends our RI in handshake
+                   if (getVersion() > 1)
+                       return;
                    _isFirstMessageOurDSM = true;
                }
            }
@@ -371,8 +442,8 @@ class OutboundEstablishState {
         _receivedIV = null;
         _receivedSignature = null;
         if (_keyBuilder != null) {
-            if (_keyBuilder.getPeerPublicValue() == null)
-                _keyFactory.returnUnused(_keyBuilder);
+            //if (_keyBuilder.getPeerPublicValue() == null)
+            //    _keyFactory.returnUnused(_keyBuilder);
             _keyBuilder = null;
         }
         // sure, there's a chance the packet was corrupted, but in practice
@@ -544,8 +615,8 @@ class OutboundEstablishState {
         }
         _confirmedSentCount++;
         _nextSend = _lastSend + delay;
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Send confirm packets, nextSend in " + delay);
+        if (_log.shouldDebug())
+            _log.debug("Send confirm packets, nextSend in " + delay + " on " + this);
         if (_currentState == OutboundState.OB_STATE_UNKNOWN || 
             _currentState == OutboundState.OB_STATE_PENDING_INTRO ||
             _currentState == OutboundState.OB_STATE_INTRODUCED ||
@@ -574,7 +645,7 @@ class OutboundEstablishState {
         _requestSentCount++;
         _nextSend = _lastSend + delay;
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Send a request packet, nextSend in " + delay);
+            _log.debug("Send a request packet, nextSend in " + delay + " on " + this);
         if (_currentState == OutboundState.OB_STATE_UNKNOWN ||
             _currentState == OutboundState.OB_STATE_INTRODUCED)
             _currentState = OutboundState.OB_STATE_REQUEST_SENT;
@@ -656,9 +727,18 @@ class OutboundEstablishState {
         _nextSend = now;
         return true;
     }
-    
-    /** how long have we been trying to establish this session? */
-    public long getLifetime() { return _context.clock().now() - _establishBegin; }
+
+    /**
+     * how long have we been trying to establish this session?
+     */
+    public long getLifetime() { return getLifetime(_context.clock().now()); }
+
+    /**
+     * how long have we been trying to establish this session?
+     * @since 0.9.57
+     */
+    public long getLifetime(long now) { return now - _establishBegin; }
+
     public long getEstablishBeginTime() { return _establishBegin; }
 
     /**
@@ -694,7 +774,7 @@ class OutboundEstablishState {
     /**
      *  Call from synchronized method only
      */
-    private void packetReceived() {
+    protected void packetReceived() {
         _nextSend = _context.clock().now();
         //if (_log.shouldLog(Log.DEBUG))
         //    _log.debug("Got a packet, nextSend == now");
@@ -703,6 +783,8 @@ class OutboundEstablishState {
     /** @since 0.8.9 */
     @Override
     public String toString() {
-        return "OES " + _remoteHostId + ' ' + _currentState;
+        return "OES " + _remotePeer.getHash().toBase64().substring(0, 6) + ' ' + _remoteHostId +
+               " lifetime: " + DataHelper.formatDuration(getLifetime()) +
+               ' ' + _currentState;
     }
 }
