@@ -1,5 +1,6 @@
 package net.i2p.router.transport.udp;
 
+import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -53,6 +54,8 @@ class OutboundMessageFragments {
 
     private volatile boolean _alive;
     private final PacketBuilder _builder;
+    // null if SSU2 not enabled
+    private final PacketBuilder2 _builder2;
 
     /** if we can handle more messages explicitly, set this to true */
     // private boolean _allowExcess; // LINT not used??
@@ -69,7 +72,8 @@ class OutboundMessageFragments {
         _transport = transport;
         // _throttle = throttle;
         _activePeers = new ConcurrentHashSet<PeerState>(256);
-        _builder = new PacketBuilder(ctx, transport);
+        _builder = transport.getBuilder();
+        _builder2 = transport.getBuilder2();
         _alive = true;
         // _allowExcess = false;
         _context.statManager().createRateStat("udp.sendVolleyTime", "Long it takes to send a full volley", "udp", UDPTransport.RATES);
@@ -104,8 +108,8 @@ class OutboundMessageFragments {
     }
 
     void dropPeer(PeerState peer) {
-        if (_log.shouldLog(Log.INFO))
-            _log.info("Dropping peer " + peer.getRemotePeer());
+        if (_log.shouldDebug())
+            _log.debug("Dropping peer " + peer.getRemotePeer());
         peer.dropOutbound();
         _activePeers.remove(peer);
     }
@@ -160,7 +164,7 @@ class OutboundMessageFragments {
             // will throw IAE if peer == null
             OutboundMessageState state = new OutboundMessageState(_context, msg, peer);
             peer.add(state);
-            add(peer, state.fragmentSize(0));
+            add(peer, state.getMinSendSize());
         } catch (IllegalArgumentException iae) {
             _transport.failed(msg, "Peer disconnected quickly");
             return;
@@ -177,7 +181,7 @@ class OutboundMessageFragments {
         if (peer == null)
             throw new RuntimeException("null peer for " + state);
         peer.add(state);
-        add(peer, state.fragmentSize(0));
+        add(peer, state.getMinSendSize());
         //_context.statManager().addRateData("udp.outboundActiveCount", active, 0);
     }
 
@@ -194,7 +198,7 @@ class OutboundMessageFragments {
         for (int i = 0; i < sz; i++) {
             OutboundMessageState state = states.get(i);
             peer.add(state);
-            int fsz = state.fragmentSize(0);
+            int fsz = state.getMinSendSize();
             if (fsz < min)
                 min = fsz;
         }
@@ -388,37 +392,55 @@ class OutboundMessageFragments {
         if (states == null || peer == null)
             return null;
 
-        // ok, simplest possible thing is to always tack on the bitfields if
-        List<Long> msgIds = peer.getCurrentFullACKs();
-        int newFullAckCount = msgIds.size();
-        msgIds.addAll(peer.getCurrentResendACKs());
-        List<ACKBitfield> partialACKBitfields = new ArrayList<ACKBitfield>();
-        peer.fetchPartialACKs(partialACKBitfields);
-        int piggybackedPartialACK = partialACKBitfields.size();
-        // getCurrentFullACKs() already makes a copy, do we need to copy again?
-        // YES because buildPacket() now removes them (maybe)
-        Set<Long> remaining = new HashSet<Long>(msgIds);
+        List<Long> msgIds;
+        int newFullAckCount;
+        List<ACKBitfield> partialACKBitfields;
+        int piggybackedPartialACK;
+        Set<Long> remaining;
+        int before;
+        if (peer.getVersion() == 1) {
+            // ok, simplest possible thing is to always tack on the bitfields if
+            msgIds = peer.getCurrentFullACKs();
+            newFullAckCount = msgIds.size();
+            msgIds.addAll(peer.getCurrentResendACKs());
+            partialACKBitfields = new ArrayList<ACKBitfield>();
+            peer.fetchPartialACKs(partialACKBitfields);
+            piggybackedPartialACK = partialACKBitfields.size();
+            // getCurrentFullACKs() already makes a copy, do we need to copy again?
+            // YES because buildPacket() now removes them (maybe)
+            remaining = new HashSet<Long>(msgIds);
+            before = remaining.size();
+        } else {
+            // all unused
+            msgIds = null;
+            newFullAckCount = 0;
+            partialACKBitfields = null;
+            piggybackedPartialACK = 0;
+            remaining = null;
+            before = 0;
+        }
 
         // build the list of fragments to send
         List<Fragment> toSend = new ArrayList<Fragment>(8);
         for (OutboundMessageState state : states) {
-            int fragments = state.getFragmentCount();
-            int queued = 0;
-            for (int i = 0; i < fragments; i++) {
-                if (state.needsSending(i)) {
-                    toSend.add(new Fragment(state, i));
-                    queued++;
-                }
-            }
+            int queued = state.push(toSend);
             // per-state stats
-            if (queued > 0 && state.getPushCount() > 1) {
-                peer.messageRetransmitted(queued);
+            if (queued > 0 && state.getMaxSends() > 1) {
+                int maxPktSz = state.fragmentSize(0);
+                if (peer.getVersion() == 1)
+                    maxPktSz += (peer.isIPv6() ? PacketBuilder.MIN_IPV6_DATA_PACKET_OVERHEAD : PacketBuilder.MIN_DATA_PACKET_OVERHEAD);
+                else
+                    maxPktSz += SSU2Payload.BLOCK_HEADER_SIZE +
+                                (peer.isIPv6() ? PacketBuilder2.MIN_IPV6_DATA_PACKET_OVERHEAD : PacketBuilder2.MIN_DATA_PACKET_OVERHEAD);
+                peer.messageRetransmitted(queued, maxPktSz);
                 // _packetsRetransmitted += toSend; // lifetime for the transport
-                _context.statManager().addRateData("udp.peerPacketsRetransmitted", peer.getPacketsRetransmitted(), peer.getPacketsTransmitted());
-                _context.statManager().addRateData("udp.packetsRetransmitted", state.getLifetime(), peer.getPacketsTransmitted());
+                long lifetime = state.getLifetime();
+                int transmitted = peer.getPacketsTransmitted();
+                _context.statManager().addRateData("udp.peerPacketsRetransmitted", peer.getPacketsRetransmitted(), transmitted);
+                _context.statManager().addRateData("udp.packetsRetransmitted", lifetime, transmitted);
                 if (_log.shouldLog(Log.INFO))
                     _log.info("Retransmitting " + state + " to " + peer);
-                _context.statManager().addRateData("udp.sendVolleyTime", state.getLifetime(), queued);
+                _context.statManager().addRateData("udp.sendVolleyTime", lifetime, queued);
             }
         }
 
@@ -441,65 +463,92 @@ class OutboundMessageFragments {
             OutboundMessageState state = next.state;
             OutNetMessage msg = state.getMessage();
             int msgType = (msg != null) ? msg.getMessageTypeId() : -1;
-            if (_log.shouldLog(Log.INFO))
-                _log.info("Building packet for " + next + " to " + peer);
+            if (_log.shouldDebug())
+                _log.debug("Building packet for " + next + " to " + peer);
             int curTotalDataSize = state.fragmentSize(next.num);
+            if (peer.getVersion() > 1) {
+                curTotalDataSize += SSU2Util.FIRST_FRAGMENT_HEADER_SIZE;
+                if (next.num > 0)
+                    curTotalDataSize += SSU2Util.DATA_FOLLOWON_EXTRA_SIZE;
+            }
             // now stuff in more fragments if they fit
             if (i +1 < toSend.size()) {
-                int maxAvail = PacketBuilder.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
-                for (int j = i + 1; j < toSend.size(); j++) {
-                    next = toSend.get(j);
-                    int nextDataSize = next.state.fragmentSize(next.num);
-                    //if (PacketBuilder.canFitAnotherFragment(peer, sendNext.size(), curTotalDataSize, nextDataSize)) {
-                    //if (_builder.canFitAnotherFragment(peer, sendNext.size(), curTotalDataSize, nextDataSize)) {
-                    if (nextDataSize <= maxAvail) {
-                        // add it
-                        toSend.remove(j);
-                        j--;
-                        sendNext.add(next);
-                        curTotalDataSize += nextDataSize;
-                        maxAvail = PacketBuilder.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
-                        if (_log.shouldLog(Log.INFO))
-                            _log.info("Adding in additional " + next + " to " + peer);
-                    }  // else too big
+                int maxAvail;
+                if (peer.getVersion() == 1)
+                    maxAvail = PacketBuilder.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
+                else
+                    maxAvail = PacketBuilder2.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
+                // if less than 16, just use it for acks, don't even try to look for a tiny fragment
+                if (maxAvail >= 16) {
+                    for (int j = i + 1; j < toSend.size(); j++) {
+                        next = toSend.get(j);
+                        int nextDataSize = next.state.fragmentSize(next.num);
+                        if (next.num > 0 && peer.getVersion() > 1)
+                            nextDataSize += SSU2Util.DATA_FOLLOWON_EXTRA_SIZE;
+                        //if (PacketBuilder.canFitAnotherFragment(peer, sendNext.size(), curTotalDataSize, nextDataSize)) {
+                        //if (_builder.canFitAnotherFragment(peer, sendNext.size(), curTotalDataSize, nextDataSize)) {
+                        if (nextDataSize <= maxAvail) {
+                            // add it
+                            toSend.remove(j);
+                            j--;
+                            sendNext.add(next);
+                            curTotalDataSize += nextDataSize;
+                            if (peer.getVersion() == 1)
+                                maxAvail = PacketBuilder.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
+                            else
+                                maxAvail = PacketBuilder2.getMaxAdditionalFragmentSize(peer, sendNext.size(), curTotalDataSize);
+                            if (_log.shouldLog(Log.INFO))
+                                _log.info("Adding in additional " + next + " to " + peer);
+                            // if less than 16, just use it for acks, don't even try to look for a tiny fragment
+                            if (maxAvail < 16)
+                                break;
+                        }  // else too big
+                    }
                 }
             }
 
-            int before = remaining.size();
-            UDPPacket pkt = _builder.buildPacket(sendNext, peer, remaining, newFullAckCount, partialACKBitfields);
+            UDPPacket pkt;
+            if (peer.getVersion() == 1) {
+                pkt = _builder.buildPacket(sendNext, peer, remaining, newFullAckCount, partialACKBitfields);
+            } else {
+                try {
+                    pkt = _builder2.buildPacket(sendNext, (PeerState2) peer);
+                } catch (IOException ioe) {
+                    pkt = null;
+                }
+            }
             if (pkt != null) {
-                if (_log.shouldLog(Log.INFO))
-                    _log.info("Built packet with " + sendNext.size() + " fragments totalling " + curTotalDataSize +
+                if (_log.shouldDebug())
+                    _log.debug("Built packet with " + sendNext.size() + " fragments totalling " + curTotalDataSize +
                               " data bytes to " + peer);
                 _context.statManager().addRateData("udp.sendFragmentsPerPacket", sendNext.size());
-            }
-            if (pkt == null) {
+            } else {
                 if (_log.shouldLog(Log.WARN))
-                    _log.info("Build packet FAIL for " + DataHelper.toString(sendNext) + " to " + peer);
+                    _log.warn("Build packet FAIL for " + DataHelper.toString(sendNext) + " to " + peer);
                 sendNext.clear();
                 continue;
             }
             rv.add(pkt);
 
-            int after = remaining.size();
-            newFullAckCount = Math.max(0, newFullAckCount - (before - after));
-
-            int piggybackedAck = 0;
-            if (msgIds.size() != remaining.size()) {
-                for (int j = 0; j < msgIds.size(); j++) {
-                    Long id = msgIds.get(j);
-                    if (!remaining.contains(id)) {
-                        peer.removeACKMessage(id);
-                        piggybackedAck++;
+            if (peer.getVersion() == 1) {
+                int after = remaining.size();
+                newFullAckCount = Math.max(0, newFullAckCount - (before - after));
+                int piggybackedAck = 0;
+                if (msgIds.size() != remaining.size()) {
+                    for (int j = 0; j < msgIds.size(); j++) {
+                        Long id = msgIds.get(j);
+                        if (!remaining.contains(id)) {
+                            peer.removeACKMessage(id);
+                            piggybackedAck++;
+                        }
                     }
                 }
+                if (piggybackedAck > 0)
+                    _context.statManager().addRateData("udp.sendPiggyback", piggybackedAck);
+                if (piggybackedPartialACK - partialACKBitfields.size() > 0)
+                    _context.statManager().addRateData("udp.sendPiggybackPartial", piggybackedPartialACK - partialACKBitfields.size(), state.getLifetime());
             }
 
-            if (piggybackedAck > 0)
-                _context.statManager().addRateData("udp.sendPiggyback", piggybackedAck);
-            if (piggybackedPartialACK - partialACKBitfields.size() > 0)
-                _context.statManager().addRateData("udp.sendPiggybackPartial", piggybackedPartialACK - partialACKBitfields.size(), state.getLifetime());
-
             // following for debugging and stats
             pkt.setFragmentCount(sendNext.size());
             pkt.setMessageType(msgType);  //type of first fragment
@@ -510,8 +559,10 @@ class OutboundMessageFragments {
 
         int sent = rv.size();
         peer.packetsTransmitted(sent);
-        if (_log.shouldLog(Log.INFO))
-            _log.info("Sent " + fragmentsToSend + " fragments of " + states.size() +
+        if (newFullAckCount <= 0)
+            peer.clearWantedACKSendSince();
+        if (_log.shouldDebug())
+            _log.debug("Sent " + fragmentsToSend + " fragments of " + states.size() +
                       " messages in " + sent + " packets to " + peer);
 
         return rv;
