@@ -2,6 +2,7 @@ package net.i2p.router.transport.udp;
 
 import java.net.DatagramPacket;
 import java.net.InetAddress;
+import java.net.Inet6Address;
 import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -16,6 +17,8 @@ import net.i2p.data.Hash;
 import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.SessionKey;
 import net.i2p.data.Signature;
+import net.i2p.data.router.RouterAddress;
+import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.TransportUtil;
 import net.i2p.util.Addresses;
@@ -180,8 +183,16 @@ class PacketBuilder {
     private static final byte DATA_FLAG_BYTE = UDPPacket.PAYLOAD_TYPE_DATA << 4;
     private static final byte PEER_TEST_FLAG_BYTE = UDPPacket.PAYLOAD_TYPE_TEST << 4;
     private static final byte SESSION_DESTROY_FLAG_BYTE = (byte) (UDPPacket.PAYLOAD_TYPE_SESSION_DESTROY << 4);
+
+    /* Higher than all other OutNetMessage priorities, but still droppable,
+     * and will be shown in the codel.UDP-Sender.drop.500 stat.
+     */
+    static final int PRIORITY_HIGH = 550;
+    private static final int PRIORITY_LOW = OutNetMessage.PRIORITY_LOWEST;
     
     /**
+     *  No state, all methods are thread-safe.
+     *
      *  @param transport may be null for unit testing only
      */
     public PacketBuilder(RouterContext ctx, UDPTransport transport) {
@@ -314,9 +325,13 @@ class PacketBuilder {
         // calculate data size
         int numFragments = fragments.size();
         int dataSize = 0;
+        int priority = 0;
         for (int i = 0; i < numFragments; i++) {
             Fragment frag = fragments.get(i);
             OutboundMessageState state = frag.state;
+            int pri = state.getPriority();
+            if (pri > priority)
+                priority = pri;
             int fragment = frag.num;
             int sz = state.fragmentSize(fragment);
             dataSize += sz;
@@ -364,8 +379,6 @@ class PacketBuilder {
             for (ACKBitfield bf : partialACKsRemaining) {
                 if (partialAcksToSend >= ABSOLUTE_MAX_ACKS)
                     break;  // ack count
-                if (bf.receivedComplete())
-                    continue;
                 // only send what we have to
                 //int acksz = 4 + (bf.fragmentCount() / 7) + 1;
                 int bits = bf.highestReceived() + 1;
@@ -413,8 +426,7 @@ class PacketBuilder {
         if (explicitToSend > 0) {
             if (msg != null)
                 msg.append(explicitToSend).append(" full acks included:");
-            DataHelper.toLong(data, off, 1, explicitToSend);
-            off++;
+            data[off++] = (byte) explicitToSend;
             Iterator<Long> iter = ackIdsRemaining.iterator();
             for (int i = 0; i < explicitToSend && iter.hasNext(); i++) {
                 Long ackId = iter.next();
@@ -473,8 +485,7 @@ class PacketBuilder {
         //if ( (msg != null) && (acksIncluded) )
         //  _log.debug(msg.toString());
         
-        DataHelper.toLong(data, off, 1, numFragments);
-        off++;
+        data[off++] = (byte) numFragments;
         
         // now write each fragment
         int sizeWritten = 0;
@@ -569,6 +580,7 @@ class PacketBuilder {
             }
         }
         
+        packet.setPriority(priority);
         return packet;
     }
     
@@ -601,7 +613,7 @@ class PacketBuilder {
         StringBuilder msg = null;
         if (_log.shouldLog(Log.DEBUG)) {
             msg = new StringBuilder(128);
-            msg.append("building ACK packet to ").append(peer.getRemotePeer().toBase64().substring(0,6));
+            msg.append("building ACK packet to ").append(peer.getRemotePeer().toBase64(), 0, 6);
         }
 
         int fullACKCount = 0;
@@ -627,8 +639,7 @@ class PacketBuilder {
         off++;
         
         if (fullACKCount > 0) {
-            DataHelper.toLong(data, off, 1, fullACKCount);
-            off++;
+            data[off++] = (byte) fullACKCount;
             for (int i = 0; i < ackBitfields.size(); i++) {
                 ACKBitfield bf = ackBitfields.get(i);
                 if (bf.receivedComplete()) {
@@ -641,8 +652,7 @@ class PacketBuilder {
         }
         
         if (partialACKCount > 0) {
-            DataHelper.toLong(data, off, 1, partialACKCount);
-            off++;
+            data[off++] = (byte) partialACKCount;
             for (int i = 0; i < ackBitfields.size(); i++) {
                 ACKBitfield bitfield = ackBitfields.get(i);
                 if (bitfield.receivedComplete()) continue;
@@ -672,8 +682,7 @@ class PacketBuilder {
             }
         }
         
-        DataHelper.toLong(data, off, 1, 0); // no fragments in this message
-        off++;
+        data[off++] = 0; // no fragments in this message
         
         if (msg != null)
             _log.debug(msg.toString());
@@ -684,6 +693,7 @@ class PacketBuilder {
         pkt.setLength(off);
         authenticate(packet, peer.getCurrentCipherKey(), peer.getCurrentMACKey());
         setTo(packet, peer.getRemoteIPAddress(), peer.getRemotePort());
+        packet.setPriority((fullACKCount > 0 || partialACKCount > 0) ? PRIORITY_HIGH : PRIORITY_LOW);
         return packet;
     }
     
@@ -722,8 +732,7 @@ class PacketBuilder {
         // now for the body
         System.arraycopy(state.getSentY(), 0, data, off, state.getSentY().length);
         off += state.getSentY().length;
-        DataHelper.toLong(data, off, 1, sentIP.length);
-        off += 1;
+        data[off++] = (byte) sentIP.length;
         System.arraycopy(sentIP, 0, data, off, sentIP.length);
         off += sentIP.length;
         DataHelper.toLong(data, off, 2, state.getSentPort());
@@ -779,6 +788,7 @@ class PacketBuilder {
         setTo(packet, to, state.getSentPort());
         SimpleByteCache.release(iv);
         packet.setMessageType(TYPE_CREAT);
+        packet.setPriority(PRIORITY_HIGH);
         return packet;
     }
     
@@ -828,8 +838,7 @@ class PacketBuilder {
         byte[] x = state.getSentX();
         System.arraycopy(x, 0, data, off, x.length);
         off += x.length;
-        DataHelper.toLong(data, off, 1, toIP.length);
-        off += 1;
+        data[off++] = (byte) toIP.length;
         System.arraycopy(toIP, 0, data, off, toIP.length);
         off += toIP.length;
         int port = state.getSentPort();
@@ -845,6 +854,7 @@ class PacketBuilder {
         authenticate(packet, state.getIntroKey(), state.getIntroKey());
         setTo(packet, to, port);
         packet.setMessageType(TYPE_SREQ);
+        packet.setPriority(PRIORITY_HIGH);
         return packet;
     }
 
@@ -949,6 +959,7 @@ class PacketBuilder {
         authenticate(packet, state.getCipherKey(), state.getMACKey());
         setTo(packet, to, state.getSentPort());
         packet.setMessageType(TYPE_CONF);
+        packet.setPriority(PRIORITY_HIGH);
         return packet;
     }
 
@@ -1045,6 +1056,7 @@ class PacketBuilder {
         pkt.setLength(off);
         authenticate(packet, cipherKey, macKey);
         setTo(packet, addr, port);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
     
@@ -1076,8 +1088,7 @@ class PacketBuilder {
         // now for the body
         DataHelper.toLong(data, off, 4, nonce);
         off += 4;
-        DataHelper.toLong(data, off, 1, 0); // neither Bob nor Charlie need Alice's IP from her
-        off++;
+        data[off++] = 0; // neither Bob nor Charlie need Alice's IP from her
         DataHelper.toLong(data, off, 2, 0); // neither Bob nor Charlie need Alice's port from her
         off += 2;
         System.arraycopy(aliceIntroKey.getData(), 0, data, off, SessionKey.KEYSIZE_BYTES);
@@ -1090,16 +1101,32 @@ class PacketBuilder {
         authenticate(packet, toCipherKey, toMACKey);
         setTo(packet, toIP, toPort);
         packet.setMessageType(TYPE_TFA);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
 
     /**
      * Build a packet as if we are either Bob or Charlie and we are helping test Alice.
-     * 
+     * Not for use as Bob, as of 0.9.52; use in-session cipher/mac keys instead.
+     *
      * @return ready to send packet, or null if there was a problem
      */
     public UDPPacket buildPeerTestToAlice(InetAddress aliceIP, int alicePort,
                                           SessionKey aliceIntroKey, SessionKey charlieIntroKey, long nonce) {
+        return buildPeerTestToAlice(aliceIP, alicePort, aliceIntroKey, aliceIntroKey, charlieIntroKey, nonce);
+    }
+
+    /**
+     * Build a packet as if we are either Bob or Charlie and we are helping test Alice.
+     * 
+     * @param aliceCipherKey the intro key if we are Charlie
+     * @param aliceMACKey the intro key if we are Charlie
+     * @return ready to send packet, or null if there was a problem
+     * @since 0.9.52
+     */
+    public UDPPacket buildPeerTestToAlice(InetAddress aliceIP, int alicePort,
+                                          SessionKey aliceCipherKey, SessionKey aliceMACKey,
+                                          SessionKey charlieIntroKey, long nonce) {
         UDPPacket packet = buildPacketHeader(PEER_TEST_FLAG_BYTE);
         DatagramPacket pkt = packet.getPacket();
         byte data[] = pkt.getData();
@@ -1111,8 +1138,7 @@ class PacketBuilder {
         DataHelper.toLong(data, off, 4, nonce);
         off += 4;
         byte ip[] = aliceIP.getAddress();
-        DataHelper.toLong(data, off, 1, ip.length);
-        off++;
+        data[off++] = (byte) ip.length;
         System.arraycopy(ip, 0, data, off, ip.length);
         off += ip.length;
         DataHelper.toLong(data, off, 2, alicePort);
@@ -1124,9 +1150,10 @@ class PacketBuilder {
         off = pad1(data, off);
         off = pad2(data, off);
         pkt.setLength(off);
-        authenticate(packet, aliceIntroKey, aliceIntroKey);
+        authenticate(packet, aliceCipherKey, aliceMACKey);
         setTo(packet, aliceIP, alicePort);
         packet.setMessageType(TYPE_TTA);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
 
@@ -1149,8 +1176,7 @@ class PacketBuilder {
         DataHelper.toLong(data, off, 4, nonce);
         off += 4;
         byte ip[] = aliceIP.getAddress();
-        DataHelper.toLong(data, off, 1, ip.length);
-        off++;
+        data[off++] = (byte) ip.length;
         System.arraycopy(ip, 0, data, off, ip.length);
         off += ip.length;
         DataHelper.toLong(data, off, 2, alicePort);
@@ -1165,6 +1191,7 @@ class PacketBuilder {
         authenticate(packet, charlieCipherKey, charlieMACKey);
         setTo(packet, charlieIP, charliePort);
         packet.setMessageType(TYPE_TBC);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
     
@@ -1187,8 +1214,7 @@ class PacketBuilder {
         DataHelper.toLong(data, off, 4, nonce);
         off += 4;
         byte ip[] = aliceIP.getAddress();
-        DataHelper.toLong(data, off, 1, ip.length);
-        off++;
+        data[off++] = (byte) ip.length;
         System.arraycopy(ip, 0, data, off, ip.length);
         off += ip.length;
         DataHelper.toLong(data, off, 2, alicePort);
@@ -1203,15 +1229,10 @@ class PacketBuilder {
         authenticate(packet, bobCipherKey, bobMACKey);
         setTo(packet, bobIP, bobPort);
         packet.setMessageType(TYPE_TCB);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
 
-    // specify these if we know what our external receive ip/port is and if its different
-    // from what bob is going to think
-    // FIXME IPv4 addr must be specified when sent over IPv6
-    private byte[] getOurExplicitIP() { return null; }
-    private int getOurExplicitPort() { return 0; }
-    
     /**
      *  build intro packets for each of the published introducers
      *
@@ -1231,17 +1252,34 @@ class PacketBuilder {
             long tag = addr.getIntroducerTag(i);
             long exp = addr.getIntroducerExpiration(i);
             // let's not use an introducer on a privileged port, sounds like trouble
-            if (ikey == null ||
-                iaddr == null || tag <= 0 ||
-                // we must use the same isValid() as EstablishmentManager.receiveRelayResponse().
-                // If an introducer isn't valid, we shouldn't send to it
-                !emgr.isValid(iaddr.getAddress(), iport) ||
-                (exp > 0 && exp < cutoff) ||
+            if (iaddr == null) {
+                // normal, SSU2 slots
+                if (_log.shouldDebug())
+                    _log.debug("Cannot build a relay request for " + state.getRemoteIdentity().calculateHash()
+                               + " slot " + i + " no address");
+                continue;
+            }
+            if (ikey == null || tag <= 0) {
+                if (_log.shouldWarn())
+                    _log.warn("Cannot build a relay request for " + state.getRemoteIdentity().calculateHash()
+                               + " slot " + i + " no key/tag");
+                continue;
+            }
+            if  (exp > 0 && exp < cutoff) {
+                if (_log.shouldWarn())
+                    _log.warn("Cannot build a relay request for " + state.getRemoteIdentity().calculateHash()
+                               + ", expired " + DataHelper.formatTime(exp)
+                               + " : " + Addresses.toString(iaddr.getAddress(), iport));
+                continue;
+            }
+            // we must use the same isValid() as EstablishmentManager.receiveRelayResponse().
+            // If an introducer isn't valid, we shouldn't send to it
+            if (!emgr.isValid(iaddr.getAddress(), iport) ||
                 // FIXME this will have already failed in isValid() above, right?
                 (Arrays.equals(iaddr.getAddress(), _transport.getExternalIP()) && !_transport.allowLocal())) {
-                if (_log.shouldLog(Log.WARN))
+                if (_log.shouldWarn())
                     _log.warn("Cannot build a relay request for " + state.getRemoteIdentity().calculateHash()
-                               + ", as the introducer address is invalid: " + iaddr + ':' + iport);
+                               + ", introducer address is invalid or blocklisted: " + Addresses.toString(iaddr.getAddress(), iport));
                 // TODO implement some sort of introducer banlist
                 continue;
             }
@@ -1259,6 +1297,13 @@ class PacketBuilder {
                 bobState = transport.getPeerState(rhid);
             }
             if (bobState != null) {
+                if (bobState.getVersion() > 1) {
+                    // TODO cross-version relaying, maybe
+                    if (_log.shouldWarn())
+                        _log.warn("Cannot build SSU1 relay request for " + state.getRemoteIdentity().calculateHash()
+                                   + " through SSU2-connected introducer " + bobState);
+                    continue;
+                }
                 // established session (since 0.9.12)
                 cipherKey = bobState.getCurrentCipherKey();
                 macKey = bobState.getCurrentMACKey();
@@ -1268,22 +1313,24 @@ class PacketBuilder {
                 cipherKey = new SessionKey(ikey);
                 macKey = cipherKey;
                 if (_log.shouldLog(Log.INFO))
-                    _log.info("Sending relay request (w/ intro key) to " + iaddr + ":" + iport);
+                    _log.info("Sending relay request (w/ intro key) to " + Addresses.toString(iaddr.getAddress(), iport));
             } else {
                 if (_log.shouldLog(Log.INFO))
-                    _log.info("Sending relay request (in-session) to " + iaddr + ":" + iport);
+                    _log.info("Sending relay request (in-session) to " + Addresses.toString(iaddr.getAddress(), iport));
             }
 
-            rv.add(buildRelayRequest(iaddr, iport, cipherKey, macKey, tag,
-                                     ourIntroKey, state.getIntroNonce()));
+            UDPPacket pkt = buildRelayRequest(iaddr, iport, cipherKey, macKey, tag, ourIntroKey, state.getIntroNonce());
+            if (pkt != null)
+                rv.add(pkt);
+            else if (_log.shouldWarn())
+                _log.warn("Cannot build a relay request for " + state.getRemoteIdentity().calculateHash()
+                          + ", as we don't have an address to send to: " + Addresses.toString(iaddr.getAddress(), iport));
         }
         return rv;
     }
     
     /**
-     *  TODO Alice IP/port in packet will always be null/0, must be fixed to
-     *  send a RelayRequest over IPv6
-     *
+     *  @return null on failure
      */
     private UDPPacket buildRelayRequest(InetAddress introHost, int introPort,
                                         SessionKey cipherKey, SessionKey macKey,
@@ -1293,30 +1340,48 @@ class PacketBuilder {
         byte data[] = pkt.getData();
         int off = HEADER_SIZE;
         
-        // FIXME must specify these if request is going over IPv6
-        byte ourIP[] = getOurExplicitIP();
-        int ourPort = getOurExplicitPort();
+        // Must specify these if request is going over IPv6 for v4 or vice versa
+        byte ourIP[];
+        int ourPort;
+        if (introHost instanceof Inet6Address) {
+            RouterAddress ra = _transport.getCurrentExternalAddress(true);
+            if (ra == null) {
+                ra = _transport.getCurrentExternalAddress(false);
+                if (ra == null)
+                    return null;
+            }
+            byte[] ip = ra.getIP();
+            if (ip == null)
+                return null;
+            if (ip.length != 16) {
+                ourIP = ip;
+                ourPort = _transport.getRequestedPort();
+            } else {
+                ourIP = null;
+                ourPort = 0;
+            }
+        } else {
+            // TODO IPv4 introducer, IPv6 introduction
+            ourIP = null;
+            ourPort = 0;
+        }
         
         // now for the body
         DataHelper.toLong(data, off, 4, introTag);
         off += 4;
         if (ourIP != null) {
-            DataHelper.toLong(data, off, 1, ourIP.length);
-            off++;
+            data[off++] = (byte) ourIP.length;
             System.arraycopy(ourIP, 0, data, off, ourIP.length);
             off += ourIP.length;
         } else {
-            DataHelper.toLong(data, off, 1, 0);
-            off++;
+            data[off++] = 0;
         }
         
         DataHelper.toLong(data, off, 2, ourPort);
         off += 2;
         
         // challenge...
-        DataHelper.toLong(data, off, 1, 0);
-        off++;
-        //off += 0; // *cough*
+        data[off++] = 0;
         
         System.arraycopy(ourIntroKey.getData(), 0, data, off, SessionKey.KEYSIZE_BYTES);
         off += SessionKey.KEYSIZE_BYTES;
@@ -1336,6 +1401,7 @@ class PacketBuilder {
         authenticate(packet, cipherKey, macKey);
         setTo(packet, introHost, introPort);
         packet.setMessageType(TYPE_RREQ);
+        packet.setPriority(PRIORITY_HIGH);
         return packet;
     }
 
@@ -1349,18 +1415,16 @@ class PacketBuilder {
         
         // now for the body
         byte ip[] = alice.getIP();
-        DataHelper.toLong(data, off, 1, ip.length);
-        off++;
+        data[off++] = (byte) ip.length;
         System.arraycopy(ip, 0, data, off, ip.length);
         off += ip.length;
         DataHelper.toLong(data, off, 2, alice.getPort());
         off += 2;
         
         int sz = request.readChallengeSize();
-        DataHelper.toLong(data, off, 1, sz);
-        off++;
+        data[off++] = (byte) sz;
         if (sz > 0) {
-            request.readChallengeSize(data, off);
+            request.readChallengeData(data, off);
             off += sz;
         }
         
@@ -1371,6 +1435,7 @@ class PacketBuilder {
         authenticate(packet, charlie.getCurrentCipherKey(), charlie.getCurrentMACKey());
         setTo(packet, charlie.getRemoteIPAddress(), charlie.getRemotePort());
         packet.setMessageType(TYPE_INTRO);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
     
@@ -1393,8 +1458,7 @@ class PacketBuilder {
 
         // now for the body
         byte charlieIP[] = charlie.getRemoteIP();
-        DataHelper.toLong(data, off, 1, charlieIP.length);
-        off++;
+        data[off++] = (byte) charlieIP.length;
         System.arraycopy(charlieIP, 0, data, off, charlieIP.length);
         off += charlieIP.length;
         DataHelper.toLong(data, off, 2, charlie.getRemotePort());
@@ -1402,8 +1466,7 @@ class PacketBuilder {
         
         // Alice IP/Port currently ignored on receive - see UDPPacketReader
         byte aliceIP[] = alice.getIP();
-        DataHelper.toLong(data, off, 1, aliceIP.length);
-        off++;
+        data[off++] = (byte) aliceIP.length;
         System.arraycopy(aliceIP, 0, data, off, aliceIP.length);
         off += aliceIP.length;
         DataHelper.toLong(data, off, 2, alice.getPort());
@@ -1419,6 +1482,7 @@ class PacketBuilder {
         authenticate(packet, cipherKey, macKey);
         setTo(packet, aliceAddr, alice.getPort());
         packet.setMessageType(TYPE_RESP);
+        packet.setPriority(PRIORITY_LOW);
         return packet;
     }
     
@@ -1437,6 +1501,7 @@ class PacketBuilder {
         setTo(packet, to, port);
         
         packet.setMessageType(TYPE_PUNCH);
+        packet.setPriority(PRIORITY_HIGH);
         return packet;
     }
     
