@@ -12,7 +12,7 @@ import net.i2p.data.DataHelper;
 import net.i2p.data.SessionKey;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.FIFOBandwidthLimiter;
-import net.i2p.router.util.CDQEntry;
+import net.i2p.router.util.CDPQEntry;
 import net.i2p.util.TryCache;
 import net.i2p.util.Addresses;
 import net.i2p.util.Log;
@@ -23,10 +23,10 @@ import net.i2p.util.SystemVersion;
  * of object instances to allow rapid reuse.
  *
  */
-class UDPPacket implements CDQEntry {
+class UDPPacket implements CDPQEntry {
     private RouterContext _context;
     private final DatagramPacket _packet;
-    private volatile short _priority;
+    private int _priority;
     private volatile long _initializeTime;
     //private volatile long _expiration;
     private final byte[] _data;
@@ -46,6 +46,7 @@ class UDPPacket implements CDQEntry {
     private int _validateCount;
     // private boolean _isInbound;
     private FIFOBandwidthLimiter.Request _bandwidthRequest;
+    private long _seqNum;
   
     private static class PacketFactory implements TryCache.ObjectFactory<UDPPacket> {
         static RouterContext context;
@@ -168,6 +169,20 @@ class UDPPacket implements CDQEntry {
         _receivedTime = 0;
         _fragmentCount = 0;
     }
+
+    /**
+     *  CDPQEntry
+     *  @since 0.9.53
+     */
+    public void setSeqNum(long num) { _seqNum = num; }
+
+    /**
+     *  CDPQEntry
+     *  @since 0.9.53
+     */
+    public long getSeqNum() { return _seqNum; }
+
+
     
   /****
     public void writeData(byte src[], int offset, int len) { 
@@ -180,7 +195,13 @@ class UDPPacket implements CDQEntry {
 
     /** */
     public synchronized DatagramPacket getPacket() { verifyNotReleased(); return _packet; }
-    public synchronized short getPriority() { verifyNotReleased(); return _priority; }
+    public int getPriority() { return _priority; }
+
+    /**
+     *  @since 0.9.53
+     */
+    public void setPriority(int pri) { _priority = pri; }
+
     //public long getExpiration() { verifyNotReleased(); return _expiration; }
     public synchronized long getBegin() { verifyNotReleased(); return _initializeTime; }
     public long getLifetime() { /** verifyNotReleased(); */ return _context.clock().now() - _initializeTime; }
@@ -295,6 +316,9 @@ class UDPPacket implements CDQEntry {
      * Decrypt this valid packet, overwriting the _data buffer's payload
      * with the decrypted data (leaving the MAC and IV unaltered)
      * 
+     * SSU 1 only.
+     * SSU 2 decryption is in PacketHandler.
+     * 
      */
     public synchronized void decrypt(SessionKey cipherKey) {
         verifyNotReleased(); 
@@ -388,12 +412,16 @@ class UDPPacket implements CDQEntry {
     
     @Override
     public String toString() {
-        verifyNotReleased(); 
+        synchronized(this) {
+            if (_released)
+                return "RELEASED PACKET";
+        }
         StringBuilder buf = new StringBuilder(256);
         buf.append(_packet.getLength());
         buf.append(" byte pkt with ");
         buf.append(Addresses.toString(_packet.getAddress().getAddress(), _packet.getPort()));
         //buf.append(" id=").append(System.identityHashCode(this));
+        buf.append(" priority=").append(_priority);
         if (_messageType >= 0)
             buf.append(" msgType=").append(_messageType);
         if (_markedType >= 0)
