@@ -13,6 +13,7 @@ import net.i2p.data.i2np.DeliveryInstructions;
 import net.i2p.data.i2np.DeliveryStatusMessage;
 import net.i2p.data.i2np.GarlicMessage;
 import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.data.i2np.OutboundTunnelBuildReplyMessage;
 import net.i2p.data.i2np.TunnelBuildReplyMessage;
 import net.i2p.data.i2np.VariableTunnelBuildReplyMessage;
 import net.i2p.router.ClientMessage;
@@ -22,6 +23,7 @@ import net.i2p.router.TunnelPoolSettings;
 import net.i2p.router.message.GarlicMessageReceiver;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import net.i2p.util.Log;
+import net.i2p.util.RandomSource;
 
 /**
  * When a message arrives at the inbound tunnel endpoint, this distributor
@@ -32,7 +34,8 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
     private final Log _log;
     private final Hash _client;
     private final GarlicMessageReceiver _receiver;
-    
+    private final String _clientNickname;
+    private final long _msgIDBloomXor;
     /**
      *  @param client null for router tunnel
      */
@@ -42,6 +45,23 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
         _log = ctx.logManager().getLog(InboundMessageDistributor.class);
         _receiver = new GarlicMessageReceiver(ctx, this, client);
         // all createRateStat in TunnelDispatcher
+        if (_client != null) {
+            TunnelPoolSettings clienttps = _context.tunnelManager().getInboundSettings(_client);
+            String nickname = clienttps != null ? clienttps.getDestinationNickname() : "UNKNOWN";
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("Initializing client (nickname: "
+                           + nickname
+                           + " b32: " + _client.toBase32()
+                           + ") InboundMessageDistributor with tunnel pool settings: " + clienttps);
+            _clientNickname = nickname;
+            _msgIDBloomXor = clienttps.getMsgIdBloomXor();
+        } else {
+            _clientNickname = "NULL/Expl";
+            _msgIDBloomXor = RandomSource.getInstance().nextLong(I2NPMessage.MAX_ID_VALUE);
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug("Initializing null or exploratory InboundMessageDistributor");
+        }
+
     }
     
     public void distribute(I2NPMessage msg, Hash target) {
@@ -50,7 +70,9 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
 
     public void distribute(I2NPMessage msg, Hash target, TunnelId tunnel) {
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("IBMD for " + _client + " to " + target + " / " + tunnel + " : " + msg);
+            _log.debug("IBMD for " + _clientNickname + " ("
+                       + ((_client != null) ? _client.toBase32() : "null")
+                       + ") to " + target + " / " + tunnel + " : " + msg);
 
         // allow messages on client tunnels even after client disconnection, as it may
         // include e.g. test messages, etc.  DataMessages will be dropped anyway
@@ -97,7 +119,8 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                         // We handle this safely, so we don't ask him again.
                         // Todo: if peer was ff and RI is not ff, queue for exploration in netdb (but that isn't part of the facade now)
                         if (_log.shouldLog(Log.WARN))
-                            _log.warn("Dropping DSM down a tunnel for " + _client.toBase32() + ": " + msg);
+                            _log.warn("Inbound DSM received down a tunnel for " + _clientNickname
+                                      + " (" + _client.toBase32() + "): " + msg);
                         // Handle safely by just updating the caps table, after doing basic validation
                         Hash key = dsm.getKey();
                         if (_context.routerHash().equals(key))
@@ -131,6 +154,7 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
 
                 case DeliveryStatusMessage.MESSAGE_TYPE:
                 case GarlicMessage.MESSAGE_TYPE:
+                case OutboundTunnelBuildReplyMessage.MESSAGE_TYPE:
                 case TunnelBuildReplyMessage.MESSAGE_TYPE:
                 case VariableTunnelBuildReplyMessage.MESSAGE_TYPE:
                     // these are safe, handled below
@@ -161,6 +185,7 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                 case DatabaseSearchReplyMessage.MESSAGE_TYPE:
                 case DeliveryStatusMessage.MESSAGE_TYPE:
                 case GarlicMessage.MESSAGE_TYPE:
+                case OutboundTunnelBuildReplyMessage.MESSAGE_TYPE:
                 case TunnelBuildReplyMessage.MESSAGE_TYPE:
                 case VariableTunnelBuildReplyMessage.MESSAGE_TYPE:
                     // these are safe, handled below
@@ -266,9 +291,10 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                                     // ... and inject it.
                                     ((LeaseSet)dsm.getEntry()).setReceivedBy(_client);
                                     if (_log.shouldLog(Log.INFO))
-                                        _log.info("Storing garlic LS down tunnel for: " + dsm.getKey() + " sent to: " +
-                                                  (_client != null ? _client.toBase32() : "router"));
-                                    _context.inNetMessagePool().add(dsm, null, null);
+                                        _log.info("Storing garlic LS down tunnel for: " + dsm.getKey() + " sent to: "
+                                                  + _clientNickname + " ("
+                                                  + (_client != null ? _client.toBase32() : ") router"));
+                                    _context.inNetMessagePool().add(dsm, null, null, _msgIDBloomXor);
                             } else {                                        
                                 if (_client != null) {
                                     // drop it, since the data we receive shouldn't include router 
@@ -276,7 +302,8 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                                     // open an attack vector)
                                     _context.statManager().addRateData("tunnel.dropDangerousClientTunnelMessage", 1, 
                                                                        DatabaseStoreMessage.MESSAGE_TYPE);
-                                    _log.error("Dropped dangerous message down a tunnel for " + _client.toBase32() + ": " + dsm, new Exception("cause"));
+                                    _log.error("Dropped dangerous message down a tunnel for " + _clientNickname
+                                               + " ("+ _client.toBase32() + ") : " + dsm, new Exception("cause"));
                                     return;
                                 }
                                 // Case 3:
@@ -285,11 +312,11 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                                 // We must send to the InNetMessagePool so the message can be matched
                                 // and the search marked as successful.
                                 // note that encrypted replies to RI lookups is currently disables in ISJ, we won't get here.
-
                                 // ... and inject it.
                                 if (_log.shouldLog(Log.INFO))
-                                    _log.info("Storing garlic RI down tunnel for: " + dsm.getKey());
-                                _context.inNetMessagePool().add(dsm, null, null);
+                                    _log.info("Storing garlic RI down tunnel (" + _clientNickname
+                                              + ") for: " + dsm.getKey());
+                                _context.inNetMessagePool().add(dsm, null, null, _msgIDBloomXor);
                             }
                 } else if (_client != null && type == DatabaseSearchReplyMessage.MESSAGE_TYPE) {
                     // DSRMs show up here now that replies are encrypted
@@ -308,21 +335,24 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
                         orig = newMsg;
                      }
                    ****/
-                    _context.inNetMessagePool().add(orig, null, null);
+                    _context.inNetMessagePool().add(orig, null, null, _msgIDBloomXor);
                 } else if (type == DataMessage.MESSAGE_TYPE) {
                         // a data message targetting the local router is how we send load tests (real
                         // data messages target destinations)
                         _context.statManager().addRateData("tunnel.handleLoadClove", 1);
                         data = null;
                         //_context.inNetMessagePool().add(data, null, null);
-                } else if (_client != null && type != DeliveryStatusMessage.MESSAGE_TYPE) {
+                } else if (_client != null && type != DeliveryStatusMessage.MESSAGE_TYPE &&
+                           type != OutboundTunnelBuildReplyMessage.MESSAGE_TYPE) {
                             // drop it, since the data we receive shouldn't include other stuff, 
                             // as that might open an attack vector
                             _context.statManager().addRateData("tunnel.dropDangerousClientTunnelMessage", 1, 
                                                                data.getType());
-                            _log.error("Dropped dangerous message down a tunnel for " + _client.toBase32() + ": " + data, new Exception("cause"));
+                            _log.error("Dropped dangerous message received down a tunnel for "
+                                       + _clientNickname + " (" + _client.toBase32() + ") : "
+                                       + data, new Exception("cause"));
                 } else {
-                            _context.inNetMessagePool().add(data, null, null);
+                        _context.inNetMessagePool().add(data, null, null, _msgIDBloomXor);
                 }
                 return;
 
@@ -366,9 +396,14 @@ class InboundMessageDistributor implements GarlicMessageReceiver.CloveReceiver {
 
             case DeliveryInstructions.DELIVERY_MODE_ROUTER: // fall through
             case DeliveryInstructions.DELIVERY_MODE_TUNNEL:
+                // Targeted messages are usually dropped, but it is safe to
+                // allow distribute() to evaluate the message.
                 if (_log.shouldLog(Log.INFO))
-                    _log.info("clove targetted " + instructions.getRouter() + ":" + instructions.getTunnelId()
-                               + ", treat recursively to prevent leakage");
+                    _log.info("Recursively handling message from targeted clove (for client:"
+                              + _clientNickname + " " + ((_client != null) ? _client.toBase32() : "null")
+                              + ", msg type: " + data.getClass().getSimpleName() + "): " + instructions.getRouter()
+                              + ":" + instructions.getTunnelId() + " msg: " + data);
+
                 distribute(data, instructions.getRouter(), instructions.getTunnelId());
                 return;
 
