@@ -11,6 +11,8 @@ import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.TunnelId;
 import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.data.i2np.OutboundTunnelBuildReplyMessage;
+import net.i2p.data.i2np.ShortTunnelBuildMessage;
 import net.i2p.data.i2np.TunnelBuildMessage;
 import net.i2p.data.i2np.TunnelBuildReplyMessage;
 import net.i2p.data.i2np.TunnelDataMessage;
@@ -20,8 +22,10 @@ import net.i2p.data.i2np.VariableTunnelBuildReplyMessage;
 import net.i2p.router.JobImpl;
 import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
+import net.i2p.router.RouterThrottleImpl;
 import net.i2p.router.Service;
 import net.i2p.router.peermanager.PeerProfile;
+import net.i2p.router.tunnel.pool.PooledTunnelCreatorConfig;
 import net.i2p.util.Log;
 
 /**
@@ -170,14 +174,14 @@ public class TunnelDispatcher implements Service {
         // estimate for RouterThrottleImpl
         ctx.statManager().createRequiredRateStat("tunnel.participatingMessageCountAvgPerTunnel", 
                                          "Estimate of participating messages per tunnel lifetime", "Tunnels", 
-                                         new long[] { 60*1000l });
+                                         new long[] { 60*1000, 20*60*1000 });
         ctx.statManager().createRateStat("tunnel.ownedMessageCount", 
                                          "How many messages are sent through a tunnel we created (period == failures)?", "Tunnels", 
                                          new long[] { 60*1000l, 10*60*1000l, 60*60*1000l });
         ctx.statManager().createRateStat("tunnel.failedCompletelyMessages", 
                                          "How many messages are sent through a tunnel that failed prematurely (period == failures)?", "Tunnels", 
                                          new long[] { 60*1000l, 10*60*1000l, 60*60*1000l });
-        ctx.statManager().createRateStat("tunnel.failedPartially", 
+        ctx.statManager().createRateStat("tunnel.failedPartiallyMessages",
                                          "How many messages are sent through a tunnel that only failed partially (period == failures)?", "Tunnels", 
                                          new long[] { 60*1000l, 10*60*1000l, 60*60*1000l });
         // following are for BatchedPreprocessor
@@ -222,6 +226,10 @@ public class TunnelDispatcher implements Service {
         ctx.statManager().createRateStat("tunnel.handleLoadClove", "When do we receive load test cloves", "Tunnels", new long[] { 60*60*1000 });
         // following is for PumpedTunnelGateway
         ctx.statManager().createRateStat("tunnel.dropGatewayOverflow", "Dropped message at GW, queue full", "Tunnels", new long[] { 60*60*1000 });
+        ctx.statManager().createRateStat("tunnel.inboundI2NPGarlicRIDSM", "IBMD Storing Garlic RI DSM",
+                                         "Tunnels", new long[] { 10*60*1000, 60*60*1000 });
+        ctx.statManager().createRateStat("tunnel.outboundTunnelEndpointFwdRIDSM", "OBTE Forwarding RI DSM",
+                                         "Tunnels", new long[] { 10*60*1000, 60*60*1000 });
     }
 
     /** for IBGW */
@@ -245,7 +253,7 @@ public class TunnelDispatcher implements Service {
      *
      *  @return success; false if Tunnel ID is a duplicate
      */
-    public boolean joinOutbound(TunnelCreatorConfig cfg) {
+    public boolean joinOutbound(PooledTunnelCreatorConfig cfg) {
         if (_log.shouldLog(Log.INFO))
             _log.info("Outbound built successfully: " + cfg);
         TunnelGateway gw;
@@ -571,39 +579,48 @@ public class TunnelDispatcher implements Service {
      */
     public void dispatch(TunnelGatewayMessage msg) {
         long before = _context.clock().now();
-        TunnelGateway gw = _inboundGateways.get(msg.getTunnelId());
+        TunnelId id = msg.getTunnelId();
+        TunnelGateway gw = _inboundGateways.get(id);
+        I2NPMessage submsg = msg.getMessage();
+        // The contained message is nulled out when written
+        if (submsg == null)
+            throw new IllegalArgumentException("TGM message is null");
         if (gw != null) {
             if (_log.shouldLog(Log.DEBUG))
                 _log.debug("dispatch where we are the inbound gateway: " + gw + ": " + msg);
             long minTime = before - Router.CLOCK_FUDGE_FACTOR;
             long maxTime = before + MAX_FUTURE_EXPIRATION;
-            if ( (msg.getMessageExpiration() < minTime) || (msg.getMessage().getMessageExpiration() < minTime) ||
-                 (msg.getMessageExpiration() > maxTime) || (msg.getMessage().getMessageExpiration() > maxTime) ) {
+            long exp = msg.getMessageExpiration();
+            long subexp = submsg.getMessageExpiration();
+            if (exp < minTime || subexp < minTime ||
+                exp > maxTime || subexp > maxTime) {
                 if (_log.shouldLog(Log.WARN))
-                    _log.warn("Not dispatching a gateway message for tunnel " + msg.getTunnelId().getTunnelId()
-                               + " as the wrapper's expiration is in " + DataHelper.formatDuration(msg.getMessageExpiration()-before)
-                               + " and/or the content's expiration is in " + DataHelper.formatDuration(msg.getMessage().getMessageExpiration()-before)
-                               + " with messageId " + msg.getUniqueId() + "/" + msg.getMessage().getUniqueId() + " and message type "
-                               + msg.getMessage().getClass().getSimpleName());
+                    _log.warn("Not dispatching a gateway message for tunnel " + id.getTunnelId()
+                               + " as the wrapper's expiration is in " + DataHelper.formatDuration(exp - before)
+                               + " and/or the content's expiration is in " + DataHelper.formatDuration(subexp - before)
+                               + " with messageId " + id + "/" + submsg.getUniqueId()
+                               + " messageType: " + submsg.getClass().getSimpleName());
                 return;
             }
-            //_context.messageHistory().tunnelDispatched("message " + msg.getUniqueId() + "/" + msg.getMessage().getUniqueId() + " on tunnel " 
+            //_context.messageHistory().tunnelDispatched("message " + msg.getRawUniqueId() + "/" + msg.getMessage().getRawUniqueId() + " on tunnel " 
             //                                               + msg.getTunnelId().getTunnelId() + " as inbound gateway");
-            _context.messageHistory().tunnelDispatched(msg.getUniqueId(), msg.getMessage().getUniqueId(), msg.getTunnelId().getTunnelId(), "inbound gateway");
+            _context.messageHistory().tunnelDispatched(msg.getUniqueId(),
+                                                       submsg.getUniqueId(),
+                                                       id.getTunnelId(), "inbound gateway");
             gw.add(msg);
             _context.statManager().addRateData("tunnel.dispatchInbound", 1);
         } else {
-            _context.messageHistory().droppedTunnelGatewayMessageUnknown(msg.getUniqueId(), msg.getTunnelId().getTunnelId());
+            _context.messageHistory().droppedTunnelGatewayMessageUnknown(msg.getUniqueId(), id.getTunnelId());
             int level = (_context.router().getUptime() > 10*60*1000 ? Log.WARN : Log.INFO);
             if (_log.shouldLog(level))
-                _log.log(level, "no matching tunnel for id=" + msg.getTunnelId().getTunnelId() 
+                _log.log(level, "no matching tunnel for id=" + id.getTunnelId() 
                            + ": gateway message expiring in " 
-                           + DataHelper.formatDuration(msg.getMessageExpiration()-_context.clock().now())
+                           + DataHelper.formatDuration(msg.getMessageExpiration() - before)
                            + "/" 
-                           + DataHelper.formatDuration(msg.getMessage().getMessageExpiration()-_context.clock().now())
-                           + " messageId " + msg.getUniqueId()
+                           + DataHelper.formatDuration(submsg.getMessageExpiration() - before)
+                           + " messageId " + id
                            + "/" + msg.getMessage().getUniqueId()
-                           + " messageType: " + msg.getMessage().getClass().getSimpleName()
+                           + " messageType: " + submsg.getClass().getSimpleName()
                            + " existing = " + _inboundGateways.size());
         }
         
@@ -767,84 +784,32 @@ public class TunnelDispatcher implements Service {
     public boolean shouldDropParticipatingMessage(Location loc, int type, int length) {
         if (length <= 0)
             return false;
-     /****
-        Don't use the tunnel.participatingBandwidth stat any more. It could be up to 3 minutes old.
-        Also, it counts inbound bandwidth, i.e. before dropping, which resulted in too many drops
-        during a burst.
-        We now use the bandwidth limiter to track outbound participating bandwidth
-        over the last few seconds.
-     ****/
 
-     /****
-        RateStat rs = _context.statManager().getRate("tunnel.participatingBandwidth");
-        if (rs == null)
-            return false;
-        Rate r = rs.getRate(60*1000);
-        if (r == null)
-            return false;
-        // weight current period higher
-        long count = r.getLastEventCount() + (3 * r.getCurrentEventCount());
-        int bw = 0;
-        if (count > 0)
-            bw = (int) ((r.getLastTotalValue() + (3 * r.getCurrentTotalValue())) / count);
-        else
-            bw = (int) r.getLifetimeAverageValue();
-
-        int usedIn = Math.min(_context.router().get1sRateIn(), _context.router().get15sRateIn());
-        if (bw < usedIn)
-            usedIn = bw;
-        if (usedIn <= 0)
-            return false;
-        int usedOut = Math.min(_context.router().get1sRate(true), _context.router().get15sRate(true));
-        if (bw < usedOut)
-            usedOut = bw;
-        if (usedOut <= 0)
-            return false;
-        int used = Math.min(usedIn, usedOut);
-     ****/
-        int used = _context.bandwidthLimiter().getCurrentParticipatingBandwidth();
-        if (used <= 0)
-            return false;
-
-        int maxKBps = Math.min(_context.bandwidthLimiter().getInboundKBytesPerSecond(),
-                               _context.bandwidthLimiter().getOutboundKBytesPerSecond());
-        float share = (float) _context.router().getSharePercentage();
-
-        // start dropping at 120% of the limit,
-        // as we rely on Throttle for long-term bandwidth control by rejecting tunnels
-        float maxBps = maxKBps * share * (1024f * 1.20f);
-        float pctDrop = (used - maxBps) / used;
-        if (pctDrop <= 0)
-            return false;
         // increase the drop probability for OBEP,
         // (except lower it for tunnel build messages type 21/22/23/24),
         // and lower it for IBGW, for network efficiency
-        double len = length;
+        float factor;
         if (loc == Location.OBEP) {
             // we don't need to check for VTBRM/TBRM as that happens at tunnel creation
-            if (type == VariableTunnelBuildMessage.MESSAGE_TYPE || type == TunnelBuildMessage.MESSAGE_TYPE)
-                len /= 1.5;
+            if (type == VariableTunnelBuildMessage.MESSAGE_TYPE || type == TunnelBuildMessage.MESSAGE_TYPE ||
+                type == ShortTunnelBuildMessage.MESSAGE_TYPE)
+                factor = 1 / 1.5f;
             else
-                len *= 1.5;
+                factor = 1.5f;
         } else if (loc == Location.IBGW) {
             // we don't need to check for VTBM/TBM as that happens at tunnel creation
-            if (type == VariableTunnelBuildReplyMessage.MESSAGE_TYPE || type == TunnelBuildReplyMessage.MESSAGE_TYPE)
-                len /= 1.5 * 1.5 * 1.5;
+            if (type == VariableTunnelBuildReplyMessage.MESSAGE_TYPE || type == TunnelBuildReplyMessage.MESSAGE_TYPE ||
+                type == OutboundTunnelBuildReplyMessage.MESSAGE_TYPE)
+                factor = 1 / (1.5f * 1.5f * 1.5f);
             else
-                len /= 1.5;
+                factor = 1 / 1.5f;
+        } else {
+            factor = 1.0f;
         }
-        // drop in proportion to size w.r.t. a standard 1024-byte message
-        // this is a little expensive but we want to adjust the curve between 0 and 1
-        // Most messages are 1024, only at the OBEP do we see other sizes
-        if ((int)len != 1024)
-            pctDrop = (float) Math.pow(pctDrop, 1024d / len);
-        float rand = _context.random().nextFloat();
-        boolean reject = rand <= pctDrop;
+        boolean reject = ! _context.bandwidthLimiter().sentParticipatingMessage(length, factor);
         if (reject) {
             if (_log.shouldLog(Log.WARN)) {
-                int availBps = (int) (((maxKBps*1024)*share) - used);
-                _log.warn("Drop part. msg. avail/max/used " + availBps + "/" + (int) maxBps + "/" 
-                          + used + " %Drop = " + pctDrop
+                _log.warn("Drop part. msg. factor=" + factor
                           + ' ' + loc + ' ' + type + ' ' + length);
             }
             _context.statManager().addRateData("tunnel.participatingMessageDropped", 1);
@@ -967,7 +932,8 @@ public class TunnelDispatcher implements Service {
             super(ctx);
             _configs = new LinkedBlockingQueue<HopConfig>();
             // 10 min no tunnels accepted + 10 min tunnel expiration
-            getTiming().setStartAfter(ctx.clock().now() + 20*60*1000);
+            long rejectStartupTime = ctx.getProperty(RouterThrottleImpl.PROP_REJECT_STARTUP_TIME, RouterThrottleImpl.DEFAULT_REJECT_STARTUP_TIME);
+            getTiming().setStartAfter(ctx.clock().now() + rejectStartupTime + 10*60*1000);
             getContext().jobQueue().addJob(LeaveTunnel.this);
         }
         
@@ -987,7 +953,7 @@ public class TunnelDispatcher implements Service {
             long now = getContext().clock().now() + LEAVE_BATCH_TIME; // leave all expiring in next 10 sec
             long nextTime = now + 10*60*1000;
             while ((cur = _configs.peek()) != null) {
-                long exp = cur.getExpiration() + (2 * Router.CLOCK_FUDGE_FACTOR) + LEAVE_BATCH_TIME;
+                long exp = cur.getExpiration() + (3 * Router.CLOCK_FUDGE_FACTOR / 2) + LEAVE_BATCH_TIME;
                 if (exp < now) {
                     _configs.poll();
                     if (_log.shouldLog(Log.INFO))
