@@ -85,10 +85,11 @@ class TunnelParticipant {
     
     public void dispatch(TunnelDataMessage msg, Hash recvFrom) {
         boolean ok = false;
+        byte[] data = msg.getData();
         if (_processor != null)
-            ok = _processor.process(msg.getData(), 0, msg.getData().length, recvFrom);
+            ok = _processor.process(data, 0, data.length, recvFrom);
         else if (_inboundEndpointProcessor != null) 
-            ok = _inboundEndpointProcessor.retrievePreprocessedData(msg.getData(), 0, msg.getData().length, recvFrom);
+            ok = _inboundEndpointProcessor.retrievePreprocessedData(data, 0, data.length, recvFrom);
         
         if (!ok) {
             if (_log.shouldLog(Log.WARN))
@@ -96,7 +97,6 @@ class TunnelParticipant {
                            + " inboundEndpoint=" + _inboundEndpointProcessor);
             if (_config != null)
                 _config.incrementProcessedMessages();
-            _context.statManager().addRateData("tunnel.corruptMessage", 1, 1);
             return;
         }
         
@@ -122,10 +122,26 @@ class TunnelParticipant {
                                                   new TimeoutJob(_context, msg), MAX_LOOKUP_TIME);
             }
         } else {
-            _inboundEndpointProcessor.getConfig().incrementProcessedMessages();
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Receive fragment: on " + _config + ": " + msg);
-            _handler.receiveTunnelMessage(msg.getData(), 0, msg.getData().length);
+            // IBEP
+            TunnelCreatorConfig cfg = _inboundEndpointProcessor.getConfig();
+            cfg.incrementProcessedMessages();
+            ok = _handler.receiveTunnelMessage(data, 0, data.length);
+            if (ok) {
+                if (_log.shouldLog(Log.DEBUG))
+                    _log.debug("Receive fragment: on " + _config + ": " + msg);
+            } else {
+                // blame everybody equally
+                int lenm1 = cfg.getLength() - 1;
+                if (lenm1 > 0) {
+                    int pct = 100 / (lenm1);
+                    for (int i = 0; i < lenm1; i++) {
+                        Hash h = cfg.getPeer(i);
+                        if (_log.shouldLog(Log.WARN))
+                            _log.warn(toString() + ": Blaming " + h + ' ' + pct + '%');
+                        _context.profileManager().tunnelFailed(h, pct);
+                    }
+                }
+            }
         }
     }
     
@@ -187,7 +203,6 @@ class TunnelParticipant {
                                                                        TunnelDataMessage.MESSAGE_TYPE, 1024))
             return;
         //_config.incrementSentMessages();
-        _context.bandwidthLimiter().sentParticipatingMessage(1024);
         long oldId = msg.getUniqueId();
         long newId = _context.random().nextLong(I2NPMessage.MAX_ID_VALUE);
         _context.messageHistory().wrap("TunnelDataMessage", oldId, "TunnelDataMessage", newId);
