@@ -20,6 +20,7 @@ import net.i2p.stat.Rate;
 import net.i2p.stat.RateStat;
 import net.i2p.stat.StatManager;
 import net.i2p.util.Log;
+import net.i2p.util.SystemVersion;
 
 /**
  * Single threaded controller of the tunnel creation process, spanning all tunnel pools.
@@ -49,6 +50,11 @@ class BuildExecutor implements Runnable {
     /** accept replies up to a minute after we gave up on them */
     private static final long GRACE_PERIOD = 60*1000;
 
+    /**
+     *  @since 0.9.53
+     */
+    enum Result { SUCCESS, REJECT, TIMEOUT, BAD_RESPONSE, DUP_ID, OTHER_FAILURE }
+
     public BuildExecutor(RouterContext ctx, TunnelPoolManager mgr) {
         _context = ctx;
         _log = ctx.logManager().getLog(getClass());
@@ -115,7 +121,7 @@ class BuildExecutor implements Runnable {
         CommSystemFacade csf = _context.commSystem();
         if (csf.getStatus() == Status.DISCONNECTED)
             return 0;
-        if (csf.isDummy() && csf.getEstablished().size() <= 0)
+        if (csf.isDummy() && csf.countActivePeers() <= 0)
             return 0;
         int maxKBps = _context.bandwidthLimiter().getOutboundKBytesPerSecond();
         int allowed = maxKBps / 6; // Max. 1 concurrent build per 6 KB/s outbound
@@ -145,7 +151,8 @@ class BuildExecutor implements Runnable {
         allowed = _context.getProperty("router.tunnelConcurrentBuilds", allowed);
 
         // expire any REALLY old requests
-        long expireBefore = _context.clock().now() + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT - GRACE_PERIOD;
+        long now = _context.clock().now();
+        long expireBefore = now + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT - GRACE_PERIOD;
         for (Iterator<PooledTunnelCreatorConfig> iter = _recentlyBuildingMap.values().iterator(); iter.hasNext(); ) {
             PooledTunnelCreatorConfig cfg = iter.next();
             if (cfg.getExpiration() <= expireBefore) {
@@ -157,7 +164,7 @@ class BuildExecutor implements Runnable {
         List<PooledTunnelCreatorConfig> expired = null;
         int concurrent = 0;
         // Todo: Make expiration variable
-        expireBefore = _context.clock().now() + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT;
+        expireBefore = now + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT;
         for (Iterator<PooledTunnelCreatorConfig> iter = _currentlyBuildingMap.values().iterator(); iter.hasNext(); ) {
             PooledTunnelCreatorConfig cfg = iter.next();
             if (cfg.getExpiration() <= expireBefore) {
@@ -202,7 +209,7 @@ class BuildExecutor implements Runnable {
 
                 TunnelPool pool = cfg.getTunnelPool();
                 if (pool != null)
-                    pool.buildComplete(cfg);
+                    pool.buildComplete(cfg, Result.TIMEOUT);
                 if (cfg.getDestination() == null) {
                     _context.statManager().addRateData("tunnel.buildExploratoryExpire", 1);
                     //if (cfg.isInbound())
@@ -219,7 +226,7 @@ class BuildExecutor implements Runnable {
             }
         }
         
-        _context.statManager().addRateData("tunnel.concurrentBuilds", concurrent, 0);
+        _context.statManager().addRateData("tunnel.concurrentBuilds", concurrent);
         
         long lag = _context.jobQueue().getMaxLag();
         if ( (lag > 2000) && (_context.router().getUptime() > 5*60*1000) ) {
@@ -301,7 +308,7 @@ class BuildExecutor implements Runnable {
 */
 
     /** Set 1.5 * LOOP_TIME < BuildRequestor.REQUEST_TIMEOUT/4 - margin */
-    private static final int LOOP_TIME = 1000;
+    private static final int LOOP_TIME = SystemVersion.isSlow() ? 1000 : 800;
 
     public void run() {
         _isRunning = true;
@@ -409,11 +416,11 @@ class BuildExecutor implements Runnable {
                                     if (_log.shouldLog(Log.DEBUG))
                                         _log.debug("We don't need more fallbacks for " + pool);
                                     i--; //0hop, we can keep going, as there's no worry about throttling
-                                    pool.buildComplete(cfg);
+                                    pool.buildComplete(cfg, Result.OTHER_FAILURE);
                                     continue;
                                 }
                                 long pTime = System.currentTimeMillis() - bef;
-                                _context.statManager().addRateData("tunnel.buildConfigTime", pTime, 0);
+                                _context.statManager().addRateData("tunnel.buildConfigTime", pTime);
                                 if (_log.shouldLog(Log.DEBUG))
                                     _log.debug("Configuring new tunnel " + i + " for " + pool + ": " + cfg);
                                 buildTunnel(cfg);
@@ -537,7 +544,7 @@ class BuildExecutor implements Runnable {
             return;
         if (cfg.getLength() > 1) {
             long buildTime = System.currentTimeMillis() - beforeBuild;
-            _context.statManager().addRateData("tunnel.buildRequestTime", buildTime, 0);
+            _context.statManager().addRateData("tunnel.buildRequestTime", buildTime);
         }
         long id = cfg.getReplyMessageId();
         if (id > 0) {
@@ -553,19 +560,25 @@ class BuildExecutor implements Runnable {
     }
     
     /**
-     *  This wakes up the executor, so call this after TunnelPool.addTunnel()
-     *  so we don't build too many.
+     *  This calls TunnelPool.buildComplete which calls TunnelPool.addTunnel()
+     *  on success, and then we wake up the executor.
+     *
+     *  On success, this also calls TunnelPoolManager to optionally start a test job,
+     *  and queues an ExpireJob.
+     *
+     *  @since 0.9.53 added result parameter
      */
-    public void buildComplete(PooledTunnelCreatorConfig cfg) {
+    public void buildComplete(PooledTunnelCreatorConfig cfg, Result result) {
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Build complete for " + cfg, new Exception());
-        cfg.getTunnelPool().buildComplete(cfg);
+            _log.debug("Build complete (" + result + ") for " + cfg);
+        cfg.getTunnelPool().buildComplete(cfg, result);
         if (cfg.getLength() > 1)
             removeFromBuilding(cfg.getReplyMessageId());
         // Only wake up the build thread if it took a reasonable amount of time -
         // this prevents high CPU usage when there is no network connection
         // (via BuildRequestor.TunnelBuildFirstHopFailJob)
-        long buildTime = _context.clock().now() + 10*60*1000- cfg.getExpiration();
+        long now = _context.clock().now();
+        long buildTime = now + 10*60*1000 - cfg.getExpiration();
         if (buildTime > 250) {
             synchronized (_currentlyBuilding) { 
                 _currentlyBuilding.notifyAll();
@@ -575,11 +588,16 @@ class BuildExecutor implements Runnable {
                 _log.info("Build complete really fast (" + buildTime + " ms) for tunnel: " + cfg);
         }
         
-        long expireBefore = _context.clock().now() + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT;
+        long expireBefore = now + 10*60*1000 - BuildRequestor.REQUEST_TIMEOUT;
         if (cfg.getExpiration() <= expireBefore) {
             if (_log.shouldLog(Log.INFO))
                 _log.info("Build complete for expired tunnel: " + cfg);
         }
+        if (result == Result.SUCCESS) {
+            _manager.buildComplete(cfg);
+            ExpireJob expireJob = new ExpireJob(_context, cfg);
+            _context.jobQueue().addJob(expireJob);
+        }
     }
     
     public boolean wasRecentlyBuilding(long replyId) {
@@ -588,10 +606,6 @@ class BuildExecutor implements Runnable {
         }
     }
     
-    public void buildSuccessful(PooledTunnelCreatorConfig cfg) {
-        _manager.buildComplete(cfg);
-    }
-    
     public void repoll() { 
         synchronized (_currentlyBuilding) { 
             _repoll = true;
@@ -635,9 +649,9 @@ class BuildExecutor implements Runnable {
         if (rv != null) {
             long requestedOn = rv.getExpiration() - 10*60*1000;
             long rtt = _context.clock().now() - requestedOn;
-            _context.statManager().addRateData("tunnel.buildReplySlow", rtt, 0);
-            if (_log.shouldLog(Log.WARN))
-                _log.warn("Got reply late (rtt = " + rtt + ") for: " + rv);
+            _context.statManager().addRateData("tunnel.buildReplySlow", rtt);
+            if (_log.shouldInfo())
+                _log.info("Got reply late (rtt = " + rtt + ") for: " + rv);
         }
         return rv;
     }
