@@ -3,24 +3,36 @@ package net.i2p.router.tunnel.pool;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.StringTokenizer;
 
+import net.i2p.crypto.EncType;
+import net.i2p.crypto.SessionKeyManager;
+import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.PublicKey;
-import net.i2p.data.router.RouterInfo;
 import net.i2p.data.TunnelId;
+import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.data.i2np.ShortTunnelBuildMessage;
 import net.i2p.data.i2np.TunnelBuildMessage;
 import net.i2p.data.i2np.VariableTunnelBuildMessage;
+import net.i2p.data.router.RouterInfo;
 import net.i2p.router.JobImpl;
+import net.i2p.router.LeaseSetKeys;
 import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
 import net.i2p.router.TunnelInfo;
 import net.i2p.router.TunnelManagerFacade;
 import net.i2p.router.TunnelPoolSettings;
-import net.i2p.router.tunnel.BuildMessageGenerator;
+import net.i2p.router.crypto.ratchet.RatchetSKM;
+import net.i2p.router.crypto.ratchet.MuxedSKM;
+import net.i2p.router.networkdb.kademlia.MessageWrapper;
+import net.i2p.router.networkdb.kademlia.MessageWrapper.OneTimeSession;
 import net.i2p.router.tunnel.HopConfig;
 import net.i2p.router.tunnel.TunnelCreatorConfig;
+import static net.i2p.router.tunnel.pool.BuildExecutor.Result.*;
 import net.i2p.util.Log;
+import net.i2p.util.SystemVersion;
 import net.i2p.util.VersionComparator;
 
 /**
@@ -28,7 +40,7 @@ import net.i2p.util.VersionComparator;
  */
 abstract class BuildRequestor {
     private static final List<Integer> ORDER = new ArrayList<Integer>(TunnelBuildMessage.MAX_RECORD_COUNT);
-    //private static final String MIN_VARIABLE_VERSION = "0.7.12";
+    private static final String MIN_NEWTBM_VERSION = "0.9.51";
     private static final boolean SEND_VARIABLE = true;
     private static final int SHORT_RECORDS = 4;
     private static final List<Integer> SHORT_ORDER = new ArrayList<Integer>(SHORT_RECORDS);
@@ -46,7 +58,6 @@ abstract class BuildRequestor {
             MEDIUM_ORDER.add(Integer.valueOf(i));
         }
     }
-
     private static final int PRIORITY = OutNetMessage.PRIORITY_MY_BUILD_REQUEST;
 
     /**
@@ -59,7 +70,7 @@ abstract class BuildRequestor {
      *  so can we use a successfully built tunnel anyway.
      *
      */
-    static final int REQUEST_TIMEOUT = 13*1000;
+    static final int REQUEST_TIMEOUT = SystemVersion.isSlow() ? 10*1000 : 5*1000;
 
     /** make this shorter than REQUEST_TIMEOUT */
     private static final int FIRST_HOP_TIMEOUT = 10*1000;
@@ -67,6 +78,8 @@ abstract class BuildRequestor {
     /** some randomization is added on to this */
     private static final int BUILD_MSG_TIMEOUT = 60*1000;
 
+    private static final int MAX_CONSECUTIVE_CLIENT_BUILD_FAILS = 6;
+
     /**
      *  "paired tunnels" means using a client's own inbound tunnel to receive the
      *  reply for an outbound build request, and using a client's own outbound tunnel
@@ -105,9 +118,8 @@ abstract class BuildRequestor {
             
             if (i > 0)
                 cfg.getConfig(i-1).setSendTunnelId(hop.getReceiveTunnelId());
-            byte iv[] = new byte[TunnelCreatorConfig.REPLY_IV_LENGTH];
-            ctx.random().nextBytes(iv);
-            cfg.setAESReplyKeys(i, ctx.keyGenerator().generateSessionKey(), iv);
+            // AES reply keys now set in createTunnelBuildMessage(),
+            // as we don't need them for short TBM
         }
         // This is in BuildExecutor.buildTunnel() now
         // And it was overwritten by the one in createTunnelBuildMessage() anyway!
@@ -138,17 +150,41 @@ abstract class BuildRequestor {
         Hash farEnd = cfg.getFarEnd();
         TunnelManagerFacade mgr = ctx.tunnelManager();
         boolean isInbound = settings.isInbound();
+        // OB only, short record only
+        SessionKeyManager replySKM = null;
         if (settings.isExploratory() || !usePairedTunnels(ctx)) {
-            if (isInbound)
+            if (isInbound) {
                 pairedTunnel = mgr.selectOutboundExploratoryTunnel(farEnd);
-            else
+            } else {
                 pairedTunnel = mgr.selectInboundExploratoryTunnel(farEnd);
+                if (pairedTunnel != null) {
+                    replySKM = ctx.sessionKeyManager();
+                }
+            }
         } else {
             // building a client tunnel
-            if (isInbound)
-                pairedTunnel = mgr.selectOutboundTunnel(settings.getDestination(), farEnd);
-            else
-                pairedTunnel = mgr.selectInboundTunnel(settings.getDestination(), farEnd);
+            int fails = pool.getConsecutiveBuildTimeouts();
+            if (fails < MAX_CONSECUTIVE_CLIENT_BUILD_FAILS) {
+                Hash from = settings.getDestination();
+                if (isInbound) {
+                    pairedTunnel = mgr.selectOutboundTunnel(from, farEnd);
+                } else {
+                    pairedTunnel = mgr.selectInboundTunnel(from, farEnd);
+                    if (pairedTunnel != null) {
+                        replySKM = ctx.clientManager().getClientSessionKeyManager(from);
+                        if (replySKM == null && cfg.getGarlicReplyKeys() != null) {
+                            // no client SKM, fall back to expl.
+                            pairedTunnel = null;
+                        }
+                    }
+                }
+            } else {
+                // Force using an expl. tunnel as the paired tunnel
+                // This prevents us from being stuck for 10 minutes if the client pool
+                // has exactly one tunnel in the other direction and it's bad
+                if (log.shouldWarn())
+                    log.warn(fails + " consecutive build timeouts for " + cfg + ", using exploratory tunnel");
+            }
             if (pairedTunnel == null) {   
                 if (isInbound) {
                     // random more reliable than closest ??
@@ -174,6 +210,9 @@ abstract class BuildRequestor {
                         // ditto
                         pairedTunnel = null;
                     }
+                    if (pairedTunnel != null) {
+                        replySKM = ctx.sessionKeyManager();
+                    }
                 }
                 if (pairedTunnel != null && log.shouldLog(Log.INFO))
                     log.info("Couldn't find a paired tunnel for " + cfg + ", using exploratory tunnel");
@@ -182,7 +221,7 @@ abstract class BuildRequestor {
         if (pairedTunnel == null) {
             if (log.shouldLog(Log.WARN))
                 log.warn("Tunnel build failed, as we couldn't find a paired tunnel for " + cfg);
-            exec.buildComplete(cfg);
+            exec.buildComplete(cfg, OTHER_FAILURE);
             // Not even an exploratory tunnel? We are in big trouble.
             // Let's not spin through here too fast.
             // But don't let a client tunnel waiting for exploratories slow things down too much,
@@ -193,31 +232,60 @@ abstract class BuildRequestor {
         }
         
         //long beforeCreate = System.currentTimeMillis();
-        TunnelBuildMessage msg = createTunnelBuildMessage(ctx, pool, cfg, pairedTunnel, exec);
+        I2NPMessage msg = createTunnelBuildMessage(ctx, pool, cfg, pairedTunnel, exec);
         //long createTime = System.currentTimeMillis()-beforeCreate;
         if (msg == null) {
             if (log.shouldLog(Log.WARN))
                 log.warn("Tunnel build failed, as we couldn't create the tunnel build message for " + cfg);
-            exec.buildComplete(cfg);
+            exec.buildComplete(cfg, OTHER_FAILURE);
             return false;
         }
         
-        //cfg.setPairedTunnel(pairedTunnel);
+        // store the ID of the paired GW so we can credit/blame the paired tunnel,
+        // see TunnelPool.updatePairedProfile()
+        if (pairedTunnel.getLength() > 1) {
+            TunnelId gw = pairedTunnel.isInbound() ?
+                          pairedTunnel.getReceiveTunnelId(0) :
+                          pairedTunnel.getSendTunnelId(0);
+            cfg.setPairedGW(gw);
+        }
         
         //long beforeDispatch = System.currentTimeMillis();
         if (cfg.isInbound()) {
+            Hash ibgw = cfg.getPeer(0);
+            // don't wrap if IBGW == OBEP
+            if (msg.getType() == ShortTunnelBuildMessage.MESSAGE_TYPE &&
+                !ibgw.equals(pairedTunnel.getEndpoint())) {
+                // STBM is garlic encrypted to the IBGW, to hide it from the OBEP
+                RouterInfo peer = ctx.netDb().lookupRouterInfoLocally(ibgw);
+                if (peer != null) {
+                    I2NPMessage enc = MessageWrapper.wrap(ctx, msg, peer);
+                    if (enc != null) {
+                        msg = enc;
+                        // log.debug("wrapping IB TBM to " + ibgw);
+                    } else {
+                        if (log.shouldWarn())
+                            log.warn("failed to wrap IB TBM to " + ibgw);
+                    }
+                } else {
+                    if (log.shouldWarn())
+                        log.warn("no RI, failed to wrap IB TBM to " + ibgw);
+                }
+            }
+
             if (log.shouldLog(Log.INFO))
                 log.info("Sending the tunnel build request " + msg.getUniqueId() + " out the tunnel " + pairedTunnel + " to " 
-                          + cfg.getPeer(0) + " for " + cfg + " waiting for the reply of "
+                          + ibgw + " for " + cfg + " waiting for the reply of "
                           + cfg.getReplyMessageId());
             // send it out a tunnel targetting the first hop
             // TODO - would be nice to have a TunnelBuildFirstHopFailJob queued if the
             // pairedTunnel is zero-hop, but no way to do that?
-            ctx.tunnelDispatcher().dispatchOutbound(msg, pairedTunnel.getSendTunnelId(0), cfg.getPeer(0));
+            ctx.tunnelDispatcher().dispatchOutbound(msg, pairedTunnel.getSendTunnelId(0), ibgw);
         } else {
             if (log.shouldLog(Log.INFO))
                 log.info("Sending the tunnel build request directly to " + cfg.getPeer(1)
                           + " for " + cfg + " waiting for the reply of " + cfg.getReplyMessageId() 
+                          + " via IB tunnel " + pairedTunnel
                           + " with msgId=" + msg.getUniqueId());
             // send it directly to the first hop
             // Add some fuzz to the TBM expiration to make it harder to guess how many hops
@@ -229,11 +297,26 @@ abstract class BuildRequestor {
             if (peer == null) {
                 if (log.shouldLog(Log.WARN))
                     log.warn("Could not find the next hop to send the outbound request to: " + cfg);
-                exec.buildComplete(cfg);
+                exec.buildComplete(cfg, OTHER_FAILURE);
                 return false;
             }
             OutNetMessage outMsg = new OutNetMessage(ctx, msg, ctx.clock().now() + FIRST_HOP_TIMEOUT, PRIORITY, peer);
             outMsg.setOnFailedSendJob(new TunnelBuildFirstHopFailJob(ctx, cfg, exec));
+            OneTimeSession ots = cfg.getGarlicReplyKeys();
+            if (ots != null && replySKM != null) {
+                if (replySKM instanceof RatchetSKM) {
+                    RatchetSKM rskm = (RatchetSKM) replySKM;
+                    rskm.tagsReceived(ots.key, ots.rtag, 2 * BUILD_MSG_TIMEOUT);
+                } else if (replySKM instanceof MuxedSKM) {
+                    MuxedSKM mskm = (MuxedSKM) replySKM;
+                    mskm.tagsReceived(ots.key, ots.rtag, 2 * BUILD_MSG_TIMEOUT);
+                } else {
+                    // non-EC client, shouldn't happen, checked at top of createTunnelBuildMessage() below
+                    if (log.shouldWarn())
+                        log.warn("Unsupported SKM for garlic reply to: " + cfg);
+                }
+                cfg.setGarlicReplyKeys(null);
+            }
             try {
                 ctx.outNetMessagePool().add(outMsg);
             } catch (RuntimeException re) {
@@ -242,23 +325,23 @@ abstract class BuildRequestor {
             }
         }
         //if (log.shouldLog(Log.DEBUG))
-        //    log.debug("Tunnel build message " + msg.getUniqueId() + " created in " + createTime
+        //    log.debug("Tunnel build message " + msg.getRawUniqueId() + " created in " + createTime
         //              + "ms and dispatched in " + (System.currentTimeMillis()-beforeDispatch));
         return true;
     }
 
-    /** @since 0.7.12 */
-/****
-we can assume everybody supports variable now...
-keep this here for the next time we change the build protocol
-    private static boolean supportsVariable(RouterContext ctx, Hash h) {
+    /**
+     * @since 0.9.51
+     */
+    private static boolean supportsShortTBM(RouterContext ctx, Hash h) {
         RouterInfo ri = ctx.netDb().lookupRouterInfoLocally(h);
         if (ri == null)
             return false;
+        if (ri.getIdentity().getPublicKey().getType() != EncType.ECIES_X25519)
+            return false;
         String v = ri.getVersion();
-        return VersionComparator.comp(v, MIN_VARIABLE_VERSION) >= 0;
+        return VersionComparator.comp(v, MIN_NEWTBM_VERSION) >= 0;
     }
-****/
 
     /**
      *  If the tunnel is short enough, and everybody in the tunnel, and the
@@ -273,53 +356,87 @@ keep this here for the next time we change the build protocol
         long replyTunnel = 0;
         Hash replyRouter;
         boolean useVariable = SEND_VARIABLE && cfg.getLength() <= MEDIUM_RECORDS;
+        boolean useShortTBM = ctx.keyManager().getPublicKey().getType() == EncType.ECIES_X25519;
+        if (useShortTBM && !cfg.isInbound() && !pool.getSettings().isExploratory()) {
+            // client must be EC also to get garlic OTBRM reply
+            LeaseSetKeys lsk = ctx.keyManager().getKeys(pool.getSettings().getDestination());
+            if (lsk != null) {
+                if (!lsk.isSupported(EncType.ECIES_X25519))
+                    useShortTBM = false;
+            } else {
+                useShortTBM = false;
+            }
+        }
+
+/*
+        // Testing, send to explicitPeers only
+        List<Hash> explicitPeers = null;
+        if (useShortTBM) {
+            String peers = ctx.getProperty("explicitPeers");
+            if (peers != null && !peers.isEmpty()) {
+                explicitPeers = new ArrayList<Hash>(4);
+                StringTokenizer tok = new StringTokenizer(peers, ",");
+                while (tok.hasMoreTokens()) {
+                    String peerStr = tok.nextToken();
+                    Hash peer = new Hash();
+                    try {
+                        peer.fromBase64(peerStr);
+                        explicitPeers.add(peer);
+                    } catch (DataFormatException dfe) {}
+                }
+                if (explicitPeers.isEmpty())
+                    useShortTBM = false;
+            } else {
+                useShortTBM = false;
+            }
+        }
+*/
 
         if (cfg.isInbound()) {
             //replyTunnel = 0; // as above
             replyRouter = ctx.routerHash();
-/****
-we can assume everybody supports variable now...
-keep this here for the next time we change the build protocol
-            if (useVariable) {
-                // check the reply OBEP and all the tunnel peers except ourselves
-                if (!supportsVariable(ctx, pairedTunnel.getPeer(pairedTunnel.getLength() - 1))) {
-                    useVariable = false;
-                } else {
-                    for (int i = 0; i < cfg.getLength() - 1; i++) {
-                        if (!supportsVariable(ctx, cfg.getPeer(i))) {
-                            useVariable = false;
-                            break;
-                        }
+            if (useShortTBM) {
+                // check all the tunnel peers except ourselves
+                for (int i = 0; i < cfg.getLength() - 1; i++) {
+                    // TODO remove explicit check
+                    if (/* !explicitPeers.contains(cfg.getPeer(i)) || */ !supportsShortTBM(ctx, cfg.getPeer(i))) {
+                        useShortTBM = false;
+                        break;
                     }
                 }
             }
-****/
         } else {
             replyTunnel = pairedTunnel.getReceiveTunnelId(0).getTunnelId();
             replyRouter = pairedTunnel.getPeer(0);
-/****
-we can assume everybody supports variable now
-keep this here for the next time we change the build protocol
-            if (useVariable) {
-                // check the reply IBGW and all the tunnel peers except ourselves
-                if (!supportsVariable(ctx, replyRouter)) {
-                    useVariable = false;
-                } else {
-                    for (int i = 1; i < cfg.getLength() - 1; i++) {
-                        if (!supportsVariable(ctx, cfg.getPeer(i))) {
-                            useVariable = false;
-                            break;
-                        }
+            if (useShortTBM) {
+                // check all the tunnel peers except ourselves
+                for (int i = 1; i < cfg.getLength(); i++) {
+                    // TODO remove explicit check
+                    if (/* !explicitPeers.contains(cfg.getPeer(i)) || */ !supportsShortTBM(ctx, cfg.getPeer(i))) {
+                        useShortTBM = false;
+                        break;
                     }
                 }
             }
-****/
         }
 
         // populate and encrypt the message
         TunnelBuildMessage msg;
         List<Integer> order;
-        if (useVariable) {
+        if (useShortTBM) {
+            int len;
+            if (cfg.getLength() <= SHORT_RECORDS) {
+                len = SHORT_RECORDS;
+                order = new ArrayList<Integer>(SHORT_ORDER);
+            } else if (cfg.getLength() <= MEDIUM_RECORDS) {
+                len = MEDIUM_RECORDS;
+                order = new ArrayList<Integer>(MEDIUM_ORDER);
+            } else {
+                len = TunnelBuildMessage.MAX_RECORD_COUNT;
+                order = new ArrayList<Integer>(ORDER);
+            }
+            msg = new ShortTunnelBuildMessage(ctx, len);
+        } else if (useVariable) {
             if (cfg.getLength() <= SHORT_RECORDS) {
                 msg = new VariableTunnelBuildMessage(ctx, SHORT_RECORDS);
                 order = new ArrayList<Integer>(SHORT_ORDER);
@@ -332,6 +449,20 @@ keep this here for the next time we change the build protocol
             order = new ArrayList<Integer>(ORDER);
         }
 
+        if (!useShortTBM) {
+            int len = cfg.getLength();
+            for (int i = 0; i < len; i++) {
+                HopConfig hop = cfg.getConfig(i);
+                // set IV/Layer keys (formerly in TunnelPool.configureNewTunnel())
+                hop.setIVKey(ctx.keyGenerator().generateSessionKey());
+                hop.setLayerKey(ctx.keyGenerator().generateSessionKey());
+                // set the AES reply keys (formerly in prepare())
+                byte iv[] = new byte[TunnelCreatorConfig.REPLY_IV_LENGTH];
+                ctx.random().nextBytes(iv);
+                cfg.setAESReplyKeys(i, ctx.keyGenerator().generateSessionKey(), iv);
+            }
+        }  // else keys are derived
+
         // This is in BuildExecutor.buildTunnel() now
         //long replyMessageId = ctx.random().nextLong(I2NPMessage.MAX_ID_VALUE);
         //cfg.setReplyMessageId(replyMessageId);
@@ -360,11 +491,17 @@ keep this here for the next time we change the build protocol
                     key = peerInfo.getIdentity().getPublicKey();
                 }
             }
-            if (log.shouldLog(Log.DEBUG))
-                log.debug(cfg.getReplyMessageId() + ": record " + i + "/" + hop + " has key " + key);
+            if (log.shouldLog(Log.DEBUG)) {
+                if (key != null)
+                    log.debug(cfg.getReplyMessageId() + ": record " + i + "/" + hop + " has key " + key);
+                else
+                    log.debug(cfg.getReplyMessageId() + ": record " + i + "/" + hop + " empty");
+            }
             BuildMessageGenerator.createRecord(i, hop, msg, cfg, replyRouter, replyTunnel, ctx, key);
         }
         BuildMessageGenerator.layeredEncrypt(ctx, msg, cfg, order);
+        //if (useShortTBM && log.shouldWarn())
+        //    log.warn("Sending STBM: " + cfg.toStringFull());
         
         return msg;
     }
@@ -374,15 +511,12 @@ keep this here for the next time we change the build protocol
         if (log.shouldLog(Log.DEBUG))
             log.debug("Build zero hop tunnel " + cfg);            
 
-        exec.buildComplete(cfg);
+        boolean ok;
         if (cfg.isInbound())
-            ctx.tunnelDispatcher().joinInbound(cfg);
+            ok = ctx.tunnelDispatcher().joinInbound(cfg);
         else
-            ctx.tunnelDispatcher().joinOutbound(cfg);
-        cfg.getTunnelPool().addTunnel(cfg);
-        exec.buildSuccessful(cfg);
-        ExpireJob expireJob = new ExpireJob(ctx, cfg);
-        ctx.jobQueue().addJob(expireJob);
+            ok = ctx.tunnelDispatcher().joinOutbound(cfg);
+        exec.buildComplete(cfg, ok ? SUCCESS : DUP_ID);
         // can it get much easier?
     }
 
@@ -405,9 +539,9 @@ keep this here for the next time we change the build protocol
         }
         public String getName() { return "Timeout contacting first peer for OB tunnel"; }
         public void runJob() {
-            _exec.buildComplete(_cfg);
+            _exec.buildComplete(_cfg, OTHER_FAILURE);
             getContext().profileManager().tunnelTimedOut(_cfg.getPeer(1));
-            getContext().statManager().addRateData("tunnel.buildFailFirstHop", 1, 0);
+            getContext().statManager().addRateData("tunnel.buildFailFirstHop", 1);
             // static, no _log
             //System.err.println("Cant contact first hop for " + _cfg);
         }
