@@ -1,7 +1,6 @@
 package net.i2p.router.tunnel.pool;
 
 import java.io.Serializable;
-import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -12,12 +11,16 @@ import java.util.Properties;
 import java.util.Set;
 import java.util.StringTokenizer;
 
+import net.i2p.CoreVersion;
 import net.i2p.crypto.EncType;
 import net.i2p.crypto.SHA256Generator;
 import net.i2p.crypto.SigType;
+import net.i2p.crypto.SipHashInline;
 import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
+import net.i2p.data.SessionKey;
+import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.LeaseSetKeys;
@@ -25,25 +28,29 @@ import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
 import net.i2p.router.TunnelPoolSettings;
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import net.i2p.router.peermanager.PeerProfile;
 import net.i2p.router.transport.TransportUtil;
-import net.i2p.router.util.HashDistance;
+import net.i2p.util.ArraySet;
 import net.i2p.util.Log;
+import net.i2p.util.SystemVersion;
 import net.i2p.util.VersionComparator;
 
 /**
- * Coordinate the selection of peers to go into a tunnel for one particular 
+ * Coordinate the selection of peers to go into a tunnel for one particular
  * pool.
  *
  */
 public abstract class TunnelPeerSelector extends ConnectChecker {
 
+    private static final String DEFAULT_EXCLUDE_CAPS = String.valueOf(Router.CAPABILITY_BW12)+String.valueOf(Router.CAPABILITY_CONGESTION_SEVERE);//+String.valueOf(Router.CAPABILITY_CONGESTION_MODERATE);
+
     protected TunnelPeerSelector(RouterContext context) {
         super(context);
     }
 
     /**
-     * Which peers should go into the next tunnel for the given settings?  
-     * 
+     * Which peers should go into the next tunnel for the given settings?
+     *
      * @return ordered list of Hash objects (one per peer) specifying what order
      *         they should appear in a tunnel (ENDPOINT FIRST).  This includes
      *         the local router in the list.  If there are no tunnels or peers
@@ -51,7 +58,7 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      *         return null.
      */
     public abstract List<Hash> selectPeers(TunnelPoolSettings settings);
-    
+
     /**
      *  @return randomized number of hops 0-7, not including ourselves
      */
@@ -77,25 +84,9 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
             length = 0;
         else if (length > 7) // as documented in tunnel.html
             length = 7;
-        /*
-        if ( (ctx.tunnelManager().getOutboundTunnelCount() <= 0) || 
-             (ctx.tunnelManager().getFreeTunnelCount() <= 0) ) {
-            Log log = ctx.logManager().getLog(TunnelPeerSelector.class);
-            // no tunnels to build tunnels with
-            if (settings.getAllowZeroHop()) {
-                if (log.shouldLog(Log.INFO))
-                    log.info("no outbound tunnels or free inbound tunnels, but we do allow zeroHop: " + settings);
-                return 0;
-            } else {
-                if (log.shouldLog(Log.WARN))
-                    log.warn("no outbound tunnels or free inbound tunnels, and we dont allow zeroHop: " + settings);
-                return -1;
-            }
-        }
-        */
         return length;
     }
-    
+
     /**
      *  For debugging, also possibly for restricted routes?
      *  Needs analysis and testing
@@ -103,6 +94,9 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      */
     protected boolean shouldSelectExplicit(TunnelPoolSettings settings) {
         if (settings.isExploratory()) return false;
+        // To test IB or OB only
+        //if (settings.isInbound()) return false;
+        //if (!settings.isInbound()) return false;
         Properties opts = settings.getUnknownOptions();
         String peers = opts.getProperty("explicitPeers");
         if (peers == null)
@@ -112,7 +106,7 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
             return true;
         return false;
     }
-    
+
     /**
      *  For debugging, also possibly for restricted routes?
      *  Needs analysis and testing
@@ -122,10 +116,10 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
         String peers = null;
         Properties opts = settings.getUnknownOptions();
         peers = opts.getProperty("explicitPeers");
-        
+
         if (peers == null)
             peers = ctx.getProperty("explicitPeers");
-        
+
         List<Hash> rv = new ArrayList<Hash>();
         StringTokenizer tok = new StringTokenizer(peers, ",");
         while (tok.hasMoreTokens()) {
@@ -133,7 +127,7 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
             Hash peer = new Hash();
             try {
                 peer.fromBase64(peerStr);
-                
+
                 if (ctx.profileOrganizer().isSelectable(peer)) {
                     rv.add(peer);
                 } else {
@@ -144,14 +138,14 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
                     log.error("Explicit peer is improperly formatted (" + peerStr + ")", dfe);
             }
         }
-        
+
         int sz = rv.size();
         if (sz == 0) {
             log.logAlways(Log.WARN, "No valid explicit peers found, building zero hop");
         } else if (sz > 1) {
             Collections.shuffle(rv, ctx.random());
         }
-        
+
         while (rv.size() > length) {
             rv.remove(0);
         }
@@ -159,12 +153,13 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
             int more = length - rv.size();
             Set<Hash> exclude = getExclude(settings.isInbound(), settings.isExploratory());
             exclude.addAll(rv);
-            Set<Hash> matches = new HashSet<Hash>(more);
-            ctx.profileOrganizer().selectFastPeers(more, exclude, matches, 0);
+            Set<Hash> matches = new ArraySet<Hash>(more);
+            // don't bother with IP restrictions here
+            ctx.profileOrganizer().selectFastPeers(more, exclude, matches);
             rv.addAll(matches);
             Collections.shuffle(rv, ctx.random());
         }
-        
+
         if (log.shouldLog(Log.INFO)) {
             StringBuilder buf = new StringBuilder();
             if (settings.getDestinationNickname() != null)
@@ -181,19 +176,29 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
             buf.append(", out of ").append(sz).append(" (not including self)");
             log.info(buf.toString());
         }
-        
+
         if (settings.isInbound())
             rv.add(0, ctx.routerHash());
         else
             rv.add(ctx.routerHash());
-        
+
         return rv;
     }
-    
-    /** 
-     * Pick peers that we want to avoid
+
+    /**
+     *  As of 0.9.58, this returns a set populated only by TunnelManager.selectPeersInTooManyTunnels(),
+     *  for passing to ProfileOrganizer.
+     *  The set will be populated via the contains() calls.
      */
-    public Set<Hash> getExclude(boolean isInbound, boolean isExploratory) {
+    protected Set<Hash> getExclude(boolean isInbound, boolean isExploratory) {
+        return new Excluder(isInbound, isExploratory);
+    }
+
+
+    /**
+     *  @since 0.9.58, previously getExclude()
+     */
+    private boolean shouldExclude(Hash h, boolean isInbound, boolean isExploratory) {
         // we may want to update this to skip 'hidden' or 'unreachable' peers, but that
         // isn't safe, since they may publish one set of routerInfo to us and another to
         // other peers.  the defaults for filterUnreachable has always been to return false,
@@ -210,133 +215,36 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
         // We could just try and exclude them as the inbound gateway but that's harder
         // (and even worse for anonymity?).
         //
-        // Defaults changed to true for inbound only in filterUnreachable below.
 
-        Set<Hash> peers = new HashSet<Hash>(8);
-        peers.addAll(ctx.profileOrganizer().selectPeersRecentlyRejecting());
-        peers.addAll(ctx.tunnelManager().selectPeersInTooManyTunnels());
-        // if (false && filterUnreachable(ctx, isInbound, isExploratory)) {
-        if (filterUnreachable(isInbound, isExploratory)) {
-            // NOTE: filterUnreachable returns true for inbound, false for outbound
-            // This is the only use for getPeersByCapability? And the whole set of datastructures in PeerManager?
-            Collection<Hash> caps = ctx.peerManager().getPeersByCapability(Router.CAPABILITY_UNREACHABLE);
-            if (caps != null)
-                peers.addAll(caps);
-            caps = ctx.profileOrganizer().selectPeersLocallyUnreachable();
-            if (caps != null)
-                peers.addAll(caps);
+        PeerProfile prof = ctx.profileOrganizer().getProfileNonblocking(h);
+        if (prof != null) {
+            long cutoff = ctx.clock().now() - (20*1000);
+            if (prof.getTunnelHistory().getLastRejectedBandwidth() > cutoff)
+                return true;
         }
+
+        // the transport layer thinks is unreachable
+        if (ctx.commSystem().wasUnreachable(h))
+            return true;
+
+        RouterInfo info = (RouterInfo) ctx.netDb().lookupLocallyWithoutValidation(h);
+        if (info == null)
+            return true;
+
+        if (filterUnreachable(isInbound, isExploratory)) {
+            String caps = info.getCapabilities();
+            if (caps.indexOf(Router.CAPABILITY_UNREACHABLE) >= 0)
+                return true;
+        }
+
         if (filterSlow(isInbound, isExploratory)) {
             // NOTE: filterSlow always returns true
-            char excl[] = getExcludeCaps(ctx);
-            if (excl != null) {
-                FloodfillNetworkDatabaseFacade fac = (FloodfillNetworkDatabaseFacade)ctx.netDb();
-                List<RouterInfo> known = fac.getKnownRouterData();
-                if (known != null) {
-                    for (int i = 0; i < known.size(); i++) {
-                        RouterInfo peer = known.get(i);
-                        boolean shouldExclude = shouldExclude(peer, excl);
-                        if (shouldExclude) {
-                            peers.add(peer.getIdentity().calculateHash());
-                            continue;
-                        }
-                        /*
-                        String cap = peer.getCapabilities();
-                        if (cap == null) {
-                            peers.add(peer.getIdentity().calculateHash());
-                            continue;
-                        }
-                        for (int j = 0; j < excl.length; j++) {
-                            if (cap.indexOf(excl[j]) >= 0) {
-                                peers.add(peer.getIdentity().calculateHash());
-                                continue;
-                            }
-                        }
-                        int maxLen = 0;
-                        if (cap.indexOf(FloodfillNetworkDatabaseFacade.CAPACITY_FLOODFILL) >= 0)
-                            maxLen++;
-                        if (cap.indexOf(Router.CAPABILITY_REACHABLE) >= 0)
-                            maxLen++;
-                        if (cap.indexOf(Router.CAPABILITY_UNREACHABLE) >= 0)
-                            maxLen++;
-                        if (cap.length() <= maxLen)
-                            peers.add(peer.getIdentity().calculateHash());
-                        // otherwise, it contains flags we aren't trying to focus on,
-                        // so don't exclude it based on published capacity
-                        
-                        if (filterUptime(ctx, isInbound, isExploratory)) {
-                            Properties opts = peer.getOptions();
-                            if (opts != null) {
-                                String val = opts.getProperty("stat_uptime");
-                                long uptimeMs = 0;
-                                if (val != null) {
-                                    long factor = 1;
-                                    if (val.endsWith("ms")) {
-                                        factor = 1;
-                                        val = val.substring(0, val.length()-2);
-                                    } else if (val.endsWith("s")) {
-                                        factor = 1000l;
-                                        val = val.substring(0, val.length()-1);
-                                    } else if (val.endsWith("m")) {
-                                        factor = 60*1000l;
-                                        val = val.substring(0, val.length()-1);
-                                    } else if (val.endsWith("h")) {
-                                        factor = 60*60*1000l;
-                                        val = val.substring(0, val.length()-1);
-                                    } else if (val.endsWith("d")) {
-                                        factor = 24*60*60*1000l;
-                                        val = val.substring(0, val.length()-1);
-                                    }
-                                    try { uptimeMs = Long.parseLong(val); } catch (NumberFormatException nfe) {}
-                                    uptimeMs *= factor;
-                                } else {
-                                    // not publishing an uptime, so exclude it
-                                    peers.add(peer.getIdentity().calculateHash());
-                                    continue;
-                                }
-                                
-                                long infoAge = ctx.clock().now() - peer.getPublished();
-                                if (infoAge < 0) {
-                                    infoAge = 0;
-                                } else if (infoAge > 24*60*60*1000) {
-				    // Only exclude long-unseen peers if we haven't just started up
-				    long DONT_EXCLUDE_PERIOD = 15*60*1000;
-				    if (ctx.router().getUptime() < DONT_EXCLUDE_PERIOD) {
-				        if (log.shouldLog(Log.DEBUG))
-				            log.debug("Not excluding a long-unseen peer, since we just started up.");
-				    } else {
-				        if (log.shouldLog(Log.DEBUG))
-				            log.debug("Excluding a long-unseen peer.");
-				        peers.add(peer.getIdentity().calculateHash());
-				    }
-                                    //peers.add(peer.getIdentity().calculateHash());
-                                    continue;
-                                } else {
-                                    if (infoAge + uptimeMs < 2*60*60*1000) {
-                                        // up for less than 2 hours, so exclude it
-                                        peers.add(peer.getIdentity().calculateHash());
-                                    }
-                                }
-                            } else {
-                                // not publishing stats, so exclude it
-                                peers.add(peer.getIdentity().calculateHash());
-                                continue;
-                            }
-                        }
-                         */
-                    }
-                }
-                /*
-                for (int i = 0; i < excludeCaps.length(); i++) {
-                    List matches = ctx.peerManager().getPeersByCapability(excludeCaps.charAt(i));
-                    if (log.shouldLog(Log.INFO))
-                        log.info("Filtering out " + matches.size() + " peers with capability " + excludeCaps.charAt(i));
-                    peers.addAll(matches);
-                }
-                 */
-            }
+            String excl = getExcludeCaps(ctx);
+            if (shouldExclude(info, excl))
+                return true;
         }
-        return peers;
+
+        return false;
     }
 
     /**
@@ -356,10 +264,10 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      *  the RI. Will not force RI lookups.
      *  Default true.
      *
-     *  @since 0.9.34
+     *  @since 0.9.34, protected since 0.9.58 for ClientPeerSelector
      */
-    private boolean allowAsOBEP(Hash h) {
-        RouterInfo ri = ctx.netDb().lookupRouterInfoLocally(h);
+    protected boolean allowAsOBEP(Hash h) {
+        RouterInfo ri = (RouterInfo) ctx.netDb().lookupLocallyWithoutValidation(h);
         if (ri == null)
             return true;
         return canConnect(ri, ANY_V4);
@@ -367,22 +275,24 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
 
     /**
      *  Should we allow as IBGW?
-     *  This just checks for IPv4 support.
+     *  This just checks for the "R" capability and IPv4 support.
      *  Will return false for hidden or IPv6-only.
      *  This is intended for tunnel candidates, where we already have
      *  the RI. Will not force RI lookups.
      *  Default true.
      *
-     *  @since 0.9.34
+     *  @since 0.9.34, protected since 0.9.58 for ClientPeerSelector
      */
-    private boolean allowAsIBGW(Hash h) {
-        RouterInfo ri = ctx.netDb().lookupRouterInfoLocally(h);
+    protected boolean allowAsIBGW(Hash h) {
+        RouterInfo ri = (RouterInfo) ctx.netDb().lookupLocallyWithoutValidation(h);
         if (ri == null)
             return true;
+        if (ri.getCapabilities().indexOf(Router.CAPABILITY_REACHABLE) < 0)
+            return false;
         return canConnect(ANY_V4, ri);
     }
-    
-    /** 
+
+    /**
      *  Pick peers that we want to avoid for the first OB hop or last IB hop.
      *  There's several cases of importance:
      *  <ol><li>Inbound and we are hidden -
@@ -403,74 +313,45 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      *      Make sure that ClientPeerSelector and TunnelPeerSelector selectPeers() call this when needed.
      *  </ol>
      *
-     *  Don't call this unless you need to.
-     *  See ClientPeerSelector and TunnelPeerSelector selectPeers().
+     *  As of 0.9.58, this a set with only toAdd, for use in ProfileOrganizer.
+     *  The set will be populated via the contains() calls.
      *
      *  @param isInbound
-     *  @return null if none
+     *  @return non-null
      *  @since 0.9.17
      */
-    protected Set<Hash> getClosestHopExclude(boolean isInbound) {
-        RouterInfo ri = ctx.router().getRouterInfo();
-        if (ri == null)
-            return null;
-
-        // we can skip this check now, uncomment if we have some new sigtype
-        //SigType type = ri.getIdentity().getSigType();
-        //if (type == SigType.DSA_SHA1)
-        //    return null;
-
-        int ourMask = isInbound ? getInboundMask(ri) : getOutboundMask(ri);
-        Set<Hash> connected = ctx.commSystem().getEstablished();
-        Set<Hash> rv = new HashSet<Hash>(256);
-        FloodfillNetworkDatabaseFacade fac = (FloodfillNetworkDatabaseFacade)ctx.netDb();
-        List<RouterInfo> known = fac.getKnownRouterData();
-        if (known != null) {
-            for (int i = 0; i < known.size(); i++) {
-                RouterInfo peer = known.get(i);
-                Hash h = peer.getIdentity().calculateHash();
-
-                // Uncomment if stricter than in shouldExclude() below
-                //String v = peer.getVersion();
-                //if (VersionComparator.comp(v, "0.9.16") < 0) {
-                //    rv.add(h);
-                //    continue;
-                //}
-
-                if (connected.contains(h))
-                    continue;
-                boolean canConnect = isInbound ? canConnect(peer, ourMask) : canConnect(ourMask, peer);
-                if (!canConnect)
-                    rv.add(h);
-            }
-        }
-        return rv;
+    protected Set<Hash> getClosestHopExclude(boolean isInbound, Set<Hash> toAdd) {
+        return new ClosestHopExcluder(isInbound, toAdd);
     }
-    
-    /** warning, this is also called by ProfileOrganizer.isSelectable() */
+
+    /**
+     *  Should the peer be excluded based on its published caps?
+     *
+     *  Warning, this is also called by ProfileOrganizer.isSelectable()
+     */
     public static boolean shouldExclude(RouterContext ctx, RouterInfo peer) {
         return shouldExclude(peer, getExcludeCaps(ctx));
     }
-    
-    private static char[] getExcludeCaps(RouterContext ctx) {
-        String excludeCaps = ctx.getProperty("router.excludePeerCaps", 
-                                             String.valueOf(Router.CAPABILITY_BW12));
-        if (excludeCaps != null) {
-            char excl[] = excludeCaps.toCharArray();
-            return excl;
-        } else {
-            return null;
-        }
-    }
-    
-    /** NTCP2 */
-    private static final String MIN_VERSION = "0.9.36";
 
-    /** warning, this is also called by ProfileOrganizer.isSelectable() */
-    private static boolean shouldExclude(RouterInfo peer, char excl[]) {
+    /**
+     *  @return non-null, possibly empty
+     */
+    private static String getExcludeCaps(RouterContext ctx) {
+        return ctx.getProperty("router.excludePeerCaps", DEFAULT_EXCLUDE_CAPS);
+    }
+
+    /** Short ECIES tunnel build messages (1.5.0) */
+    private static final String MIN_VERSION = "0.9.51";
+
+    /**
+     *  Should the peer be excluded based on its published caps?
+     *
+     *  Warning, this is also called by ProfileOrganizer.isSelectable()
+     */
+    private static boolean shouldExclude(RouterInfo peer, String excl) {
         String cap = peer.getCapabilities();
-        for (int j = 0; j < excl.length; j++) {
-            if (cap.indexOf(excl[j]) >= 0) {
+        for (int j = 0; j < excl.length(); j++) {
+            if (cap.indexOf(excl.charAt(j)) >= 0) {
                 return true;
             }
         }
@@ -495,105 +376,78 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
 
         // minimum version check
         String v = peer.getVersion();
+        if (v.equals("0.9.52")) {
+            // c++ bug in 2.40.0/0.9.52, drops SSU messages
+            for (RouterAddress addr : peer.getAddresses()) {
+                if (addr.getCost() == 9 && addr.getTransportStyle().equals("SSU"))
+                    return true;
+            }
+            return false;
+        }
+        // quick check to skip the comparator
+        if (v.equals(CoreVersion.PUBLISHED_VERSION))
+            return false;
         if (VersionComparator.comp(v, MIN_VERSION) < 0)
             return true;
-
-        // uptime is always spoofed to 90m, so just remove all this
-      /******
-        String val = peer.getOption("stat_uptime");
-        if (val != null) {
-            long uptimeMs = 0;
-                long factor = 1;
-                if (val.endsWith("ms")) {
-                    factor = 1;
-                    val = val.substring(0, val.length()-2);
-                } else if (val.endsWith("s")) {
-                    factor = 1000l;
-                    val = val.substring(0, val.length()-1);
-                } else if (val.endsWith("m")) {
-                    factor = 60*1000l;
-                    val = val.substring(0, val.length()-1);
-                } else if (val.endsWith("h")) {
-                    factor = 60*60*1000l;
-                    val = val.substring(0, val.length()-1);
-                } else if (val.endsWith("d")) {
-                    factor = 24*60*60*1000l;
-                    val = val.substring(0, val.length()-1);
-                }
-                try { uptimeMs = Long.parseLong(val); } catch (NumberFormatException nfe) {}
-                uptimeMs *= factor;
-
-            long infoAge = ctx.clock().now() - peer.getPublished();
-            if (infoAge < 0) {
-                return false;
-            } else if (infoAge > 5*24*60*60*1000) {
-                // Only exclude long-unseen peers if we haven't just started up
-                if (ctx.router().getUptime() < DONT_EXCLUDE_PERIOD) {
-                    if (log.shouldLog(Log.DEBUG))
-                        log.debug("Not excluding a long-unseen peer, since we just started up.");
-                    return false;
-                } else {
-                    if (log.shouldLog(Log.DEBUG))
-                        log.debug("Excluding a long-unseen peer.");
-                    return true;
-                }
-            } else {
-                if ( (infoAge + uptimeMs < 90*60*1000) && (ctx.router().getUptime() > DONT_EXCLUDE_PERIOD) ) {
-                    // up for less than 90 min (which is really 1h since an uptime of 1h-2h is published as 90m),
-                    // so exclude it
-                    return true;
-                } else {
-                    return false;
-                }
-            }
-        } else {
-            // not publishing an uptime, so exclude it
-            return true;
-        }
-      ******/
         return false;
     }
-    
+
     private static final String PROP_OUTBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE = "router.outboundExploratoryExcludeUnreachable";
     private static final String PROP_OUTBOUND_CLIENT_EXCLUDE_UNREACHABLE = "router.outboundClientExcludeUnreachable";
     private static final String PROP_INBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE = "router.inboundExploratoryExcludeUnreachable";
     private static final String PROP_INBOUND_CLIENT_EXCLUDE_UNREACHABLE = "router.inboundClientExcludeUnreachable";
-    
+
     private static final boolean DEFAULT_OUTBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE = false;
     private static final boolean DEFAULT_OUTBOUND_CLIENT_EXCLUDE_UNREACHABLE = false;
     // see comments at getExclude() above
-    private static final boolean DEFAULT_INBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE = true;
-    private static final boolean DEFAULT_INBOUND_CLIENT_EXCLUDE_UNREACHABLE = true;
-    
+    private static final boolean DEFAULT_INBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE = false;
+    private static final boolean DEFAULT_INBOUND_CLIENT_EXCLUDE_UNREACHABLE = false;
+
     /**
-     * do we want to skip peers who haven't been up for long?
-     * @return true for inbound, false for outbound, unless configured otherwise
+     * do we want to skip unreachable peers?
+     * @return true if yes
      */
-    protected boolean filterUnreachable(boolean isInbound, boolean isExploratory) {
+    private boolean filterUnreachable(boolean isInbound, boolean isExploratory) {
+        if (!isExploratory)
+            return true;
+        // give them some cover without killing our success rate
+        return ctx.random().nextInt(4) != 0;
+/*
+        if (SystemVersion.isSlow() || ctx.router().getUptime() < 65*60*1000)
+            return true;
         if (isExploratory) {
-            if (isInbound)
+            if (isInbound) {
+                if (ctx.router().isHidden())
+                    return true;
                 return ctx.getProperty(PROP_INBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE, DEFAULT_INBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE);
-            else
+            } else {
                 return ctx.getProperty(PROP_OUTBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE, DEFAULT_OUTBOUND_EXPLORATORY_EXCLUDE_UNREACHABLE);
+            }
         } else {
-            if (isInbound)
+            if (isInbound) {
+                if (ctx.router().isHidden())
+                    return true;
                 return ctx.getProperty(PROP_INBOUND_CLIENT_EXCLUDE_UNREACHABLE, DEFAULT_INBOUND_CLIENT_EXCLUDE_UNREACHABLE);
-            else 
+            } else {
                 return ctx.getProperty(PROP_OUTBOUND_CLIENT_EXCLUDE_UNREACHABLE, DEFAULT_OUTBOUND_CLIENT_EXCLUDE_UNREACHABLE);
+            }
         }
+*/
     }
 
-    
+
     private static final String PROP_OUTBOUND_EXPLORATORY_EXCLUDE_SLOW = "router.outboundExploratoryExcludeSlow";
     private static final String PROP_OUTBOUND_CLIENT_EXCLUDE_SLOW = "router.outboundClientExcludeSlow";
     private static final String PROP_INBOUND_EXPLORATORY_EXCLUDE_SLOW = "router.inboundExploratoryExcludeSlow";
     private static final String PROP_INBOUND_CLIENT_EXCLUDE_SLOW = "router.inboundClientExcludeSlow";
-    
+
     /**
      * do we want to skip peers that are slow?
      * @return true unless configured otherwise
      */
     protected boolean filterSlow(boolean isInbound, boolean isExploratory) {
+        return true;
+/*
         if (isExploratory) {
             if (isInbound)
                 return ctx.getProperty(PROP_INBOUND_EXPLORATORY_EXCLUDE_SLOW, true);
@@ -602,42 +456,16 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
         } else {
             if (isInbound)
                 return ctx.getProperty(PROP_INBOUND_CLIENT_EXCLUDE_SLOW, true);
-            else 
-                return ctx.getProperty(PROP_OUTBOUND_CLIENT_EXCLUDE_SLOW, true);
-        }        
-    }
-    
-/****
-    private static final String PROP_OUTBOUND_EXPLORATORY_EXCLUDE_UPTIME = "router.outboundExploratoryExcludeUptime";
-    private static final String PROP_OUTBOUND_CLIENT_EXCLUDE_UPTIME = "router.outboundClientExcludeUptime";
-    private static final String PROP_INBOUND_EXPLORATORY_EXCLUDE_UPTIME = "router.inboundExploratoryExcludeUptime";
-    private static final String PROP_INBOUND_CLIENT_EXCLUDE_UPTIME = "router.inboundClientExcludeUptime";
-****/
-    
-    /**
-     * do we want to skip peers who haven't been up for long?
-     * @return true unless configured otherwise
-     */
-/****
-    protected boolean filterUptime(boolean isInbound, boolean isExploratory) {
-        if (isExploratory) {
-            if (isInbound)
-                return ctx.getProperty(PROP_INBOUND_EXPLORATORY_EXCLUDE_UPTIME, true);
             else
-                return ctx.getProperty(PROP_OUTBOUND_EXPLORATORY_EXCLUDE_UPTIME, true);
-        } else {
-            if (isInbound)
-                return ctx.getProperty(PROP_INBOUND_CLIENT_EXCLUDE_UPTIME, true);
-            else 
-                return ctx.getProperty(PROP_OUTBOUND_CLIENT_EXCLUDE_UPTIME, true);
+                return ctx.getProperty(PROP_OUTBOUND_CLIENT_EXCLUDE_SLOW, true);
         }
+*/
     }
-****/
 
     /** see HashComparator */
-    protected void orderPeers(List<Hash> rv, Hash hash) {
+    protected void orderPeers(List<Hash> rv, SessionKey key) {
         if (rv.size() > 1)
-            Collections.sort(rv, new HashComparator(hash));
+            Collections.sort(rv, new HashComparator(key));
     }
 
     /**
@@ -647,34 +475,40 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      *  to maximize his chances of those two routers being at opposite
      *  ends of a tunnel.
      *
-     *  Previous:
+     *  Previous Previous:
      *     d(l, h) - d(r, h)
      *
-     *  Now:
+     *  Previous:
      *     d((H(l+h), h) - d(H(r+h), h)
+     *
+     *  Now:
+     *     SipHash using h to generate the SipHash keys
+     *     then siphash(l) - siphash(r)
      */
     private static class HashComparator implements Comparator<Hash>, Serializable {
-        private final Hash _hash, tmp;
-        private final byte[] data;
+        private final long k0, k1;
 
-        /** not thread safe */
-        private HashComparator(Hash h) {
-            _hash = h;
-            tmp = new Hash(new byte[Hash.HASH_LENGTH]);
-            data = new byte[2*Hash.HASH_LENGTH];
-            System.arraycopy(_hash.getData(), 0, data, Hash.HASH_LENGTH, Hash.HASH_LENGTH);
+        /**
+         * not thread safe
+         *
+         * @param k container for sort keys, not used as a Hash
+         */
+        private HashComparator(SessionKey k) {
+            byte[] b = k.getData();
+            // we use the first half of the random key in ProfileOrganizer.getSubTier(),
+            // so use the last half here
+            k0 = DataHelper.fromLong8(b, 16);
+            k1 = DataHelper.fromLong8(b, 24);
         }
 
         public int compare(Hash l, Hash r) {
-            System.arraycopy(l.getData(), 0, data, 0, Hash.HASH_LENGTH);
-            byte[] tb = tmp.getData();
-            // don't use caching version of calculateHash()
-            SHA256Generator.getInstance().calculateHash(data, 0, 2*Hash.HASH_LENGTH, tb, 0);
-            BigInteger ll = HashDistance.getDistance(_hash, tmp);
-            System.arraycopy(r.getData(), 0, data, 0, Hash.HASH_LENGTH);
-            SHA256Generator.getInstance().calculateHash(data, 0, 2*Hash.HASH_LENGTH, tb, 0);
-            BigInteger rr = HashDistance.getDistance(_hash, tmp);
-            return ll.compareTo(rr);
+            long lh = SipHashInline.hash24(k0, k1, l.getData());
+            long rh = SipHashInline.hash24(k0, k1, r.getData());
+            if (lh > rh)
+                return 1;
+            if (lh < rh)
+                return -1;
+            return 0;
         }
     }
 
@@ -682,21 +516,24 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
      *  Connectivity check.
      *  Check that each hop can connect to the next, including us.
      *  Check that the OBEP is not IPv6-only, and the IBGW is
-     *  not hidden or IPv6-only.
+     *  reachable and not hidden or IPv6-only.
      *  Tells the profile manager to blame the hop, and returns false on failure.
      *
      *  @param tunnel ENDPOINT FIRST, GATEWAY LAST!!!!, length 2 or greater
      *  @return ok
      *  @since 0.9.34
      */
-    protected boolean checkTunnel(boolean isInbound, List<Hash> tunnel) {
+    protected boolean checkTunnel(boolean isInbound, boolean isExploratory, List<Hash> tunnel) {
         if (!checkTunnel(tunnel))
             return false;
+        // client OBEP/IBGW checks now in CPS
+        if (!isExploratory)
+            return true;
         if (isInbound) {
             Hash h = tunnel.get(tunnel.size() - 1);
             if (!allowAsIBGW(h)) {
                 if (log.shouldWarn())
-                    log.warn("Picked IPv6-only or hidden peer for IBGW: " + h);
+                    log.warn("Picked IPv6-only or unreachable peer for IBGW: " + h);
                 // treat as a timeout in the profile
                 // tunnelRejected() would set the last heard from time
                 ctx.profileManager().tunnelTimedOut(h);
@@ -748,4 +585,119 @@ public abstract class TunnelPeerSelector extends ConnectChecker {
         }
         return rv;
     }
+
+    /**
+     *  A Set of Hashes that automatically adds to the
+     *  Set in the contains() check.
+     *
+     *  So we don't need to generate the exclude set up front.
+     *
+     *  @since 0.9.58
+     */
+    protected class Excluder extends ExcluderBase {
+        private final boolean _isIn, _isExpl;
+
+        /**
+         *  Automatically adds selectPeersInTooManyTunnels(), unless i2np.allowLocal.
+         */
+        public Excluder(boolean isInbound, boolean isExploratory) {
+            super(ctx.getBooleanProperty("i2np.allowLocal") ? new HashSet<Hash>()
+                                                            : ctx.tunnelManager().selectPeersInTooManyTunnels());
+            _isIn = isInbound;
+            _isExpl = isExploratory;
+        }
+
+        /**
+         *  Does not add selectPeersInTooManyTunnels().
+         *  Makes a copy of toAdd
+         *
+         *  @param toAdd initial contents, copied
+         */
+        public Excluder(boolean isInbound, boolean isExploratory, Set<Hash> toAdd) {
+            super(new HashSet<Hash>(toAdd));
+            _isIn = isInbound;
+            _isExpl = isExploratory;
+        }
+
+        /**
+         *  Overridden to automatically check our exclusion criteria
+         *  and add the Hash to the set if the criteria are met.
+         *
+         *  @param o a Hash
+         *  @return true if peer should be excluded
+         */
+        @Override
+        public boolean contains(Object o) {
+            if (s.contains(o))
+                return true;
+            Hash h = (Hash) o;
+            if (shouldExclude(h, _isIn, _isExpl)) {
+                s.add(h);
+                //if (log.shouldDebug())
+                //    log.debug("TPS exclude " + h.toBase64());
+                return true;
+            }
+            return false;
+        }
+    }
+
+    /**
+     *  Only for hidden mode and other tough situations.
+     *  See checkClosestHop boolean.
+     *  Not for hidden inbound; use SANFP instead.
+     *
+     *  @since 0.9.58
+     */
+    private class ClosestHopExcluder extends ExcluderBase {
+        private final boolean isIn;
+        private final int ourMask;
+
+        /**
+         *  Automatically check if peer can connect to us (for inbound)
+         *  or we can connect to it (for outbound)
+         *  and add the Hash to the set if not.
+         *
+         *  @param set not copied, contents will be modified by all methods
+         */
+        public ClosestHopExcluder(boolean isInbound, Set<Hash> set) {
+            super(set);
+            isIn = isInbound;
+            RouterInfo ri = ctx.router().getRouterInfo();
+            if (ri != null)
+                ourMask = isInbound ? getInboundMask(ri) : getOutboundMask(ri);
+            else
+                ourMask = 0xff;
+        }
+
+        /**
+         *  Automatically check if peer can connect to us (for inbound)
+         *  or we can connect to it (for outbound)
+         *  and add the Hash to the set if not.
+         *
+         *  @param o a Hash
+         *  @return true if peer should be excluded
+         */
+        public boolean contains(Object o) {
+            if (s.contains(o))
+                return true;
+            Hash h = (Hash) o;
+            if (ctx.commSystem().isEstablished(h))
+                return false;
+            boolean canConnect;
+            RouterInfo peer = (RouterInfo) ctx.netDb().lookupLocallyWithoutValidation(h);
+            if (peer == null) {
+                canConnect = false;
+            } else if (isIn) {
+                canConnect = canConnect(peer, ourMask);
+            } else {
+                canConnect = canConnect(ourMask, peer);
+            }
+            if (!canConnect) {
+                s.add(h);
+                //if (log.shouldDebug())
+                //    log.debug("TPS closest exclude "  h.toBase64());
+            }
+            return !canConnect;
+        }
+    }
 }
