@@ -10,6 +10,7 @@ import java.util.List;
 import java.util.Properties;
 import java.util.Set;
 import java.util.TreeSet;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
@@ -47,6 +48,7 @@ public class TunnelPool {
     private long _lastLifetimeProcessed;
     private final String _rateName;
     private final long _firstInstalled;
+    private final AtomicInteger _consecutiveBuildTimeouts = new AtomicInteger();
 
     private static final int TUNNEL_LIFETIME = 10*60*1000;
     /** if less than one success in this many, reduce quantity (exploratory only) */
@@ -112,7 +114,7 @@ public class TunnelPool {
             }
 
             if (ls != null)
-                _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
+                requestLeaseSet(ls);
         }
         _context.statManager().createRequiredRateStat(_rateName,
                                "Tunnel Bandwidth (Bytes/sec)", "Tunnels", 
@@ -127,6 +129,7 @@ public class TunnelPool {
         synchronized (_inProgress) {
             _inProgress.clear();
         }
+        _consecutiveBuildTimeouts.set(0);
     }
 
     /** 
@@ -163,6 +166,7 @@ public class TunnelPool {
     private TunnelInfo selectTunnel(boolean allowRecurseOnFail) {
         boolean avoidZeroHop = !_settings.getAllowZeroHop();
         
+        long now = _context.clock().now();
         synchronized (_tunnels) {
             if (_tunnels.isEmpty()) {
                 if (_log.shouldLog(Log.WARN))
@@ -178,7 +182,7 @@ public class TunnelPool {
                         if (_lastSelectedIdx >= _tunnels.size())
                             _lastSelectedIdx = 0;
                         TunnelInfo info = _tunnels.get(_lastSelectedIdx);
-                        if ( (info.getLength() > 1) && (info.getExpiration() > _context.clock().now()) ) {
+                        if (info.getLength() > 1 && info.getExpiration() > now) {
                             // avoid outbound tunnels where the 1st hop is backlogged
                             if (_settings.isInbound() || !_context.commSystem().isBacklogged(info.getPeer(1))) {
                                 return info;
@@ -198,7 +202,7 @@ public class TunnelPool {
                 // randomly
                 for (int i = 0; i < _tunnels.size(); i++) {
                     TunnelInfo info = _tunnels.get(i);
-                    if (info.getExpiration() > _context.clock().now()) {
+                    if (info.getExpiration() > now) {
                         // avoid outbound tunnels where the 1st hop is backlogged
                         if (_settings.isInbound() || info.getLength() <= 1 ||
                             !_context.commSystem().isBacklogged(info.getPeer(1))) {
@@ -240,12 +244,13 @@ public class TunnelPool {
     TunnelInfo selectTunnel(Hash closestTo) {
         boolean avoidZeroHop = !_settings.getAllowZeroHop();
         TunnelInfo rv = null;
+        long now = _context.clock().now();
         synchronized (_tunnels) {
             if (!_tunnels.isEmpty()) {
                 if (_tunnels.size() > 1)
                     Collections.sort(_tunnels, new TunnelInfoComparator(closestTo, avoidZeroHop));
                 for (TunnelInfo info : _tunnels) {
-                    if (info.getExpiration() > _context.clock().now()) {
+                    if (info.getExpiration() > now) {
                         rv = info;
                         break;
                     }
@@ -261,6 +266,9 @@ public class TunnelPool {
         return rv;
     }
     
+    /**
+     *  @param gatewayId for inbound, the GW rcv tunnel ID; for outbound, the GW send tunnel ID.
+     */
     public TunnelInfo getTunnel(TunnelId gatewayId) {
         synchronized (_tunnels) {
             for (int i = 0; i < _tunnels.size(); i++) {
@@ -326,8 +334,28 @@ public class TunnelPool {
         if (_settings.getLength() == 0 && _settings.getLengthVariance() == 0)
             return 1;
         int rv = _settings.getTotalQuantity();
-        if (!_settings.isExploratory())
+        if (!_settings.isExploratory()) {
+            if (rv <= 1)
+                return rv;
+            // throttle client tunnel builds in times of congestion
+            int fails = _consecutiveBuildTimeouts.get();
+            if (fails > 4) {
+                if (fails > 12) {
+                    rv = 1;
+                    if (_log.shouldWarn())
+                        _log.warn("Limit to 1 tunnel after " + fails + " consec. build timeouts on " + this);
+                } else if (fails > 8) {
+                    rv = Math.max(1, rv / 3);
+                    if (_log.shouldWarn())
+                        _log.warn("Limit to " + rv + " tunnels after " + fails + " consec. build timeouts on " + this);
+                } else if (rv > 2) {
+                    rv--;
+                    if (_log.shouldWarn())
+                        _log.warn("Limit to " + rv + " tunnels after " + fails + " consec. build timeouts on " + this);
+                }
+            }
             return rv;
+        }
         // TODO high-bw non-ff also
         if (_context.netDb().floodfillEnabled() &&
             _context.router().getUptime() > 5*60*1000) {
@@ -449,7 +477,7 @@ public class TunnelPool {
     /**
      *  Add to the pool.
      */
-    void addTunnel(TunnelInfo info) {
+    protected void addTunnel(TunnelInfo info) {
         if (_log.shouldLog(Log.DEBUG))
             _log.debug(toString() + ": Adding tunnel " + info /* , new Exception("Creator") */ );
         LeaseSet ls = null;
@@ -460,7 +488,7 @@ public class TunnelPool {
         }
         
         if (ls != null)
-            _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
+            requestLeaseSet(ls);
     }
     
     /**
@@ -492,7 +520,7 @@ public class TunnelPool {
         
         if (_alive && _settings.isInbound() && !_settings.isExploratory()) {
             if (ls != null) {
-                _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
+                requestLeaseSet(ls);
             } else {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn(toString() + ": unable to build a new leaseSet on removal (" + remaining 
@@ -533,8 +561,6 @@ public class TunnelPool {
      *  Remove the tunnel.
      */
     private void fail(TunnelInfo cfg) {
-        if (_log.shouldLog(Log.WARN))
-            _log.warn(toString() + ": Tunnel failed: " + cfg);
         LeaseSet ls = null;
         synchronized (_tunnels) {
             boolean removed = _tunnels.remove(cfg);
@@ -543,7 +569,9 @@ public class TunnelPool {
             if (_settings.isInbound() && !_settings.isExploratory())
                 ls = locked_buildNewLeaseSet();
         }
-        
+
+        if (_log.shouldLog(Log.WARN))
+            _log.warn(toString() + ": Tunnel failed: " + cfg);
         _manager.tunnelFailed();
         
         _lifetimeProcessed += cfg.getProcessedMessagesCount();
@@ -551,7 +579,7 @@ public class TunnelPool {
         
         if (_settings.isInbound() && !_settings.isExploratory()) {
             if (ls != null) {
-                _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
+                requestLeaseSet(ls);
             }
         }
     }
@@ -590,7 +618,7 @@ public class TunnelPool {
         long et = now - _lastRateUpdate;
         if (et > 2*60*1000) {
             long bw = 1024 * (_lifetimeProcessed - _lastLifetimeProcessed) * 1000 / et;   // Bps
-            _context.statManager().addRateData(_rateName, bw, 0);
+            _context.statManager().addRateData(_rateName, bw);
             _lastRateUpdate = now;
             _lastLifetimeProcessed = _lifetimeProcessed;
         }
@@ -606,13 +634,28 @@ public class TunnelPool {
                 ls = locked_buildNewLeaseSet();
             }
             if (ls != null) {
-                _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
-                Set<Hash> aliases = _settings.getAliases();
-                if (aliases != null && !aliases.isEmpty()) {
-                    for (Hash h : aliases) {
-                        _context.clientManager().requestLeaseSet(h, ls);
-                    }
-                }
+                requestLeaseSet(ls);
+            }
+        }
+    }
+
+    /**
+     *  Request lease set from client for the primary and all aliases.
+     *
+     *  @param ls non-null
+     *  @since 0.9.49
+     */
+    private void requestLeaseSet(LeaseSet ls) {
+        _context.clientManager().requestLeaseSet(_settings.getDestination(), ls);
+        Set<Hash> aliases = _settings.getAliases();
+        if (aliases != null && !aliases.isEmpty()) {
+            for (Hash h : aliases) {
+                 // don't corrupt other requests
+                 LeaseSet ls2 = new LeaseSet();
+                 for (int i = 0; i < ls.getLeaseCount(); i++) {
+                     ls2.addLease(ls.getLease(i));
+                 }
+                _context.clientManager().requestLeaseSet(h, ls2);
             }
         }
     }
@@ -709,6 +752,8 @@ public class TunnelPool {
     /**
      * Build a leaseSet with the required tunnels that aren't about to expire.
      * Caller must synchronize on _tunnels.
+     * The returned LeaseSet will be incomplete; it will not have the destination
+     * set and will not be signed. Only the leases will be included.
      *
      * @return null on failure
      */
@@ -718,8 +763,8 @@ public class TunnelPool {
 
         int wanted = Math.min(_settings.getQuantity(), LeaseSet.MAX_LEASES);
         if (_tunnels.size() < wanted) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn(toString() + ": Not enough tunnels (" + _tunnels.size() + ", wanted " + wanted + ")");
+            if (_log.shouldInfo())
+                _log.info(toString() + ": Not enough tunnels (" + _tunnels.size() + ", wanted " + wanted + ")");
             // see comment below
             if (_tunnels.isEmpty())
                 return null;
@@ -780,8 +825,8 @@ public class TunnelPool {
         // So we will generate a succession of leases at startup. That's OK.
         // Do we want a config option for this, or are there times when we shouldn't do this?
         if (leases.size() < wanted) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn(toString() + ": Not enough leases (" + leases.size() + ", wanted " + wanted + ")");
+            if (_log.shouldInfo())
+                _log.info(toString() + ": Not enough leases (" + leases.size() + ", wanted " + wanted + ")");
             if (leases.isEmpty())
                 return null;
         }
@@ -926,7 +971,7 @@ public class TunnelPool {
                        + " soon " + expireSoon + " later " + expireLater
                        + " std " + wanted + " inProgress " + inProgress + " fallback " + fallback 
                        + " for " + toString());
-            _context.statManager().addRateData(rateName, rv + inProgress, 0);
+            _context.statManager().addRateData(rateName, rv + inProgress);
             return rv;
         }
 
@@ -979,7 +1024,7 @@ public class TunnelPool {
         
         int rv = countHowManyToBuild(allowZeroHop, expire30s, expire90s, expire150s, expire210s, expire270s, 
                                    expireLater, wanted, inProgress, fallback);
-        _context.statManager().addRateData(rateName, (rv > 0 || inProgress > 0) ? 1 : 0, 0);
+        _context.statManager().addRateData(rateName, (rv > 0 || inProgress > 0) ? 1 : 0);
         return rv;
 
     }
@@ -1093,6 +1138,9 @@ public class TunnelPool {
     }
     
     /**
+     *  This only sets the peers and creation/expiration times in the configuration.
+     *  For the crypto, see BuildRequestor and BuildMessageGenerator.
+     *
      *  @return null on failure
      */
     PooledTunnelCreatorConfig configureNewTunnel() { return configureNewTunnel(false); }
@@ -1161,8 +1209,7 @@ public class TunnelPool {
             HopConfig hop = cfg.getConfig(j);
             hop.setCreation(now);
             hop.setExpiration(expiration);
-            hop.setIVKey(_context.keyGenerator().generateSessionKey());
-            hop.setLayerKey(_context.keyGenerator().generateSessionKey());
+            // IV and Layer key now set in BuildRequestor.createTunnelBuildMessage()
             // tunnelIds will be updated during building, and as the creator, we
             // don't need to worry about prev/next hop
         }
@@ -1178,15 +1225,103 @@ public class TunnelPool {
         }
         return cfg;
     }
-    
+
     /**
-     *  Remove from the _inprogress list
+     *  Remove from the _inprogress list and call addTunnel() if result is SUCCESS.
+     *  Updates consecutive build timeout count.
+     *
+     *  @since 0.9.53 added result parameter
      */
-    void buildComplete(PooledTunnelCreatorConfig cfg) {
+    void buildComplete(PooledTunnelCreatorConfig cfg, BuildExecutor.Result result) {
+        if (cfg.getTunnelPool() != this) {
+            _log.error("Wrong pool " + cfg + " for " + this, new Exception());
+            return;
+        }
+
         synchronized (_inProgress) { _inProgress.remove(cfg); }
-        //_manager.buildComplete(cfg);
+
+        switch (result) {
+            case SUCCESS:
+                _consecutiveBuildTimeouts.set(0);
+                addTunnel(cfg);
+                updatePairedProfile(cfg, true);
+                break;
+
+            case REJECT:
+            case BAD_RESPONSE:
+            case DUP_ID:
+                _consecutiveBuildTimeouts.set(0);
+                updatePairedProfile(cfg, true);
+                break;
+
+            case TIMEOUT:
+                _consecutiveBuildTimeouts.incrementAndGet();
+                updatePairedProfile(cfg, false);
+                break;
+
+            case OTHER_FAILURE:
+            default:
+                break;
+        }
     }
-    
+
+    /**
+     *  @since 0.9.53
+     */
+    int getConsecutiveBuildTimeouts() {
+        return _consecutiveBuildTimeouts.get();
+    }
+
+    /**
+     *  Update the paired tunnel profiles by treating the build as a tunnel test
+     *
+     *  @param cfg the build for this tunnel, to lookup the paired tunnel
+     *  @param success did the paired tunnel pass the message through
+     *  @since 0.9.53
+     */
+    private void updatePairedProfile(PooledTunnelCreatorConfig cfg, boolean success) {
+       // will be null if paired tunnel is 0-hop
+       TunnelId pairedGW = cfg.getPairedGW();
+       if (pairedGW == null)
+           return;
+       if (!success) {
+           // don't blame the paired tunnel for exploratory build failures
+           if (_settings.isExploratory())
+               return;
+           // don't blame the paired tunnel if there might be some other problem
+           if (getConsecutiveBuildTimeouts() > 3)
+               return;
+       }
+       TunnelPool pool;
+       PooledTunnelCreatorConfig paired = null;
+       if (!_settings.isExploratory()) {
+           Hash dest = _settings.getDestination();
+           if (_settings.isInbound())
+               pool = _manager.getOutboundPool(dest);
+           else
+               pool = _manager.getInboundPool(dest);
+           if (pool != null)
+               paired = (PooledTunnelCreatorConfig) pool.getTunnel(pairedGW);
+       }
+       if (paired == null) { // not found or exploratory
+           if (_settings.isInbound())
+               pool = _manager.getOutboundExploratoryPool();
+           else
+               pool = _manager.getInboundExploratoryPool();
+           paired = (PooledTunnelCreatorConfig) pool.getTunnel(pairedGW);
+       }
+       if (paired != null && paired.getLength() > 1) {
+           //_log.info("Updating paired cfg, success? " + success + ' ' + paired);
+           if (success) {
+               long requestedOn = cfg.getExpiration() - 10*60*1000;
+               int rtt = (int) (_context.clock().now() - requestedOn);
+               paired.testSuccessful(rtt);
+           } else {
+               paired.tunnelFailed();
+           }
+       }
+    }
+
     @Override
     public String toString() {
         if (_settings.isExploratory()) {
@@ -1203,7 +1338,7 @@ public class TunnelPool {
             if (_settings.getDestinationNickname() != null)
                 rv.append(_settings.getDestinationNickname());
             else
-                rv.append(_settings.getDestination().toBase64().substring(0,4));
+                rv.append(_settings.getDestination().toBase64(), 0, 4);
             return rv.toString();
         }
             
