@@ -24,7 +24,7 @@ import net.i2p.router.TunnelPoolSettings;
 import net.i2p.router.tunnel.TunnelDispatcher;
 import net.i2p.util.I2PThread;
 import net.i2p.util.Log;
-import net.i2p.util.ObjectCounter;
+import net.i2p.util.ObjectCounterUnsafe;
 import net.i2p.util.SimpleTimer;
 
 /**
@@ -54,12 +54,6 @@ public class TunnelPoolManager implements TunnelManagerFacade {
         _context = ctx;
         _log = ctx.logManager().getLog(TunnelPoolManager.class);
         
-        //HandlerJobBuilder builder = new HandleTunnelCreateMessageJob.Builder(ctx);
-        //ctx.inNetMessagePool().registerHandlerJobBuilder(TunnelCreateMessage.MESSAGE_TYPE, builder);
-        //HandlerJobBuilder b = new TunnelMessageHandlerBuilder(ctx);
-        //ctx.inNetMessagePool().registerHandlerJobBuilder(TunnelGatewayMessage.MESSAGE_TYPE, b);
-        //ctx.inNetMessagePool().registerHandlerJobBuilder(TunnelDataMessage.MESSAGE_TYPE, b);
-
         _clientInboundPools = new ConcurrentHashMap<Hash, TunnelPool>(4);
         _clientOutboundPools = new ConcurrentHashMap<Hash, TunnelPool>(4);
         _clientPeerSelector = new ClientPeerSelector(ctx);
@@ -119,7 +113,7 @@ public class TunnelPoolManager implements TunnelManagerFacade {
 
     /**
      * Pick a random inbound tunnel from the given destination's pool.
-     * Warning - selectOutboundTunnel(Hash, Hash) is preferred.
+     * Warning - selectInboundTunnel(Hash, Hash) is preferred.
      *
      * @param destination if null, returns inbound exploratory tunnel
      * @return null if none
@@ -130,8 +124,8 @@ public class TunnelPoolManager implements TunnelManagerFacade {
         if (pool != null) {
             return pool.selectTunnel();
         }
-        if (_log.shouldLog(Log.ERROR))
-            _log.error("Want the inbound tunnel for " + destination.toBase32() +
+        if (_log.shouldWarn())
+            _log.warn("Want the inbound tunnel for " + destination.toBase32() +
                      " but there isn't a pool?", new Exception());
         return null;
     }
@@ -206,8 +200,8 @@ public class TunnelPoolManager implements TunnelManagerFacade {
         if (pool != null) {
             return pool.selectTunnel(closestTo);
         }
-        if (_log.shouldLog(Log.ERROR))
-            _log.error("Want the inbound tunnel for " + destination.toBase32() +
+        if (_log.shouldWarn())
+            _log.warn("Want the inbound tunnel for " + destination.toBase32() +
                      " but there isn't a pool?", new Exception());
         return null;
     }
@@ -335,6 +329,8 @@ public class TunnelPoolManager implements TunnelManagerFacade {
     }
 
     public boolean isValidTunnel(Hash client, TunnelInfo tunnel) {
+        if (tunnel.getTunnelFailed())
+            return false;
         if (tunnel.getExpiration() < _context.clock().now())
             return false;
         TunnelPool pool;
@@ -571,7 +567,7 @@ public class TunnelPoolManager implements TunnelManagerFacade {
     void buildComplete(PooledTunnelCreatorConfig cfg) {
         if (cfg.getLength() > 1 &&
             !_context.router().gracefulShutdownInProgress() &&
-            (!_context.getBooleanPropertyDefaultTrue("router.disableTunnelTesting") ||
+            (!_context.getBooleanProperty("router.disableTunnelTesting") ||
              _context.router().isHidden() ||
              _context.router().getRouterInfo().getAddressCount() <= 0)) {
             TunnelPool pool = cfg.getTunnelPool();
@@ -649,7 +645,7 @@ public class TunnelPoolManager implements TunnelManagerFacade {
     }
 
     /** @return total number of non-fallback expl. + client tunnels */
-    private int countTunnelsPerPeer(ObjectCounter<Hash> lc) {
+    private int countTunnelsPerPeer(ObjectCounterUnsafe<Hash> lc) {
         List<TunnelPool> pools = new ArrayList<TunnelPool>();
         listPools(pools);
         int tunnelCount = 0;
@@ -685,7 +681,7 @@ public class TunnelPoolManager implements TunnelManagerFacade {
      *  @return Set of peers that should not be allowed in another tunnel
      */
     public Set<Hash> selectPeersInTooManyTunnels() {
-        ObjectCounter<Hash> lc = new ObjectCounter<Hash>();
+        ObjectCounterUnsafe<Hash> lc = new ObjectCounterUnsafe<Hash>();
         int tunnelCount = countTunnelsPerPeer(lc);
         Set<Hash> rv = new HashSet<Hash>();
         if (tunnelCount >= 4 && _context.router().getUptime() > 10*60*1000) {
